DEFAULT_SYSTEM_CONFIG_FILENAMES = (
    "/etc/harp.yaml",
    "/etc/harp.yml",
)

DEFAULT_APPLICATIONS = (
    "http_client",
    "proxy",
    "storage",
    "dashboard",
    "harp_apps.contrib.sentry",
    "telemetry",
    "janitor",
    "rules",
)
