"""make message headers nullable

Revision ID: 29d104241dae
Revises: 4bdd9e1d790d
Create Date: 2025-10-30 11:25:43.000000

"""

from typing import Sequence, Union

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "29d104241dae"
down_revision: Union[str, None] = "4bdd9e1d790d"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("messages", "headers", existing_type=sa.VARCHAR(length=40), nullable=True, existing_nullable=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("messages", "headers", existing_type=sa.VARCHAR(length=40), nullable=False, existing_nullable=True)
    # ### end Alembic commands ###
