import"./react-C1sqhVEE.js";import{d as s,_ as i,e as r}from"./Settings-C7_TiiRH.js";import{j as l}from"./ui-cEB64NN_.js";import{u as n,i as m,a as p,b as d,c as u,d as c,e as h,f}from"./echarts-gihz4qUC.js";n([m,p,d,u,c,h,f]);const g=t=>{const a=t[0];let e=`<strong>${a.name}</strong><br/>`;return e+=`<small>${a.value}</small>`,e},y=t=>new Date(t).toLocaleTimeString(void 0,{hour:"2-digit",minute:"2-digit"}),w=({data:t,color:a=void 0})=>{const e={...s,grid:{...s.grid,left:8,right:8,top:64,bottom:24},xAxis:{type:"category",data:t.map(o=>y(o.datetime))},yAxis:{axisLabel:{show:!1}},tooltip:{...s.tooltip,formatter:g},series:[{data:t.map(o=>o.value),type:"bar",smooth:!1,symbol:"none",lineStyle:{width:4},color:a}]};return l(i,{echarts:r,option:e,notMerge:!0,lazyUpdate:!0,opts:{height:240,width:"auto"}})};export{w as default};
