"""
This application implements the :doc:`Rules Engine </apps/rules/index>`.

.. versionadded:: 0.6

.. warning:: Early prototype. The API may change, a lot.

.. todo::

    Syntax and logic error reporting at compile time. Especially, can we detect invalid variable tentatively used in
    user scripts?

"""

__title__ = "Rules Engine"
