import"./react-C1sqhVEE.js";import{O as i,a as e,c as o}from"./Pane-kIG3Tumw.js";import"./reactQuery-CcoxLypZ.js";import{u as m,T as n}from"./TransactionDetailOnQuerySuccess-CxaclI7p.js";import{j as t}from"./ui-cEB64NN_.js";import{d as u}from"./reactRouter-B7BPABFu.js";import"./sentry-Cc0xHm77.js";import"./index-D8wxMOat.js";import"./syntaxHighlighter-C6zraU67.js";import"./dateFns-DhBcwMxz.js";function O(){const{id:a}=u(),r=m(a);return t(o,{title:t(e,{title:"Transaction details"})},t(i,{query:r},s=>t(n,{query:s})))}export{O as default};
