"""
The Typing (:mod:`harp.typing`) package provides contains everything related to types and typing.

Contents
--------

"""

from .global_settings import GlobalSettings
from .signs import Default, Maybe, NotSet

__title__ = "Typing"

__all__ = [
    "Default",
    "GlobalSettings",
    "Maybe",
    "NotSet",
]
