/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */var Kl=function(r,t){return Kl=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,i){e.__proto__=i}||function(e,i){for(var n in i)Object.prototype.hasOwnProperty.call(i,n)&&(e[n]=i[n])},Kl(r,t)};function k(r,t){if(typeof t!="function"&&t!==null)throw new TypeError("Class extends value "+String(t)+" is not a constructor or null");Kl(r,t);function e(){this.constructor=r}r.prototype=t===null?Object.create(t):(e.prototype=t.prototype,new e)}var Ql=function(){return Ql=Object.assign||function(t){for(var e,i=1,n=arguments.length;i<n;i++){e=arguments[i];for(var a in e)Object.prototype.hasOwnProperty.call(e,a)&&(t[a]=e[a])}return t},Ql.apply(this,arguments)};function k0(r,t){var e={};for(var i in r)Object.prototype.hasOwnProperty.call(r,i)&&t.indexOf(i)<0&&(e[i]=r[i]);if(r!=null&&typeof Object.getOwnPropertySymbols=="function")for(var n=0,i=Object.getOwnPropertySymbols(r);n<i.length;n++)t.indexOf(i[n])<0&&Object.prototype.propertyIsEnumerable.call(r,i[n])&&(e[i[n]]=r[i[n]]);return e}function B0(r,t,e,i){var n=arguments.length,a=n<3?t:i===null?i=Object.getOwnPropertyDescriptor(t,e):i,o;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")a=Reflect.decorate(r,t,e,i);else for(var s=r.length-1;s>=0;s--)(o=r[s])&&(a=(n<3?o(a):n>3?o(t,e,a):o(t,e))||a);return n>3&&a&&Object.defineProperty(t,e,a),a}function N0(r,t){return function(e,i){t(e,i,r)}}function F0(r,t){if(typeof Reflect=="object"&&typeof Reflect.metadata=="function")return Reflect.metadata(r,t)}function z0(r,t,e,i){function n(a){return a instanceof e?a:new e(function(o){o(a)})}return new(e||(e=Promise))(function(a,o){function s(f){try{u(i.next(f))}catch(h){o(h)}}function l(f){try{u(i.throw(f))}catch(h){o(h)}}function u(f){f.done?a(f.value):n(f.value).then(s,l)}u((i=i.apply(r,t||[])).next())})}function V0(r,t){var e={label:0,sent:function(){if(a[0]&1)throw a[1];return a[1]},trys:[],ops:[]},i,n,a,o;return o={next:s(0),throw:s(1),return:s(2)},typeof Symbol=="function"&&(o[Symbol.iterator]=function(){return this}),o;function s(u){return function(f){return l([u,f])}}function l(u){if(i)throw new TypeError("Generator is already executing.");for(;e;)try{if(i=1,n&&(a=u[0]&2?n.return:u[0]?n.throw||((a=n.return)&&a.call(n),0):n.next)&&!(a=a.call(n,u[1])).done)return a;switch(n=0,a&&(u=[u[0]&2,a.value]),u[0]){case 0:case 1:a=u;break;case 4:return e.label++,{value:u[1],done:!1};case 5:e.label++,n=u[1],u=[0];continue;case 7:u=e.ops.pop(),e.trys.pop();continue;default:if(a=e.trys,!(a=a.length>0&&a[a.length-1])&&(u[0]===6||u[0]===2)){e=0;continue}if(u[0]===3&&(!a||u[1]>a[0]&&u[1]<a[3])){e.label=u[1];break}if(u[0]===6&&e.label<a[1]){e.label=a[1],a=u;break}if(a&&e.label<a[2]){e.label=a[2],e.ops.push(u);break}a[2]&&e.ops.pop(),e.trys.pop();continue}u=t.call(r,e)}catch(f){u=[6,f],n=0}finally{i=a=0}if(u[0]&5)throw u[1];return{value:u[0]?u[1]:void 0,done:!0}}}var nf=Object.create?function(r,t,e,i){i===void 0&&(i=e),Object.defineProperty(r,i,{enumerable:!0,get:function(){return t[e]}})}:function(r,t,e,i){i===void 0&&(i=e),r[i]=t[e]};function H0(r,t){for(var e in r)e!=="default"&&!Object.prototype.hasOwnProperty.call(t,e)&&nf(t,r,e)}function jl(r){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&r[t],i=0;if(e)return e.call(r);if(r&&typeof r.length=="number")return{next:function(){return r&&i>=r.length&&(r=void 0),{value:r&&r[i++],done:!r}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")}function yp(r,t){var e=typeof Symbol=="function"&&r[Symbol.iterator];if(!e)return r;var i=e.call(r),n,a=[],o;try{for(;(t===void 0||t-- >0)&&!(n=i.next()).done;)a.push(n.value)}catch(s){o={error:s}}finally{try{n&&!n.done&&(e=i.return)&&e.call(i)}finally{if(o)throw o.error}}return a}function G0(){for(var r=[],t=0;t<arguments.length;t++)r=r.concat(yp(arguments[t]));return r}function W0(){for(var r=0,t=0,e=arguments.length;t<e;t++)r+=arguments[t].length;for(var i=Array(r),n=0,t=0;t<e;t++)for(var a=arguments[t],o=0,s=a.length;o<s;o++,n++)i[n]=a[o];return i}function U0(r,t,e){if(e||arguments.length===2)for(var i=0,n=t.length,a;i<n;i++)(a||!(i in t))&&(a||(a=Array.prototype.slice.call(t,0,i)),a[i]=t[i]);return r.concat(a||t)}function zn(r){return this instanceof zn?(this.v=r,this):new zn(r)}function Y0(r,t,e){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var i=e.apply(r,t||[]),n,a=[];return n={},o("next"),o("throw"),o("return"),n[Symbol.asyncIterator]=function(){return this},n;function o(c){i[c]&&(n[c]=function(v){return new Promise(function(d,y){a.push([c,v,d,y])>1||s(c,v)})})}function s(c,v){try{l(i[c](v))}catch(d){h(a[0][3],d)}}function l(c){c.value instanceof zn?Promise.resolve(c.value.v).then(u,f):h(a[0][2],c)}function u(c){s("next",c)}function f(c){s("throw",c)}function h(c,v){c(v),a.shift(),a.length&&s(a[0][0],a[0][1])}}function X0(r){var t,e;return t={},i("next"),i("throw",function(n){throw n}),i("return"),t[Symbol.iterator]=function(){return this},t;function i(n,a){t[n]=r[n]?function(o){return(e=!e)?{value:zn(r[n](o)),done:n==="return"}:a?a(o):o}:a}}function $0(r){if(!Symbol.asyncIterator)throw new TypeError("Symbol.asyncIterator is not defined.");var t=r[Symbol.asyncIterator],e;return t?t.call(r):(r=typeof jl=="function"?jl(r):r[Symbol.iterator](),e={},i("next"),i("throw"),i("return"),e[Symbol.asyncIterator]=function(){return this},e);function i(a){e[a]=r[a]&&function(o){return new Promise(function(s,l){o=r[a](o),n(s,l,o.done,o.value)})}}function n(a,o,s,l){Promise.resolve(l).then(function(u){a({value:u,done:s})},o)}}function Z0(r,t){return Object.defineProperty?Object.defineProperty(r,"raw",{value:t}):r.raw=t,r}var q0=Object.create?function(r,t){Object.defineProperty(r,"default",{enumerable:!0,value:t})}:function(r,t){r.default=t};function K0(r){if(r&&r.__esModule)return r;var t={};if(r!=null)for(var e in r)e!=="default"&&Object.prototype.hasOwnProperty.call(r,e)&&nf(t,r,e);return q0(t,r),t}function Q0(r){return r&&r.__esModule?r:{default:r}}function j0(r,t,e,i){if(e==="a"&&!i)throw new TypeError("Private accessor was defined without a getter");if(typeof t=="function"?r!==t||!i:!t.has(r))throw new TypeError("Cannot read private member from an object whose class did not declare it");return e==="m"?i:e==="a"?i.call(r):i?i.value:t.get(r)}function J0(r,t,e,i,n){if(i==="m")throw new TypeError("Private method is not writable");if(i==="a"&&!n)throw new TypeError("Private accessor was defined without a setter");if(typeof t=="function"?r!==t||!n:!t.has(r))throw new TypeError("Cannot write private member to an object whose class did not declare it");return i==="a"?n.call(r,e):n?n.value=e:t.set(r,e),e}const KI=Object.freeze(Object.defineProperty({__proto__:null,get __assign(){return Ql},__asyncDelegator:X0,__asyncGenerator:Y0,__asyncValues:$0,__await:zn,__awaiter:z0,__classPrivateFieldGet:j0,__classPrivateFieldSet:J0,__createBinding:nf,__decorate:B0,__exportStar:H0,__extends:k,__generator:V0,__importDefault:Q0,__importStar:K0,__makeTemplateObject:Z0,__metadata:F0,__param:N0,__read:yp,__rest:k0,__spread:G0,__spreadArray:U0,__spreadArrays:W0,__values:jl},Symbol.toStringTag,{value:"Module"}));var af=12,t_="sans-serif",ai=af+"px "+t_,e_=20,r_=100,i_="007LLmW'55;N0500LLLLLLLLLL00NNNLzWW\\\\WQb\\0FWLg\\bWb\\WQ\\WrWWQ000CL5LLFLL0LL**F*gLLLL5F0LF\\FFF5.5N";function n_(r){var t={};if(typeof JSON>"u")return t;for(var e=0;e<r.length;e++){var i=String.fromCharCode(e+32),n=(r.charCodeAt(e)-e_)/r_;t[i]=n}return t}var a_=n_(i_),je={createCanvas:function(){return typeof document<"u"&&document.createElement("canvas")},measureText:function(){var r,t;return function(e,i){if(!r){var n=je.createCanvas();r=n&&n.getContext("2d")}if(r)return t!==i&&(t=r.font=i||ai),r.measureText(e);e=e||"",i=i||ai;var a=/((?:\d+)?\.?\d*)px/.exec(i),o=a&&+a[1]||af,s=0;if(i.indexOf("mono")>=0)s=o*e.length;else for(var l=0;l<e.length;l++){var u=a_[e[l]];s+=u==null?o:u*o}return{width:s}}}(),loadImage:function(r,t,e){var i=new Image;return i.onload=t,i.onerror=e,i.src=r,i}};function o_(r){for(var t in je)r[t]&&(je[t]=r[t])}var mp=Ne(["Function","RegExp","Date","Error","CanvasGradient","CanvasPattern","Image","Canvas"],function(r,t){return r["[object "+t+"]"]=!0,r},{}),_p=Ne(["Int8","Uint8","Uint8Clamped","Int16","Uint16","Int32","Uint32","Float32","Float64"],function(r,t){return r["[object "+t+"Array]"]=!0,r},{}),Yi=Object.prototype.toString,Ko=Array.prototype,s_=Ko.forEach,l_=Ko.filter,of=Ko.slice,u_=Ko.map,xh=(function(){}).constructor,ya=xh?xh.prototype:null,sf="__proto__",f_=2311;function lf(){return f_++}function Qo(){for(var r=[],t=0;t<arguments.length;t++)r[t]=arguments[t];typeof console<"u"&&console.error.apply(console,r)}function U(r){if(r==null||typeof r!="object")return r;var t=r,e=Yi.call(r);if(e==="[object Array]"){if(!Bi(r)){t=[];for(var i=0,n=r.length;i<n;i++)t[i]=U(r[i])}}else if(_p[e]){if(!Bi(r)){var a=r.constructor;if(a.from)t=a.from(r);else{t=new a(r.length);for(var i=0,n=r.length;i<n;i++)t[i]=r[i]}}}else if(!mp[e]&&!Bi(r)&&!Gi(r)){t={};for(var o in r)r.hasOwnProperty(o)&&o!==sf&&(t[o]=U(r[o]))}return t}function J(r,t,e){if(!z(t)||!z(r))return e?U(t):r;for(var i in t)if(t.hasOwnProperty(i)&&i!==sf){var n=r[i],a=t[i];z(a)&&z(n)&&!N(a)&&!N(n)&&!Gi(a)&&!Gi(n)&&!Jl(a)&&!Jl(n)&&!Bi(a)&&!Bi(n)?J(n,a,e):(e||!(i in r))&&(r[i]=U(t[i]))}return r}function h_(r,t){for(var e=r[0],i=1,n=r.length;i<n;i++)e=J(e,r[i],t);return e}function O(r,t){if(Object.assign)Object.assign(r,t);else for(var e in t)t.hasOwnProperty(e)&&e!==sf&&(r[e]=t[e]);return r}function nt(r,t,e){for(var i=ft(t),n=0;n<i.length;n++){var a=i[n];(e?t[a]!=null:r[a]==null)&&(r[a]=t[a])}return r}var v_=je.createCanvas;function ot(r,t){if(r){if(r.indexOf)return r.indexOf(t);for(var e=0,i=r.length;e<i;e++)if(r[e]===t)return e}return-1}function uf(r,t){var e=r.prototype;function i(){}i.prototype=t.prototype,r.prototype=new i;for(var n in e)e.hasOwnProperty(n)&&(r.prototype[n]=e[n]);r.prototype.constructor=r,r.superClass=t}function de(r,t,e){if(r="prototype"in r?r.prototype:r,t="prototype"in t?t.prototype:t,Object.getOwnPropertyNames)for(var i=Object.getOwnPropertyNames(t),n=0;n<i.length;n++){var a=i[n];a!=="constructor"&&(e?t[a]!=null:r[a]==null)&&(r[a]=t[a])}else nt(r,t,e)}function Nt(r){return!r||typeof r=="string"?!1:typeof r.length=="number"}function T(r,t,e){if(r&&t)if(r.forEach&&r.forEach===s_)r.forEach(t,e);else if(r.length===+r.length)for(var i=0,n=r.length;i<n;i++)t.call(e,r[i],i,r);else for(var a in r)r.hasOwnProperty(a)&&t.call(e,r[a],a,r)}function H(r,t,e){if(!r)return[];if(!t)return jo(r);if(r.map&&r.map===u_)return r.map(t,e);for(var i=[],n=0,a=r.length;n<a;n++)i.push(t.call(e,r[n],n,r));return i}function Ne(r,t,e,i){if(r&&t){for(var n=0,a=r.length;n<a;n++)e=t.call(i,e,r[n],n,r);return e}}function wt(r,t,e){if(!r)return[];if(!t)return jo(r);if(r.filter&&r.filter===l_)return r.filter(t,e);for(var i=[],n=0,a=r.length;n<a;n++)t.call(e,r[n],n,r)&&i.push(r[n]);return i}function c_(r,t,e){if(r&&t){for(var i=0,n=r.length;i<n;i++)if(t.call(e,r[i],i,r))return r[i]}}function ft(r){if(!r)return[];if(Object.keys)return Object.keys(r);var t=[];for(var e in r)r.hasOwnProperty(e)&&t.push(e);return t}function d_(r,t){for(var e=[],i=2;i<arguments.length;i++)e[i-2]=arguments[i];return function(){return r.apply(t,e.concat(of.call(arguments)))}}var K=ya&&Y(ya.bind)?ya.call.bind(ya.bind):d_;function kt(r){for(var t=[],e=1;e<arguments.length;e++)t[e-1]=arguments[e];return function(){return r.apply(this,t.concat(of.call(arguments)))}}function N(r){return Array.isArray?Array.isArray(r):Yi.call(r)==="[object Array]"}function Y(r){return typeof r=="function"}function V(r){return typeof r=="string"}function wo(r){return Yi.call(r)==="[object String]"}function ht(r){return typeof r=="number"}function z(r){var t=typeof r;return t==="function"||!!r&&t==="object"}function Jl(r){return!!mp[Yi.call(r)]}function Ft(r){return!!_p[Yi.call(r)]}function Gi(r){return typeof r=="object"&&typeof r.nodeType=="number"&&typeof r.ownerDocument=="object"}function sa(r){return r.colorStops!=null}function Sp(r){return r.image!=null}function p_(r){return Yi.call(r)==="[object RegExp]"}function Vn(r){return r!==r}function oi(){for(var r=[],t=0;t<arguments.length;t++)r[t]=arguments[t];for(var e=0,i=r.length;e<i;e++)if(r[e]!=null)return r[e]}function Q(r,t){return r??t}function Cn(r,t,e){return r??t??e}function jo(r){for(var t=[],e=1;e<arguments.length;e++)t[e-1]=arguments[e];return of.apply(r,t)}function ff(r){if(typeof r=="number")return[r,r,r,r];var t=r.length;return t===2?[r[0],r[1],r[0],r[1]]:t===3?[r[0],r[1],r[2],r[1]]:r}function Ce(r,t){if(!r)throw new Error(t)}function be(r){return r==null?null:typeof r.trim=="function"?r.trim():r.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"")}var wp="__ec_primitive__";function Hn(r){r[wp]=!0}function Bi(r){return r[wp]}var g_=function(){function r(){this.data={}}return r.prototype.delete=function(t){var e=this.has(t);return e&&delete this.data[t],e},r.prototype.has=function(t){return this.data.hasOwnProperty(t)},r.prototype.get=function(t){return this.data[t]},r.prototype.set=function(t,e){return this.data[t]=e,this},r.prototype.keys=function(){return ft(this.data)},r.prototype.forEach=function(t){var e=this.data;for(var i in e)e.hasOwnProperty(i)&&t(e[i],i)},r}(),bp=typeof Map=="function";function y_(){return bp?new Map:new g_}var xp=function(){function r(t){var e=N(t);this.data=y_();var i=this;t instanceof r?t.each(n):t&&T(t,n);function n(a,o){e?i.set(a,o):i.set(o,a)}}return r.prototype.hasKey=function(t){return this.data.has(t)},r.prototype.get=function(t){return this.data.get(t)},r.prototype.set=function(t,e){return this.data.set(t,e),e},r.prototype.each=function(t,e){this.data.forEach(function(i,n){t.call(e,i,n)})},r.prototype.keys=function(){var t=this.data.keys();return bp?Array.from(t):t},r.prototype.removeKey=function(t){this.data.delete(t)},r}();function $(r){return new xp(r)}function Tp(r,t){for(var e=new r.constructor(r.length+t.length),i=0;i<r.length;i++)e[i]=r[i];for(var n=r.length,i=0;i<t.length;i++)e[i+n]=t[i];return e}function la(r,t){var e;if(Object.create)e=Object.create(r);else{var i=function(){};i.prototype=r,e=new i}return t&&O(e,t),e}function hf(r){var t=r.style;t.webkitUserSelect="none",t.userSelect="none",t.webkitTapHighlightColor="rgba(0,0,0,0)",t["-webkit-touch-callout"]="none"}function si(r,t){return r.hasOwnProperty(t)}function At(){}var Mp=180/Math.PI;const QI=Object.freeze(Object.defineProperty({__proto__:null,HashMap:xp,RADIAN_TO_DEGREE:Mp,assert:Ce,bind:K,clone:U,concatArray:Tp,createCanvas:v_,createHashMap:$,createObject:la,curry:kt,defaults:nt,disableUserSelect:hf,each:T,eqNaN:Vn,extend:O,filter:wt,find:c_,guid:lf,hasOwn:si,indexOf:ot,inherits:uf,isArray:N,isArrayLike:Nt,isBuiltInObject:Jl,isDom:Gi,isFunction:Y,isGradientObject:sa,isImagePatternObject:Sp,isNumber:ht,isObject:z,isPrimitive:Bi,isRegExp:p_,isString:V,isStringSafe:wo,isTypedArray:Ft,keys:ft,logError:Qo,map:H,merge:J,mergeAll:h_,mixin:de,noop:At,normalizeCssArray:ff,reduce:Ne,retrieve:oi,retrieve2:Q,retrieve3:Cn,setAsPrimitive:Hn,slice:jo,trim:be},Symbol.toStringTag,{value:"Module"}));var m_=function(){function r(){this.firefox=!1,this.ie=!1,this.edge=!1,this.newEdge=!1,this.weChat=!1}return r}(),__=function(){function r(){this.browser=new m_,this.node=!1,this.wxa=!1,this.worker=!1,this.svgSupported=!1,this.touchEventsSupported=!1,this.pointerEventsSupported=!1,this.domSupported=!1,this.transformSupported=!1,this.transform3dSupported=!1,this.hasGlobalWindow=typeof window<"u"}return r}(),Xr=new __;typeof wx=="object"&&typeof wx.getSystemInfoSync=="function"?(Xr.wxa=!0,Xr.touchEventsSupported=!0):typeof document>"u"&&typeof self<"u"?Xr.worker=!0:typeof navigator>"u"||navigator.userAgent.indexOf("Node.js")===0?(Xr.node=!0,Xr.svgSupported=!0):S_(navigator.userAgent,Xr);function S_(r,t){var e=t.browser,i=r.match(/Firefox\/([\d.]+)/),n=r.match(/MSIE\s([\d.]+)/)||r.match(/Trident\/.+?rv:(([\d.]+))/),a=r.match(/Edge?\/([\d.]+)/),o=/micromessenger/i.test(r);i&&(e.firefox=!0,e.version=i[1]),n&&(e.ie=!0,e.version=n[1]),a&&(e.edge=!0,e.version=a[1],e.newEdge=+a[1].split(".")[0]>18),o&&(e.weChat=!0),t.svgSupported=typeof SVGRect<"u",t.touchEventsSupported="ontouchstart"in window&&!e.ie&&!e.edge,t.pointerEventsSupported="onpointerdown"in window&&(e.edge||e.ie&&+e.version>=11),t.domSupported=typeof document<"u";var s=document.documentElement.style;t.transform3dSupported=(e.ie&&"transition"in s||e.edge||"WebKitCSSMatrix"in window&&"m11"in new WebKitCSSMatrix||"MozPerspective"in s)&&!("OTransition"in s),t.transformSupported=t.transform3dSupported||e.ie&&+e.version>=9}const Z=Xr;var w_=".",Mr="___EC__COMPONENT__CONTAINER___",Cp="___EC__EXTENDED_CLASS___";function Ee(r){var t={main:"",sub:""};if(r){var e=r.split(w_);t.main=e[0]||"",t.sub=e[1]||""}return t}function b_(r){Ce(/^[a-zA-Z0-9_]+([.][a-zA-Z0-9_]+)?$/.test(r),'componentType "'+r+'" illegal')}function x_(r){return!!(r&&r[Cp])}function vf(r,t){r.$constructor=r,r.extend=function(e){var i=this,n;return T_(i)?n=function(a){k(o,a);function o(){return a.apply(this,arguments)||this}return o}(i):(n=function(){(e.$constructor||i).apply(this,arguments)},uf(n,this)),O(n.prototype,e),n[Cp]=!0,n.extend=this.extend,n.superCall=D_,n.superApply=A_,n.superClass=i,n}}function T_(r){return Y(r)&&/^class\s/.test(Function.prototype.toString.call(r))}function Dp(r,t){r.extend=t.extend}var M_=Math.round(Math.random()*10);function C_(r){var t=["__\0is_clz",M_++].join("_");r.prototype[t]=!0,r.isInstance=function(e){return!!(e&&e[t])}}function D_(r,t){for(var e=[],i=2;i<arguments.length;i++)e[i-2]=arguments[i];return this.superClass.prototype[t].apply(r,e)}function A_(r,t,e){return this.superClass.prototype[t].apply(r,e)}function Jo(r){var t={};r.registerClass=function(i){var n=i.type||i.prototype.type;if(n){b_(n),i.prototype.type=n;var a=Ee(n);if(!a.sub)t[a.main]=i;else if(a.sub!==Mr){var o=e(a);o[a.sub]=i}}return i},r.getClass=function(i,n,a){var o=t[i];if(o&&o[Mr]&&(o=n?o[n]:null),a&&!o)throw new Error(n?"Component "+i+"."+(n||"")+" is used but not imported.":i+".type should be specified.");return o},r.getClassesByMainType=function(i){var n=Ee(i),a=[],o=t[n.main];return o&&o[Mr]?T(o,function(s,l){l!==Mr&&a.push(s)}):a.push(o),a},r.hasClass=function(i){var n=Ee(i);return!!t[n.main]},r.getAllClassMainTypes=function(){var i=[];return T(t,function(n,a){i.push(a)}),i},r.hasSubTypes=function(i){var n=Ee(i),a=t[n.main];return a&&a[Mr]};function e(i){var n=t[i.main];return(!n||!n[Mr])&&(n=t[i.main]={},n[Mr]=!0),n}}function Gn(r,t){for(var e=0;e<r.length;e++)r[e][1]||(r[e][1]=r[e][0]);return t=t||!1,function(i,n,a){for(var o={},s=0;s<r.length;s++){var l=r[s][1];if(!(n&&ot(n,l)>=0||a&&ot(a,l)<0)){var u=i.getShallow(l,t);u!=null&&(o[r[s][0]]=u)}}return o}}var I_=[["fill","color"],["shadowBlur"],["shadowOffsetX"],["shadowOffsetY"],["opacity"],["shadowColor"]],L_=Gn(I_),P_=function(){function r(){}return r.prototype.getAreaStyle=function(t,e){return L_(this,t,e)},r}(),Ap=function(){function r(t){this.value=t}return r}(),R_=function(){function r(){this._len=0}return r.prototype.insert=function(t){var e=new Ap(t);return this.insertEntry(e),e},r.prototype.insertEntry=function(t){this.head?(this.tail.next=t,t.prev=this.tail,t.next=null,this.tail=t):this.head=this.tail=t,this._len++},r.prototype.remove=function(t){var e=t.prev,i=t.next;e?e.next=i:this.head=i,i?i.prev=e:this.tail=e,t.next=t.prev=null,this._len--},r.prototype.len=function(){return this._len},r.prototype.clear=function(){this.head=this.tail=null,this._len=0},r}(),ua=function(){function r(t){this._list=new R_,this._maxSize=10,this._map={},this._maxSize=t}return r.prototype.put=function(t,e){var i=this._list,n=this._map,a=null;if(n[t]==null){var o=i.len(),s=this._lastRemovedEntry;if(o>=this._maxSize&&o>0){var l=i.head;i.remove(l),delete n[l.key],a=l.value,this._lastRemovedEntry=l}s?s.value=e:s=new Ap(e),s.key=t,i.insertEntry(s),n[t]=s}return a},r.prototype.get=function(t){var e=this._map[t],i=this._list;if(e!=null)return e!==i.tail&&(i.remove(e),i.insertEntry(e)),e.value},r.prototype.clear=function(){this._list.clear(),this._map={}},r.prototype.len=function(){return this._list.len()},r}(),tu=new ua(50);function E_(r){if(typeof r=="string"){var t=tu.get(r);return t&&t.image}else return r}function Ip(r,t,e,i,n){if(r)if(typeof r=="string"){if(t&&t.__zrImageSrc===r||!e)return t;var a=tu.get(r),o={hostEl:e,cb:i,cbPayload:n};return a?(t=a.image,!ts(t)&&a.pending.push(o)):(t=je.loadImage(r,Th,Th),t.__zrImageSrc=r,tu.put(r,t.__cachedImgObj={image:t,pending:[o]})),t}else return r;else return t}function Th(){var r=this.__cachedImgObj;this.onload=this.onerror=this.__cachedImgObj=null;for(var t=0;t<r.pending.length;t++){var e=r.pending[t],i=e.cb;i&&i(this,e.cbPayload),e.hostEl.dirty()}r.pending.length=0}function ts(r){return r&&r.width&&r.height}function mr(){return[1,0,0,1,0,0]}function fa(r){return r[0]=1,r[1]=0,r[2]=0,r[3]=1,r[4]=0,r[5]=0,r}function cf(r,t){return r[0]=t[0],r[1]=t[1],r[2]=t[2],r[3]=t[3],r[4]=t[4],r[5]=t[5],r}function _r(r,t,e){var i=t[0]*e[0]+t[2]*e[1],n=t[1]*e[0]+t[3]*e[1],a=t[0]*e[2]+t[2]*e[3],o=t[1]*e[2]+t[3]*e[3],s=t[0]*e[4]+t[2]*e[5]+t[4],l=t[1]*e[4]+t[3]*e[5]+t[5];return r[0]=i,r[1]=n,r[2]=a,r[3]=o,r[4]=s,r[5]=l,r}function bo(r,t,e){return r[0]=t[0],r[1]=t[1],r[2]=t[2],r[3]=t[3],r[4]=t[4]+e[0],r[5]=t[5]+e[1],r}function es(r,t,e,i){i===void 0&&(i=[0,0]);var n=t[0],a=t[2],o=t[4],s=t[1],l=t[3],u=t[5],f=Math.sin(e),h=Math.cos(e);return r[0]=n*h+s*f,r[1]=-n*f+s*h,r[2]=a*h+l*f,r[3]=-a*f+h*l,r[4]=h*(o-i[0])+f*(u-i[1])+i[0],r[5]=h*(u-i[1])-f*(o-i[0])+i[1],r}function Lp(r,t,e){var i=e[0],n=e[1];return r[0]=t[0]*i,r[1]=t[1]*n,r[2]=t[2]*i,r[3]=t[3]*n,r[4]=t[4]*i,r[5]=t[5]*n,r}function ha(r,t){var e=t[0],i=t[2],n=t[4],a=t[1],o=t[3],s=t[5],l=e*o-a*i;return l?(l=1/l,r[0]=o*l,r[1]=-a*l,r[2]=-i*l,r[3]=e*l,r[4]=(i*s-o*n)*l,r[5]=(a*n-e*s)*l,r):null}function O_(r){var t=mr();return cf(t,r),t}const jI=Object.freeze(Object.defineProperty({__proto__:null,clone:O_,copy:cf,create:mr,identity:fa,invert:ha,mul:_r,rotate:es,scale:Lp,translate:bo},Symbol.toStringTag,{value:"Module"}));var ut=function(){function r(t,e){this.x=t||0,this.y=e||0}return r.prototype.copy=function(t){return this.x=t.x,this.y=t.y,this},r.prototype.clone=function(){return new r(this.x,this.y)},r.prototype.set=function(t,e){return this.x=t,this.y=e,this},r.prototype.equal=function(t){return t.x===this.x&&t.y===this.y},r.prototype.add=function(t){return this.x+=t.x,this.y+=t.y,this},r.prototype.scale=function(t){this.x*=t,this.y*=t},r.prototype.scaleAndAdd=function(t,e){this.x+=t.x*e,this.y+=t.y*e},r.prototype.sub=function(t){return this.x-=t.x,this.y-=t.y,this},r.prototype.dot=function(t){return this.x*t.x+this.y*t.y},r.prototype.len=function(){return Math.sqrt(this.x*this.x+this.y*this.y)},r.prototype.lenSquare=function(){return this.x*this.x+this.y*this.y},r.prototype.normalize=function(){var t=this.len();return this.x/=t,this.y/=t,this},r.prototype.distance=function(t){var e=this.x-t.x,i=this.y-t.y;return Math.sqrt(e*e+i*i)},r.prototype.distanceSquare=function(t){var e=this.x-t.x,i=this.y-t.y;return e*e+i*i},r.prototype.negate=function(){return this.x=-this.x,this.y=-this.y,this},r.prototype.transform=function(t){if(t){var e=this.x,i=this.y;return this.x=t[0]*e+t[2]*i+t[4],this.y=t[1]*e+t[3]*i+t[5],this}},r.prototype.toArray=function(t){return t[0]=this.x,t[1]=this.y,t},r.prototype.fromArray=function(t){this.x=t[0],this.y=t[1]},r.set=function(t,e,i){t.x=e,t.y=i},r.copy=function(t,e){t.x=e.x,t.y=e.y},r.len=function(t){return Math.sqrt(t.x*t.x+t.y*t.y)},r.lenSquare=function(t){return t.x*t.x+t.y*t.y},r.dot=function(t,e){return t.x*e.x+t.y*e.y},r.add=function(t,e,i){t.x=e.x+i.x,t.y=e.y+i.y},r.sub=function(t,e,i){t.x=e.x-i.x,t.y=e.y-i.y},r.scale=function(t,e,i){t.x=e.x*i,t.y=e.y*i},r.scaleAndAdd=function(t,e,i,n){t.x=e.x+i.x*n,t.y=e.y+i.y*n},r.lerp=function(t,e,i,n){var a=1-n;t.x=a*e.x+n*i.x,t.y=a*e.y+n*i.y},r}(),ma=Math.min,_a=Math.max,Cr=new ut,Dr=new ut,Ar=new ut,Ir=new ut,ji=new ut,Ji=new ut,k_=function(){function r(t,e,i,n){i<0&&(t=t+i,i=-i),n<0&&(e=e+n,n=-n),this.x=t,this.y=e,this.width=i,this.height=n}return r.prototype.union=function(t){var e=ma(t.x,this.x),i=ma(t.y,this.y);isFinite(this.x)&&isFinite(this.width)?this.width=_a(t.x+t.width,this.x+this.width)-e:this.width=t.width,isFinite(this.y)&&isFinite(this.height)?this.height=_a(t.y+t.height,this.y+this.height)-i:this.height=t.height,this.x=e,this.y=i},r.prototype.applyTransform=function(t){r.applyTransform(this,this,t)},r.prototype.calculateTransform=function(t){var e=this,i=t.width/e.width,n=t.height/e.height,a=mr();return bo(a,a,[-e.x,-e.y]),Lp(a,a,[i,n]),bo(a,a,[t.x,t.y]),a},r.prototype.intersect=function(t,e){if(!t)return!1;t instanceof r||(t=r.create(t));var i=this,n=i.x,a=i.x+i.width,o=i.y,s=i.y+i.height,l=t.x,u=t.x+t.width,f=t.y,h=t.y+t.height,c=!(a<l||u<n||s<f||h<o);if(e){var v=1/0,d=0,y=Math.abs(a-l),p=Math.abs(u-n),g=Math.abs(s-f),m=Math.abs(h-o),_=Math.min(y,p),S=Math.min(g,m);a<l||u<n?_>d&&(d=_,y<p?ut.set(Ji,-y,0):ut.set(Ji,p,0)):_<v&&(v=_,y<p?ut.set(ji,y,0):ut.set(ji,-p,0)),s<f||h<o?S>d&&(d=S,g<m?ut.set(Ji,0,-g):ut.set(Ji,0,m)):_<v&&(v=_,g<m?ut.set(ji,0,g):ut.set(ji,0,-m))}return e&&ut.copy(e,c?ji:Ji),c},r.prototype.contain=function(t,e){var i=this;return t>=i.x&&t<=i.x+i.width&&e>=i.y&&e<=i.y+i.height},r.prototype.clone=function(){return new r(this.x,this.y,this.width,this.height)},r.prototype.copy=function(t){r.copy(this,t)},r.prototype.plain=function(){return{x:this.x,y:this.y,width:this.width,height:this.height}},r.prototype.isFinite=function(){return isFinite(this.x)&&isFinite(this.y)&&isFinite(this.width)&&isFinite(this.height)},r.prototype.isZero=function(){return this.width===0||this.height===0},r.create=function(t){return new r(t.x,t.y,t.width,t.height)},r.copy=function(t,e){t.x=e.x,t.y=e.y,t.width=e.width,t.height=e.height},r.applyTransform=function(t,e,i){if(!i){t!==e&&r.copy(t,e);return}if(i[1]<1e-5&&i[1]>-1e-5&&i[2]<1e-5&&i[2]>-1e-5){var n=i[0],a=i[3],o=i[4],s=i[5];t.x=e.x*n+o,t.y=e.y*a+s,t.width=e.width*n,t.height=e.height*a,t.width<0&&(t.x+=t.width,t.width=-t.width),t.height<0&&(t.y+=t.height,t.height=-t.height);return}Cr.x=Ar.x=e.x,Cr.y=Ir.y=e.y,Dr.x=Ir.x=e.x+e.width,Dr.y=Ar.y=e.y+e.height,Cr.transform(i),Ir.transform(i),Dr.transform(i),Ar.transform(i),t.x=ma(Cr.x,Dr.x,Ar.x,Ir.x),t.y=ma(Cr.y,Dr.y,Ar.y,Ir.y);var l=_a(Cr.x,Dr.x,Ar.x,Ir.x),u=_a(Cr.y,Dr.y,Ar.y,Ir.y);t.width=l-t.x,t.height=u-t.y},r}();const et=k_;var Mh={};function Qt(r,t){t=t||ai;var e=Mh[t];e||(e=Mh[t]=new ua(500));var i=e.get(r);return i==null&&(i=je.measureText(r,t).width,e.put(r,i)),i}function Ch(r,t,e,i){var n=Qt(r,t),a=pf(t),o=_n(0,n,e),s=Pi(0,a,i),l=new et(o,s,n,a);return l}function df(r,t,e,i){var n=((r||"")+"").split(`
`),a=n.length;if(a===1)return Ch(n[0],t,e,i);for(var o=new et(0,0,0,0),s=0;s<n.length;s++){var l=Ch(n[s],t,e,i);s===0?o.copy(l):o.union(l)}return o}function _n(r,t,e){return e==="right"?r-=t:e==="center"&&(r-=t/2),r}function Pi(r,t,e){return e==="middle"?r-=t/2:e==="bottom"&&(r-=t),r}function pf(r){return Qt("国",r)}function Fe(r,t){return typeof r=="string"?r.lastIndexOf("%")>=0?parseFloat(r)/100*t:parseFloat(r):r}function xo(r,t,e){var i=t.position||"inside",n=t.distance!=null?t.distance:5,a=e.height,o=e.width,s=a/2,l=e.x,u=e.y,f="left",h="top";if(i instanceof Array)l+=Fe(i[0],e.width),u+=Fe(i[1],e.height),f=null,h=null;else switch(i){case"left":l-=n,u+=s,f="right",h="middle";break;case"right":l+=n+o,u+=s,h="middle";break;case"top":l+=o/2,u-=n,f="center",h="bottom";break;case"bottom":l+=o/2,u+=a+n,f="center";break;case"inside":l+=o/2,u+=s,f="center",h="middle";break;case"insideLeft":l+=n,u+=s,h="middle";break;case"insideRight":l+=o-n,u+=s,f="right",h="middle";break;case"insideTop":l+=o/2,u+=n,f="center";break;case"insideBottom":l+=o/2,u+=a-n,f="center",h="bottom";break;case"insideTopLeft":l+=n,u+=n;break;case"insideTopRight":l+=o-n,u+=n,f="right";break;case"insideBottomLeft":l+=n,u+=a-n,h="bottom";break;case"insideBottomRight":l+=o-n,u+=a-n,f="right",h="bottom";break}return r=r||{},r.x=l,r.y=u,r.align=f,r.verticalAlign=h,r}var Ls=/\{([a-zA-Z0-9_]+)\|([^}]*)\}/g;function Pp(r,t,e,i,n){if(!t)return"";var a=(r+"").split(`
`);n=Rp(t,e,i,n);for(var o=0,s=a.length;o<s;o++)a[o]=Ep(a[o],n);return a.join(`
`)}function Rp(r,t,e,i){i=i||{};var n=O({},i);n.font=t,e=Q(e,"..."),n.maxIterations=Q(i.maxIterations,2);var a=n.minChar=Q(i.minChar,0);n.cnCharWidth=Qt("国",t);var o=n.ascCharWidth=Qt("a",t);n.placeholder=Q(i.placeholder,"");for(var s=r=Math.max(0,r-1),l=0;l<a&&s>=o;l++)s-=o;var u=Qt(e,t);return u>s&&(e="",u=0),s=r-u,n.ellipsis=e,n.ellipsisWidth=u,n.contentWidth=s,n.containerWidth=r,n}function Ep(r,t){var e=t.containerWidth,i=t.font,n=t.contentWidth;if(!e)return"";var a=Qt(r,i);if(a<=e)return r;for(var o=0;;o++){if(a<=n||o>=t.maxIterations){r+=t.ellipsis;break}var s=o===0?B_(r,n,t.ascCharWidth,t.cnCharWidth):a>0?Math.floor(r.length*n/a):0;r=r.substr(0,s),a=Qt(r,i)}return r===""&&(r=t.placeholder),r}function B_(r,t,e,i){for(var n=0,a=0,o=r.length;a<o&&n<t;a++){var s=r.charCodeAt(a);n+=0<=s&&s<=127?e:i}return a}function N_(r,t){r!=null&&(r+="");var e=t.overflow,i=t.padding,n=t.font,a=e==="truncate",o=pf(n),s=Q(t.lineHeight,o),l=!!t.backgroundColor,u=t.lineOverflow==="truncate",f=t.width,h;f!=null&&(e==="break"||e==="breakAll")?h=r?Op(r,t.font,f,e==="breakAll",0).lines:[]:h=r?r.split(`
`):[];var c=h.length*s,v=Q(t.height,c);if(c>v&&u){var d=Math.floor(v/s);h=h.slice(0,d)}if(r&&a&&f!=null)for(var y=Rp(f,n,t.ellipsis,{minChar:t.truncateMinChar,placeholder:t.placeholder}),p=0;p<h.length;p++)h[p]=Ep(h[p],y);for(var g=v,m=0,p=0;p<h.length;p++)m=Math.max(Qt(h[p],n),m);f==null&&(f=m);var _=m;return i&&(g+=i[0]+i[2],_+=i[1]+i[3],f+=i[1]+i[3]),l&&(_=f),{lines:h,height:v,outerWidth:_,outerHeight:g,lineHeight:s,calculatedLineHeight:o,contentWidth:m,contentHeight:c,width:f}}var F_=function(){function r(){}return r}(),Dh=function(){function r(t){this.tokens=[],t&&(this.tokens=t)}return r}(),z_=function(){function r(){this.width=0,this.height=0,this.contentWidth=0,this.contentHeight=0,this.outerWidth=0,this.outerHeight=0,this.lines=[]}return r}();function V_(r,t){var e=new z_;if(r!=null&&(r+=""),!r)return e;for(var i=t.width,n=t.height,a=t.overflow,o=(a==="break"||a==="breakAll")&&i!=null?{width:i,accumWidth:0,breakAll:a==="breakAll"}:null,s=Ls.lastIndex=0,l;(l=Ls.exec(r))!=null;){var u=l.index;u>s&&Ps(e,r.substring(s,u),t,o),Ps(e,l[2],t,o,l[1]),s=Ls.lastIndex}s<r.length&&Ps(e,r.substring(s,r.length),t,o);var f=[],h=0,c=0,v=t.padding,d=a==="truncate",y=t.lineOverflow==="truncate";function p(B,F,G){B.width=F,B.lineHeight=G,h+=G,c=Math.max(c,F)}t:for(var g=0;g<e.lines.length;g++){for(var m=e.lines[g],_=0,S=0,b=0;b<m.tokens.length;b++){var w=m.tokens[b],x=w.styleName&&t.rich[w.styleName]||{},C=w.textPadding=x.padding,D=C?C[1]+C[3]:0,A=w.font=x.font||t.font;w.contentHeight=pf(A);var M=Q(x.height,w.contentHeight);if(w.innerHeight=M,C&&(M+=C[0]+C[2]),w.height=M,w.lineHeight=Cn(x.lineHeight,t.lineHeight,M),w.align=x&&x.align||t.align,w.verticalAlign=x&&x.verticalAlign||"middle",y&&n!=null&&h+w.lineHeight>n){b>0?(m.tokens=m.tokens.slice(0,b),p(m,S,_),e.lines=e.lines.slice(0,g+1)):e.lines=e.lines.slice(0,g);break t}var I=x.width,L=I==null||I==="auto";if(typeof I=="string"&&I.charAt(I.length-1)==="%")w.percentWidth=I,f.push(w),w.contentWidth=Qt(w.text,A);else{if(L){var P=x.backgroundColor,R=P&&P.image;R&&(R=E_(R),ts(R)&&(w.width=Math.max(w.width,R.width*M/R.height)))}var E=d&&i!=null?i-S:null;E!=null&&E<w.width?!L||E<D?(w.text="",w.width=w.contentWidth=0):(w.text=Pp(w.text,E-D,A,t.ellipsis,{minChar:t.truncateMinChar}),w.width=w.contentWidth=Qt(w.text,A)):w.contentWidth=Qt(w.text,A)}w.width+=D,S+=w.width,x&&(_=Math.max(_,w.lineHeight))}p(m,S,_)}e.outerWidth=e.width=Q(i,c),e.outerHeight=e.height=Q(n,h),e.contentHeight=h,e.contentWidth=c,v&&(e.outerWidth+=v[1]+v[3],e.outerHeight+=v[0]+v[2]);for(var g=0;g<f.length;g++){var w=f[g],W=w.percentWidth;w.width=parseInt(W,10)/100*e.width}return e}function Ps(r,t,e,i,n){var a=t==="",o=n&&e.rich[n]||{},s=r.lines,l=o.font||e.font,u=!1,f,h;if(i){var c=o.padding,v=c?c[1]+c[3]:0;if(o.width!=null&&o.width!=="auto"){var d=Fe(o.width,i.width)+v;s.length>0&&d+i.accumWidth>i.width&&(f=t.split(`
`),u=!0),i.accumWidth=d}else{var y=Op(t,l,i.width,i.breakAll,i.accumWidth);i.accumWidth=y.accumWidth+v,h=y.linesWidths,f=y.lines}}else f=t.split(`
`);for(var p=0;p<f.length;p++){var g=f[p],m=new F_;if(m.styleName=n,m.text=g,m.isLineHolder=!g&&!a,typeof o.width=="number"?m.width=o.width:m.width=h?h[p]:Qt(g,l),!p&&!u){var _=(s[s.length-1]||(s[0]=new Dh)).tokens,S=_.length;S===1&&_[0].isLineHolder?_[0]=m:(g||!S||a)&&_.push(m)}else s.push(new Dh([m]))}}function H_(r){var t=r.charCodeAt(0);return t>=32&&t<=591||t>=880&&t<=4351||t>=4608&&t<=5119||t>=7680&&t<=8303}var G_=Ne(",&?/;] ".split(""),function(r,t){return r[t]=!0,r},{});function W_(r){return H_(r)?!!G_[r]:!0}function Op(r,t,e,i,n){for(var a=[],o=[],s="",l="",u=0,f=0,h=0;h<r.length;h++){var c=r.charAt(h);if(c===`
`){l&&(s+=l,f+=u),a.push(s),o.push(f),s="",l="",u=0,f=0;continue}var v=Qt(c,t),d=i?!1:!W_(c);if(a.length?f+v>e:n+f+v>e){f?(s||l)&&(d?(s||(s=l,l="",u=0,f=u),a.push(s),o.push(f-u),l+=c,u+=v,s="",f=u):(l&&(s+=l,l="",u=0),a.push(s),o.push(f),s=c,f=v)):d?(a.push(l),o.push(u),l=c,u=v):(a.push(c),o.push(v));continue}f+=v,d?(l+=c,u+=v):(l&&(s+=l,l="",u=0),s+=c)}return!a.length&&!s&&(s=r,l="",u=0),l&&(s+=l),s&&(a.push(s),o.push(f)),a.length===1&&(f+=n),{accumWidth:f,lines:a,linesWidths:o}}function vi(r,t){return r==null&&(r=0),t==null&&(t=0),[r,t]}function U_(r,t){return r[0]=t[0],r[1]=t[1],r}function kp(r){return[r[0],r[1]]}function Y_(r,t,e){return r[0]=t,r[1]=e,r}function eu(r,t,e){return r[0]=t[0]+e[0],r[1]=t[1]+e[1],r}function X_(r,t,e,i){return r[0]=t[0]+e[0]*i,r[1]=t[1]+e[1]*i,r}function Bp(r,t,e){return r[0]=t[0]-e[0],r[1]=t[1]-e[1],r}function gf(r){return Math.sqrt(yf(r))}var $_=gf;function yf(r){return r[0]*r[0]+r[1]*r[1]}var Z_=yf;function q_(r,t,e){return r[0]=t[0]*e[0],r[1]=t[1]*e[1],r}function K_(r,t,e){return r[0]=t[0]/e[0],r[1]=t[1]/e[1],r}function Q_(r,t){return r[0]*t[0]+r[1]*t[1]}function no(r,t,e){return r[0]=t[0]*e,r[1]=t[1]*e,r}function Np(r,t){var e=gf(t);return e===0?(r[0]=0,r[1]=0):(r[0]=t[0]/e,r[1]=t[1]/e),r}function To(r,t){return Math.sqrt((r[0]-t[0])*(r[0]-t[0])+(r[1]-t[1])*(r[1]-t[1]))}var Fp=To;function zp(r,t){return(r[0]-t[0])*(r[0]-t[0])+(r[1]-t[1])*(r[1]-t[1])}var Jr=zp;function j_(r,t){return r[0]=-t[0],r[1]=-t[1],r}function J_(r,t,e,i){return r[0]=t[0]+i*(e[0]-t[0]),r[1]=t[1]+i*(e[1]-t[1]),r}function Bt(r,t,e){var i=t[0],n=t[1];return r[0]=e[0]*i+e[2]*n+e[4],r[1]=e[1]*i+e[3]*n+e[5],r}function cr(r,t,e){return r[0]=Math.min(t[0],e[0]),r[1]=Math.min(t[1],e[1]),r}function dr(r,t,e){return r[0]=Math.max(t[0],e[0]),r[1]=Math.max(t[1],e[1]),r}const JI=Object.freeze(Object.defineProperty({__proto__:null,add:eu,applyTransform:Bt,clone:kp,copy:U_,create:vi,dist:Fp,distSquare:Jr,distance:To,distanceSquare:zp,div:K_,dot:Q_,len:gf,lenSquare:yf,length:$_,lengthSquare:Z_,lerp:J_,max:dr,min:cr,mul:q_,negate:j_,normalize:Np,scale:no,scaleAndAdd:X_,set:Y_,sub:Bp},Symbol.toStringTag,{value:"Module"}));var Ah=fa,Ih=5e-5;function Lr(r){return r>Ih||r<-Ih}var Pr=[],gi=[],Rs=mr(),Es=Math.abs,t1=function(){function r(){}return r.prototype.getLocalTransform=function(t){return r.getLocalTransform(this,t)},r.prototype.setPosition=function(t){this.x=t[0],this.y=t[1]},r.prototype.setScale=function(t){this.scaleX=t[0],this.scaleY=t[1]},r.prototype.setSkew=function(t){this.skewX=t[0],this.skewY=t[1]},r.prototype.setOrigin=function(t){this.originX=t[0],this.originY=t[1]},r.prototype.needLocalTransform=function(){return Lr(this.rotation)||Lr(this.x)||Lr(this.y)||Lr(this.scaleX-1)||Lr(this.scaleY-1)||Lr(this.skewX)||Lr(this.skewY)},r.prototype.updateTransform=function(){var t=this.parent&&this.parent.transform,e=this.needLocalTransform(),i=this.transform;if(!(e||t)){i&&(Ah(i),this.invTransform=null);return}i=i||mr(),e?this.getLocalTransform(i):Ah(i),t&&(e?_r(i,t,i):cf(i,t)),this.transform=i,this._resolveGlobalScaleRatio(i)},r.prototype._resolveGlobalScaleRatio=function(t){var e=this.globalScaleRatio;if(e!=null&&e!==1){this.getGlobalScale(Pr);var i=Pr[0]<0?-1:1,n=Pr[1]<0?-1:1,a=((Pr[0]-i)*e+i)/Pr[0]||0,o=((Pr[1]-n)*e+n)/Pr[1]||0;t[0]*=a,t[1]*=a,t[2]*=o,t[3]*=o}this.invTransform=this.invTransform||mr(),ha(this.invTransform,t)},r.prototype.getComputedTransform=function(){for(var t=this,e=[];t;)e.push(t),t=t.parent;for(;t=e.pop();)t.updateTransform();return this.transform},r.prototype.setLocalTransform=function(t){if(t){var e=t[0]*t[0]+t[1]*t[1],i=t[2]*t[2]+t[3]*t[3],n=Math.atan2(t[1],t[0]),a=Math.PI/2+n-Math.atan2(t[3],t[2]);i=Math.sqrt(i)*Math.cos(a),e=Math.sqrt(e),this.skewX=a,this.skewY=0,this.rotation=-n,this.x=+t[4],this.y=+t[5],this.scaleX=e,this.scaleY=i,this.originX=0,this.originY=0}},r.prototype.decomposeTransform=function(){if(this.transform){var t=this.parent,e=this.transform;t&&t.transform&&(t.invTransform=t.invTransform||mr(),_r(gi,t.invTransform,e),e=gi);var i=this.originX,n=this.originY;(i||n)&&(Rs[4]=i,Rs[5]=n,_r(gi,e,Rs),gi[4]-=i,gi[5]-=n,e=gi),this.setLocalTransform(e)}},r.prototype.getGlobalScale=function(t){var e=this.transform;return t=t||[],e?(t[0]=Math.sqrt(e[0]*e[0]+e[1]*e[1]),t[1]=Math.sqrt(e[2]*e[2]+e[3]*e[3]),e[0]<0&&(t[0]=-t[0]),e[3]<0&&(t[1]=-t[1]),t):(t[0]=1,t[1]=1,t)},r.prototype.transformCoordToLocal=function(t,e){var i=[t,e],n=this.invTransform;return n&&Bt(i,i,n),i},r.prototype.transformCoordToGlobal=function(t,e){var i=[t,e],n=this.transform;return n&&Bt(i,i,n),i},r.prototype.getLineScale=function(){var t=this.transform;return t&&Es(t[0]-1)>1e-10&&Es(t[3]-1)>1e-10?Math.sqrt(Es(t[0]*t[3]-t[2]*t[1])):1},r.prototype.copyTransform=function(t){e1(this,t)},r.getLocalTransform=function(t,e){e=e||[];var i=t.originX||0,n=t.originY||0,a=t.scaleX,o=t.scaleY,s=t.anchorX,l=t.anchorY,u=t.rotation||0,f=t.x,h=t.y,c=t.skewX?Math.tan(t.skewX):0,v=t.skewY?Math.tan(-t.skewY):0;if(i||n||s||l){var d=i+s,y=n+l;e[4]=-d*a-c*y*o,e[5]=-y*o-v*d*a}else e[4]=e[5]=0;return e[0]=a,e[3]=o,e[1]=v*a,e[2]=c*o,u&&es(e,e,u),e[4]+=i+f,e[5]+=n+h,e},r.initDefaultProps=function(){var t=r.prototype;t.scaleX=t.scaleY=t.globalScaleRatio=1,t.x=t.y=t.originX=t.originY=t.skewX=t.skewY=t.rotation=t.anchorX=t.anchorY=0}(),r}(),Wn=["x","y","originX","originY","anchorX","anchorY","rotation","scaleX","scaleY","skewX","skewY"];function e1(r,t){for(var e=0;e<Wn.length;e++){var i=Wn[e];r[i]=t[i]}}const mf=t1;var Dn={linear:function(r){return r},quadraticIn:function(r){return r*r},quadraticOut:function(r){return r*(2-r)},quadraticInOut:function(r){return(r*=2)<1?.5*r*r:-.5*(--r*(r-2)-1)},cubicIn:function(r){return r*r*r},cubicOut:function(r){return--r*r*r+1},cubicInOut:function(r){return(r*=2)<1?.5*r*r*r:.5*((r-=2)*r*r+2)},quarticIn:function(r){return r*r*r*r},quarticOut:function(r){return 1- --r*r*r*r},quarticInOut:function(r){return(r*=2)<1?.5*r*r*r*r:-.5*((r-=2)*r*r*r-2)},quinticIn:function(r){return r*r*r*r*r},quinticOut:function(r){return--r*r*r*r*r+1},quinticInOut:function(r){return(r*=2)<1?.5*r*r*r*r*r:.5*((r-=2)*r*r*r*r+2)},sinusoidalIn:function(r){return 1-Math.cos(r*Math.PI/2)},sinusoidalOut:function(r){return Math.sin(r*Math.PI/2)},sinusoidalInOut:function(r){return .5*(1-Math.cos(Math.PI*r))},exponentialIn:function(r){return r===0?0:Math.pow(1024,r-1)},exponentialOut:function(r){return r===1?1:1-Math.pow(2,-10*r)},exponentialInOut:function(r){return r===0?0:r===1?1:(r*=2)<1?.5*Math.pow(1024,r-1):.5*(-Math.pow(2,-10*(r-1))+2)},circularIn:function(r){return 1-Math.sqrt(1-r*r)},circularOut:function(r){return Math.sqrt(1- --r*r)},circularInOut:function(r){return(r*=2)<1?-.5*(Math.sqrt(1-r*r)-1):.5*(Math.sqrt(1-(r-=2)*r)+1)},elasticIn:function(r){var t,e=.1,i=.4;return r===0?0:r===1?1:(!e||e<1?(e=1,t=i/4):t=i*Math.asin(1/e)/(2*Math.PI),-(e*Math.pow(2,10*(r-=1))*Math.sin((r-t)*(2*Math.PI)/i)))},elasticOut:function(r){var t,e=.1,i=.4;return r===0?0:r===1?1:(!e||e<1?(e=1,t=i/4):t=i*Math.asin(1/e)/(2*Math.PI),e*Math.pow(2,-10*r)*Math.sin((r-t)*(2*Math.PI)/i)+1)},elasticInOut:function(r){var t,e=.1,i=.4;return r===0?0:r===1?1:(!e||e<1?(e=1,t=i/4):t=i*Math.asin(1/e)/(2*Math.PI),(r*=2)<1?-.5*(e*Math.pow(2,10*(r-=1))*Math.sin((r-t)*(2*Math.PI)/i)):e*Math.pow(2,-10*(r-=1))*Math.sin((r-t)*(2*Math.PI)/i)*.5+1)},backIn:function(r){var t=1.70158;return r*r*((t+1)*r-t)},backOut:function(r){var t=1.70158;return--r*r*((t+1)*r+t)+1},backInOut:function(r){var t=2.5949095;return(r*=2)<1?.5*(r*r*((t+1)*r-t)):.5*((r-=2)*r*((t+1)*r+t)+2)},bounceIn:function(r){return 1-Dn.bounceOut(1-r)},bounceOut:function(r){return r<1/2.75?7.5625*r*r:r<2/2.75?7.5625*(r-=1.5/2.75)*r+.75:r<2.5/2.75?7.5625*(r-=2.25/2.75)*r+.9375:7.5625*(r-=2.625/2.75)*r+.984375},bounceInOut:function(r){return r<.5?Dn.bounceIn(r*2)*.5:Dn.bounceOut(r*2-1)*.5+.5}},Sa=Math.pow,Sr=Math.sqrt,Mo=1e-8,Vp=1e-4,Lh=Sr(3),wa=1/3,Re=vi(),se=vi(),Ni=vi();function pr(r){return r>-Mo&&r<Mo}function Hp(r){return r>Mo||r<-Mo}function bt(r,t,e,i,n){var a=1-n;return a*a*(a*r+3*n*t)+n*n*(n*i+3*a*e)}function Ph(r,t,e,i,n){var a=1-n;return 3*(((t-r)*a+2*(e-t)*n)*a+(i-e)*n*n)}function Co(r,t,e,i,n,a){var o=i+3*(t-e)-r,s=3*(e-t*2+r),l=3*(t-r),u=r-n,f=s*s-3*o*l,h=s*l-9*o*u,c=l*l-3*s*u,v=0;if(pr(f)&&pr(h))if(pr(s))a[0]=0;else{var d=-l/s;d>=0&&d<=1&&(a[v++]=d)}else{var y=h*h-4*f*c;if(pr(y)){var p=h/f,d=-s/o+p,g=-p/2;d>=0&&d<=1&&(a[v++]=d),g>=0&&g<=1&&(a[v++]=g)}else if(y>0){var m=Sr(y),_=f*s+1.5*o*(-h+m),S=f*s+1.5*o*(-h-m);_<0?_=-Sa(-_,wa):_=Sa(_,wa),S<0?S=-Sa(-S,wa):S=Sa(S,wa);var d=(-s-(_+S))/(3*o);d>=0&&d<=1&&(a[v++]=d)}else{var b=(2*f*s-3*o*h)/(2*Sr(f*f*f)),w=Math.acos(b)/3,x=Sr(f),C=Math.cos(w),d=(-s-2*x*C)/(3*o),g=(-s+x*(C+Lh*Math.sin(w)))/(3*o),D=(-s+x*(C-Lh*Math.sin(w)))/(3*o);d>=0&&d<=1&&(a[v++]=d),g>=0&&g<=1&&(a[v++]=g),D>=0&&D<=1&&(a[v++]=D)}}return v}function Gp(r,t,e,i,n){var a=6*e-12*t+6*r,o=9*t+3*i-3*r-9*e,s=3*t-3*r,l=0;if(pr(o)){if(Hp(a)){var u=-s/a;u>=0&&u<=1&&(n[l++]=u)}}else{var f=a*a-4*o*s;if(pr(f))n[0]=-a/(2*o);else if(f>0){var h=Sr(f),u=(-a+h)/(2*o),c=(-a-h)/(2*o);u>=0&&u<=1&&(n[l++]=u),c>=0&&c<=1&&(n[l++]=c)}}return l}function Do(r,t,e,i,n,a){var o=(t-r)*n+r,s=(e-t)*n+t,l=(i-e)*n+e,u=(s-o)*n+o,f=(l-s)*n+s,h=(f-u)*n+u;a[0]=r,a[1]=o,a[2]=u,a[3]=h,a[4]=h,a[5]=f,a[6]=l,a[7]=i}function r1(r,t,e,i,n,a,o,s,l,u,f){var h,c=.005,v=1/0,d,y,p,g;Re[0]=l,Re[1]=u;for(var m=0;m<1;m+=.05)se[0]=bt(r,e,n,o,m),se[1]=bt(t,i,a,s,m),p=Jr(Re,se),p<v&&(h=m,v=p);v=1/0;for(var _=0;_<32&&!(c<Vp);_++)d=h-c,y=h+c,se[0]=bt(r,e,n,o,d),se[1]=bt(t,i,a,s,d),p=Jr(se,Re),d>=0&&p<v?(h=d,v=p):(Ni[0]=bt(r,e,n,o,y),Ni[1]=bt(t,i,a,s,y),g=Jr(Ni,Re),y<=1&&g<v?(h=y,v=g):c*=.5);return Sr(v)}function i1(r,t,e,i,n,a,o,s,l){for(var u=r,f=t,h=0,c=1/l,v=1;v<=l;v++){var d=v*c,y=bt(r,e,n,o,d),p=bt(t,i,a,s,d),g=y-u,m=p-f;h+=Math.sqrt(g*g+m*m),u=y,f=p}return h}function Ht(r,t,e,i){var n=1-i;return n*(n*r+2*i*t)+i*i*e}function Rh(r,t,e,i){return 2*((1-i)*(t-r)+i*(e-t))}function n1(r,t,e,i,n){var a=r-2*t+e,o=2*(t-r),s=r-i,l=0;if(pr(a)){if(Hp(o)){var u=-s/o;u>=0&&u<=1&&(n[l++]=u)}}else{var f=o*o-4*a*s;if(pr(f)){var u=-o/(2*a);u>=0&&u<=1&&(n[l++]=u)}else if(f>0){var h=Sr(f),u=(-o+h)/(2*a),c=(-o-h)/(2*a);u>=0&&u<=1&&(n[l++]=u),c>=0&&c<=1&&(n[l++]=c)}}return l}function Wp(r,t,e){var i=r+e-2*t;return i===0?.5:(r-t)/i}function Ao(r,t,e,i,n){var a=(t-r)*i+r,o=(e-t)*i+t,s=(o-a)*i+a;n[0]=r,n[1]=a,n[2]=s,n[3]=s,n[4]=o,n[5]=e}function a1(r,t,e,i,n,a,o,s,l){var u,f=.005,h=1/0;Re[0]=o,Re[1]=s;for(var c=0;c<1;c+=.05){se[0]=Ht(r,e,n,c),se[1]=Ht(t,i,a,c);var v=Jr(Re,se);v<h&&(u=c,h=v)}h=1/0;for(var d=0;d<32&&!(f<Vp);d++){var y=u-f,p=u+f;se[0]=Ht(r,e,n,y),se[1]=Ht(t,i,a,y);var v=Jr(se,Re);if(y>=0&&v<h)u=y,h=v;else{Ni[0]=Ht(r,e,n,p),Ni[1]=Ht(t,i,a,p);var g=Jr(Ni,Re);p<=1&&g<h?(u=p,h=g):f*=.5}}return Sr(h)}function o1(r,t,e,i,n,a,o){for(var s=r,l=t,u=0,f=1/o,h=1;h<=o;h++){var c=h*f,v=Ht(r,e,n,c),d=Ht(t,i,a,c),y=v-s,p=d-l;u+=Math.sqrt(y*y+p*p),s=v,l=d}return u}var s1=/cubic-bezier\(([0-9,\.e ]+)\)/;function Up(r){var t=r&&s1.exec(r);if(t){var e=t[1].split(","),i=+be(e[0]),n=+be(e[1]),a=+be(e[2]),o=+be(e[3]);if(isNaN(i+n+a+o))return;var s=[];return function(l){return l<=0?0:l>=1?1:Co(0,i,a,1,l,s)&&bt(0,n,o,1,s[0])}}}var l1=function(){function r(t){this._inited=!1,this._startTime=0,this._pausedTime=0,this._paused=!1,this._life=t.life||1e3,this._delay=t.delay||0,this.loop=t.loop||!1,this.onframe=t.onframe||At,this.ondestroy=t.ondestroy||At,this.onrestart=t.onrestart||At,t.easing&&this.setEasing(t.easing)}return r.prototype.step=function(t,e){if(this._inited||(this._startTime=t+this._delay,this._inited=!0),this._paused){this._pausedTime+=e;return}var i=this._life,n=t-this._startTime-this._pausedTime,a=n/i;a<0&&(a=0),a=Math.min(a,1);var o=this.easingFunc,s=o?o(a):a;if(this.onframe(s),a===1)if(this.loop){var l=n%i;this._startTime=t-l,this._pausedTime=0,this.onrestart()}else return!0;return!1},r.prototype.pause=function(){this._paused=!0},r.prototype.resume=function(){this._paused=!1},r.prototype.setEasing=function(t){this.easing=t,this.easingFunc=Y(t)?t:Dn[t]||Up(t)},r}(),Eh={transparent:[0,0,0,0],aliceblue:[240,248,255,1],antiquewhite:[250,235,215,1],aqua:[0,255,255,1],aquamarine:[127,255,212,1],azure:[240,255,255,1],beige:[245,245,220,1],bisque:[255,228,196,1],black:[0,0,0,1],blanchedalmond:[255,235,205,1],blue:[0,0,255,1],blueviolet:[138,43,226,1],brown:[165,42,42,1],burlywood:[222,184,135,1],cadetblue:[95,158,160,1],chartreuse:[127,255,0,1],chocolate:[210,105,30,1],coral:[255,127,80,1],cornflowerblue:[100,149,237,1],cornsilk:[255,248,220,1],crimson:[220,20,60,1],cyan:[0,255,255,1],darkblue:[0,0,139,1],darkcyan:[0,139,139,1],darkgoldenrod:[184,134,11,1],darkgray:[169,169,169,1],darkgreen:[0,100,0,1],darkgrey:[169,169,169,1],darkkhaki:[189,183,107,1],darkmagenta:[139,0,139,1],darkolivegreen:[85,107,47,1],darkorange:[255,140,0,1],darkorchid:[153,50,204,1],darkred:[139,0,0,1],darksalmon:[233,150,122,1],darkseagreen:[143,188,143,1],darkslateblue:[72,61,139,1],darkslategray:[47,79,79,1],darkslategrey:[47,79,79,1],darkturquoise:[0,206,209,1],darkviolet:[148,0,211,1],deeppink:[255,20,147,1],deepskyblue:[0,191,255,1],dimgray:[105,105,105,1],dimgrey:[105,105,105,1],dodgerblue:[30,144,255,1],firebrick:[178,34,34,1],floralwhite:[255,250,240,1],forestgreen:[34,139,34,1],fuchsia:[255,0,255,1],gainsboro:[220,220,220,1],ghostwhite:[248,248,255,1],gold:[255,215,0,1],goldenrod:[218,165,32,1],gray:[128,128,128,1],green:[0,128,0,1],greenyellow:[173,255,47,1],grey:[128,128,128,1],honeydew:[240,255,240,1],hotpink:[255,105,180,1],indianred:[205,92,92,1],indigo:[75,0,130,1],ivory:[255,255,240,1],khaki:[240,230,140,1],lavender:[230,230,250,1],lavenderblush:[255,240,245,1],lawngreen:[124,252,0,1],lemonchiffon:[255,250,205,1],lightblue:[173,216,230,1],lightcoral:[240,128,128,1],lightcyan:[224,255,255,1],lightgoldenrodyellow:[250,250,210,1],lightgray:[211,211,211,1],lightgreen:[144,238,144,1],lightgrey:[211,211,211,1],lightpink:[255,182,193,1],lightsalmon:[255,160,122,1],lightseagreen:[32,178,170,1],lightskyblue:[135,206,250,1],lightslategray:[119,136,153,1],lightslategrey:[119,136,153,1],lightsteelblue:[176,196,222,1],lightyellow:[255,255,224,1],lime:[0,255,0,1],limegreen:[50,205,50,1],linen:[250,240,230,1],magenta:[255,0,255,1],maroon:[128,0,0,1],mediumaquamarine:[102,205,170,1],mediumblue:[0,0,205,1],mediumorchid:[186,85,211,1],mediumpurple:[147,112,219,1],mediumseagreen:[60,179,113,1],mediumslateblue:[123,104,238,1],mediumspringgreen:[0,250,154,1],mediumturquoise:[72,209,204,1],mediumvioletred:[199,21,133,1],midnightblue:[25,25,112,1],mintcream:[245,255,250,1],mistyrose:[255,228,225,1],moccasin:[255,228,181,1],navajowhite:[255,222,173,1],navy:[0,0,128,1],oldlace:[253,245,230,1],olive:[128,128,0,1],olivedrab:[107,142,35,1],orange:[255,165,0,1],orangered:[255,69,0,1],orchid:[218,112,214,1],palegoldenrod:[238,232,170,1],palegreen:[152,251,152,1],paleturquoise:[175,238,238,1],palevioletred:[219,112,147,1],papayawhip:[255,239,213,1],peachpuff:[255,218,185,1],peru:[205,133,63,1],pink:[255,192,203,1],plum:[221,160,221,1],powderblue:[176,224,230,1],purple:[128,0,128,1],red:[255,0,0,1],rosybrown:[188,143,143,1],royalblue:[65,105,225,1],saddlebrown:[139,69,19,1],salmon:[250,128,114,1],sandybrown:[244,164,96,1],seagreen:[46,139,87,1],seashell:[255,245,238,1],sienna:[160,82,45,1],silver:[192,192,192,1],skyblue:[135,206,235,1],slateblue:[106,90,205,1],slategray:[112,128,144,1],slategrey:[112,128,144,1],snow:[255,250,250,1],springgreen:[0,255,127,1],steelblue:[70,130,180,1],tan:[210,180,140,1],teal:[0,128,128,1],thistle:[216,191,216,1],tomato:[255,99,71,1],turquoise:[64,224,208,1],violet:[238,130,238,1],wheat:[245,222,179,1],white:[255,255,255,1],whitesmoke:[245,245,245,1],yellow:[255,255,0,1],yellowgreen:[154,205,50,1]};function Te(r){return r=Math.round(r),r<0?0:r>255?255:r}function u1(r){return r=Math.round(r),r<0?0:r>360?360:r}function Un(r){return r<0?0:r>1?1:r}function Os(r){var t=r;return t.length&&t.charAt(t.length-1)==="%"?Te(parseFloat(t)/100*255):Te(parseInt(t,10))}function ti(r){var t=r;return t.length&&t.charAt(t.length-1)==="%"?Un(parseFloat(t)/100):Un(parseFloat(t))}function ks(r,t,e){return e<0?e+=1:e>1&&(e-=1),e*6<1?r+(t-r)*e*6:e*2<1?t:e*3<2?r+(t-r)*(2/3-e)*6:r}function gr(r,t,e){return r+(t-r)*e}function ie(r,t,e,i,n){return r[0]=t,r[1]=e,r[2]=i,r[3]=n,r}function ru(r,t){return r[0]=t[0],r[1]=t[1],r[2]=t[2],r[3]=t[3],r}var Yp=new ua(20),ba=null;function yi(r,t){ba&&ru(ba,t),ba=Yp.put(r,ba||t.slice())}function jt(r,t){if(r){t=t||[];var e=Yp.get(r);if(e)return ru(t,e);r=r+"";var i=r.replace(/ /g,"").toLowerCase();if(i in Eh)return ru(t,Eh[i]),yi(r,t),t;var n=i.length;if(i.charAt(0)==="#"){if(n===4||n===5){var a=parseInt(i.slice(1,4),16);if(!(a>=0&&a<=4095)){ie(t,0,0,0,1);return}return ie(t,(a&3840)>>4|(a&3840)>>8,a&240|(a&240)>>4,a&15|(a&15)<<4,n===5?parseInt(i.slice(4),16)/15:1),yi(r,t),t}else if(n===7||n===9){var a=parseInt(i.slice(1,7),16);if(!(a>=0&&a<=16777215)){ie(t,0,0,0,1);return}return ie(t,(a&16711680)>>16,(a&65280)>>8,a&255,n===9?parseInt(i.slice(7),16)/255:1),yi(r,t),t}return}var o=i.indexOf("("),s=i.indexOf(")");if(o!==-1&&s+1===n){var l=i.substr(0,o),u=i.substr(o+1,s-(o+1)).split(","),f=1;switch(l){case"rgba":if(u.length!==4)return u.length===3?ie(t,+u[0],+u[1],+u[2],1):ie(t,0,0,0,1);f=ti(u.pop());case"rgb":if(u.length>=3)return ie(t,Os(u[0]),Os(u[1]),Os(u[2]),u.length===3?f:ti(u[3])),yi(r,t),t;ie(t,0,0,0,1);return;case"hsla":if(u.length!==4){ie(t,0,0,0,1);return}return u[3]=ti(u[3]),iu(u,t),yi(r,t),t;case"hsl":if(u.length!==3){ie(t,0,0,0,1);return}return iu(u,t),yi(r,t),t;default:return}}ie(t,0,0,0,1)}}function iu(r,t){var e=(parseFloat(r[0])%360+360)%360/360,i=ti(r[1]),n=ti(r[2]),a=n<=.5?n*(i+1):n+i-n*i,o=n*2-a;return t=t||[],ie(t,Te(ks(o,a,e+1/3)*255),Te(ks(o,a,e)*255),Te(ks(o,a,e-1/3)*255),1),r.length===4&&(t[3]=r[3]),t}function f1(r){if(r){var t=r[0]/255,e=r[1]/255,i=r[2]/255,n=Math.min(t,e,i),a=Math.max(t,e,i),o=a-n,s=(a+n)/2,l,u;if(o===0)l=0,u=0;else{s<.5?u=o/(a+n):u=o/(2-a-n);var f=((a-t)/6+o/2)/o,h=((a-e)/6+o/2)/o,c=((a-i)/6+o/2)/o;t===a?l=c-h:e===a?l=1/3+f-c:i===a&&(l=2/3+h-f),l<0&&(l+=1),l>1&&(l-=1)}var v=[l*360,u,s];return r[3]!=null&&v.push(r[3]),v}}function nu(r,t){var e=jt(r);if(e){for(var i=0;i<3;i++)t<0?e[i]=e[i]*(1-t)|0:e[i]=(255-e[i])*t+e[i]|0,e[i]>255?e[i]=255:e[i]<0&&(e[i]=0);return ke(e,e.length===4?"rgba":"rgb")}}function h1(r){var t=jt(r);if(t)return((1<<24)+(t[0]<<16)+(t[1]<<8)+ +t[2]).toString(16).slice(1)}function An(r,t,e){if(!(!(t&&t.length)||!(r>=0&&r<=1))){e=e||[];var i=r*(t.length-1),n=Math.floor(i),a=Math.ceil(i),o=t[n],s=t[a],l=i-n;return e[0]=Te(gr(o[0],s[0],l)),e[1]=Te(gr(o[1],s[1],l)),e[2]=Te(gr(o[2],s[2],l)),e[3]=Un(gr(o[3],s[3],l)),e}}var v1=An;function _f(r,t,e){if(!(!(t&&t.length)||!(r>=0&&r<=1))){var i=r*(t.length-1),n=Math.floor(i),a=Math.ceil(i),o=jt(t[n]),s=jt(t[a]),l=i-n,u=ke([Te(gr(o[0],s[0],l)),Te(gr(o[1],s[1],l)),Te(gr(o[2],s[2],l)),Un(gr(o[3],s[3],l))],"rgba");return e?{color:u,leftIndex:n,rightIndex:a,value:i}:u}}var c1=_f;function ao(r,t,e,i){var n=jt(r);if(r)return n=f1(n),t!=null&&(n[0]=u1(t)),e!=null&&(n[1]=ti(e)),i!=null&&(n[2]=ti(i)),ke(iu(n),"rgba")}function Xp(r,t){var e=jt(r);if(e&&t!=null)return e[3]=Un(t),ke(e,"rgba")}function ke(r,t){if(!(!r||!r.length)){var e=r[0]+","+r[1]+","+r[2];return(t==="rgba"||t==="hsva"||t==="hsla")&&(e+=","+r[3]),t+"("+e+")"}}function Yn(r,t){var e=jt(r);return e?(.299*e[0]+.587*e[1]+.114*e[2])*e[3]/255+(1-e[3])*t:0}function d1(){return ke([Math.round(Math.random()*255),Math.round(Math.random()*255),Math.round(Math.random()*255)],"rgb")}var Oh=new ua(100);function au(r){if(V(r)){var t=Oh.get(r);return t||(t=nu(r,-.1),Oh.put(r,t)),t}else if(sa(r)){var e=O({},r);return e.colorStops=H(r.colorStops,function(i){return{offset:i.offset,color:nu(i.color,-.1)}}),e}return r}const tL=Object.freeze(Object.defineProperty({__proto__:null,fastLerp:An,fastMapToColor:v1,lerp:_f,lift:nu,liftColor:au,lum:Yn,mapToColor:c1,modifyAlpha:Xp,modifyHSL:ao,parse:jt,random:d1,stringify:ke,toHex:h1},Symbol.toStringTag,{value:"Module"}));function p1(r){return r.type==="linear"}function g1(r){return r.type==="radial"}(function(){return Z.hasGlobalWindow&&Y(window.btoa)?function(r){return window.btoa(unescape(encodeURIComponent(r)))}:typeof Buffer<"u"?function(r){return Buffer.from(r).toString("base64")}:function(r){return null}})();var ou=Array.prototype.slice;function Ze(r,t,e){return(t-r)*e+r}function Bs(r,t,e,i){for(var n=t.length,a=0;a<n;a++)r[a]=Ze(t[a],e[a],i);return r}function y1(r,t,e,i){for(var n=t.length,a=n&&t[0].length,o=0;o<n;o++){r[o]||(r[o]=[]);for(var s=0;s<a;s++)r[o][s]=Ze(t[o][s],e[o][s],i)}return r}function xa(r,t,e,i){for(var n=t.length,a=0;a<n;a++)r[a]=t[a]+e[a]*i;return r}function kh(r,t,e,i){for(var n=t.length,a=n&&t[0].length,o=0;o<n;o++){r[o]||(r[o]=[]);for(var s=0;s<a;s++)r[o][s]=t[o][s]+e[o][s]*i}return r}function m1(r,t){for(var e=r.length,i=t.length,n=e>i?t:r,a=Math.min(e,i),o=n[a-1]||{color:[0,0,0,0],offset:0},s=a;s<Math.max(e,i);s++)n.push({offset:o.offset,color:o.color.slice()})}function _1(r,t,e){var i=r,n=t;if(!(!i.push||!n.push)){var a=i.length,o=n.length;if(a!==o){var s=a>o;if(s)i.length=o;else for(var l=a;l<o;l++)i.push(e===1?n[l]:ou.call(n[l]))}for(var u=i[0]&&i[0].length,l=0;l<i.length;l++)if(e===1)isNaN(i[l])&&(i[l]=n[l]);else for(var f=0;f<u;f++)isNaN(i[l][f])&&(i[l][f]=n[l][f])}}function oo(r){if(Nt(r)){var t=r.length;if(Nt(r[0])){for(var e=[],i=0;i<t;i++)e.push(ou.call(r[i]));return e}return ou.call(r)}return r}function so(r){return r[0]=Math.floor(r[0])||0,r[1]=Math.floor(r[1])||0,r[2]=Math.floor(r[2])||0,r[3]=r[3]==null?1:r[3],"rgba("+r.join(",")+")"}function S1(r){return Nt(r&&r[0])?2:1}var Ta=0,lo=1,$p=2,Sn=3,su=4,lu=5,Bh=6;function Nh(r){return r===su||r===lu}function Ma(r){return r===lo||r===$p}var tn=[0,0,0,0],w1=function(){function r(t){this.keyframes=[],this.discrete=!1,this._invalid=!1,this._needsSort=!1,this._lastFr=0,this._lastFrP=0,this.propName=t}return r.prototype.isFinished=function(){return this._finished},r.prototype.setFinished=function(){this._finished=!0,this._additiveTrack&&this._additiveTrack.setFinished()},r.prototype.needsAnimate=function(){return this.keyframes.length>=1},r.prototype.getAdditiveTrack=function(){return this._additiveTrack},r.prototype.addKeyframe=function(t,e,i){this._needsSort=!0;var n=this.keyframes,a=n.length,o=!1,s=Bh,l=e;if(Nt(e)){var u=S1(e);s=u,(u===1&&!ht(e[0])||u===2&&!ht(e[0][0]))&&(o=!0)}else if(ht(e)&&!Vn(e))s=Ta;else if(V(e))if(!isNaN(+e))s=Ta;else{var f=jt(e);f&&(l=f,s=Sn)}else if(sa(e)){var h=O({},l);h.colorStops=H(e.colorStops,function(v){return{offset:v.offset,color:jt(v.color)}}),p1(e)?s=su:g1(e)&&(s=lu),l=h}a===0?this.valType=s:(s!==this.valType||s===Bh)&&(o=!0),this.discrete=this.discrete||o;var c={time:t,value:l,rawValue:e,percent:0};return i&&(c.easing=i,c.easingFunc=Y(i)?i:Dn[i]||Up(i)),n.push(c),c},r.prototype.prepare=function(t,e){var i=this.keyframes;this._needsSort&&i.sort(function(y,p){return y.time-p.time});for(var n=this.valType,a=i.length,o=i[a-1],s=this.discrete,l=Ma(n),u=Nh(n),f=0;f<a;f++){var h=i[f],c=h.value,v=o.value;h.percent=h.time/t,s||(l&&f!==a-1?_1(c,v,n):u&&m1(c.colorStops,v.colorStops))}if(!s&&n!==lu&&e&&this.needsAnimate()&&e.needsAnimate()&&n===e.valType&&!e._finished){this._additiveTrack=e;for(var d=i[0].value,f=0;f<a;f++)n===Ta?i[f].additiveValue=i[f].value-d:n===Sn?i[f].additiveValue=xa([],i[f].value,d,-1):Ma(n)&&(i[f].additiveValue=n===lo?xa([],i[f].value,d,-1):kh([],i[f].value,d,-1))}},r.prototype.step=function(t,e){if(!this._finished){this._additiveTrack&&this._additiveTrack._finished&&(this._additiveTrack=null);var i=this._additiveTrack!=null,n=i?"additiveValue":"value",a=this.valType,o=this.keyframes,s=o.length,l=this.propName,u=a===Sn,f,h=this._lastFr,c=Math.min,v,d;if(s===1)v=d=o[0];else{if(e<0)f=0;else if(e<this._lastFrP){var y=c(h+1,s-1);for(f=y;f>=0&&!(o[f].percent<=e);f--);f=c(f,s-2)}else{for(f=h;f<s&&!(o[f].percent>e);f++);f=c(f-1,s-2)}d=o[f+1],v=o[f]}if(v&&d){this._lastFr=f,this._lastFrP=e;var p=d.percent-v.percent,g=p===0?1:c((e-v.percent)/p,1);d.easingFunc&&(g=d.easingFunc(g));var m=i?this._additiveValue:u?tn:t[l];if((Ma(a)||u)&&!m&&(m=this._additiveValue=[]),this.discrete)t[l]=g<1?v.rawValue:d.rawValue;else if(Ma(a))a===lo?Bs(m,v[n],d[n],g):y1(m,v[n],d[n],g);else if(Nh(a)){var _=v[n],S=d[n],b=a===su;t[l]={type:b?"linear":"radial",x:Ze(_.x,S.x,g),y:Ze(_.y,S.y,g),colorStops:H(_.colorStops,function(x,C){var D=S.colorStops[C];return{offset:Ze(x.offset,D.offset,g),color:so(Bs([],x.color,D.color,g))}}),global:S.global},b?(t[l].x2=Ze(_.x2,S.x2,g),t[l].y2=Ze(_.y2,S.y2,g)):t[l].r=Ze(_.r,S.r,g)}else if(u)Bs(m,v[n],d[n],g),i||(t[l]=so(m));else{var w=Ze(v[n],d[n],g);i?this._additiveValue=w:t[l]=w}i&&this._addToTarget(t)}}},r.prototype._addToTarget=function(t){var e=this.valType,i=this.propName,n=this._additiveValue;e===Ta?t[i]=t[i]+n:e===Sn?(jt(t[i],tn),xa(tn,tn,n,1),t[i]=so(tn)):e===lo?xa(t[i],t[i],n,1):e===$p&&kh(t[i],t[i],n,1)},r}(),Sf=function(){function r(t,e,i,n){if(this._tracks={},this._trackKeys=[],this._maxTime=0,this._started=0,this._clip=null,this._target=t,this._loop=e,e&&n){Qo("Can' use additive animation on looped animation.");return}this._additiveAnimators=n,this._allowDiscrete=i}return r.prototype.getMaxTime=function(){return this._maxTime},r.prototype.getDelay=function(){return this._delay},r.prototype.getLoop=function(){return this._loop},r.prototype.getTarget=function(){return this._target},r.prototype.changeTarget=function(t){this._target=t},r.prototype.when=function(t,e,i){return this.whenWithKeys(t,e,ft(e),i)},r.prototype.whenWithKeys=function(t,e,i,n){for(var a=this._tracks,o=0;o<i.length;o++){var s=i[o],l=a[s];if(!l){l=a[s]=new w1(s);var u=void 0,f=this._getAdditiveTrack(s);if(f){var h=f.keyframes,c=h[h.length-1];u=c&&c.value,f.valType===Sn&&u&&(u=so(u))}else u=this._target[s];if(u==null)continue;t>0&&l.addKeyframe(0,oo(u),n),this._trackKeys.push(s)}l.addKeyframe(t,oo(e[s]),n)}return this._maxTime=Math.max(this._maxTime,t),this},r.prototype.pause=function(){this._clip.pause(),this._paused=!0},r.prototype.resume=function(){this._clip.resume(),this._paused=!1},r.prototype.isPaused=function(){return!!this._paused},r.prototype.duration=function(t){return this._maxTime=t,this._force=!0,this},r.prototype._doneCallback=function(){this._setTracksFinished(),this._clip=null;var t=this._doneCbs;if(t)for(var e=t.length,i=0;i<e;i++)t[i].call(this)},r.prototype._abortedCallback=function(){this._setTracksFinished();var t=this.animation,e=this._abortedCbs;if(t&&t.removeClip(this._clip),this._clip=null,e)for(var i=0;i<e.length;i++)e[i].call(this)},r.prototype._setTracksFinished=function(){for(var t=this._tracks,e=this._trackKeys,i=0;i<e.length;i++)t[e[i]].setFinished()},r.prototype._getAdditiveTrack=function(t){var e,i=this._additiveAnimators;if(i)for(var n=0;n<i.length;n++){var a=i[n].getTrack(t);a&&(e=a)}return e},r.prototype.start=function(t){if(!(this._started>0)){this._started=1;for(var e=this,i=[],n=this._maxTime||0,a=0;a<this._trackKeys.length;a++){var o=this._trackKeys[a],s=this._tracks[o],l=this._getAdditiveTrack(o),u=s.keyframes,f=u.length;if(s.prepare(n,l),s.needsAnimate())if(!this._allowDiscrete&&s.discrete){var h=u[f-1];h&&(e._target[s.propName]=h.rawValue),s.setFinished()}else i.push(s)}if(i.length||this._force){var c=new l1({life:n,loop:this._loop,delay:this._delay||0,onframe:function(v){e._started=2;var d=e._additiveAnimators;if(d){for(var y=!1,p=0;p<d.length;p++)if(d[p]._clip){y=!0;break}y||(e._additiveAnimators=null)}for(var p=0;p<i.length;p++)i[p].step(e._target,v);var g=e._onframeCbs;if(g)for(var p=0;p<g.length;p++)g[p](e._target,v)},ondestroy:function(){e._doneCallback()}});this._clip=c,this.animation&&this.animation.addClip(c),t&&c.setEasing(t)}else this._doneCallback();return this}},r.prototype.stop=function(t){if(this._clip){var e=this._clip;t&&e.onframe(1),this._abortedCallback()}},r.prototype.delay=function(t){return this._delay=t,this},r.prototype.during=function(t){return t&&(this._onframeCbs||(this._onframeCbs=[]),this._onframeCbs.push(t)),this},r.prototype.done=function(t){return t&&(this._doneCbs||(this._doneCbs=[]),this._doneCbs.push(t)),this},r.prototype.aborted=function(t){return t&&(this._abortedCbs||(this._abortedCbs=[]),this._abortedCbs.push(t)),this},r.prototype.getClip=function(){return this._clip},r.prototype.getTrack=function(t){return this._tracks[t]},r.prototype.getTracks=function(){var t=this;return H(this._trackKeys,function(e){return t._tracks[e]})},r.prototype.stopTracks=function(t,e){if(!t.length||!this._clip)return!0;for(var i=this._tracks,n=this._trackKeys,a=0;a<t.length;a++){var o=i[t[a]];o&&!o.isFinished()&&(e?o.step(this._target,1):this._started===1&&o.step(this._target,0),o.setFinished())}for(var s=!0,a=0;a<n.length;a++)if(!i[n[a]].isFinished()){s=!1;break}return s&&this._abortedCallback(),s},r.prototype.saveTo=function(t,e,i){if(t){e=e||this._trackKeys;for(var n=0;n<e.length;n++){var a=e[n],o=this._tracks[a];if(!(!o||o.isFinished())){var s=o.keyframes,l=s[i?0:s.length-1];l&&(t[a]=oo(l.rawValue))}}}},r.prototype.__changeFinalValue=function(t,e){e=e||ft(t);for(var i=0;i<e.length;i++){var n=e[i],a=this._tracks[n];if(a){var o=a.keyframes;if(o.length>1){var s=o.pop();a.addKeyframe(s.time,t[n]),a.prepare(this._maxTime,a.getAdditiveTrack())}}}},r}(),Ge=function(){function r(t){t&&(this._$eventProcessor=t)}return r.prototype.on=function(t,e,i,n){this._$handlers||(this._$handlers={});var a=this._$handlers;if(typeof e=="function"&&(n=i,i=e,e=null),!i||!t)return this;var o=this._$eventProcessor;e!=null&&o&&o.normalizeQuery&&(e=o.normalizeQuery(e)),a[t]||(a[t]=[]);for(var s=0;s<a[t].length;s++)if(a[t][s].h===i)return this;var l={h:i,query:e,ctx:n||this,callAtLast:i.zrEventfulCallAtLast},u=a[t].length-1,f=a[t][u];return f&&f.callAtLast?a[t].splice(u,0,l):a[t].push(l),this},r.prototype.isSilent=function(t){var e=this._$handlers;return!e||!e[t]||!e[t].length},r.prototype.off=function(t,e){var i=this._$handlers;if(!i)return this;if(!t)return this._$handlers={},this;if(e){if(i[t]){for(var n=[],a=0,o=i[t].length;a<o;a++)i[t][a].h!==e&&n.push(i[t][a]);i[t]=n}i[t]&&i[t].length===0&&delete i[t]}else delete i[t];return this},r.prototype.trigger=function(t){for(var e=[],i=1;i<arguments.length;i++)e[i-1]=arguments[i];if(!this._$handlers)return this;var n=this._$handlers[t],a=this._$eventProcessor;if(n)for(var o=e.length,s=n.length,l=0;l<s;l++){var u=n[l];if(!(a&&a.filter&&u.query!=null&&!a.filter(t,u.query)))switch(o){case 0:u.h.call(u.ctx);break;case 1:u.h.call(u.ctx,e[0]);break;case 2:u.h.call(u.ctx,e[0],e[1]);break;default:u.h.apply(u.ctx,e);break}}return a&&a.afterTrigger&&a.afterTrigger(t),this},r.prototype.triggerWithContext=function(t){for(var e=[],i=1;i<arguments.length;i++)e[i-1]=arguments[i];if(!this._$handlers)return this;var n=this._$handlers[t],a=this._$eventProcessor;if(n)for(var o=e.length,s=e[o-1],l=n.length,u=0;u<l;u++){var f=n[u];if(!(a&&a.filter&&f.query!=null&&!a.filter(t,f.query)))switch(o){case 0:f.h.call(s);break;case 1:f.h.call(s,e[0]);break;case 2:f.h.call(s,e[0],e[1]);break;default:f.h.apply(s,e.slice(1,o-1));break}}return a&&a.afterTrigger&&a.afterTrigger(t),this},r}(),Zp=1;Z.hasGlobalWindow&&(Zp=Math.max(window.devicePixelRatio||window.screen&&window.screen.deviceXDPI/window.screen.logicalXDPI||1,1));var Io=Zp,uu=.4,fu="#333",hu="#ccc",b1="#eee",Kt=1,wn=2,Ri=4,Ns="__zr_normal__",Fs=Wn.concat(["ignore"]),x1=Ne(Wn,function(r,t){return r[t]=!0,r},{ignore:!1}),mi={},T1=new et(0,0,0,0),rs=function(){function r(t){this.id=lf(),this.animators=[],this.currentStates=[],this.states={},this._init(t)}return r.prototype._init=function(t){this.attr(t)},r.prototype.drift=function(t,e,i){switch(this.draggable){case"horizontal":e=0;break;case"vertical":t=0;break}var n=this.transform;n||(n=this.transform=[1,0,0,1,0,0]),n[4]+=t,n[5]+=e,this.decomposeTransform(),this.markRedraw()},r.prototype.beforeUpdate=function(){},r.prototype.afterUpdate=function(){},r.prototype.update=function(){this.updateTransform(),this.__dirty&&this.updateInnerText()},r.prototype.updateInnerText=function(t){var e=this._textContent;if(e&&(!e.ignore||t)){this.textConfig||(this.textConfig={});var i=this.textConfig,n=i.local,a=e.innerTransformable,o=void 0,s=void 0,l=!1;a.parent=n?this:null;var u=!1;if(a.copyTransform(e),i.position!=null){var f=T1;i.layoutRect?f.copy(i.layoutRect):f.copy(this.getBoundingRect()),n||f.applyTransform(this.transform),this.calculateTextPosition?this.calculateTextPosition(mi,i,f):xo(mi,i,f),a.x=mi.x,a.y=mi.y,o=mi.align,s=mi.verticalAlign;var h=i.origin;if(h&&i.rotation!=null){var c=void 0,v=void 0;h==="center"?(c=f.width*.5,v=f.height*.5):(c=Fe(h[0],f.width),v=Fe(h[1],f.height)),u=!0,a.originX=-a.x+c+(n?0:f.x),a.originY=-a.y+v+(n?0:f.y)}}i.rotation!=null&&(a.rotation=i.rotation);var d=i.offset;d&&(a.x+=d[0],a.y+=d[1],u||(a.originX=-d[0],a.originY=-d[1]));var y=i.inside==null?typeof i.position=="string"&&i.position.indexOf("inside")>=0:i.inside,p=this._innerTextDefaultStyle||(this._innerTextDefaultStyle={}),g=void 0,m=void 0,_=void 0;y&&this.canBeInsideText()?(g=i.insideFill,m=i.insideStroke,(g==null||g==="auto")&&(g=this.getInsideTextFill()),(m==null||m==="auto")&&(m=this.getInsideTextStroke(g),_=!0)):(g=i.outsideFill,m=i.outsideStroke,(g==null||g==="auto")&&(g=this.getOutsideFill()),(m==null||m==="auto")&&(m=this.getOutsideStroke(g),_=!0)),g=g||"#000",(g!==p.fill||m!==p.stroke||_!==p.autoStroke||o!==p.align||s!==p.verticalAlign)&&(l=!0,p.fill=g,p.stroke=m,p.autoStroke=_,p.align=o,p.verticalAlign=s,e.setDefaultTextStyle(p)),e.__dirty|=Kt,l&&e.dirtyStyle(!0)}},r.prototype.canBeInsideText=function(){return!0},r.prototype.getInsideTextFill=function(){return"#fff"},r.prototype.getInsideTextStroke=function(t){return"#000"},r.prototype.getOutsideFill=function(){return this.__zr&&this.__zr.isDarkMode()?hu:fu},r.prototype.getOutsideStroke=function(t){var e=this.__zr&&this.__zr.getBackgroundColor(),i=typeof e=="string"&&jt(e);i||(i=[255,255,255,1]);for(var n=i[3],a=this.__zr.isDarkMode(),o=0;o<3;o++)i[o]=i[o]*n+(a?0:255)*(1-n);return i[3]=1,ke(i,"rgba")},r.prototype.traverse=function(t,e){},r.prototype.attrKV=function(t,e){t==="textConfig"?this.setTextConfig(e):t==="textContent"?this.setTextContent(e):t==="clipPath"?this.setClipPath(e):t==="extra"?(this.extra=this.extra||{},O(this.extra,e)):this[t]=e},r.prototype.hide=function(){this.ignore=!0,this.markRedraw()},r.prototype.show=function(){this.ignore=!1,this.markRedraw()},r.prototype.attr=function(t,e){if(typeof t=="string")this.attrKV(t,e);else if(z(t))for(var i=t,n=ft(i),a=0;a<n.length;a++){var o=n[a];this.attrKV(o,t[o])}return this.markRedraw(),this},r.prototype.saveCurrentToNormalState=function(t){this._innerSaveToNormal(t);for(var e=this._normalState,i=0;i<this.animators.length;i++){var n=this.animators[i],a=n.__fromStateTransition;if(!(n.getLoop()||a&&a!==Ns)){var o=n.targetName,s=o?e[o]:e;n.saveTo(s)}}},r.prototype._innerSaveToNormal=function(t){var e=this._normalState;e||(e=this._normalState={}),t.textConfig&&!e.textConfig&&(e.textConfig=this.textConfig),this._savePrimaryToNormal(t,e,Fs)},r.prototype._savePrimaryToNormal=function(t,e,i){for(var n=0;n<i.length;n++){var a=i[n];t[a]!=null&&!(a in e)&&(e[a]=this[a])}},r.prototype.hasState=function(){return this.currentStates.length>0},r.prototype.getState=function(t){return this.states[t]},r.prototype.ensureState=function(t){var e=this.states;return e[t]||(e[t]={}),e[t]},r.prototype.clearStates=function(t){this.useState(Ns,!1,t)},r.prototype.useState=function(t,e,i,n){var a=t===Ns,o=this.hasState();if(!(!o&&a)){var s=this.currentStates,l=this.stateTransition;if(!(ot(s,t)>=0&&(e||s.length===1))){var u;if(this.stateProxy&&!a&&(u=this.stateProxy(t)),u||(u=this.states&&this.states[t]),!u&&!a){Qo("State "+t+" not exists.");return}a||this.saveCurrentToNormalState(u);var f=!!(u&&u.hoverLayer||n);f&&this._toggleHoverLayerFlag(!0),this._applyStateObj(t,u,this._normalState,e,!i&&!this.__inHover&&l&&l.duration>0,l);var h=this._textContent,c=this._textGuide;return h&&h.useState(t,e,i,f),c&&c.useState(t,e,i,f),a?(this.currentStates=[],this._normalState={}):e?this.currentStates.push(t):this.currentStates=[t],this._updateAnimationTargets(),this.markRedraw(),!f&&this.__inHover&&(this._toggleHoverLayerFlag(!1),this.__dirty&=~Kt),u}}},r.prototype.useStates=function(t,e,i){if(!t.length)this.clearStates();else{var n=[],a=this.currentStates,o=t.length,s=o===a.length;if(s){for(var l=0;l<o;l++)if(t[l]!==a[l]){s=!1;break}}if(s)return;for(var l=0;l<o;l++){var u=t[l],f=void 0;this.stateProxy&&(f=this.stateProxy(u,t)),f||(f=this.states[u]),f&&n.push(f)}var h=n[o-1],c=!!(h&&h.hoverLayer||i);c&&this._toggleHoverLayerFlag(!0);var v=this._mergeStates(n),d=this.stateTransition;this.saveCurrentToNormalState(v),this._applyStateObj(t.join(","),v,this._normalState,!1,!e&&!this.__inHover&&d&&d.duration>0,d);var y=this._textContent,p=this._textGuide;y&&y.useStates(t,e,c),p&&p.useStates(t,e,c),this._updateAnimationTargets(),this.currentStates=t.slice(),this.markRedraw(),!c&&this.__inHover&&(this._toggleHoverLayerFlag(!1),this.__dirty&=~Kt)}},r.prototype.isSilent=function(){for(var t=this.silent,e=this.parent;!t&&e;){if(e.silent){t=!0;break}e=e.parent}return t},r.prototype._updateAnimationTargets=function(){for(var t=0;t<this.animators.length;t++){var e=this.animators[t];e.targetName&&e.changeTarget(this[e.targetName])}},r.prototype.removeState=function(t){var e=ot(this.currentStates,t);if(e>=0){var i=this.currentStates.slice();i.splice(e,1),this.useStates(i)}},r.prototype.replaceState=function(t,e,i){var n=this.currentStates.slice(),a=ot(n,t),o=ot(n,e)>=0;a>=0?o?n.splice(a,1):n[a]=e:i&&!o&&n.push(e),this.useStates(n)},r.prototype.toggleState=function(t,e){e?this.useState(t,!0):this.removeState(t)},r.prototype._mergeStates=function(t){for(var e={},i,n=0;n<t.length;n++){var a=t[n];O(e,a),a.textConfig&&(i=i||{},O(i,a.textConfig))}return i&&(e.textConfig=i),e},r.prototype._applyStateObj=function(t,e,i,n,a,o){var s=!(e&&n);e&&e.textConfig?(this.textConfig=O({},n?this.textConfig:i.textConfig),O(this.textConfig,e.textConfig)):s&&i.textConfig&&(this.textConfig=i.textConfig);for(var l={},u=!1,f=0;f<Fs.length;f++){var h=Fs[f],c=a&&x1[h];e&&e[h]!=null?c?(u=!0,l[h]=e[h]):this[h]=e[h]:s&&i[h]!=null&&(c?(u=!0,l[h]=i[h]):this[h]=i[h])}if(!a)for(var f=0;f<this.animators.length;f++){var v=this.animators[f],d=v.targetName;v.getLoop()||v.__changeFinalValue(d?(e||i)[d]:e||i)}u&&this._transitionState(t,l,o)},r.prototype._attachComponent=function(t){if(!(t.__zr&&!t.__hostTarget)&&t!==this){var e=this.__zr;e&&t.addSelfToZr(e),t.__zr=e,t.__hostTarget=this}},r.prototype._detachComponent=function(t){t.__zr&&t.removeSelfFromZr(t.__zr),t.__zr=null,t.__hostTarget=null},r.prototype.getClipPath=function(){return this._clipPath},r.prototype.setClipPath=function(t){this._clipPath&&this._clipPath!==t&&this.removeClipPath(),this._attachComponent(t),this._clipPath=t,this.markRedraw()},r.prototype.removeClipPath=function(){var t=this._clipPath;t&&(this._detachComponent(t),this._clipPath=null,this.markRedraw())},r.prototype.getTextContent=function(){return this._textContent},r.prototype.setTextContent=function(t){var e=this._textContent;e!==t&&(e&&e!==t&&this.removeTextContent(),t.innerTransformable=new mf,this._attachComponent(t),this._textContent=t,this.markRedraw())},r.prototype.setTextConfig=function(t){this.textConfig||(this.textConfig={}),O(this.textConfig,t),this.markRedraw()},r.prototype.removeTextConfig=function(){this.textConfig=null,this.markRedraw()},r.prototype.removeTextContent=function(){var t=this._textContent;t&&(t.innerTransformable=null,this._detachComponent(t),this._textContent=null,this._innerTextDefaultStyle=null,this.markRedraw())},r.prototype.getTextGuideLine=function(){return this._textGuide},r.prototype.setTextGuideLine=function(t){this._textGuide&&this._textGuide!==t&&this.removeTextGuideLine(),this._attachComponent(t),this._textGuide=t,this.markRedraw()},r.prototype.removeTextGuideLine=function(){var t=this._textGuide;t&&(this._detachComponent(t),this._textGuide=null,this.markRedraw())},r.prototype.markRedraw=function(){this.__dirty|=Kt;var t=this.__zr;t&&(this.__inHover?t.refreshHover():t.refresh()),this.__hostTarget&&this.__hostTarget.markRedraw()},r.prototype.dirty=function(){this.markRedraw()},r.prototype._toggleHoverLayerFlag=function(t){this.__inHover=t;var e=this._textContent,i=this._textGuide;e&&(e.__inHover=t),i&&(i.__inHover=t)},r.prototype.addSelfToZr=function(t){if(this.__zr!==t){this.__zr=t;var e=this.animators;if(e)for(var i=0;i<e.length;i++)t.animation.addAnimator(e[i]);this._clipPath&&this._clipPath.addSelfToZr(t),this._textContent&&this._textContent.addSelfToZr(t),this._textGuide&&this._textGuide.addSelfToZr(t)}},r.prototype.removeSelfFromZr=function(t){if(this.__zr){this.__zr=null;var e=this.animators;if(e)for(var i=0;i<e.length;i++)t.animation.removeAnimator(e[i]);this._clipPath&&this._clipPath.removeSelfFromZr(t),this._textContent&&this._textContent.removeSelfFromZr(t),this._textGuide&&this._textGuide.removeSelfFromZr(t)}},r.prototype.animate=function(t,e,i){var n=t?this[t]:this,a=new Sf(n,e,i);return t&&(a.targetName=t),this.addAnimator(a,t),a},r.prototype.addAnimator=function(t,e){var i=this.__zr,n=this;t.during(function(){n.updateDuringAnimation(e)}).done(function(){var a=n.animators,o=ot(a,t);o>=0&&a.splice(o,1)}),this.animators.push(t),i&&i.animation.addAnimator(t),i&&i.wakeUp()},r.prototype.updateDuringAnimation=function(t){this.markRedraw()},r.prototype.stopAnimation=function(t,e){for(var i=this.animators,n=i.length,a=[],o=0;o<n;o++){var s=i[o];!t||t===s.scope?s.stop(e):a.push(s)}return this.animators=a,this},r.prototype.animateTo=function(t,e,i){zs(this,t,e,i)},r.prototype.animateFrom=function(t,e,i){zs(this,t,e,i,!0)},r.prototype._transitionState=function(t,e,i,n){for(var a=zs(this,e,i,n),o=0;o<a.length;o++)a[o].__fromStateTransition=t},r.prototype.getBoundingRect=function(){return null},r.prototype.getPaintRect=function(){return null},r.initDefaultProps=function(){var t=r.prototype;t.type="element",t.name="",t.ignore=t.silent=t.isGroup=t.draggable=t.dragging=t.ignoreClip=t.__inHover=!1,t.__dirty=Kt;function e(i,n,a,o){Object.defineProperty(t,i,{get:function(){if(!this[n]){var l=this[n]=[];s(this,l)}return this[n]},set:function(l){this[a]=l[0],this[o]=l[1],this[n]=l,s(this,l)}});function s(l,u){Object.defineProperty(u,0,{get:function(){return l[a]},set:function(f){l[a]=f}}),Object.defineProperty(u,1,{get:function(){return l[o]},set:function(f){l[o]=f}})}}Object.defineProperty&&(e("position","_legacyPos","x","y"),e("scale","_legacyScale","scaleX","scaleY"),e("origin","_legacyOrigin","originX","originY"))}(),r}();de(rs,Ge);de(rs,mf);function zs(r,t,e,i,n){e=e||{};var a=[];qp(r,"",r,t,e,i,a,n);var o=a.length,s=!1,l=e.done,u=e.aborted,f=function(){s=!0,o--,o<=0&&(s?l&&l():u&&u())},h=function(){o--,o<=0&&(s?l&&l():u&&u())};o||l&&l(),a.length>0&&e.during&&a[0].during(function(d,y){e.during(y)});for(var c=0;c<a.length;c++){var v=a[c];f&&v.done(f),h&&v.aborted(h),e.force&&v.duration(e.duration),v.start(e.easing)}return a}function Vs(r,t,e){for(var i=0;i<e;i++)r[i]=t[i]}function M1(r){return Nt(r[0])}function C1(r,t,e){if(Nt(t[e]))if(Nt(r[e])||(r[e]=[]),Ft(t[e])){var i=t[e].length;r[e].length!==i&&(r[e]=new t[e].constructor(i),Vs(r[e],t[e],i))}else{var n=t[e],a=r[e],o=n.length;if(M1(n))for(var s=n[0].length,l=0;l<o;l++)a[l]?Vs(a[l],n[l],s):a[l]=Array.prototype.slice.call(n[l]);else Vs(a,n,o);a.length=n.length}else r[e]=t[e]}function D1(r,t){return r===t||Nt(r)&&Nt(t)&&A1(r,t)}function A1(r,t){var e=r.length;if(e!==t.length)return!1;for(var i=0;i<e;i++)if(r[i]!==t[i])return!1;return!0}function qp(r,t,e,i,n,a,o,s){for(var l=ft(i),u=n.duration,f=n.delay,h=n.additive,c=n.setToFinal,v=!z(a),d=r.animators,y=[],p=0;p<l.length;p++){var g=l[p],m=i[g];if(m!=null&&e[g]!=null&&(v||a[g]))if(z(m)&&!Nt(m)&&!sa(m)){if(t){s||(e[g]=m,r.updateDuringAnimation(t));continue}qp(r,g,e[g],m,n,a&&a[g],o,s)}else y.push(g);else s||(e[g]=m,r.updateDuringAnimation(t),y.push(g))}var _=y.length;if(!h&&_)for(var S=0;S<d.length;S++){var b=d[S];if(b.targetName===t){var w=b.stopTracks(y);if(w){var x=ot(d,b);d.splice(x,1)}}}if(n.force||(y=wt(y,function(M){return!D1(i[M],e[M])}),_=y.length),_>0||n.force&&!o.length){var C=void 0,D=void 0,A=void 0;if(s){D={},c&&(C={});for(var S=0;S<_;S++){var g=y[S];D[g]=e[g],c?C[g]=i[g]:e[g]=i[g]}}else if(c){A={};for(var S=0;S<_;S++){var g=y[S];A[g]=oo(e[g]),C1(e,i,g)}}var b=new Sf(e,!1,!1,h?wt(d,function(I){return I.targetName===t}):null);b.targetName=t,n.scope&&(b.scope=n.scope),c&&C&&b.whenWithKeys(0,C,y),A&&b.whenWithKeys(0,A,y),b.whenWithKeys(u??500,s?D:i,y).delay(f||0),r.addAnimator(b,t),o.push(b)}}var vu="__zr_style_"+Math.round(Math.random()*10),ei={shadowBlur:0,shadowOffsetX:0,shadowOffsetY:0,shadowColor:"#000",opacity:1,blend:"source-over"},is={style:{shadowBlur:!0,shadowOffsetX:!0,shadowOffsetY:!0,shadowColor:!0,opacity:!0}};ei[vu]=!0;var Fh=["z","z2","invisible"],I1=["invisible"],L1=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype._init=function(e){for(var i=ft(e),n=0;n<i.length;n++){var a=i[n];a==="style"?this.useStyle(e[a]):r.prototype.attrKV.call(this,a,e[a])}this.style||this.useStyle({})},t.prototype.beforeBrush=function(){},t.prototype.afterBrush=function(){},t.prototype.innerBeforeBrush=function(){},t.prototype.innerAfterBrush=function(){},t.prototype.shouldBePainted=function(e,i,n,a){var o=this.transform;if(this.ignore||this.invisible||this.style.opacity===0||this.culling&&P1(this,e,i)||o&&!o[0]&&!o[3])return!1;if(n&&this.__clipPaths){for(var s=0;s<this.__clipPaths.length;++s)if(this.__clipPaths[s].isZeroArea())return!1}if(a&&this.parent)for(var l=this.parent;l;){if(l.ignore)return!1;l=l.parent}return!0},t.prototype.contain=function(e,i){return this.rectContain(e,i)},t.prototype.traverse=function(e,i){e.call(i,this)},t.prototype.rectContain=function(e,i){var n=this.transformCoordToLocal(e,i),a=this.getBoundingRect();return a.contain(n[0],n[1])},t.prototype.getPaintRect=function(){var e=this._paintRect;if(!this._paintRect||this.__dirty){var i=this.transform,n=this.getBoundingRect(),a=this.style,o=a.shadowBlur||0,s=a.shadowOffsetX||0,l=a.shadowOffsetY||0;e=this._paintRect||(this._paintRect=new et(0,0,0,0)),i?et.applyTransform(e,n,i):e.copy(n),(o||s||l)&&(e.width+=o*2+Math.abs(s),e.height+=o*2+Math.abs(l),e.x=Math.min(e.x,e.x+s-o),e.y=Math.min(e.y,e.y+l-o));var u=this.dirtyRectTolerance;e.isZero()||(e.x=Math.floor(e.x-u),e.y=Math.floor(e.y-u),e.width=Math.ceil(e.width+1+u*2),e.height=Math.ceil(e.height+1+u*2))}return e},t.prototype.setPrevPaintRect=function(e){e?(this._prevPaintRect=this._prevPaintRect||new et(0,0,0,0),this._prevPaintRect.copy(e)):this._prevPaintRect=null},t.prototype.getPrevPaintRect=function(){return this._prevPaintRect},t.prototype.animateStyle=function(e){return this.animate("style",e)},t.prototype.updateDuringAnimation=function(e){e==="style"?this.dirtyStyle():this.markRedraw()},t.prototype.attrKV=function(e,i){e!=="style"?r.prototype.attrKV.call(this,e,i):this.style?this.setStyle(i):this.useStyle(i)},t.prototype.setStyle=function(e,i){return typeof e=="string"?this.style[e]=i:O(this.style,e),this.dirtyStyle(),this},t.prototype.dirtyStyle=function(e){e||this.markRedraw(),this.__dirty|=wn,this._rect&&(this._rect=null)},t.prototype.dirty=function(){this.dirtyStyle()},t.prototype.styleChanged=function(){return!!(this.__dirty&wn)},t.prototype.styleUpdated=function(){this.__dirty&=~wn},t.prototype.createStyle=function(e){return la(ei,e)},t.prototype.useStyle=function(e){e[vu]||(e=this.createStyle(e)),this.__inHover?this.__hoverStyle=e:this.style=e,this.dirtyStyle()},t.prototype.isStyleObject=function(e){return e[vu]},t.prototype._innerSaveToNormal=function(e){r.prototype._innerSaveToNormal.call(this,e);var i=this._normalState;e.style&&!i.style&&(i.style=this._mergeStyle(this.createStyle(),this.style)),this._savePrimaryToNormal(e,i,Fh)},t.prototype._applyStateObj=function(e,i,n,a,o,s){r.prototype._applyStateObj.call(this,e,i,n,a,o,s);var l=!(i&&a),u;if(i&&i.style?o?a?u=i.style:(u=this._mergeStyle(this.createStyle(),n.style),this._mergeStyle(u,i.style)):(u=this._mergeStyle(this.createStyle(),a?this.style:n.style),this._mergeStyle(u,i.style)):l&&(u=n.style),u)if(o){var f=this.style;if(this.style=this.createStyle(l?{}:f),l)for(var h=ft(f),c=0;c<h.length;c++){var v=h[c];v in u&&(u[v]=u[v],this.style[v]=f[v])}for(var d=ft(u),c=0;c<d.length;c++){var v=d[c];this.style[v]=this.style[v]}this._transitionState(e,{style:u},s,this.getAnimationStyleProps())}else this.useStyle(u);for(var y=this.__inHover?I1:Fh,c=0;c<y.length;c++){var v=y[c];i&&i[v]!=null?this[v]=i[v]:l&&n[v]!=null&&(this[v]=n[v])}},t.prototype._mergeStates=function(e){for(var i=r.prototype._mergeStates.call(this,e),n,a=0;a<e.length;a++){var o=e[a];o.style&&(n=n||{},this._mergeStyle(n,o.style))}return n&&(i.style=n),i},t.prototype._mergeStyle=function(e,i){return O(e,i),e},t.prototype.getAnimationStyleProps=function(){return is},t.initDefaultProps=function(){var e=t.prototype;e.type="displayable",e.invisible=!1,e.z=0,e.z2=0,e.zlevel=0,e.culling=!1,e.cursor="pointer",e.rectHover=!1,e.incremental=!1,e._rect=null,e.dirtyRectTolerance=0,e.__dirty=Kt|wn}(),t}(rs),Hs=new et(0,0,0,0),Gs=new et(0,0,0,0);function P1(r,t,e){return Hs.copy(r.getBoundingRect()),r.transform&&Hs.applyTransform(r.transform),Gs.width=t,Gs.height=e,!Hs.intersect(Gs)}const va=L1;var le=Math.min,ue=Math.max,Ws=Math.sin,Us=Math.cos,Rr=Math.PI*2,Ca=vi(),Da=vi(),Aa=vi();function zh(r,t,e,i,n,a){n[0]=le(r,e),n[1]=le(t,i),a[0]=ue(r,e),a[1]=ue(t,i)}var Vh=[],Hh=[];function R1(r,t,e,i,n,a,o,s,l,u){var f=Gp,h=bt,c=f(r,e,n,o,Vh);l[0]=1/0,l[1]=1/0,u[0]=-1/0,u[1]=-1/0;for(var v=0;v<c;v++){var d=h(r,e,n,o,Vh[v]);l[0]=le(d,l[0]),u[0]=ue(d,u[0])}c=f(t,i,a,s,Hh);for(var v=0;v<c;v++){var y=h(t,i,a,s,Hh[v]);l[1]=le(y,l[1]),u[1]=ue(y,u[1])}l[0]=le(r,l[0]),u[0]=ue(r,u[0]),l[0]=le(o,l[0]),u[0]=ue(o,u[0]),l[1]=le(t,l[1]),u[1]=ue(t,u[1]),l[1]=le(s,l[1]),u[1]=ue(s,u[1])}function E1(r,t,e,i,n,a,o,s){var l=Wp,u=Ht,f=ue(le(l(r,e,n),1),0),h=ue(le(l(t,i,a),1),0),c=u(r,e,n,f),v=u(t,i,a,h);o[0]=le(r,n,c),o[1]=le(t,a,v),s[0]=ue(r,n,c),s[1]=ue(t,a,v)}function O1(r,t,e,i,n,a,o,s,l){var u=cr,f=dr,h=Math.abs(n-a);if(h%Rr<1e-4&&h>1e-4){s[0]=r-e,s[1]=t-i,l[0]=r+e,l[1]=t+i;return}if(Ca[0]=Us(n)*e+r,Ca[1]=Ws(n)*i+t,Da[0]=Us(a)*e+r,Da[1]=Ws(a)*i+t,u(s,Ca,Da),f(l,Ca,Da),n=n%Rr,n<0&&(n=n+Rr),a=a%Rr,a<0&&(a=a+Rr),n>a&&!o?a+=Rr:n<a&&o&&(n+=Rr),o){var c=a;a=n,n=c}for(var v=0;v<a;v+=Math.PI/2)v>n&&(Aa[0]=Us(v)*e+r,Aa[1]=Ws(v)*i+t,u(s,Aa,s),f(l,Aa,l))}var at={M:1,L:2,C:3,Q:4,A:5,Z:6,R:7},Er=[],Or=[],De=[],ir=[],Ae=[],Ie=[],Ys=Math.min,Xs=Math.max,kr=Math.cos,Br=Math.sin,Xe=Math.abs,cu=Math.PI,hr=cu*2,$s=typeof Float32Array<"u",en=[];function Zs(r){var t=Math.round(r/cu*1e8)/1e8;return t%2*cu}function k1(r,t){var e=Zs(r[0]);e<0&&(e+=hr);var i=e-r[0],n=r[1];n+=i,!t&&n-e>=hr?n=e+hr:t&&e-n>=hr?n=e-hr:!t&&e>n?n=e+(hr-Zs(e-n)):t&&e<n&&(n=e-(hr-Zs(n-e))),r[0]=e,r[1]=n}var li=function(){function r(t){this.dpr=1,this._xi=0,this._yi=0,this._x0=0,this._y0=0,this._len=0,t&&(this._saveData=!1),this._saveData&&(this.data=[])}return r.prototype.increaseVersion=function(){this._version++},r.prototype.getVersion=function(){return this._version},r.prototype.setScale=function(t,e,i){i=i||0,i>0&&(this._ux=Xe(i/Io/t)||0,this._uy=Xe(i/Io/e)||0)},r.prototype.setDPR=function(t){this.dpr=t},r.prototype.setContext=function(t){this._ctx=t},r.prototype.getContext=function(){return this._ctx},r.prototype.beginPath=function(){return this._ctx&&this._ctx.beginPath(),this.reset(),this},r.prototype.reset=function(){this._saveData&&(this._len=0),this._pathSegLen&&(this._pathSegLen=null,this._pathLen=0),this._version++},r.prototype.moveTo=function(t,e){return this._drawPendingPt(),this.addData(at.M,t,e),this._ctx&&this._ctx.moveTo(t,e),this._x0=t,this._y0=e,this._xi=t,this._yi=e,this},r.prototype.lineTo=function(t,e){var i=Xe(t-this._xi),n=Xe(e-this._yi),a=i>this._ux||n>this._uy;if(this.addData(at.L,t,e),this._ctx&&a&&this._ctx.lineTo(t,e),a)this._xi=t,this._yi=e,this._pendingPtDist=0;else{var o=i*i+n*n;o>this._pendingPtDist&&(this._pendingPtX=t,this._pendingPtY=e,this._pendingPtDist=o)}return this},r.prototype.bezierCurveTo=function(t,e,i,n,a,o){return this._drawPendingPt(),this.addData(at.C,t,e,i,n,a,o),this._ctx&&this._ctx.bezierCurveTo(t,e,i,n,a,o),this._xi=a,this._yi=o,this},r.prototype.quadraticCurveTo=function(t,e,i,n){return this._drawPendingPt(),this.addData(at.Q,t,e,i,n),this._ctx&&this._ctx.quadraticCurveTo(t,e,i,n),this._xi=i,this._yi=n,this},r.prototype.arc=function(t,e,i,n,a,o){this._drawPendingPt(),en[0]=n,en[1]=a,k1(en,o),n=en[0],a=en[1];var s=a-n;return this.addData(at.A,t,e,i,i,n,s,0,o?0:1),this._ctx&&this._ctx.arc(t,e,i,n,a,o),this._xi=kr(a)*i+t,this._yi=Br(a)*i+e,this},r.prototype.arcTo=function(t,e,i,n,a){return this._drawPendingPt(),this._ctx&&this._ctx.arcTo(t,e,i,n,a),this},r.prototype.rect=function(t,e,i,n){return this._drawPendingPt(),this._ctx&&this._ctx.rect(t,e,i,n),this.addData(at.R,t,e,i,n),this},r.prototype.closePath=function(){this._drawPendingPt(),this.addData(at.Z);var t=this._ctx,e=this._x0,i=this._y0;return t&&t.closePath(),this._xi=e,this._yi=i,this},r.prototype.fill=function(t){t&&t.fill(),this.toStatic()},r.prototype.stroke=function(t){t&&t.stroke(),this.toStatic()},r.prototype.len=function(){return this._len},r.prototype.setData=function(t){var e=t.length;!(this.data&&this.data.length===e)&&$s&&(this.data=new Float32Array(e));for(var i=0;i<e;i++)this.data[i]=t[i];this._len=e},r.prototype.appendPath=function(t){t instanceof Array||(t=[t]);for(var e=t.length,i=0,n=this._len,a=0;a<e;a++)i+=t[a].len();$s&&this.data instanceof Float32Array&&(this.data=new Float32Array(n+i));for(var a=0;a<e;a++)for(var o=t[a].data,s=0;s<o.length;s++)this.data[n++]=o[s];this._len=n},r.prototype.addData=function(t,e,i,n,a,o,s,l,u){if(this._saveData){var f=this.data;this._len+arguments.length>f.length&&(this._expandData(),f=this.data);for(var h=0;h<arguments.length;h++)f[this._len++]=arguments[h]}},r.prototype._drawPendingPt=function(){this._pendingPtDist>0&&(this._ctx&&this._ctx.lineTo(this._pendingPtX,this._pendingPtY),this._pendingPtDist=0)},r.prototype._expandData=function(){if(!(this.data instanceof Array)){for(var t=[],e=0;e<this._len;e++)t[e]=this.data[e];this.data=t}},r.prototype.toStatic=function(){if(this._saveData){this._drawPendingPt();var t=this.data;t instanceof Array&&(t.length=this._len,$s&&this._len>11&&(this.data=new Float32Array(t)))}},r.prototype.getBoundingRect=function(){De[0]=De[1]=Ae[0]=Ae[1]=Number.MAX_VALUE,ir[0]=ir[1]=Ie[0]=Ie[1]=-Number.MAX_VALUE;var t=this.data,e=0,i=0,n=0,a=0,o;for(o=0;o<this._len;){var s=t[o++],l=o===1;switch(l&&(e=t[o],i=t[o+1],n=e,a=i),s){case at.M:e=n=t[o++],i=a=t[o++],Ae[0]=n,Ae[1]=a,Ie[0]=n,Ie[1]=a;break;case at.L:zh(e,i,t[o],t[o+1],Ae,Ie),e=t[o++],i=t[o++];break;case at.C:R1(e,i,t[o++],t[o++],t[o++],t[o++],t[o],t[o+1],Ae,Ie),e=t[o++],i=t[o++];break;case at.Q:E1(e,i,t[o++],t[o++],t[o],t[o+1],Ae,Ie),e=t[o++],i=t[o++];break;case at.A:var u=t[o++],f=t[o++],h=t[o++],c=t[o++],v=t[o++],d=t[o++]+v;o+=1;var y=!t[o++];l&&(n=kr(v)*h+u,a=Br(v)*c+f),O1(u,f,h,c,v,d,y,Ae,Ie),e=kr(d)*h+u,i=Br(d)*c+f;break;case at.R:n=e=t[o++],a=i=t[o++];var p=t[o++],g=t[o++];zh(n,a,n+p,a+g,Ae,Ie);break;case at.Z:e=n,i=a;break}cr(De,De,Ae),dr(ir,ir,Ie)}return o===0&&(De[0]=De[1]=ir[0]=ir[1]=0),new et(De[0],De[1],ir[0]-De[0],ir[1]-De[1])},r.prototype._calculateLength=function(){var t=this.data,e=this._len,i=this._ux,n=this._uy,a=0,o=0,s=0,l=0;this._pathSegLen||(this._pathSegLen=[]);for(var u=this._pathSegLen,f=0,h=0,c=0;c<e;){var v=t[c++],d=c===1;d&&(a=t[c],o=t[c+1],s=a,l=o);var y=-1;switch(v){case at.M:a=s=t[c++],o=l=t[c++];break;case at.L:{var p=t[c++],g=t[c++],m=p-a,_=g-o;(Xe(m)>i||Xe(_)>n||c===e-1)&&(y=Math.sqrt(m*m+_*_),a=p,o=g);break}case at.C:{var S=t[c++],b=t[c++],p=t[c++],g=t[c++],w=t[c++],x=t[c++];y=i1(a,o,S,b,p,g,w,x,10),a=w,o=x;break}case at.Q:{var S=t[c++],b=t[c++],p=t[c++],g=t[c++];y=o1(a,o,S,b,p,g,10),a=p,o=g;break}case at.A:var C=t[c++],D=t[c++],A=t[c++],M=t[c++],I=t[c++],L=t[c++],P=L+I;c+=1,d&&(s=kr(I)*A+C,l=Br(I)*M+D),y=Xs(A,M)*Ys(hr,Math.abs(L)),a=kr(P)*A+C,o=Br(P)*M+D;break;case at.R:{s=a=t[c++],l=o=t[c++];var R=t[c++],E=t[c++];y=R*2+E*2;break}case at.Z:{var m=s-a,_=l-o;y=Math.sqrt(m*m+_*_),a=s,o=l;break}}y>=0&&(u[h++]=y,f+=y)}return this._pathLen=f,f},r.prototype.rebuildPath=function(t,e){var i=this.data,n=this._ux,a=this._uy,o=this._len,s,l,u,f,h,c,v=e<1,d,y,p=0,g=0,m,_=0,S,b;if(!(v&&(this._pathSegLen||this._calculateLength(),d=this._pathSegLen,y=this._pathLen,m=e*y,!m)))t:for(var w=0;w<o;){var x=i[w++],C=w===1;switch(C&&(u=i[w],f=i[w+1],s=u,l=f),x!==at.L&&_>0&&(t.lineTo(S,b),_=0),x){case at.M:s=u=i[w++],l=f=i[w++],t.moveTo(u,f);break;case at.L:{h=i[w++],c=i[w++];var D=Xe(h-u),A=Xe(c-f);if(D>n||A>a){if(v){var M=d[g++];if(p+M>m){var I=(m-p)/M;t.lineTo(u*(1-I)+h*I,f*(1-I)+c*I);break t}p+=M}t.lineTo(h,c),u=h,f=c,_=0}else{var L=D*D+A*A;L>_&&(S=h,b=c,_=L)}break}case at.C:{var P=i[w++],R=i[w++],E=i[w++],W=i[w++],B=i[w++],F=i[w++];if(v){var M=d[g++];if(p+M>m){var I=(m-p)/M;Do(u,P,E,B,I,Er),Do(f,R,W,F,I,Or),t.bezierCurveTo(Er[1],Or[1],Er[2],Or[2],Er[3],Or[3]);break t}p+=M}t.bezierCurveTo(P,R,E,W,B,F),u=B,f=F;break}case at.Q:{var P=i[w++],R=i[w++],E=i[w++],W=i[w++];if(v){var M=d[g++];if(p+M>m){var I=(m-p)/M;Ao(u,P,E,I,Er),Ao(f,R,W,I,Or),t.quadraticCurveTo(Er[1],Or[1],Er[2],Or[2]);break t}p+=M}t.quadraticCurveTo(P,R,E,W),u=E,f=W;break}case at.A:var G=i[w++],q=i[w++],it=i[w++],lt=i[w++],vt=i[w++],$t=i[w++],ct=i[w++],Ye=!i[w++],pi=it>lt?it:lt,Zt=Xe(it-lt)>.001,St=vt+$t,X=!1;if(v){var M=d[g++];p+M>m&&(St=vt+$t*(m-p)/M,X=!0),p+=M}if(Zt&&t.ellipse?t.ellipse(G,q,it,lt,ct,vt,St,Ye):t.arc(G,q,pi,vt,St,Ye),X)break t;C&&(s=kr(vt)*it+G,l=Br(vt)*lt+q),u=kr(St)*it+G,f=Br(St)*lt+q;break;case at.R:s=u=i[w],l=f=i[w+1],h=i[w++],c=i[w++];var j=i[w++],Tr=i[w++];if(v){var M=d[g++];if(p+M>m){var Lt=m-p;t.moveTo(h,c),t.lineTo(h+Ys(Lt,j),c),Lt-=j,Lt>0&&t.lineTo(h+j,c+Ys(Lt,Tr)),Lt-=Tr,Lt>0&&t.lineTo(h+Xs(j-Lt,0),c+Tr),Lt-=j,Lt>0&&t.lineTo(h,c+Xs(Tr-Lt,0));break t}p+=M}t.rect(h,c,j,Tr);break;case at.Z:if(v){var M=d[g++];if(p+M>m){var I=(m-p)/M;t.lineTo(u*(1-I)+s*I,f*(1-I)+l*I);break t}p+=M}t.closePath(),u=s,f=l}}},r.prototype.clone=function(){var t=new r,e=this.data;return t.data=e.slice?e.slice():Array.prototype.slice.call(e),t._len=this._len,t},r.CMD=at,r.initDefaultProps=function(){var t=r.prototype;t._saveData=!0,t._ux=0,t._uy=0,t._pendingPtDist=0,t._version=0}(),r}();function _i(r,t,e,i,n,a,o){if(n===0)return!1;var s=n,l=0,u=r;if(o>t+s&&o>i+s||o<t-s&&o<i-s||a>r+s&&a>e+s||a<r-s&&a<e-s)return!1;if(r!==e)l=(t-i)/(r-e),u=(r*i-e*t)/(r-e);else return Math.abs(a-r)<=s/2;var f=l*a-o+u,h=f*f/(l*l+1);return h<=s/2*s/2}function B1(r,t,e,i,n,a,o,s,l,u,f){if(l===0)return!1;var h=l;if(f>t+h&&f>i+h&&f>a+h&&f>s+h||f<t-h&&f<i-h&&f<a-h&&f<s-h||u>r+h&&u>e+h&&u>n+h&&u>o+h||u<r-h&&u<e-h&&u<n-h&&u<o-h)return!1;var c=r1(r,t,e,i,n,a,o,s,u,f);return c<=h/2}function N1(r,t,e,i,n,a,o,s,l){if(o===0)return!1;var u=o;if(l>t+u&&l>i+u&&l>a+u||l<t-u&&l<i-u&&l<a-u||s>r+u&&s>e+u&&s>n+u||s<r-u&&s<e-u&&s<n-u)return!1;var f=a1(r,t,e,i,n,a,s,l);return f<=u/2}var Gh=Math.PI*2;function Ia(r){return r%=Gh,r<0&&(r+=Gh),r}var rn=Math.PI*2;function F1(r,t,e,i,n,a,o,s,l){if(o===0)return!1;var u=o;s-=r,l-=t;var f=Math.sqrt(s*s+l*l);if(f-u>e||f+u<e)return!1;if(Math.abs(i-n)%rn<1e-4)return!0;if(a){var h=i;i=Ia(n),n=Ia(h)}else i=Ia(i),n=Ia(n);i>n&&(n+=rn);var c=Math.atan2(l,s);return c<0&&(c+=rn),c>=i&&c<=n||c+rn>=i&&c+rn<=n}function qe(r,t,e,i,n,a){if(a>t&&a>i||a<t&&a<i||i===t)return 0;var o=(a-t)/(i-t),s=i<t?1:-1;(o===1||o===0)&&(s=i<t?.5:-.5);var l=o*(e-r)+r;return l===n?1/0:l>n?s:0}var nr=li.CMD,Nr=Math.PI*2,z1=1e-4;function V1(r,t){return Math.abs(r-t)<z1}var Rt=[-1,-1,-1],oe=[-1,-1];function H1(){var r=oe[0];oe[0]=oe[1],oe[1]=r}function G1(r,t,e,i,n,a,o,s,l,u){if(u>t&&u>i&&u>a&&u>s||u<t&&u<i&&u<a&&u<s)return 0;var f=Co(t,i,a,s,u,Rt);if(f===0)return 0;for(var h=0,c=-1,v=void 0,d=void 0,y=0;y<f;y++){var p=Rt[y],g=p===0||p===1?.5:1,m=bt(r,e,n,o,p);m<l||(c<0&&(c=Gp(t,i,a,s,oe),oe[1]<oe[0]&&c>1&&H1(),v=bt(t,i,a,s,oe[0]),c>1&&(d=bt(t,i,a,s,oe[1]))),c===2?p<oe[0]?h+=v<t?g:-g:p<oe[1]?h+=d<v?g:-g:h+=s<d?g:-g:p<oe[0]?h+=v<t?g:-g:h+=s<v?g:-g)}return h}function W1(r,t,e,i,n,a,o,s){if(s>t&&s>i&&s>a||s<t&&s<i&&s<a)return 0;var l=n1(t,i,a,s,Rt);if(l===0)return 0;var u=Wp(t,i,a);if(u>=0&&u<=1){for(var f=0,h=Ht(t,i,a,u),c=0;c<l;c++){var v=Rt[c]===0||Rt[c]===1?.5:1,d=Ht(r,e,n,Rt[c]);d<o||(Rt[c]<u?f+=h<t?v:-v:f+=a<h?v:-v)}return f}else{var v=Rt[0]===0||Rt[0]===1?.5:1,d=Ht(r,e,n,Rt[0]);return d<o?0:a<t?v:-v}}function U1(r,t,e,i,n,a,o,s){if(s-=t,s>e||s<-e)return 0;var l=Math.sqrt(e*e-s*s);Rt[0]=-l,Rt[1]=l;var u=Math.abs(i-n);if(u<1e-4)return 0;if(u>=Nr-1e-4){i=0,n=Nr;var f=a?1:-1;return o>=Rt[0]+r&&o<=Rt[1]+r?f:0}if(i>n){var h=i;i=n,n=h}i<0&&(i+=Nr,n+=Nr);for(var c=0,v=0;v<2;v++){var d=Rt[v];if(d+r>o){var y=Math.atan2(s,d),f=a?1:-1;y<0&&(y=Nr+y),(y>=i&&y<=n||y+Nr>=i&&y+Nr<=n)&&(y>Math.PI/2&&y<Math.PI*1.5&&(f=-f),c+=f)}}return c}function Kp(r,t,e,i,n){for(var a=r.data,o=r.len(),s=0,l=0,u=0,f=0,h=0,c,v,d=0;d<o;){var y=a[d++],p=d===1;switch(y===nr.M&&d>1&&(e||(s+=qe(l,u,f,h,i,n))),p&&(l=a[d],u=a[d+1],f=l,h=u),y){case nr.M:f=a[d++],h=a[d++],l=f,u=h;break;case nr.L:if(e){if(_i(l,u,a[d],a[d+1],t,i,n))return!0}else s+=qe(l,u,a[d],a[d+1],i,n)||0;l=a[d++],u=a[d++];break;case nr.C:if(e){if(B1(l,u,a[d++],a[d++],a[d++],a[d++],a[d],a[d+1],t,i,n))return!0}else s+=G1(l,u,a[d++],a[d++],a[d++],a[d++],a[d],a[d+1],i,n)||0;l=a[d++],u=a[d++];break;case nr.Q:if(e){if(N1(l,u,a[d++],a[d++],a[d],a[d+1],t,i,n))return!0}else s+=W1(l,u,a[d++],a[d++],a[d],a[d+1],i,n)||0;l=a[d++],u=a[d++];break;case nr.A:var g=a[d++],m=a[d++],_=a[d++],S=a[d++],b=a[d++],w=a[d++];d+=1;var x=!!(1-a[d++]);c=Math.cos(b)*_+g,v=Math.sin(b)*S+m,p?(f=c,h=v):s+=qe(l,u,c,v,i,n);var C=(i-g)*S/_+g;if(e){if(F1(g,m,S,b,b+w,x,t,C,n))return!0}else s+=U1(g,m,S,b,b+w,x,C,n);l=Math.cos(b+w)*_+g,u=Math.sin(b+w)*S+m;break;case nr.R:f=l=a[d++],h=u=a[d++];var D=a[d++],A=a[d++];if(c=f+D,v=h+A,e){if(_i(f,h,c,h,t,i,n)||_i(c,h,c,v,t,i,n)||_i(c,v,f,v,t,i,n)||_i(f,v,f,h,t,i,n))return!0}else s+=qe(c,h,c,v,i,n),s+=qe(f,v,f,h,i,n);break;case nr.Z:if(e){if(_i(l,u,f,h,t,i,n))return!0}else s+=qe(l,u,f,h,i,n);l=f,u=h;break}}return!e&&!V1(u,h)&&(s+=qe(l,u,f,h,i,n)||0),s!==0}function Y1(r,t,e){return Kp(r,0,!1,t,e)}function X1(r,t,e,i){return Kp(r,t,!0,e,i)}var Qp=nt({fill:"#000",stroke:null,strokePercent:1,fillOpacity:1,strokeOpacity:1,lineDashOffset:0,lineWidth:1,lineCap:"butt",miterLimit:10,strokeNoScale:!1,strokeFirst:!1},ei),$1={style:nt({fill:!0,stroke:!0,strokePercent:!0,fillOpacity:!0,strokeOpacity:!0,lineDashOffset:!0,lineWidth:!0,miterLimit:!0},is.style)},qs=Wn.concat(["invisible","culling","z","z2","zlevel","parent"]),st=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.update=function(){var e=this;r.prototype.update.call(this);var i=this.style;if(i.decal){var n=this._decalEl=this._decalEl||new t;n.buildPath===t.prototype.buildPath&&(n.buildPath=function(l){e.buildPath(l,e.shape)}),n.silent=!0;var a=n.style;for(var o in i)a[o]!==i[o]&&(a[o]=i[o]);a.fill=i.fill?i.decal:null,a.decal=null,a.shadowColor=null,i.strokeFirst&&(a.stroke=null);for(var s=0;s<qs.length;++s)n[qs[s]]=this[qs[s]];n.__dirty|=Kt}else this._decalEl&&(this._decalEl=null)},t.prototype.getDecalElement=function(){return this._decalEl},t.prototype._init=function(e){var i=ft(e);this.shape=this.getDefaultShape();var n=this.getDefaultStyle();n&&this.useStyle(n);for(var a=0;a<i.length;a++){var o=i[a],s=e[o];o==="style"?this.style?O(this.style,s):this.useStyle(s):o==="shape"?O(this.shape,s):r.prototype.attrKV.call(this,o,s)}this.style||this.useStyle({})},t.prototype.getDefaultStyle=function(){return null},t.prototype.getDefaultShape=function(){return{}},t.prototype.canBeInsideText=function(){return this.hasFill()},t.prototype.getInsideTextFill=function(){var e=this.style.fill;if(e!=="none"){if(V(e)){var i=Yn(e,0);return i>.5?fu:i>.2?b1:hu}else if(e)return hu}return fu},t.prototype.getInsideTextStroke=function(e){var i=this.style.fill;if(V(i)){var n=this.__zr,a=!!(n&&n.isDarkMode()),o=Yn(e,0)<uu;if(a===o)return i}},t.prototype.buildPath=function(e,i,n){},t.prototype.pathUpdated=function(){this.__dirty&=~Ri},t.prototype.getUpdatedPathProxy=function(e){return!this.path&&this.createPathProxy(),this.path.beginPath(),this.buildPath(this.path,this.shape,e),this.path},t.prototype.createPathProxy=function(){this.path=new li(!1)},t.prototype.hasStroke=function(){var e=this.style,i=e.stroke;return!(i==null||i==="none"||!(e.lineWidth>0))},t.prototype.hasFill=function(){var e=this.style,i=e.fill;return i!=null&&i!=="none"},t.prototype.getBoundingRect=function(){var e=this._rect,i=this.style,n=!e;if(n){var a=!1;this.path||(a=!0,this.createPathProxy());var o=this.path;(a||this.__dirty&Ri)&&(o.beginPath(),this.buildPath(o,this.shape,!1),this.pathUpdated()),e=o.getBoundingRect()}if(this._rect=e,this.hasStroke()&&this.path&&this.path.len()>0){var s=this._rectStroke||(this._rectStroke=e.clone());if(this.__dirty||n){s.copy(e);var l=i.strokeNoScale?this.getLineScale():1,u=i.lineWidth;if(!this.hasFill()){var f=this.strokeContainThreshold;u=Math.max(u,f??4)}l>1e-10&&(s.width+=u/l,s.height+=u/l,s.x-=u/l/2,s.y-=u/l/2)}return s}return e},t.prototype.contain=function(e,i){var n=this.transformCoordToLocal(e,i),a=this.getBoundingRect(),o=this.style;if(e=n[0],i=n[1],a.contain(e,i)){var s=this.path;if(this.hasStroke()){var l=o.lineWidth,u=o.strokeNoScale?this.getLineScale():1;if(u>1e-10&&(this.hasFill()||(l=Math.max(l,this.strokeContainThreshold)),X1(s,l/u,e,i)))return!0}if(this.hasFill())return Y1(s,e,i)}return!1},t.prototype.dirtyShape=function(){this.__dirty|=Ri,this._rect&&(this._rect=null),this._decalEl&&this._decalEl.dirtyShape(),this.markRedraw()},t.prototype.dirty=function(){this.dirtyStyle(),this.dirtyShape()},t.prototype.animateShape=function(e){return this.animate("shape",e)},t.prototype.updateDuringAnimation=function(e){e==="style"?this.dirtyStyle():e==="shape"?this.dirtyShape():this.markRedraw()},t.prototype.attrKV=function(e,i){e==="shape"?this.setShape(i):r.prototype.attrKV.call(this,e,i)},t.prototype.setShape=function(e,i){var n=this.shape;return n||(n=this.shape={}),typeof e=="string"?n[e]=i:O(n,e),this.dirtyShape(),this},t.prototype.shapeChanged=function(){return!!(this.__dirty&Ri)},t.prototype.createStyle=function(e){return la(Qp,e)},t.prototype._innerSaveToNormal=function(e){r.prototype._innerSaveToNormal.call(this,e);var i=this._normalState;e.shape&&!i.shape&&(i.shape=O({},this.shape))},t.prototype._applyStateObj=function(e,i,n,a,o,s){r.prototype._applyStateObj.call(this,e,i,n,a,o,s);var l=!(i&&a),u;if(i&&i.shape?o?a?u=i.shape:(u=O({},n.shape),O(u,i.shape)):(u=O({},a?this.shape:n.shape),O(u,i.shape)):l&&(u=n.shape),u)if(o){this.shape=O({},this.shape);for(var f={},h=ft(u),c=0;c<h.length;c++){var v=h[c];typeof u[v]=="object"?this.shape[v]=u[v]:f[v]=u[v]}this._transitionState(e,{shape:f},s)}else this.shape=u,this.dirtyShape()},t.prototype._mergeStates=function(e){for(var i=r.prototype._mergeStates.call(this,e),n,a=0;a<e.length;a++){var o=e[a];o.shape&&(n=n||{},this._mergeStyle(n,o.shape))}return n&&(i.shape=n),i},t.prototype.getAnimationStyleProps=function(){return $1},t.prototype.isZeroArea=function(){return!1},t.extend=function(e){var i=function(a){k(o,a);function o(s){var l=a.call(this,s)||this;return e.init&&e.init.call(l,s),l}return o.prototype.getDefaultStyle=function(){return U(e.style)},o.prototype.getDefaultShape=function(){return U(e.shape)},o}(t);for(var n in e)typeof e[n]=="function"&&(i.prototype[n]=e[n]);return i},t.initDefaultProps=function(){var e=t.prototype;e.type="path",e.strokeContainThreshold=5,e.segmentIgnoreThreshold=0,e.subPixelOptimize=!1,e.autoBatch=!1,e.__dirty=Kt|wn|Ri}(),t}(va),Z1=nt({strokeFirst:!0,font:ai,x:0,y:0,textAlign:"left",textBaseline:"top",miterLimit:2},Qp),Lo=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.hasStroke=function(){var e=this.style,i=e.stroke;return i!=null&&i!=="none"&&e.lineWidth>0},t.prototype.hasFill=function(){var e=this.style,i=e.fill;return i!=null&&i!=="none"},t.prototype.createStyle=function(e){return la(Z1,e)},t.prototype.setBoundingRect=function(e){this._rect=e},t.prototype.getBoundingRect=function(){var e=this.style;if(!this._rect){var i=e.text;i!=null?i+="":i="";var n=df(i,e.font,e.textAlign,e.textBaseline);if(n.x+=e.x||0,n.y+=e.y||0,this.hasStroke()){var a=e.lineWidth;n.x-=a/2,n.y-=a/2,n.width+=a,n.height+=a}this._rect=n}return this._rect},t.initDefaultProps=function(){var e=t.prototype;e.dirtyRectTolerance=10}(),t}(va);Lo.prototype.type="tspan";var q1=nt({x:0,y:0},ei),K1={style:nt({x:!0,y:!0,width:!0,height:!0,sx:!0,sy:!0,sWidth:!0,sHeight:!0},is.style)};function Q1(r){return!!(r&&typeof r!="string"&&r.width&&r.height)}var jp=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.createStyle=function(e){return la(q1,e)},t.prototype._getSize=function(e){var i=this.style,n=i[e];if(n!=null)return n;var a=Q1(i.image)?i.image:this.__image;if(!a)return 0;var o=e==="width"?"height":"width",s=i[o];return s==null?a[e]:a[e]/a[o]*s},t.prototype.getWidth=function(){return this._getSize("width")},t.prototype.getHeight=function(){return this._getSize("height")},t.prototype.getAnimationStyleProps=function(){return K1},t.prototype.getBoundingRect=function(){var e=this.style;return this._rect||(this._rect=new et(e.x||0,e.y||0,this.getWidth(),this.getHeight())),this._rect},t}(va);jp.prototype.type="image";const tr=jp;function j1(r,t){var e=t.x,i=t.y,n=t.width,a=t.height,o=t.r,s,l,u,f;n<0&&(e=e+n,n=-n),a<0&&(i=i+a,a=-a),typeof o=="number"?s=l=u=f=o:o instanceof Array?o.length===1?s=l=u=f=o[0]:o.length===2?(s=u=o[0],l=f=o[1]):o.length===3?(s=o[0],l=f=o[1],u=o[2]):(s=o[0],l=o[1],u=o[2],f=o[3]):s=l=u=f=0;var h;s+l>n&&(h=s+l,s*=n/h,l*=n/h),u+f>n&&(h=u+f,u*=n/h,f*=n/h),l+u>a&&(h=l+u,l*=a/h,u*=a/h),s+f>a&&(h=s+f,s*=a/h,f*=a/h),r.moveTo(e+s,i),r.lineTo(e+n-l,i),l!==0&&r.arc(e+n-l,i+l,l,-Math.PI/2,0),r.lineTo(e+n,i+a-u),u!==0&&r.arc(e+n-u,i+a-u,u,0,Math.PI/2),r.lineTo(e+f,i+a),f!==0&&r.arc(e+f,i+a-f,f,Math.PI/2,Math.PI),r.lineTo(e,i+s),s!==0&&r.arc(e+s,i+s,s,Math.PI,Math.PI*1.5)}var Ei=Math.round;function Jp(r,t,e){if(t){var i=t.x1,n=t.x2,a=t.y1,o=t.y2;r.x1=i,r.x2=n,r.y1=a,r.y2=o;var s=e&&e.lineWidth;return s&&(Ei(i*2)===Ei(n*2)&&(r.x1=r.x2=qr(i,s,!0)),Ei(a*2)===Ei(o*2)&&(r.y1=r.y2=qr(a,s,!0))),r}}function tg(r,t,e){if(t){var i=t.x,n=t.y,a=t.width,o=t.height;r.x=i,r.y=n,r.width=a,r.height=o;var s=e&&e.lineWidth;return s&&(r.x=qr(i,s,!0),r.y=qr(n,s,!0),r.width=Math.max(qr(i+a,s,!1)-r.x,a===0?0:1),r.height=Math.max(qr(n+o,s,!1)-r.y,o===0?0:1)),r}}function qr(r,t,e){if(!t)return r;var i=Ei(r*2);return(i+Ei(t))%2===0?i/2:(i+(e?1:-1))/2}var J1=function(){function r(){this.x=0,this.y=0,this.width=0,this.height=0}return r}(),tS={},eg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new J1},t.prototype.buildPath=function(e,i){var n,a,o,s;if(this.subPixelOptimize){var l=tg(tS,i,this.style);n=l.x,a=l.y,o=l.width,s=l.height,l.r=i.r,i=l}else n=i.x,a=i.y,o=i.width,s=i.height;i.r?j1(e,i):e.rect(n,a,o,s)},t.prototype.isZeroArea=function(){return!this.shape.width||!this.shape.height},t}(st);eg.prototype.type="rect";const gt=eg;var Wh={fill:"#000"},Uh=2,eS={style:nt({fill:!0,stroke:!0,fillOpacity:!0,strokeOpacity:!0,lineWidth:!0,fontSize:!0,lineHeight:!0,width:!0,height:!0,textShadowColor:!0,textShadowBlur:!0,textShadowOffsetX:!0,textShadowOffsetY:!0,backgroundColor:!0,padding:!0,borderColor:!0,borderWidth:!0,borderRadius:!0},is.style)},rg=function(r){k(t,r);function t(e){var i=r.call(this)||this;return i.type="text",i._children=[],i._defaultStyle=Wh,i.attr(e),i}return t.prototype.childrenRef=function(){return this._children},t.prototype.update=function(){r.prototype.update.call(this),this.styleChanged()&&this._updateSubTexts();for(var e=0;e<this._children.length;e++){var i=this._children[e];i.zlevel=this.zlevel,i.z=this.z,i.z2=this.z2,i.culling=this.culling,i.cursor=this.cursor,i.invisible=this.invisible}},t.prototype.updateTransform=function(){var e=this.innerTransformable;e?(e.updateTransform(),e.transform&&(this.transform=e.transform)):r.prototype.updateTransform.call(this)},t.prototype.getLocalTransform=function(e){var i=this.innerTransformable;return i?i.getLocalTransform(e):r.prototype.getLocalTransform.call(this,e)},t.prototype.getComputedTransform=function(){return this.__hostTarget&&(this.__hostTarget.getComputedTransform(),this.__hostTarget.updateInnerText(!0)),r.prototype.getComputedTransform.call(this)},t.prototype._updateSubTexts=function(){this._childCursor=0,oS(this.style),this.style.rich?this._updateRichTexts():this._updatePlainTexts(),this._children.length=this._childCursor,this.styleUpdated()},t.prototype.addSelfToZr=function(e){r.prototype.addSelfToZr.call(this,e);for(var i=0;i<this._children.length;i++)this._children[i].__zr=e},t.prototype.removeSelfFromZr=function(e){r.prototype.removeSelfFromZr.call(this,e);for(var i=0;i<this._children.length;i++)this._children[i].__zr=null},t.prototype.getBoundingRect=function(){if(this.styleChanged()&&this._updateSubTexts(),!this._rect){for(var e=new et(0,0,0,0),i=this._children,n=[],a=null,o=0;o<i.length;o++){var s=i[o],l=s.getBoundingRect(),u=s.getLocalTransform(n);u?(e.copy(l),e.applyTransform(u),a=a||e.clone(),a.union(e)):(a=a||l.clone(),a.union(l))}this._rect=a||e}return this._rect},t.prototype.setDefaultTextStyle=function(e){this._defaultStyle=e||Wh},t.prototype.setTextContent=function(e){},t.prototype._mergeStyle=function(e,i){if(!i)return e;var n=i.rich,a=e.rich||n&&{};return O(e,i),n&&a?(this._mergeRich(a,n),e.rich=a):a&&(e.rich=a),e},t.prototype._mergeRich=function(e,i){for(var n=ft(i),a=0;a<n.length;a++){var o=n[a];e[o]=e[o]||{},O(e[o],i[o])}},t.prototype.getAnimationStyleProps=function(){return eS},t.prototype._getOrCreateChild=function(e){var i=this._children[this._childCursor];return(!i||!(i instanceof e))&&(i=new e),this._children[this._childCursor++]=i,i.__zr=this.__zr,i.parent=this,i},t.prototype._updatePlainTexts=function(){var e=this.style,i=e.font||ai,n=e.padding,a=Qh(e),o=N_(a,e),s=Ks(e),l=!!e.backgroundColor,u=o.outerHeight,f=o.outerWidth,h=o.contentWidth,c=o.lines,v=o.lineHeight,d=this._defaultStyle,y=e.x||0,p=e.y||0,g=e.align||d.align||"left",m=e.verticalAlign||d.verticalAlign||"top",_=y,S=Pi(p,o.contentHeight,m);if(s||n){var b=_n(y,f,g),w=Pi(p,u,m);s&&this._renderBackground(e,e,b,w,f,u)}S+=v/2,n&&(_=Kh(y,g,n),m==="top"?S+=n[0]:m==="bottom"&&(S-=n[2]));for(var x=0,C=!1,D=qh("fill"in e?e.fill:(C=!0,d.fill)),A=Zh("stroke"in e?e.stroke:!l&&(!d.autoStroke||C)?(x=Uh,d.stroke):null),M=e.textShadowBlur>0,I=e.width!=null&&(e.overflow==="truncate"||e.overflow==="break"||e.overflow==="breakAll"),L=o.calculatedLineHeight,P=0;P<c.length;P++){var R=this._getOrCreateChild(Lo),E=R.createStyle();R.useStyle(E),E.text=c[P],E.x=_,E.y=S,g&&(E.textAlign=g),E.textBaseline="middle",E.opacity=e.opacity,E.strokeFirst=!0,M&&(E.shadowBlur=e.textShadowBlur||0,E.shadowColor=e.textShadowColor||"transparent",E.shadowOffsetX=e.textShadowOffsetX||0,E.shadowOffsetY=e.textShadowOffsetY||0),E.stroke=A,E.fill=D,A&&(E.lineWidth=e.lineWidth||x,E.lineDash=e.lineDash,E.lineDashOffset=e.lineDashOffset||0),E.font=i,Xh(E,e),S+=v,I&&R.setBoundingRect(new et(_n(E.x,e.width,E.textAlign),Pi(E.y,L,E.textBaseline),h,L))}},t.prototype._updateRichTexts=function(){var e=this.style,i=Qh(e),n=V_(i,e),a=n.width,o=n.outerWidth,s=n.outerHeight,l=e.padding,u=e.x||0,f=e.y||0,h=this._defaultStyle,c=e.align||h.align,v=e.verticalAlign||h.verticalAlign,d=_n(u,o,c),y=Pi(f,s,v),p=d,g=y;l&&(p+=l[3],g+=l[0]);var m=p+a;Ks(e)&&this._renderBackground(e,e,d,y,o,s);for(var _=!!e.backgroundColor,S=0;S<n.lines.length;S++){for(var b=n.lines[S],w=b.tokens,x=w.length,C=b.lineHeight,D=b.width,A=0,M=p,I=m,L=x-1,P=void 0;A<x&&(P=w[A],!P.align||P.align==="left");)this._placeToken(P,e,C,g,M,"left",_),D-=P.width,M+=P.width,A++;for(;L>=0&&(P=w[L],P.align==="right");)this._placeToken(P,e,C,g,I,"right",_),D-=P.width,I-=P.width,L--;for(M+=(a-(M-p)-(m-I)-D)/2;A<=L;)P=w[A],this._placeToken(P,e,C,g,M+P.width/2,"center",_),M+=P.width,A++;g+=C}},t.prototype._placeToken=function(e,i,n,a,o,s,l){var u=i.rich[e.styleName]||{};u.text=e.text;var f=e.verticalAlign,h=a+n/2;f==="top"?h=a+e.height/2:f==="bottom"&&(h=a+n-e.height/2);var c=!e.isLineHolder&&Ks(u);c&&this._renderBackground(u,i,s==="right"?o-e.width:s==="center"?o-e.width/2:o,h-e.height/2,e.width,e.height);var v=!!u.backgroundColor,d=e.textPadding;d&&(o=Kh(o,s,d),h-=e.height/2-d[0]-e.innerHeight/2);var y=this._getOrCreateChild(Lo),p=y.createStyle();y.useStyle(p);var g=this._defaultStyle,m=!1,_=0,S=qh("fill"in u?u.fill:"fill"in i?i.fill:(m=!0,g.fill)),b=Zh("stroke"in u?u.stroke:"stroke"in i?i.stroke:!v&&!l&&(!g.autoStroke||m)?(_=Uh,g.stroke):null),w=u.textShadowBlur>0||i.textShadowBlur>0;p.text=e.text,p.x=o,p.y=h,w&&(p.shadowBlur=u.textShadowBlur||i.textShadowBlur||0,p.shadowColor=u.textShadowColor||i.textShadowColor||"transparent",p.shadowOffsetX=u.textShadowOffsetX||i.textShadowOffsetX||0,p.shadowOffsetY=u.textShadowOffsetY||i.textShadowOffsetY||0),p.textAlign=s,p.textBaseline="middle",p.font=e.font||ai,p.opacity=Cn(u.opacity,i.opacity,1),Xh(p,u),b&&(p.lineWidth=Cn(u.lineWidth,i.lineWidth,_),p.lineDash=Q(u.lineDash,i.lineDash),p.lineDashOffset=i.lineDashOffset||0,p.stroke=b),S&&(p.fill=S);var x=e.contentWidth,C=e.contentHeight;y.setBoundingRect(new et(_n(p.x,x,p.textAlign),Pi(p.y,C,p.textBaseline),x,C))},t.prototype._renderBackground=function(e,i,n,a,o,s){var l=e.backgroundColor,u=e.borderWidth,f=e.borderColor,h=l&&l.image,c=l&&!h,v=e.borderRadius,d=this,y,p;if(c||e.lineHeight||u&&f){y=this._getOrCreateChild(gt),y.useStyle(y.createStyle()),y.style.fill=null;var g=y.shape;g.x=n,g.y=a,g.width=o,g.height=s,g.r=v,y.dirtyShape()}if(c){var m=y.style;m.fill=l||null,m.fillOpacity=Q(e.fillOpacity,1)}else if(h){p=this._getOrCreateChild(tr),p.onload=function(){d.dirtyStyle()};var _=p.style;_.image=l.image,_.x=n,_.y=a,_.width=o,_.height=s}if(u&&f){var m=y.style;m.lineWidth=u,m.stroke=f,m.strokeOpacity=Q(e.strokeOpacity,1),m.lineDash=e.borderDash,m.lineDashOffset=e.borderDashOffset||0,y.strokeContainThreshold=0,y.hasFill()&&y.hasStroke()&&(m.strokeFirst=!0,m.lineWidth*=2)}var S=(y||p).style;S.shadowBlur=e.shadowBlur||0,S.shadowColor=e.shadowColor||"transparent",S.shadowOffsetX=e.shadowOffsetX||0,S.shadowOffsetY=e.shadowOffsetY||0,S.opacity=Cn(e.opacity,i.opacity,1)},t.makeFont=function(e){var i="";return aS(e)&&(i=[e.fontStyle,e.fontWeight,nS(e.fontSize),e.fontFamily||"sans-serif"].join(" ")),i&&be(i)||e.textFont||e.font},t}(va),rS={left:!0,right:1,center:1},iS={top:1,bottom:1,middle:1},Yh=["fontStyle","fontWeight","fontSize","fontFamily"];function nS(r){return typeof r=="string"&&(r.indexOf("px")!==-1||r.indexOf("rem")!==-1||r.indexOf("em")!==-1)?r:isNaN(+r)?af+"px":r+"px"}function Xh(r,t){for(var e=0;e<Yh.length;e++){var i=Yh[e],n=t[i];n!=null&&(r[i]=n)}}function aS(r){return r.fontSize!=null||r.fontFamily||r.fontWeight}function oS(r){return $h(r),T(r.rich,$h),r}function $h(r){if(r){r.font=rg.makeFont(r);var t=r.align;t==="middle"&&(t="center"),r.align=t==null||rS[t]?t:"left";var e=r.verticalAlign;e==="center"&&(e="middle"),r.verticalAlign=e==null||iS[e]?e:"top";var i=r.padding;i&&(r.padding=ff(r.padding))}}function Zh(r,t){return r==null||t<=0||r==="transparent"||r==="none"?null:r.image||r.colorStops?"#000":r}function qh(r){return r==null||r==="none"?null:r.image||r.colorStops?"#000":r}function Kh(r,t,e){return t==="right"?r-e[1]:t==="center"?r+e[3]/2-e[1]/2:r+e[3]}function Qh(r){var t=r.text;return t!=null&&(t+=""),t}function Ks(r){return!!(r.backgroundColor||r.lineHeight||r.borderWidth&&r.borderColor)}const yt=rg;var jh=1e-4,ig=20;function sS(r){return r.replace(/^\s+|\s+$/g,"")}function ze(r,t,e,i){var n=t[0],a=t[1],o=e[0],s=e[1],l=a-n,u=s-o;if(l===0)return u===0?o:(o+s)/2;if(i)if(l>0){if(r<=n)return o;if(r>=a)return s}else{if(r>=n)return o;if(r<=a)return s}else{if(r===n)return o;if(r===a)return s}return(r-n)/l*u+o}function Ot(r,t){switch(r){case"center":case"middle":r="50%";break;case"left":case"top":r="0%";break;case"right":case"bottom":r="100%";break}return V(r)?sS(r).match(/%$/)?parseFloat(r)/100*t:parseFloat(r):r==null?NaN:+r}function pt(r,t,e){return t==null&&(t=10),t=Math.min(Math.max(0,t),ig),r=(+r).toFixed(t),e?r:+r}function wf(r){return r.sort(function(t,e){return t-e}),r}function Oe(r){if(r=+r,isNaN(r))return 0;if(r>1e-14){for(var t=1,e=0;e<15;e++,t*=10)if(Math.round(r*t)/t===r)return e}return ng(r)}function ng(r){var t=r.toString().toLowerCase(),e=t.indexOf("e"),i=e>0?+t.slice(e+1):0,n=e>0?e:t.length,a=t.indexOf("."),o=a<0?0:n-1-a;return Math.max(0,o-i)}function ag(r,t){var e=Math.log,i=Math.LN10,n=Math.floor(e(r[1]-r[0])/i),a=Math.round(e(Math.abs(t[1]-t[0]))/i),o=Math.min(Math.max(-n+a,0),20);return isFinite(o)?o:20}function lS(r,t,e){if(!r[t])return 0;var i=uS(r,e);return i[t]||0}function uS(r,t){var e=Ne(r,function(v,d){return v+(isNaN(d)?0:d)},0);if(e===0)return[];for(var i=Math.pow(10,t),n=H(r,function(v){return(isNaN(v)?0:v)/e*i*100}),a=i*100,o=H(n,function(v){return Math.floor(v)}),s=Ne(o,function(v,d){return v+d},0),l=H(n,function(v,d){return v-o[d]});s<a;){for(var u=Number.NEGATIVE_INFINITY,f=null,h=0,c=l.length;h<c;++h)l[h]>u&&(u=l[h],f=h);++o[f],l[f]=0,++s}return H(o,function(v){return v/i})}function fS(r,t){var e=Math.max(Oe(r),Oe(t)),i=r+t;return e>ig?i:pt(i,e)}var hS=9007199254740991;function bf(r){var t=Math.PI*2;return(r%t+t)%t}function Xn(r){return r>-jh&&r<jh}var vS=/^(?:(\d{4})(?:[-\/](\d{1,2})(?:[-\/](\d{1,2})(?:[T ](\d{1,2})(?::(\d{1,2})(?::(\d{1,2})(?:[.,](\d+))?)?)?(Z|[\+\-]\d\d:?\d\d)?)?)?)?)?$/;function ve(r){if(r instanceof Date)return r;if(V(r)){var t=vS.exec(r);if(!t)return new Date(NaN);if(t[8]){var e=+t[4]||0;return t[8].toUpperCase()!=="Z"&&(e-=+t[8].slice(0,3)),new Date(Date.UTC(+t[1],+(t[2]||1)-1,+t[3]||1,e,+(t[5]||0),+t[6]||0,t[7]?+t[7].substring(0,3):0))}else return new Date(+t[1],+(t[2]||1)-1,+t[3]||1,+t[4]||0,+(t[5]||0),+t[6]||0,t[7]?+t[7].substring(0,3):0)}else if(r==null)return new Date(NaN);return new Date(Math.round(r))}function og(r){return Math.pow(10,ns(r))}function ns(r){if(r===0)return 0;var t=Math.floor(Math.log(r)/Math.LN10);return r/Math.pow(10,t)>=10&&t++,t}function xf(r,t){var e=ns(r),i=Math.pow(10,e),n=r/i,a;return t?n<1.5?a=1:n<2.5?a=2:n<4?a=3:n<7?a=5:a=10:n<1?a=1:n<2?a=2:n<3?a=3:n<5?a=5:a=10,r=a*i,e>=-20?+r.toFixed(e<0?-e:0):r}function cS(r,t){var e=(r.length-1)*t+1,i=Math.floor(e),n=+r[i-1],a=e-i;return a?n+a*(r[i]-n):n}function du(r){r.sort(function(l,u){return s(l,u,0)?-1:1});for(var t=-1/0,e=1,i=0;i<r.length;){for(var n=r[i].interval,a=r[i].close,o=0;o<2;o++)n[o]<=t&&(n[o]=t,a[o]=o?1:1-e),t=n[o],e=a[o];n[0]===n[1]&&a[0]*a[1]!==1?r.splice(i,1):i++}return r;function s(l,u,f){return l.interval[f]<u.interval[f]||l.interval[f]===u.interval[f]&&(l.close[f]-u.close[f]===(f?-1:1)||!f&&s(l,u,1))}}function $n(r){var t=parseFloat(r);return t==r&&(t!==0||!V(r)||r.indexOf("x")<=0)?t:NaN}function sg(r){return!isNaN($n(r))}function lg(){return Math.round(Math.random()*9)}function ug(r,t){return t===0?r:ug(t,r%t)}function Jh(r,t){return r==null?t:t==null?r:r*t/ug(r,t)}function Gt(r){throw new Error(r)}function tv(r,t,e){return(t-r)*e+r}var fg="series\0",dS="\0_ec_\0";function Tt(r){return r instanceof Array?r:r==null?[]:[r]}function ev(r,t,e){if(r){r[t]=r[t]||{},r.emphasis=r.emphasis||{},r.emphasis[t]=r.emphasis[t]||{};for(var i=0,n=e.length;i<n;i++){var a=e[i];!r.emphasis[t].hasOwnProperty(a)&&r[t].hasOwnProperty(a)&&(r.emphasis[t][a]=r[t][a])}}}var rv=["fontStyle","fontWeight","fontSize","fontFamily","rich","tag","color","textBorderColor","textBorderWidth","width","height","lineHeight","align","verticalAlign","baseline","shadowColor","shadowBlur","shadowOffsetX","shadowOffsetY","textShadowColor","textShadowBlur","textShadowOffsetX","textShadowOffsetY","backgroundColor","borderColor","borderWidth","borderRadius","padding"];function ca(r){return z(r)&&!N(r)&&!(r instanceof Date)?r.value:r}function pS(r){return z(r)&&!(r instanceof Array)}function gS(r,t,e){var i=e==="normalMerge",n=e==="replaceMerge",a=e==="replaceAll";r=r||[],t=(t||[]).slice();var o=$();T(t,function(l,u){if(!z(l)){t[u]=null;return}});var s=yS(r,o,e);return(i||n)&&mS(s,r,o,t),i&&_S(s,t),i||n?SS(s,t,n):a&&wS(s,t),bS(s),s}function yS(r,t,e){var i=[];if(e==="replaceAll")return i;for(var n=0;n<r.length;n++){var a=r[n];a&&a.id!=null&&t.set(a.id,n),i.push({existing:e==="replaceMerge"||Zn(a)?null:a,newOption:null,keyInfo:null,brandNew:null})}return i}function mS(r,t,e,i){T(i,function(n,a){if(!(!n||n.id==null)){var o=In(n.id),s=e.get(o);if(s!=null){var l=r[s];Ce(!l.newOption,'Duplicated option on id "'+o+'".'),l.newOption=n,l.existing=t[s],i[a]=null}}})}function _S(r,t){T(t,function(e,i){if(!(!e||e.name==null))for(var n=0;n<r.length;n++){var a=r[n].existing;if(!r[n].newOption&&a&&(a.id==null||e.id==null)&&!Zn(e)&&!Zn(a)&&hg("name",a,e)){r[n].newOption=e,t[i]=null;return}}})}function SS(r,t,e){T(t,function(i){if(i){for(var n,a=0;(n=r[a])&&(n.newOption||Zn(n.existing)||n.existing&&i.id!=null&&!hg("id",i,n.existing));)a++;n?(n.newOption=i,n.brandNew=e):r.push({newOption:i,brandNew:e,existing:null,keyInfo:null}),a++}})}function wS(r,t){T(t,function(e){r.push({newOption:e,brandNew:!0,existing:null,keyInfo:null})})}function bS(r){var t=$();T(r,function(e){var i=e.existing;i&&t.set(i.id,e)}),T(r,function(e){var i=e.newOption;Ce(!i||i.id==null||!t.get(i.id)||t.get(i.id)===e,"id duplicates: "+(i&&i.id)),i&&i.id!=null&&t.set(i.id,e),!e.keyInfo&&(e.keyInfo={})}),T(r,function(e,i){var n=e.existing,a=e.newOption,o=e.keyInfo;if(z(a)){if(o.name=a.name!=null?In(a.name):n?n.name:fg+i,n)o.id=In(n.id);else if(a.id!=null)o.id=In(a.id);else{var s=0;do o.id="\0"+o.name+"\0"+s++;while(t.get(o.id))}t.set(o.id,e)}})}function hg(r,t,e){var i=Me(t[r],null),n=Me(e[r],null);return i!=null&&n!=null&&i===n}function In(r){return Me(r,"")}function Me(r,t){return r==null?t:V(r)?r:ht(r)||wo(r)?r+"":t}function vg(r){var t=r.name;return!!(t&&t.indexOf(fg))}function Zn(r){return r&&r.id!=null&&In(r.id).indexOf(dS)===0}function xS(r,t,e){T(r,function(i){var n=i.newOption;z(n)&&(i.keyInfo.mainType=t,i.keyInfo.subType=TS(t,n,i.existing,e))})}function TS(r,t,e,i){var n=t.type?t.type:e?e.subType:i.determineSubType(r,t);return n}function MS(r,t){var e={},i={};return n(r||[],e),n(t||[],i,e),[a(e),a(i)];function n(o,s,l){for(var u=0,f=o.length;u<f;u++){var h=Me(o[u].seriesId,null);if(h==null)return;for(var c=Tt(o[u].dataIndex),v=l&&l[h],d=0,y=c.length;d<y;d++){var p=c[d];v&&v[p]?v[p]=null:(s[h]||(s[h]={}))[p]=1}}}function a(o,s){var l=[];for(var u in o)if(o.hasOwnProperty(u)&&o[u]!=null)if(s)l.push(+u);else{var f=a(o[u],!0);f.length&&l.push({seriesId:u,dataIndex:f})}return l}}function ui(r,t){if(t.dataIndexInside!=null)return t.dataIndexInside;if(t.dataIndex!=null)return N(t.dataIndex)?H(t.dataIndex,function(e){return r.indexOfRawIndex(e)}):r.indexOfRawIndex(t.dataIndex);if(t.name!=null)return N(t.name)?H(t.name,function(e){return r.indexOfName(e)}):r.indexOfName(t.name)}function _t(){var r="__ec_inner_"+CS++;return function(t){return t[r]||(t[r]={})}}var CS=lg();function Qs(r,t,e){var i=Tf(t,e),n=i.mainTypeSpecified,a=i.queryOptionMap,o=i.others,s=o,l=e?e.defaultMainType:null;return!n&&l&&a.set(l,{}),a.each(function(u,f){var h=da(r,f,u,{useDefault:l===f,enableAll:e&&e.enableAll!=null?e.enableAll:!0,enableNone:e&&e.enableNone!=null?e.enableNone:!0});s[f+"Models"]=h.models,s[f+"Model"]=h.models[0]}),s}function Tf(r,t){var e;if(V(r)){var i={};i[r+"Index"]=0,e=i}else e=r;var n=$(),a={},o=!1;return T(e,function(s,l){if(l==="dataIndex"||l==="dataIndexInside"){a[l]=s;return}var u=l.match(/^(\w+)(Index|Id|Name)$/)||[],f=u[1],h=(u[2]||"").toLowerCase();if(!(!f||!h||t&&t.includeMainTypes&&ot(t.includeMainTypes,f)<0)){o=o||!!f;var c=n.get(f)||n.set(f,{});c[h]=s}}),{mainTypeSpecified:o,queryOptionMap:n,others:a}}var xe={useDefault:!0,enableAll:!1,enableNone:!1};function da(r,t,e,i){i=i||xe;var n=e.index,a=e.id,o=e.name,s={models:null,specified:n!=null||a!=null||o!=null};if(!s.specified){var l=void 0;return s.models=i.useDefault&&(l=r.getComponent(t))?[l]:[],s}return n==="none"||n===!1?(Ce(i.enableNone,'`"none"` or `false` is not a valid value on index option.'),s.models=[],s):(n==="all"&&(Ce(i.enableAll,'`"all"` is not a valid value on index option.'),n=a=o=null),s.models=r.queryComponents({mainType:t,index:n,id:a,name:o}),s)}function cg(r,t,e){r.setAttribute?r.setAttribute(t,e):r[t]=e}function DS(r,t){return r.getAttribute?r.getAttribute(t):r[t]}function AS(r){return r==="auto"?Z.domSupported?"html":"richText":r||"html"}function IS(r,t,e,i,n){var a=t==null||t==="auto";if(i==null)return i;if(ht(i)){var o=tv(e||0,i,n);return pt(o,a?Math.max(Oe(e||0),Oe(i)):t)}else{if(V(i))return n<1?e:i;for(var s=[],l=e,u=i,f=Math.max(l?l.length:0,u.length),h=0;h<f;++h){var c=r.getDimensionInfo(h);if(c&&c.type==="ordinal")s[h]=(n<1&&l?l:u)[h];else{var v=l&&l[h]?l[h]:0,d=u[h],o=tv(v,d,n);s[h]=pt(o,a?Math.max(Oe(v),Oe(d)):t)}}return s}}var tt=_t(),LS=function(r,t,e,i){if(i){var n=tt(i);n.dataIndex=e,n.dataType=t,n.seriesIndex=r,n.ssrType="chart",i.type==="group"&&i.traverse(function(a){var o=tt(a);o.seriesIndex=r,o.dataIndex=e,o.dataType=t,o.ssrType="chart"})}},iv=1,nv={},dg=_t(),Mf=_t(),Cf=0,as=1,os=2,Ve=["emphasis","blur","select"],av=["normal","emphasis","blur","select"],PS=10,RS=9,ri="highlight",uo="downplay",Ln="select",fo="unselect",Pn="toggleSelect";function Si(r){return r!=null&&r!=="none"}function ss(r,t,e){r.onHoverStateChange&&(r.hoverState||0)!==e&&r.onHoverStateChange(t),r.hoverState=e}function pg(r){ss(r,"emphasis",os)}function gg(r){r.hoverState===os&&ss(r,"normal",Cf)}function Df(r){ss(r,"blur",as)}function yg(r){r.hoverState===as&&ss(r,"normal",Cf)}function ES(r){r.selected=!0}function OS(r){r.selected=!1}function ov(r,t,e){t(r,e)}function er(r,t,e){ov(r,t,e),r.isGroup&&r.traverse(function(i){ov(i,t,e)})}function sv(r,t){switch(t){case"emphasis":r.hoverState=os;break;case"normal":r.hoverState=Cf;break;case"blur":r.hoverState=as;break;case"select":r.selected=!0}}function kS(r,t,e,i){for(var n=r.style,a={},o=0;o<t.length;o++){var s=t[o],l=n[s];a[s]=l??(i&&i[s])}for(var o=0;o<r.animators.length;o++){var u=r.animators[o];u.__fromStateTransition&&u.__fromStateTransition.indexOf(e)<0&&u.targetName==="style"&&u.saveTo(a,t)}return a}function BS(r,t,e,i){var n=e&&ot(e,"select")>=0,a=!1;if(r instanceof st){var o=dg(r),s=n&&o.selectFill||o.normalFill,l=n&&o.selectStroke||o.normalStroke;if(Si(s)||Si(l)){i=i||{};var u=i.style||{};u.fill==="inherit"?(a=!0,i=O({},i),u=O({},u),u.fill=s):!Si(u.fill)&&Si(s)?(a=!0,i=O({},i),u=O({},u),u.fill=au(s)):!Si(u.stroke)&&Si(l)&&(a||(i=O({},i),u=O({},u)),u.stroke=au(l)),i.style=u}}if(i&&i.z2==null){a||(i=O({},i));var f=r.z2EmphasisLift;i.z2=r.z2+(f??PS)}return i}function NS(r,t,e){if(e&&e.z2==null){e=O({},e);var i=r.z2SelectLift;e.z2=r.z2+(i??RS)}return e}function FS(r,t,e){var i=ot(r.currentStates,t)>=0,n=r.style.opacity,a=i?null:kS(r,["opacity"],t,{opacity:1});e=e||{};var o=e.style||{};return o.opacity==null&&(e=O({},e),o=O({opacity:i?n:a.opacity*.1},o),e.style=o),e}function js(r,t){var e=this.states[r];if(this.style){if(r==="emphasis")return BS(this,r,t,e);if(r==="blur")return FS(this,r,e);if(r==="select")return NS(this,r,e)}return e}function zS(r){r.stateProxy=js;var t=r.getTextContent(),e=r.getTextGuideLine();t&&(t.stateProxy=js),e&&(e.stateProxy=js)}function lv(r,t){!wg(r,t)&&!r.__highByOuter&&er(r,pg)}function uv(r,t){!wg(r,t)&&!r.__highByOuter&&er(r,gg)}function Po(r,t){r.__highByOuter|=1<<(t||0),er(r,pg)}function Ro(r,t){!(r.__highByOuter&=~(1<<(t||0)))&&er(r,gg)}function VS(r){er(r,Df)}function mg(r){er(r,yg)}function _g(r){er(r,ES)}function Sg(r){er(r,OS)}function wg(r,t){return r.__highDownSilentOnTouch&&t.zrByTouch}function bg(r){var t=r.getModel(),e=[],i=[];t.eachComponent(function(n,a){var o=Mf(a),s=n==="series",l=s?r.getViewOfSeriesModel(a):r.getViewOfComponentModel(a);!s&&i.push(l),o.isBlured&&(l.group.traverse(function(u){yg(u)}),s&&e.push(a)),o.isBlured=!1}),T(i,function(n){n&&n.toggleBlurSeries&&n.toggleBlurSeries(e,!1,t)})}function pu(r,t,e,i){var n=i.getModel();e=e||"coordinateSystem";function a(u,f){for(var h=0;h<f.length;h++){var c=u.getItemGraphicEl(f[h]);c&&mg(c)}}if(r!=null&&!(!t||t==="none")){var o=n.getSeriesByIndex(r),s=o.coordinateSystem;s&&s.master&&(s=s.master);var l=[];n.eachSeries(function(u){var f=o===u,h=u.coordinateSystem;h&&h.master&&(h=h.master);var c=h&&s?h===s:f;if(!(e==="series"&&!f||e==="coordinateSystem"&&!c||t==="series"&&f)){var v=i.getViewOfSeriesModel(u);if(v.group.traverse(function(p){p.__highByOuter&&f&&t==="self"||Df(p)}),Nt(t))a(u.getData(),t);else if(z(t))for(var d=ft(t),y=0;y<d.length;y++)a(u.getData(d[y]),t[d[y]]);l.push(u),Mf(u).isBlured=!0}}),n.eachComponent(function(u,f){if(u!=="series"){var h=i.getViewOfComponentModel(f);h&&h.toggleBlurSeries&&h.toggleBlurSeries(l,!0,n)}})}}function gu(r,t,e){if(!(r==null||t==null)){var i=e.getModel().getComponent(r,t);if(i){Mf(i).isBlured=!0;var n=e.getViewOfComponentModel(i);!n||!n.focusBlurEnabled||n.group.traverse(function(a){Df(a)})}}}function HS(r,t,e){var i=r.seriesIndex,n=r.getData(t.dataType);if(n){var a=ui(n,t);a=(N(a)?a[0]:a)||0;var o=n.getItemGraphicEl(a);if(!o)for(var s=n.count(),l=0;!o&&l<s;)o=n.getItemGraphicEl(l++);if(o){var u=tt(o);pu(i,u.focus,u.blurScope,e)}else{var f=r.get(["emphasis","focus"]),h=r.get(["emphasis","blurScope"]);f!=null&&pu(i,f,h,e)}}}function Af(r,t,e,i){var n={focusSelf:!1,dispatchers:null};if(r==null||r==="series"||t==null||e==null)return n;var a=i.getModel().getComponent(r,t);if(!a)return n;var o=i.getViewOfComponentModel(a);if(!o||!o.findHighDownDispatchers)return n;for(var s=o.findHighDownDispatchers(e),l,u=0;u<s.length;u++)if(tt(s[u]).focus==="self"){l=!0;break}return{focusSelf:l,dispatchers:s}}function GS(r,t,e){var i=tt(r),n=Af(i.componentMainType,i.componentIndex,i.componentHighDownName,e),a=n.dispatchers,o=n.focusSelf;a?(o&&gu(i.componentMainType,i.componentIndex,e),T(a,function(s){return lv(s,t)})):(pu(i.seriesIndex,i.focus,i.blurScope,e),i.focus==="self"&&gu(i.componentMainType,i.componentIndex,e),lv(r,t))}function WS(r,t,e){bg(e);var i=tt(r),n=Af(i.componentMainType,i.componentIndex,i.componentHighDownName,e).dispatchers;n?T(n,function(a){return uv(a,t)}):uv(r,t)}function US(r,t,e){if(_u(t)){var i=t.dataType,n=r.getData(i),a=ui(n,t);N(a)||(a=[a]),r[t.type===Pn?"toggleSelect":t.type===Ln?"select":"unselect"](a,i)}}function fv(r){var t=r.getAllData();T(t,function(e){var i=e.data,n=e.type;i.eachItemGraphicEl(function(a,o){r.isSelected(o,n)?_g(a):Sg(a)})})}function YS(r){var t=[];return r.eachSeries(function(e){var i=e.getAllData();T(i,function(n){n.data;var a=n.type,o=e.getSelectedDataIndices();if(o.length>0){var s={dataIndex:o,seriesIndex:e.seriesIndex};a!=null&&(s.dataType=a),t.push(s)}})}),t}function xg(r,t,e){If(r,!0),er(r,zS),$S(r,t,e)}function XS(r){If(r,!1)}function Eo(r,t,e,i){i?XS(r):xg(r,t,e)}function $S(r,t,e){var i=tt(r);t!=null?(i.focus=t,i.blurScope=e):i.focus&&(i.focus=null)}var hv=["emphasis","blur","select"],ZS={itemStyle:"getItemStyle",lineStyle:"getLineStyle",areaStyle:"getAreaStyle"};function yu(r,t,e,i){e=e||"itemStyle";for(var n=0;n<hv.length;n++){var a=hv[n],o=t.getModel([a,e]),s=r.ensureState(a);s.style=o[ZS[e]]()}}function If(r,t){var e=t===!1,i=r;r.highDownSilentOnTouch&&(i.__highDownSilentOnTouch=r.highDownSilentOnTouch),(!e||i.__highDownDispatcher)&&(i.__highByOuter=i.__highByOuter||0,i.__highDownDispatcher=!e)}function mu(r){return!!(r&&r.__highDownDispatcher)}function qS(r){var t=nv[r];return t==null&&iv<=32&&(t=nv[r]=iv++),t}function _u(r){var t=r.type;return t===Ln||t===fo||t===Pn}function vv(r){var t=r.type;return t===ri||t===uo}function KS(r){var t=dg(r);t.normalFill=r.style.fill,t.normalStroke=r.style.stroke;var e=r.states.select||{};t.selectFill=e.style&&e.style.fill||null,t.selectStroke=e.style&&e.style.stroke||null}var wi=li.CMD,QS=[[],[],[]],cv=Math.sqrt,jS=Math.atan2;function JS(r,t){if(t){var e=r.data,i=r.len(),n,a,o,s,l,u,f=wi.M,h=wi.C,c=wi.L,v=wi.R,d=wi.A,y=wi.Q;for(o=0,s=0;o<i;){switch(n=e[o++],s=o,a=0,n){case f:a=1;break;case c:a=1;break;case h:a=3;break;case y:a=2;break;case d:var p=t[4],g=t[5],m=cv(t[0]*t[0]+t[1]*t[1]),_=cv(t[2]*t[2]+t[3]*t[3]),S=jS(-t[1]/_,t[0]/m);e[o]*=m,e[o++]+=p,e[o]*=_,e[o++]+=g,e[o++]*=m,e[o++]*=_,e[o++]+=S,e[o++]+=S,o+=2,s=o;break;case v:u[0]=e[o++],u[1]=e[o++],Bt(u,u,t),e[s++]=u[0],e[s++]=u[1],u[0]+=e[o++],u[1]+=e[o++],Bt(u,u,t),e[s++]=u[0],e[s++]=u[1]}for(l=0;l<a;l++){var b=QS[l];b[0]=e[o++],b[1]=e[o++],Bt(b,b,t),e[s++]=b[0],e[s++]=b[1]}}r.increaseVersion()}}var Js=Math.sqrt,La=Math.sin,Pa=Math.cos,nn=Math.PI;function dv(r){return Math.sqrt(r[0]*r[0]+r[1]*r[1])}function Su(r,t){return(r[0]*t[0]+r[1]*t[1])/(dv(r)*dv(t))}function pv(r,t){return(r[0]*t[1]<r[1]*t[0]?-1:1)*Math.acos(Su(r,t))}function gv(r,t,e,i,n,a,o,s,l,u,f){var h=l*(nn/180),c=Pa(h)*(r-e)/2+La(h)*(t-i)/2,v=-1*La(h)*(r-e)/2+Pa(h)*(t-i)/2,d=c*c/(o*o)+v*v/(s*s);d>1&&(o*=Js(d),s*=Js(d));var y=(n===a?-1:1)*Js((o*o*(s*s)-o*o*(v*v)-s*s*(c*c))/(o*o*(v*v)+s*s*(c*c)))||0,p=y*o*v/s,g=y*-s*c/o,m=(r+e)/2+Pa(h)*p-La(h)*g,_=(t+i)/2+La(h)*p+Pa(h)*g,S=pv([1,0],[(c-p)/o,(v-g)/s]),b=[(c-p)/o,(v-g)/s],w=[(-1*c-p)/o,(-1*v-g)/s],x=pv(b,w);if(Su(b,w)<=-1&&(x=nn),Su(b,w)>=1&&(x=0),x<0){var C=Math.round(x/nn*1e6)/1e6;x=nn*2+C%2*nn}f.addData(u,m,_,o,s,S,x,h,a)}var tw=/([mlvhzcqtsa])([^mlvhzcqtsa]*)/ig,ew=/-?([0-9]*\.)?[0-9]+([eE]-?[0-9]+)?/g;function rw(r){var t=new li;if(!r)return t;var e=0,i=0,n=e,a=i,o,s=li.CMD,l=r.match(tw);if(!l)return t;for(var u=0;u<l.length;u++){for(var f=l[u],h=f.charAt(0),c=void 0,v=f.match(ew)||[],d=v.length,y=0;y<d;y++)v[y]=parseFloat(v[y]);for(var p=0;p<d;){var g=void 0,m=void 0,_=void 0,S=void 0,b=void 0,w=void 0,x=void 0,C=e,D=i,A=void 0,M=void 0;switch(h){case"l":e+=v[p++],i+=v[p++],c=s.L,t.addData(c,e,i);break;case"L":e=v[p++],i=v[p++],c=s.L,t.addData(c,e,i);break;case"m":e+=v[p++],i+=v[p++],c=s.M,t.addData(c,e,i),n=e,a=i,h="l";break;case"M":e=v[p++],i=v[p++],c=s.M,t.addData(c,e,i),n=e,a=i,h="L";break;case"h":e+=v[p++],c=s.L,t.addData(c,e,i);break;case"H":e=v[p++],c=s.L,t.addData(c,e,i);break;case"v":i+=v[p++],c=s.L,t.addData(c,e,i);break;case"V":i=v[p++],c=s.L,t.addData(c,e,i);break;case"C":c=s.C,t.addData(c,v[p++],v[p++],v[p++],v[p++],v[p++],v[p++]),e=v[p-2],i=v[p-1];break;case"c":c=s.C,t.addData(c,v[p++]+e,v[p++]+i,v[p++]+e,v[p++]+i,v[p++]+e,v[p++]+i),e+=v[p-2],i+=v[p-1];break;case"S":g=e,m=i,A=t.len(),M=t.data,o===s.C&&(g+=e-M[A-4],m+=i-M[A-3]),c=s.C,C=v[p++],D=v[p++],e=v[p++],i=v[p++],t.addData(c,g,m,C,D,e,i);break;case"s":g=e,m=i,A=t.len(),M=t.data,o===s.C&&(g+=e-M[A-4],m+=i-M[A-3]),c=s.C,C=e+v[p++],D=i+v[p++],e+=v[p++],i+=v[p++],t.addData(c,g,m,C,D,e,i);break;case"Q":C=v[p++],D=v[p++],e=v[p++],i=v[p++],c=s.Q,t.addData(c,C,D,e,i);break;case"q":C=v[p++]+e,D=v[p++]+i,e+=v[p++],i+=v[p++],c=s.Q,t.addData(c,C,D,e,i);break;case"T":g=e,m=i,A=t.len(),M=t.data,o===s.Q&&(g+=e-M[A-4],m+=i-M[A-3]),e=v[p++],i=v[p++],c=s.Q,t.addData(c,g,m,e,i);break;case"t":g=e,m=i,A=t.len(),M=t.data,o===s.Q&&(g+=e-M[A-4],m+=i-M[A-3]),e+=v[p++],i+=v[p++],c=s.Q,t.addData(c,g,m,e,i);break;case"A":_=v[p++],S=v[p++],b=v[p++],w=v[p++],x=v[p++],C=e,D=i,e=v[p++],i=v[p++],c=s.A,gv(C,D,e,i,w,x,_,S,b,c,t);break;case"a":_=v[p++],S=v[p++],b=v[p++],w=v[p++],x=v[p++],C=e,D=i,e+=v[p++],i+=v[p++],c=s.A,gv(C,D,e,i,w,x,_,S,b,c,t);break}}(h==="z"||h==="Z")&&(c=s.Z,t.addData(c),e=n,i=a),o=c}return t.toStatic(),t}var Tg=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.applyTransform=function(e){},t}(st);function Mg(r){return r.setData!=null}function Cg(r,t){var e=rw(r),i=O({},t);return i.buildPath=function(n){if(Mg(n)){n.setData(e.data);var a=n.getContext();a&&n.rebuildPath(a,1)}else{var a=n;e.rebuildPath(a,1)}},i.applyTransform=function(n){JS(e,n),this.dirtyShape()},i}function iw(r,t){return new Tg(Cg(r,t))}function nw(r,t){var e=Cg(r,t),i=function(n){k(a,n);function a(o){var s=n.call(this,o)||this;return s.applyTransform=e.applyTransform,s.buildPath=e.buildPath,s}return a}(Tg);return i}function aw(r,t){for(var e=[],i=r.length,n=0;n<i;n++){var a=r[n];e.push(a.getUpdatedPathProxy(!0))}var o=new st(t);return o.createPathProxy(),o.buildPath=function(s){if(Mg(s)){s.appendPath(e);var l=s.getContext();l&&s.rebuildPath(l,1)}},o}var Dg=function(r){k(t,r);function t(e){var i=r.call(this)||this;return i.isGroup=!0,i._children=[],i.attr(e),i}return t.prototype.childrenRef=function(){return this._children},t.prototype.children=function(){return this._children.slice()},t.prototype.childAt=function(e){return this._children[e]},t.prototype.childOfName=function(e){for(var i=this._children,n=0;n<i.length;n++)if(i[n].name===e)return i[n]},t.prototype.childCount=function(){return this._children.length},t.prototype.add=function(e){return e&&e!==this&&e.parent!==this&&(this._children.push(e),this._doAdd(e)),this},t.prototype.addBefore=function(e,i){if(e&&e!==this&&e.parent!==this&&i&&i.parent===this){var n=this._children,a=n.indexOf(i);a>=0&&(n.splice(a,0,e),this._doAdd(e))}return this},t.prototype.replace=function(e,i){var n=ot(this._children,e);return n>=0&&this.replaceAt(i,n),this},t.prototype.replaceAt=function(e,i){var n=this._children,a=n[i];if(e&&e!==this&&e.parent!==this&&e!==a){n[i]=e,a.parent=null;var o=this.__zr;o&&a.removeSelfFromZr(o),this._doAdd(e)}return this},t.prototype._doAdd=function(e){e.parent&&e.parent.remove(e),e.parent=this;var i=this.__zr;i&&i!==e.__zr&&e.addSelfToZr(i),i&&i.refresh()},t.prototype.remove=function(e){var i=this.__zr,n=this._children,a=ot(n,e);return a<0?this:(n.splice(a,1),e.parent=null,i&&e.removeSelfFromZr(i),i&&i.refresh(),this)},t.prototype.removeAll=function(){for(var e=this._children,i=this.__zr,n=0;n<e.length;n++){var a=e[n];i&&a.removeSelfFromZr(i),a.parent=null}return e.length=0,this},t.prototype.eachChild=function(e,i){for(var n=this._children,a=0;a<n.length;a++){var o=n[a];e.call(i,o,a)}return this},t.prototype.traverse=function(e,i){for(var n=0;n<this._children.length;n++){var a=this._children[n],o=e.call(i,a);a.isGroup&&!o&&a.traverse(e,i)}return this},t.prototype.addSelfToZr=function(e){r.prototype.addSelfToZr.call(this,e);for(var i=0;i<this._children.length;i++){var n=this._children[i];n.addSelfToZr(e)}},t.prototype.removeSelfFromZr=function(e){r.prototype.removeSelfFromZr.call(this,e);for(var i=0;i<this._children.length;i++){var n=this._children[i];n.removeSelfFromZr(e)}},t.prototype.getBoundingRect=function(e){for(var i=new et(0,0,0,0),n=e||this._children,a=[],o=null,s=0;s<n.length;s++){var l=n[s];if(!(l.ignore||l.invisible)){var u=l.getBoundingRect(),f=l.getLocalTransform(a);f?(et.applyTransform(i,u,f),o=o||i.clone(),o.union(i)):(o=o||u.clone(),o.union(u))}}return o||i},t}(rs);Dg.prototype.type="group";const mt=Dg;var ow=function(){function r(){this.cx=0,this.cy=0,this.r=0}return r}(),Ag=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new ow},t.prototype.buildPath=function(e,i){e.moveTo(i.cx+i.r,i.cy),e.arc(i.cx,i.cy,i.r,0,Math.PI*2)},t}(st);Ag.prototype.type="circle";const ls=Ag;var sw=function(){function r(){this.cx=0,this.cy=0,this.rx=0,this.ry=0}return r}(),Ig=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new sw},t.prototype.buildPath=function(e,i){var n=.5522848,a=i.cx,o=i.cy,s=i.rx,l=i.ry,u=s*n,f=l*n;e.moveTo(a-s,o),e.bezierCurveTo(a-s,o-f,a-u,o-l,a,o-l),e.bezierCurveTo(a+u,o-l,a+s,o-f,a+s,o),e.bezierCurveTo(a+s,o+f,a+u,o+l,a,o+l),e.bezierCurveTo(a-u,o+l,a-s,o+f,a-s,o),e.closePath()},t}(st);Ig.prototype.type="ellipse";const Lf=Ig;var Lg=Math.PI,tl=Lg*2,Fr=Math.sin,bi=Math.cos,lw=Math.acos,Mt=Math.atan2,yv=Math.abs,Rn=Math.sqrt,bn=Math.max,Le=Math.min,_e=1e-4;function uw(r,t,e,i,n,a,o,s){var l=e-r,u=i-t,f=o-n,h=s-a,c=h*l-f*u;if(!(c*c<_e))return c=(f*(t-a)-h*(r-n))/c,[r+c*l,t+c*u]}function Ra(r,t,e,i,n,a,o){var s=r-e,l=t-i,u=(o?a:-a)/Rn(s*s+l*l),f=u*l,h=-u*s,c=r+f,v=t+h,d=e+f,y=i+h,p=(c+d)/2,g=(v+y)/2,m=d-c,_=y-v,S=m*m+_*_,b=n-a,w=c*y-d*v,x=(_<0?-1:1)*Rn(bn(0,b*b*S-w*w)),C=(w*_-m*x)/S,D=(-w*m-_*x)/S,A=(w*_+m*x)/S,M=(-w*m+_*x)/S,I=C-p,L=D-g,P=A-p,R=M-g;return I*I+L*L>P*P+R*R&&(C=A,D=M),{cx:C,cy:D,x0:-f,y0:-h,x1:C*(n/b-1),y1:D*(n/b-1)}}function fw(r){var t;if(N(r)){var e=r.length;if(!e)return r;e===1?t=[r[0],r[0],0,0]:e===2?t=[r[0],r[0],r[1],r[1]]:e===3?t=r.concat(r[2]):t=r}else t=[r,r,r,r];return t}function hw(r,t){var e,i=bn(t.r,0),n=bn(t.r0||0,0),a=i>0,o=n>0;if(!(!a&&!o)){if(a||(i=n,n=0),n>i){var s=i;i=n,n=s}var l=t.startAngle,u=t.endAngle;if(!(isNaN(l)||isNaN(u))){var f=t.cx,h=t.cy,c=!!t.clockwise,v=yv(u-l),d=v>tl&&v%tl;if(d>_e&&(v=d),!(i>_e))r.moveTo(f,h);else if(v>tl-_e)r.moveTo(f+i*bi(l),h+i*Fr(l)),r.arc(f,h,i,l,u,!c),n>_e&&(r.moveTo(f+n*bi(u),h+n*Fr(u)),r.arc(f,h,n,u,l,c));else{var y=void 0,p=void 0,g=void 0,m=void 0,_=void 0,S=void 0,b=void 0,w=void 0,x=void 0,C=void 0,D=void 0,A=void 0,M=void 0,I=void 0,L=void 0,P=void 0,R=i*bi(l),E=i*Fr(l),W=n*bi(u),B=n*Fr(u),F=v>_e;if(F){var G=t.cornerRadius;G&&(e=fw(G),y=e[0],p=e[1],g=e[2],m=e[3]);var q=yv(i-n)/2;if(_=Le(q,g),S=Le(q,m),b=Le(q,y),w=Le(q,p),D=x=bn(_,S),A=C=bn(b,w),(x>_e||C>_e)&&(M=i*bi(u),I=i*Fr(u),L=n*bi(l),P=n*Fr(l),v<Lg)){var it=uw(R,E,L,P,M,I,W,B);if(it){var lt=R-it[0],vt=E-it[1],$t=M-it[0],ct=I-it[1],Ye=1/Fr(lw((lt*$t+vt*ct)/(Rn(lt*lt+vt*vt)*Rn($t*$t+ct*ct)))/2),pi=Rn(it[0]*it[0]+it[1]*it[1]);D=Le(x,(i-pi)/(Ye+1)),A=Le(C,(n-pi)/(Ye-1))}}}if(!F)r.moveTo(f+R,h+E);else if(D>_e){var Zt=Le(g,D),St=Le(m,D),X=Ra(L,P,R,E,i,Zt,c),j=Ra(M,I,W,B,i,St,c);r.moveTo(f+X.cx+X.x0,h+X.cy+X.y0),D<x&&Zt===St?r.arc(f+X.cx,h+X.cy,D,Mt(X.y0,X.x0),Mt(j.y0,j.x0),!c):(Zt>0&&r.arc(f+X.cx,h+X.cy,Zt,Mt(X.y0,X.x0),Mt(X.y1,X.x1),!c),r.arc(f,h,i,Mt(X.cy+X.y1,X.cx+X.x1),Mt(j.cy+j.y1,j.cx+j.x1),!c),St>0&&r.arc(f+j.cx,h+j.cy,St,Mt(j.y1,j.x1),Mt(j.y0,j.x0),!c))}else r.moveTo(f+R,h+E),r.arc(f,h,i,l,u,!c);if(!(n>_e)||!F)r.lineTo(f+W,h+B);else if(A>_e){var Zt=Le(y,A),St=Le(p,A),X=Ra(W,B,M,I,n,-St,c),j=Ra(R,E,L,P,n,-Zt,c);r.lineTo(f+X.cx+X.x0,h+X.cy+X.y0),A<C&&Zt===St?r.arc(f+X.cx,h+X.cy,A,Mt(X.y0,X.x0),Mt(j.y0,j.x0),!c):(St>0&&r.arc(f+X.cx,h+X.cy,St,Mt(X.y0,X.x0),Mt(X.y1,X.x1),!c),r.arc(f,h,n,Mt(X.cy+X.y1,X.cx+X.x1),Mt(j.cy+j.y1,j.cx+j.x1),c),Zt>0&&r.arc(f+j.cx,h+j.cy,Zt,Mt(j.y1,j.x1),Mt(j.y0,j.x0),!c))}else r.lineTo(f+W,h+B),r.arc(f,h,n,u,l,c)}r.closePath()}}}var vw=function(){function r(){this.cx=0,this.cy=0,this.r0=0,this.r=0,this.startAngle=0,this.endAngle=Math.PI*2,this.clockwise=!0,this.cornerRadius=0}return r}(),Pg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new vw},t.prototype.buildPath=function(e,i){hw(e,i)},t.prototype.isZeroArea=function(){return this.shape.startAngle===this.shape.endAngle||this.shape.r===this.shape.r0},t}(st);Pg.prototype.type="sector";const Xi=Pg;var cw=function(){function r(){this.cx=0,this.cy=0,this.r=0,this.r0=0}return r}(),Rg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new cw},t.prototype.buildPath=function(e,i){var n=i.cx,a=i.cy,o=Math.PI*2;e.moveTo(n+i.r,a),e.arc(n,a,i.r,0,o,!1),e.moveTo(n+i.r0,a),e.arc(n,a,i.r0,0,o,!0)},t}(st);Rg.prototype.type="ring";const Pf=Rg;function dw(r,t,e,i){var n=[],a=[],o=[],s=[],l,u,f,h;if(i){f=[1/0,1/0],h=[-1/0,-1/0];for(var c=0,v=r.length;c<v;c++)cr(f,f,r[c]),dr(h,h,r[c]);cr(f,f,i[0]),dr(h,h,i[1])}for(var c=0,v=r.length;c<v;c++){var d=r[c];if(e)l=r[c?c-1:v-1],u=r[(c+1)%v];else if(c===0||c===v-1){n.push(kp(r[c]));continue}else l=r[c-1],u=r[c+1];Bp(a,u,l),no(a,a,t);var y=To(d,l),p=To(d,u),g=y+p;g!==0&&(y/=g,p/=g),no(o,a,-y),no(s,a,p);var m=eu([],d,o),_=eu([],d,s);i&&(dr(m,m,f),cr(m,m,h),dr(_,_,f),cr(_,_,h)),n.push(m),n.push(_)}return e&&n.push(n.shift()),n}function Eg(r,t,e){var i=t.smooth,n=t.points;if(n&&n.length>=2){if(i){var a=dw(n,i,e,t.smoothConstraint);r.moveTo(n[0][0],n[0][1]);for(var o=n.length,s=0;s<(e?o:o-1);s++){var l=a[s*2],u=a[s*2+1],f=n[(s+1)%o];r.bezierCurveTo(l[0],l[1],u[0],u[1],f[0],f[1])}}else{r.moveTo(n[0][0],n[0][1]);for(var s=1,h=n.length;s<h;s++)r.lineTo(n[s][0],n[s][1])}e&&r.closePath()}}var pw=function(){function r(){this.points=null,this.smooth=0,this.smoothConstraint=null}return r}(),Og=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultShape=function(){return new pw},t.prototype.buildPath=function(e,i){Eg(e,i,!0)},t}(st);Og.prototype.type="polygon";const us=Og;var gw=function(){function r(){this.points=null,this.percent=1,this.smooth=0,this.smoothConstraint=null}return r}(),kg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultStyle=function(){return{stroke:"#000",fill:null}},t.prototype.getDefaultShape=function(){return new gw},t.prototype.buildPath=function(e,i){Eg(e,i,!1)},t}(st);kg.prototype.type="polyline";const Rf=kg;var yw={},mw=function(){function r(){this.x1=0,this.y1=0,this.x2=0,this.y2=0,this.percent=1}return r}(),Bg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultStyle=function(){return{stroke:"#000",fill:null}},t.prototype.getDefaultShape=function(){return new mw},t.prototype.buildPath=function(e,i){var n,a,o,s;if(this.subPixelOptimize){var l=Jp(yw,i,this.style);n=l.x1,a=l.y1,o=l.x2,s=l.y2}else n=i.x1,a=i.y1,o=i.x2,s=i.y2;var u=i.percent;u!==0&&(e.moveTo(n,a),u<1&&(o=n*(1-u)+o*u,s=a*(1-u)+s*u),e.lineTo(o,s))},t.prototype.pointAt=function(e){var i=this.shape;return[i.x1*(1-e)+i.x2*e,i.y1*(1-e)+i.y2*e]},t}(st);Bg.prototype.type="line";const xr=Bg;var zt=[],_w=function(){function r(){this.x1=0,this.y1=0,this.x2=0,this.y2=0,this.cpx1=0,this.cpy1=0,this.percent=1}return r}();function mv(r,t,e){var i=r.cpx2,n=r.cpy2;return i!=null||n!=null?[(e?Ph:bt)(r.x1,r.cpx1,r.cpx2,r.x2,t),(e?Ph:bt)(r.y1,r.cpy1,r.cpy2,r.y2,t)]:[(e?Rh:Ht)(r.x1,r.cpx1,r.x2,t),(e?Rh:Ht)(r.y1,r.cpy1,r.y2,t)]}var Ng=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultStyle=function(){return{stroke:"#000",fill:null}},t.prototype.getDefaultShape=function(){return new _w},t.prototype.buildPath=function(e,i){var n=i.x1,a=i.y1,o=i.x2,s=i.y2,l=i.cpx1,u=i.cpy1,f=i.cpx2,h=i.cpy2,c=i.percent;c!==0&&(e.moveTo(n,a),f==null||h==null?(c<1&&(Ao(n,l,o,c,zt),l=zt[1],o=zt[2],Ao(a,u,s,c,zt),u=zt[1],s=zt[2]),e.quadraticCurveTo(l,u,o,s)):(c<1&&(Do(n,l,f,o,c,zt),l=zt[1],f=zt[2],o=zt[3],Do(a,u,h,s,c,zt),u=zt[1],h=zt[2],s=zt[3]),e.bezierCurveTo(l,u,f,h,o,s)))},t.prototype.pointAt=function(e){return mv(this.shape,e,!1)},t.prototype.tangentAt=function(e){var i=mv(this.shape,e,!0);return Np(i,i)},t}(st);Ng.prototype.type="bezier-curve";const Ef=Ng;var Sw=function(){function r(){this.cx=0,this.cy=0,this.r=0,this.startAngle=0,this.endAngle=Math.PI*2,this.clockwise=!0}return r}(),Fg=function(r){k(t,r);function t(e){return r.call(this,e)||this}return t.prototype.getDefaultStyle=function(){return{stroke:"#000",fill:null}},t.prototype.getDefaultShape=function(){return new Sw},t.prototype.buildPath=function(e,i){var n=i.cx,a=i.cy,o=Math.max(i.r,0),s=i.startAngle,l=i.endAngle,u=i.clockwise,f=Math.cos(s),h=Math.sin(s);e.moveTo(f*o+n,h*o+a),e.arc(n,a,o,s,l,!u)},t}(st);Fg.prototype.type="arc";const fs=Fg;var ww=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="compound",e}return t.prototype._updatePathDirty=function(){for(var e=this.shape.paths,i=this.shapeChanged(),n=0;n<e.length;n++)i=i||e[n].shapeChanged();i&&this.dirtyShape()},t.prototype.beforeBrush=function(){this._updatePathDirty();for(var e=this.shape.paths||[],i=this.getGlobalScale(),n=0;n<e.length;n++)e[n].path||e[n].createPathProxy(),e[n].path.setScale(i[0],i[1],e[n].segmentIgnoreThreshold)},t.prototype.buildPath=function(e,i){for(var n=i.paths||[],a=0;a<n.length;a++)n[a].buildPath(e,n[a].shape,!0)},t.prototype.afterBrush=function(){for(var e=this.shape.paths||[],i=0;i<e.length;i++)e[i].pathUpdated()},t.prototype.getBoundingRect=function(){return this._updatePathDirty.call(this),st.prototype.getBoundingRect.call(this)},t}(st);const zg=ww;var Vg=function(){function r(t){this.colorStops=t||[]}return r.prototype.addColorStop=function(t,e){this.colorStops.push({offset:t,color:e})},r}(),bw=function(r){k(t,r);function t(e,i,n,a,o,s){var l=r.call(this,o)||this;return l.x=e??0,l.y=i??0,l.x2=n??1,l.y2=a??0,l.type="linear",l.global=s||!1,l}return t}(Vg);const hs=bw;var xw=function(r){k(t,r);function t(e,i,n,a,o){var s=r.call(this,a)||this;return s.x=e??.5,s.y=i??.5,s.r=n??.5,s.type="radial",s.global=o||!1,s}return t}(Vg);const Hg=xw;var zr=[0,0],Vr=[0,0],Ea=new ut,Oa=new ut,Tw=function(){function r(t,e){this._corners=[],this._axes=[],this._origin=[0,0];for(var i=0;i<4;i++)this._corners[i]=new ut;for(var i=0;i<2;i++)this._axes[i]=new ut;t&&this.fromBoundingRect(t,e)}return r.prototype.fromBoundingRect=function(t,e){var i=this._corners,n=this._axes,a=t.x,o=t.y,s=a+t.width,l=o+t.height;if(i[0].set(a,o),i[1].set(s,o),i[2].set(s,l),i[3].set(a,l),e)for(var u=0;u<4;u++)i[u].transform(e);ut.sub(n[0],i[1],i[0]),ut.sub(n[1],i[3],i[0]),n[0].normalize(),n[1].normalize();for(var u=0;u<2;u++)this._origin[u]=n[u].dot(i[0])},r.prototype.intersect=function(t,e){var i=!0,n=!e;return Ea.set(1/0,1/0),Oa.set(0,0),!this._intersectCheckOneSide(this,t,Ea,Oa,n,1)&&(i=!1,n)||!this._intersectCheckOneSide(t,this,Ea,Oa,n,-1)&&(i=!1,n)||n||ut.copy(e,i?Ea:Oa),i},r.prototype._intersectCheckOneSide=function(t,e,i,n,a,o){for(var s=!0,l=0;l<2;l++){var u=this._axes[l];if(this._getProjMinMaxOnAxis(l,t._corners,zr),this._getProjMinMaxOnAxis(l,e._corners,Vr),zr[1]<Vr[0]||zr[0]>Vr[1]){if(s=!1,a)return s;var f=Math.abs(Vr[0]-zr[1]),h=Math.abs(zr[0]-Vr[1]);Math.min(f,h)>n.len()&&(f<h?ut.scale(n,u,-f*o):ut.scale(n,u,h*o))}else if(i){var f=Math.abs(Vr[0]-zr[1]),h=Math.abs(zr[0]-Vr[1]);Math.min(f,h)<i.len()&&(f<h?ut.scale(i,u,f*o):ut.scale(i,u,-h*o))}}return s},r.prototype._getProjMinMaxOnAxis=function(t,e,i){for(var n=this._axes[t],a=this._origin,o=e[0].dot(n)+a[t],s=o,l=o,u=1;u<e.length;u++){var f=e[u].dot(n)+a[t];s=Math.min(f,s),l=Math.max(f,l)}i[0]=s,i[1]=l},r}();const Oo=Tw;var Mw=[],Cw=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.notClear=!0,e.incremental=!0,e._displayables=[],e._temporaryDisplayables=[],e._cursor=0,e}return t.prototype.traverse=function(e,i){e.call(i,this)},t.prototype.useStyle=function(){this.style={}},t.prototype.getCursor=function(){return this._cursor},t.prototype.innerAfterBrush=function(){this._cursor=this._displayables.length},t.prototype.clearDisplaybles=function(){this._displayables=[],this._temporaryDisplayables=[],this._cursor=0,this.markRedraw(),this.notClear=!1},t.prototype.clearTemporalDisplayables=function(){this._temporaryDisplayables=[]},t.prototype.addDisplayable=function(e,i){i?this._temporaryDisplayables.push(e):this._displayables.push(e),this.markRedraw()},t.prototype.addDisplayables=function(e,i){i=i||!1;for(var n=0;n<e.length;n++)this.addDisplayable(e[n],i)},t.prototype.getDisplayables=function(){return this._displayables},t.prototype.getTemporalDisplayables=function(){return this._temporaryDisplayables},t.prototype.eachPendingDisplayable=function(e){for(var i=this._cursor;i<this._displayables.length;i++)e&&e(this._displayables[i]);for(var i=0;i<this._temporaryDisplayables.length;i++)e&&e(this._temporaryDisplayables[i])},t.prototype.update=function(){this.updateTransform();for(var e=this._cursor;e<this._displayables.length;e++){var i=this._displayables[e];i.parent=this,i.update(),i.parent=null}for(var e=0;e<this._temporaryDisplayables.length;e++){var i=this._temporaryDisplayables[e];i.parent=this,i.update(),i.parent=null}},t.prototype.getBoundingRect=function(){if(!this._rect){for(var e=new et(1/0,1/0,-1/0,-1/0),i=0;i<this._displayables.length;i++){var n=this._displayables[i],a=n.getBoundingRect().clone();n.needLocalTransform()&&a.applyTransform(n.getLocalTransform(Mw)),e.union(a)}this._rect=e}return this._rect},t.prototype.contain=function(e,i){var n=this.transformCoordToLocal(e,i),a=this.getBoundingRect();if(a.contain(n[0],n[1]))for(var o=0;o<this._displayables.length;o++){var s=this._displayables[o];if(s.contain(e,i))return!0}return!1},t}(va);const Gg=Cw;var Dw=_t();function Aw(r,t,e,i,n){var a;if(t&&t.ecModel){var o=t.ecModel.getUpdatePayload();a=o&&o.animation}var s=t&&t.isAnimationEnabled(),l=r==="update";if(s){var u=void 0,f=void 0,h=void 0;i?(u=Q(i.duration,200),f=Q(i.easing,"cubicOut"),h=0):(u=t.getShallow(l?"animationDurationUpdate":"animationDuration"),f=t.getShallow(l?"animationEasingUpdate":"animationEasing"),h=t.getShallow(l?"animationDelayUpdate":"animationDelay")),a&&(a.duration!=null&&(u=a.duration),a.easing!=null&&(f=a.easing),a.delay!=null&&(h=a.delay)),Y(h)&&(h=h(e,n)),Y(u)&&(u=u(e));var c={duration:u||0,delay:h,easing:f};return c}else return null}function Of(r,t,e,i,n,a,o){var s=!1,l;Y(n)?(o=a,a=n,n=null):z(n)&&(a=n.cb,o=n.during,s=n.isFrom,l=n.removeOpt,n=n.dataIndex);var u=r==="leave";u||t.stopAnimation("leave");var f=Aw(r,i,n,u?l||{}:null,i&&i.getAnimationDelayParams?i.getAnimationDelayParams(t,n):null);if(f&&f.duration>0){var h=f.duration,c=f.delay,v=f.easing,d={duration:h,delay:c||0,easing:v,done:a,force:!!a||!!o,setToFinal:!u,scope:r,during:o};s?t.animateFrom(e,d):t.animateTo(e,d)}else t.stopAnimation(),!s&&t.attr(e),o&&o(1),a&&a()}function te(r,t,e,i,n,a){Of("update",r,t,e,i,n,a)}function He(r,t,e,i,n,a){Of("enter",r,t,e,i,n,a)}function En(r){if(!r.__zr)return!0;for(var t=0;t<r.animators.length;t++){var e=r.animators[t];if(e.scope==="leave")return!0}return!1}function ko(r,t,e,i,n,a){En(r)||Of("leave",r,t,e,i,n,a)}function _v(r,t,e,i){r.removeTextContent(),r.removeTextGuideLine(),ko(r,{style:{opacity:0}},t,e,i)}function wu(r,t,e){function i(){r.parent&&r.parent.remove(r)}r.isGroup?r.traverse(function(n){n.isGroup||_v(n,t,e,i)}):_v(r,t,e,i)}function Wg(r){Dw(r).oldStyle=r.style}var Bo=Math.max,No=Math.min,bu={};function Ug(r){return st.extend(r)}var Iw=nw;function Yg(r,t){return Iw(r,t)}function pe(r,t){bu[r]=t}function Xg(r){if(bu.hasOwnProperty(r))return bu[r]}function vs(r,t,e,i){var n=iw(r,t);return e&&(i==="center"&&(e=$g(e,n.getBoundingRect())),Bf(n,e)),n}function kf(r,t,e){var i=new tr({style:{image:r,x:t.x,y:t.y,width:t.width,height:t.height},onload:function(n){if(e==="center"){var a={width:n.width,height:n.height};i.setStyle($g(t,a))}}});return i}function $g(r,t){var e=t.width/t.height,i=r.height*e,n;i<=r.width?n=r.height:(i=r.width,n=i/e);var a=r.x+r.width/2,o=r.y+r.height/2;return{x:a-i/2,y:o-n/2,width:i,height:n}}var Zg=aw;function Bf(r,t){if(r.applyTransform){var e=r.getBoundingRect(),i=e.calculateTransform(t);r.applyTransform(i)}}function qn(r,t){return Jp(r,r,{lineWidth:t}),r}function Lw(r){return tg(r.shape,r.shape,r.style),r}var Pw=qr;function On(r,t){for(var e=fa([]);r&&r!==t;)_r(e,r.getLocalTransform(),e),r=r.parent;return e}function Fi(r,t,e){return t&&!Nt(t)&&(t=mf.getLocalTransform(t)),e&&(t=ha([],t)),Bt([],r,t)}function qg(r,t,e){var i=t[4]===0||t[5]===0||t[0]===0?1:Math.abs(2*t[4]/t[0]),n=t[4]===0||t[5]===0||t[2]===0?1:Math.abs(2*t[4]/t[2]),a=[r==="left"?-i:r==="right"?i:0,r==="top"?-n:r==="bottom"?n:0];return a=Fi(a,t,e),Math.abs(a[0])>Math.abs(a[1])?a[0]>0?"right":"left":a[1]>0?"bottom":"top"}function Sv(r){return!r.isGroup}function Rw(r){return r.shape!=null}function Kg(r,t,e){if(!r||!t)return;function i(o){var s={};return o.traverse(function(l){Sv(l)&&l.anid&&(s[l.anid]=l)}),s}function n(o){var s={x:o.x,y:o.y,rotation:o.rotation};return Rw(o)&&(s.shape=O({},o.shape)),s}var a=i(r);t.traverse(function(o){if(Sv(o)&&o.anid){var s=a[o.anid];if(s){var l=n(o);o.attr(n(s)),te(o,l,e,tt(o).dataIndex)}}})}function Qg(r,t){return H(r,function(e){var i=e[0];i=Bo(i,t.x),i=No(i,t.x+t.width);var n=e[1];return n=Bo(n,t.y),n=No(n,t.y+t.height),[i,n]})}function jg(r,t){var e=Bo(r.x,t.x),i=No(r.x+r.width,t.x+t.width),n=Bo(r.y,t.y),a=No(r.y+r.height,t.y+t.height);if(i>=e&&a>=n)return{x:e,y:n,width:i-e,height:a-n}}function Nf(r,t,e){var i=O({rectHover:!0},t),n=i.style={strokeNoScale:!0};if(e=e||{x:-1,y:-1,width:2,height:2},r)return r.indexOf("image://")===0?(n.image=r.slice(8),nt(n,e),new tr(i)):vs(r.replace("path://",""),i,e,"center")}function Ew(r,t,e,i,n){for(var a=0,o=n[n.length-1];a<n.length;a++){var s=n[a];if(Jg(r,t,e,i,s[0],s[1],o[0],o[1]))return!0;o=s}}function Jg(r,t,e,i,n,a,o,s){var l=e-r,u=i-t,f=o-n,h=s-a,c=el(f,h,l,u);if(Ow(c))return!1;var v=r-n,d=t-a,y=el(v,d,l,u)/c;if(y<0||y>1)return!1;var p=el(v,d,f,h)/c;return!(p<0||p>1)}function el(r,t,e,i){return r*i-e*t}function Ow(r){return r<=1e-6&&r>=-1e-6}function ty(r){var t=r.itemTooltipOption,e=r.componentModel,i=r.itemName,n=V(t)?{formatter:t}:t,a=e.mainType,o=e.componentIndex,s={componentType:a,name:i,$vars:["name"]};s[a+"Index"]=o;var l=r.formatterParamsExtra;l&&T(ft(l),function(f){si(s,f)||(s[f]=l[f],s.$vars.push(f))});var u=tt(r.el);u.componentMainType=a,u.componentIndex=o,u.tooltipConfig={name:i,option:nt({content:i,encodeHTMLContent:!0,formatterParams:s},n)}}function wv(r,t){var e;r.isGroup&&(e=t(r)),e||r.traverse(t)}function cs(r,t){if(r)if(N(r))for(var e=0;e<r.length;e++)wv(r[e],t);else wv(r,t)}pe("circle",ls);pe("ellipse",Lf);pe("sector",Xi);pe("ring",Pf);pe("polygon",us);pe("polyline",Rf);pe("rect",gt);pe("line",xr);pe("bezierCurve",Ef);pe("arc",fs);const kw=Object.freeze(Object.defineProperty({__proto__:null,Arc:fs,BezierCurve:Ef,BoundingRect:et,Circle:ls,CompoundPath:zg,Ellipse:Lf,Group:mt,Image:tr,IncrementalDisplayable:Gg,Line:xr,LinearGradient:hs,OrientedBoundingRect:Oo,Path:st,Point:ut,Polygon:us,Polyline:Rf,RadialGradient:Hg,Rect:gt,Ring:Pf,Sector:Xi,Text:yt,applyTransform:Fi,clipPointsByRect:Qg,clipRectByRect:jg,createIcon:Nf,extendPath:Yg,extendShape:Ug,getShapeClass:Xg,getTransform:On,groupTransition:Kg,initProps:He,isElementRemoved:En,lineLineIntersect:Jg,linePolygonIntersect:Ew,makeImage:kf,makePath:vs,mergePath:Zg,registerShape:pe,removeElement:ko,removeElementWithFadeOut:wu,resizePath:Bf,setTooltipConfig:ty,subPixelOptimize:Pw,subPixelOptimizeLine:qn,subPixelOptimizeRect:Lw,transformDirection:qg,traverseElements:cs,updateProps:te},Symbol.toStringTag,{value:"Module"}));var ds={};function Bw(r,t){for(var e=0;e<Ve.length;e++){var i=Ve[e],n=t[i],a=r.ensureState(i);a.style=a.style||{},a.style.text=n}var o=r.currentStates.slice();r.clearStates(!0),r.setStyle({text:t.normal}),r.useStates(o,!0)}function bv(r,t,e){var i=r.labelFetcher,n=r.labelDataIndex,a=r.labelDimIndex,o=t.normal,s;i&&(s=i.getFormattedLabel(n,"normal",null,a,o&&o.get("formatter"),e!=null?{interpolatedValue:e}:null)),s==null&&(s=Y(r.defaultText)?r.defaultText(n,r,e):r.defaultText);for(var l={normal:s},u=0;u<Ve.length;u++){var f=Ve[u],h=t[f];l[f]=Q(i?i.getFormattedLabel(n,f,null,a,h&&h.get("formatter")):null,s)}return l}function Ff(r,t,e,i){e=e||ds;for(var n=r instanceof yt,a=!1,o=0;o<av.length;o++){var s=t[av[o]];if(s&&s.getShallow("show")){a=!0;break}}var l=n?r:r.getTextContent();if(a){n||(l||(l=new yt,r.setTextContent(l)),r.stateProxy&&(l.stateProxy=r.stateProxy));var u=bv(e,t),f=t.normal,h=!!f.getShallow("show"),c=he(f,i&&i.normal,e,!1,!n);c.text=u.normal,n||r.setTextConfig(xv(f,e,!1));for(var o=0;o<Ve.length;o++){var v=Ve[o],s=t[v];if(s){var d=l.ensureState(v),y=!!Q(s.getShallow("show"),h);if(y!==h&&(d.ignore=!y),d.style=he(s,i&&i[v],e,!0,!n),d.style.text=u[v],!n){var p=r.ensureState(v);p.textConfig=xv(s,e,!0)}}}l.silent=!!f.getShallow("silent"),l.style.x!=null&&(c.x=l.style.x),l.style.y!=null&&(c.y=l.style.y),l.ignore=!h,l.useStyle(c),l.dirty(),e.enableTextSetter&&(gs(l).setLabelText=function(g){var m=bv(e,t,g);Bw(l,m)})}else l&&(l.ignore=!0);r.dirty()}function ps(r,t){t=t||"label";for(var e={normal:r.getModel(t)},i=0;i<Ve.length;i++){var n=Ve[i];e[n]=r.getModel([n,t])}return e}function he(r,t,e,i,n){var a={};return Nw(a,r,e,i,n),t&&O(a,t),a}function xv(r,t,e){t=t||{};var i={},n,a=r.getShallow("rotate"),o=Q(r.getShallow("distance"),e?null:5),s=r.getShallow("offset");return n=r.getShallow("position")||(e?null:"inside"),n==="outside"&&(n=t.defaultOutsidePosition||"top"),n!=null&&(i.position=n),s!=null&&(i.offset=s),a!=null&&(a*=Math.PI/180,i.rotation=a),o!=null&&(i.distance=o),i.outsideFill=r.get("color")==="inherit"?t.inheritColor||null:"auto",i}function Nw(r,t,e,i,n){e=e||ds;var a=t.ecModel,o=a&&a.option.textStyle,s=Fw(t),l;if(s){l={};for(var u in s)if(s.hasOwnProperty(u)){var f=t.getModel(["rich",u]);Dv(l[u]={},f,o,e,i,n,!1,!0)}}l&&(r.rich=l);var h=t.get("overflow");h&&(r.overflow=h);var c=t.get("minMargin");c!=null&&(r.margin=c),Dv(r,t,o,e,i,n,!0,!1)}function Fw(r){for(var t;r&&r!==r.ecModel;){var e=(r.option||ds).rich;if(e){t=t||{};for(var i=ft(e),n=0;n<i.length;n++){var a=i[n];t[a]=1}}r=r.parentModel}return t}var Tv=["fontStyle","fontWeight","fontSize","fontFamily","textShadowColor","textShadowBlur","textShadowOffsetX","textShadowOffsetY"],Mv=["align","lineHeight","width","height","tag","verticalAlign","ellipsis"],Cv=["padding","borderWidth","borderRadius","borderDashOffset","backgroundColor","borderColor","shadowColor","shadowBlur","shadowOffsetX","shadowOffsetY"];function Dv(r,t,e,i,n,a,o,s){e=!n&&e||ds;var l=i&&i.inheritColor,u=t.getShallow("color"),f=t.getShallow("textBorderColor"),h=Q(t.getShallow("opacity"),e.opacity);(u==="inherit"||u==="auto")&&(l?u=l:u=null),(f==="inherit"||f==="auto")&&(l?f=l:f=null),a||(u=u||e.color,f=f||e.textBorderColor),u!=null&&(r.fill=u),f!=null&&(r.stroke=f);var c=Q(t.getShallow("textBorderWidth"),e.textBorderWidth);c!=null&&(r.lineWidth=c);var v=Q(t.getShallow("textBorderType"),e.textBorderType);v!=null&&(r.lineDash=v);var d=Q(t.getShallow("textBorderDashOffset"),e.textBorderDashOffset);d!=null&&(r.lineDashOffset=d),!n&&h==null&&!s&&(h=i&&i.defaultOpacity),h!=null&&(r.opacity=h),!n&&!a&&r.fill==null&&i.inheritColor&&(r.fill=i.inheritColor);for(var y=0;y<Tv.length;y++){var p=Tv[y],g=Q(t.getShallow(p),e[p]);g!=null&&(r[p]=g)}for(var y=0;y<Mv.length;y++){var p=Mv[y],g=t.getShallow(p);g!=null&&(r[p]=g)}if(r.verticalAlign==null){var m=t.getShallow("baseline");m!=null&&(r.verticalAlign=m)}if(!o||!i.disableBox){for(var y=0;y<Cv.length;y++){var p=Cv[y],g=t.getShallow(p);g!=null&&(r[p]=g)}var _=t.getShallow("borderType");_!=null&&(r.borderDash=_),(r.backgroundColor==="auto"||r.backgroundColor==="inherit")&&l&&(r.backgroundColor=l),(r.borderColor==="auto"||r.borderColor==="inherit")&&l&&(r.borderColor=l)}}function zw(r,t){var e=t&&t.getModel("textStyle");return be([r.fontStyle||e&&e.getShallow("fontStyle")||"",r.fontWeight||e&&e.getShallow("fontWeight")||"",(r.fontSize||e&&e.getShallow("fontSize")||12)+"px",r.fontFamily||e&&e.getShallow("fontFamily")||"sans-serif"].join(" "))}var gs=_t();function Vw(r,t,e,i){if(r){var n=gs(r);n.prevValue=n.value,n.value=e;var a=t.normal;n.valueAnimation=a.get("valueAnimation"),n.valueAnimation&&(n.precision=a.get("precision"),n.defaultInterpolatedText=i,n.statesModels=t)}}var Hw=["textStyle","color"],rl=["fontStyle","fontWeight","fontSize","fontFamily","padding","lineHeight","rich","width","height","overflow"],il=new yt,Gw=function(){function r(){}return r.prototype.getTextColor=function(t){var e=this.ecModel;return this.getShallow("color")||(!t&&e?e.get(Hw):null)},r.prototype.getFont=function(){return zw({fontStyle:this.getShallow("fontStyle"),fontWeight:this.getShallow("fontWeight"),fontSize:this.getShallow("fontSize"),fontFamily:this.getShallow("fontFamily")},this.ecModel)},r.prototype.getTextRect=function(t){for(var e={text:t,verticalAlign:this.getShallow("verticalAlign")||this.getShallow("baseline")},i=0;i<rl.length;i++)e[rl[i]]=this.getShallow(rl[i]);return il.useStyle(e),il.update(),il.getBoundingRect()},r}(),ey=[["lineWidth","width"],["stroke","color"],["opacity"],["shadowBlur"],["shadowOffsetX"],["shadowOffsetY"],["shadowColor"],["lineDash","type"],["lineDashOffset","dashOffset"],["lineCap","cap"],["lineJoin","join"],["miterLimit"]],Ww=Gn(ey),Uw=function(){function r(){}return r.prototype.getLineStyle=function(t){return Ww(this,t)},r}(),ry=[["fill","color"],["stroke","borderColor"],["lineWidth","borderWidth"],["opacity"],["shadowBlur"],["shadowOffsetX"],["shadowOffsetY"],["shadowColor"],["lineDash","borderType"],["lineDashOffset","borderDashOffset"],["lineCap","borderCap"],["lineJoin","borderJoin"],["miterLimit","borderMiterLimit"]],Yw=Gn(ry),Xw=function(){function r(){}return r.prototype.getItemStyle=function(t,e){return Yw(this,t,e)},r}(),ci=function(){function r(t,e,i){this.parentModel=e,this.ecModel=i,this.option=t}return r.prototype.init=function(t,e,i){},r.prototype.mergeOption=function(t,e){J(this.option,t,!0)},r.prototype.get=function(t,e){return t==null?this.option:this._doGet(this.parsePath(t),!e&&this.parentModel)},r.prototype.getShallow=function(t,e){var i=this.option,n=i==null?i:i[t];if(n==null&&!e){var a=this.parentModel;a&&(n=a.getShallow(t))}return n},r.prototype.getModel=function(t,e){var i=t!=null,n=i?this.parsePath(t):null,a=i?this._doGet(n):this.option;return e=e||this.parentModel&&this.parentModel.getModel(this.resolveParentPath(n)),new r(a,e,this.ecModel)},r.prototype.isEmpty=function(){return this.option==null},r.prototype.restoreData=function(){},r.prototype.clone=function(){var t=this.constructor;return new t(U(this.option))},r.prototype.parsePath=function(t){return typeof t=="string"?t.split("."):t},r.prototype.resolveParentPath=function(t){return t},r.prototype.isAnimationEnabled=function(){if(!Z.node&&this.option){if(this.option.animation!=null)return!!this.option.animation;if(this.parentModel)return this.parentModel.isAnimationEnabled()}},r.prototype._doGet=function(t,e){var i=this.option;if(!t)return i;for(var n=0;n<t.length&&!(t[n]&&(i=i&&typeof i=="object"?i[t[n]]:null,i==null));n++);return i==null&&e&&(i=e._doGet(this.resolveParentPath(t),e.parentModel)),i},r}();vf(ci);C_(ci);de(ci,Uw);de(ci,Xw);de(ci,P_);de(ci,Gw);const xt=ci;function an(r){return r==null?0:r.length||1}function Av(r){return r}var $w=function(){function r(t,e,i,n,a,o){this._old=t,this._new=e,this._oldKeyGetter=i||Av,this._newKeyGetter=n||Av,this.context=a,this._diffModeMultiple=o==="multiple"}return r.prototype.add=function(t){return this._add=t,this},r.prototype.update=function(t){return this._update=t,this},r.prototype.updateManyToOne=function(t){return this._updateManyToOne=t,this},r.prototype.updateOneToMany=function(t){return this._updateOneToMany=t,this},r.prototype.updateManyToMany=function(t){return this._updateManyToMany=t,this},r.prototype.remove=function(t){return this._remove=t,this},r.prototype.execute=function(){this[this._diffModeMultiple?"_executeMultiple":"_executeOneToOne"]()},r.prototype._executeOneToOne=function(){var t=this._old,e=this._new,i={},n=new Array(t.length),a=new Array(e.length);this._initIndexMap(t,null,n,"_oldKeyGetter"),this._initIndexMap(e,i,a,"_newKeyGetter");for(var o=0;o<t.length;o++){var s=n[o],l=i[s],u=an(l);if(u>1){var f=l.shift();l.length===1&&(i[s]=l[0]),this._update&&this._update(f,o)}else u===1?(i[s]=null,this._update&&this._update(l,o)):this._remove&&this._remove(o)}this._performRestAdd(a,i)},r.prototype._executeMultiple=function(){var t=this._old,e=this._new,i={},n={},a=[],o=[];this._initIndexMap(t,i,a,"_oldKeyGetter"),this._initIndexMap(e,n,o,"_newKeyGetter");for(var s=0;s<a.length;s++){var l=a[s],u=i[l],f=n[l],h=an(u),c=an(f);if(h>1&&c===1)this._updateManyToOne&&this._updateManyToOne(f,u),n[l]=null;else if(h===1&&c>1)this._updateOneToMany&&this._updateOneToMany(f,u),n[l]=null;else if(h===1&&c===1)this._update&&this._update(f,u),n[l]=null;else if(h>1&&c>1)this._updateManyToMany&&this._updateManyToMany(f,u),n[l]=null;else if(h>1)for(var v=0;v<h;v++)this._remove&&this._remove(u[v]);else this._remove&&this._remove(u)}this._performRestAdd(o,n)},r.prototype._performRestAdd=function(t,e){for(var i=0;i<t.length;i++){var n=t[i],a=e[n],o=an(a);if(o>1)for(var s=0;s<o;s++)this._add&&this._add(a[s]);else o===1&&this._add&&this._add(a);e[n]=null}},r.prototype._initIndexMap=function(t,e,i,n){for(var a=this._diffModeMultiple,o=0;o<t.length;o++){var s="_ec_"+this[n](t[o],o);if(a||(i[o]=s),!!e){var l=e[s],u=an(l);u===0?(e[s]=o,a&&i.push(s)):u===1?e[s]=[l,o]:l.push(o)}}},r}(),iy=$(["tooltip","label","itemName","itemId","itemGroupId","itemChildGroupId","seriesName"]),ge="original",Yt="arrayRows",We="objectRows",rr="keyedColumns",wr="typedArray",ny="unknown",Be="column",$i="row",qt={Must:1,Might:2,Not:3},ay=_t();function Zw(r){ay(r).datasetMap=$()}function qw(r,t,e){var i={},n=oy(t);if(!n||!r)return i;var a=[],o=[],s=t.ecModel,l=ay(s).datasetMap,u=n.uid+"_"+e.seriesLayoutBy,f,h;r=r.slice(),T(r,function(y,p){var g=z(y)?y:r[p]={name:y};g.type==="ordinal"&&f==null&&(f=p,h=d(g)),i[g.name]=[]});var c=l.get(u)||l.set(u,{categoryWayDim:h,valueWayDim:0});T(r,function(y,p){var g=y.name,m=d(y);if(f==null){var _=c.valueWayDim;v(i[g],_,m),v(o,_,m),c.valueWayDim+=m}else if(f===p)v(i[g],0,m),v(a,0,m);else{var _=c.categoryWayDim;v(i[g],_,m),v(o,_,m),c.categoryWayDim+=m}});function v(y,p,g){for(var m=0;m<g;m++)y.push(p+m)}function d(y){var p=y.dimsDef;return p?p.length:1}return a.length&&(i.itemName=a),o.length&&(i.seriesName=o),i}function oy(r){var t=r.get("data",!0);if(!t)return da(r.ecModel,"dataset",{index:r.get("datasetIndex",!0),id:r.get("datasetId",!0)},xe).models[0]}function Kw(r){return!r.get("transform",!0)&&!r.get("fromTransformResult",!0)?[]:da(r.ecModel,"dataset",{index:r.get("fromDatasetIndex",!0),id:r.get("fromDatasetId",!0)},xe).models}function sy(r,t){return Qw(r.data,r.sourceFormat,r.seriesLayoutBy,r.dimensionsDefine,r.startIndex,t)}function Qw(r,t,e,i,n,a){var o,s=5;if(Ft(r))return qt.Not;var l,u;if(i){var f=i[a];z(f)?(l=f.name,u=f.type):V(f)&&(l=f)}if(u!=null)return u==="ordinal"?qt.Must:qt.Not;if(t===Yt){var h=r;if(e===$i){for(var c=h[a],v=0;v<(c||[]).length&&v<s;v++)if((o=S(c[n+v]))!=null)return o}else for(var v=0;v<h.length&&v<s;v++){var d=h[n+v];if(d&&(o=S(d[a]))!=null)return o}}else if(t===We){var y=r;if(!l)return qt.Not;for(var v=0;v<y.length&&v<s;v++){var p=y[v];if(p&&(o=S(p[l]))!=null)return o}}else if(t===rr){var g=r;if(!l)return qt.Not;var c=g[l];if(!c||Ft(c))return qt.Not;for(var v=0;v<c.length&&v<s;v++)if((o=S(c[v]))!=null)return o}else if(t===ge)for(var m=r,v=0;v<m.length&&v<s;v++){var p=m[v],_=ca(p);if(!N(_))return qt.Not;if((o=S(_[a]))!=null)return o}function S(b){var w=V(b);if(b!=null&&Number.isFinite(Number(b))&&b!=="")return w?qt.Might:qt.Not;if(w&&b!=="-")return qt.Must}return qt.Not}var ys=function(){function r(t){this.data=t.data||(t.sourceFormat===rr?{}:[]),this.sourceFormat=t.sourceFormat||ny,this.seriesLayoutBy=t.seriesLayoutBy||Be,this.startIndex=t.startIndex||0,this.dimensionsDetectedCount=t.dimensionsDetectedCount,this.metaRawOption=t.metaRawOption;var e=this.dimensionsDefine=t.dimensionsDefine;if(e)for(var i=0;i<e.length;i++){var n=e[i];n.type==null&&sy(this,i)===qt.Must&&(n.type="ordinal")}}return r}();function zf(r){return r instanceof ys}function xu(r,t,e){e=e||uy(r);var i=t.seriesLayoutBy,n=Jw(r,e,i,t.sourceHeader,t.dimensions),a=new ys({data:r,sourceFormat:e,seriesLayoutBy:i,dimensionsDefine:n.dimensionsDefine,startIndex:n.startIndex,dimensionsDetectedCount:n.dimensionsDetectedCount,metaRawOption:U(t)});return a}function ly(r){return new ys({data:r,sourceFormat:Ft(r)?wr:ge})}function jw(r){return new ys({data:r.data,sourceFormat:r.sourceFormat,seriesLayoutBy:r.seriesLayoutBy,dimensionsDefine:U(r.dimensionsDefine),startIndex:r.startIndex,dimensionsDetectedCount:r.dimensionsDetectedCount})}function uy(r){var t=ny;if(Ft(r))t=wr;else if(N(r)){r.length===0&&(t=Yt);for(var e=0,i=r.length;e<i;e++){var n=r[e];if(n!=null){if(N(n)||Ft(n)){t=Yt;break}else if(z(n)){t=We;break}}}}else if(z(r)){for(var a in r)if(si(r,a)&&Nt(r[a])){t=rr;break}}return t}function Jw(r,t,e,i,n){var a,o;if(!r)return{dimensionsDefine:Iv(n),startIndex:o,dimensionsDetectedCount:a};if(t===Yt){var s=r;i==="auto"||i==null?Lv(function(u){u!=null&&u!=="-"&&(V(u)?o==null&&(o=1):o=0)},e,s,10):o=ht(i)?i:i?1:0,!n&&o===1&&(n=[],Lv(function(u,f){n[f]=u!=null?u+"":""},e,s,1/0)),a=n?n.length:e===$i?s.length:s[0]?s[0].length:null}else if(t===We)n||(n=tb(r));else if(t===rr)n||(n=[],T(r,function(u,f){n.push(f)}));else if(t===ge){var l=ca(r[0]);a=N(l)&&l.length||1}return{startIndex:o,dimensionsDefine:Iv(n),dimensionsDetectedCount:a}}function tb(r){for(var t=0,e;t<r.length&&!(e=r[t++]););if(e)return ft(e)}function Iv(r){if(r){var t=$();return H(r,function(e,i){e=z(e)?e:{name:e};var n={name:e.name,displayName:e.displayName,type:e.type};if(n.name==null)return n;n.name+="",n.displayName==null&&(n.displayName=n.name);var a=t.get(n.name);return a?n.name+="-"+a.count++:t.set(n.name,{count:1}),n})}}function Lv(r,t,e,i){if(t===$i)for(var n=0;n<e.length&&n<i;n++)r(e[n]?e[n][0]:null,n);else for(var a=e[0]||[],n=0;n<a.length&&n<i;n++)r(a[n],n)}function fy(r){var t=r.sourceFormat;return t===We||t===rr}var Hr,Gr,Wr,Pv,Rv,hy=function(){function r(t,e){var i=zf(t)?t:ly(t);this._source=i;var n=this._data=i.data;i.sourceFormat===wr&&(this._offset=0,this._dimSize=e,this._data=n),Rv(this,n,i)}return r.prototype.getSource=function(){return this._source},r.prototype.count=function(){return 0},r.prototype.getItem=function(t,e){},r.prototype.appendData=function(t){},r.prototype.clean=function(){},r.protoInitialize=function(){var t=r.prototype;t.pure=!1,t.persistent=!0}(),r.internalField=function(){var t;Rv=function(o,s,l){var u=l.sourceFormat,f=l.seriesLayoutBy,h=l.startIndex,c=l.dimensionsDefine,v=Pv[Vf(u,f)];if(O(o,v),u===wr)o.getItem=e,o.count=n,o.fillStorage=i;else{var d=vy(u,f);o.getItem=K(d,null,s,h,c);var y=cy(u,f);o.count=K(y,null,s,h,c)}};var e=function(o,s){o=o-this._offset,s=s||[];for(var l=this._data,u=this._dimSize,f=u*o,h=0;h<u;h++)s[h]=l[f+h];return s},i=function(o,s,l,u){for(var f=this._data,h=this._dimSize,c=0;c<h;c++){for(var v=u[c],d=v[0]==null?1/0:v[0],y=v[1]==null?-1/0:v[1],p=s-o,g=l[c],m=0;m<p;m++){var _=f[m*h+c];g[o+m]=_,_<d&&(d=_),_>y&&(y=_)}v[0]=d,v[1]=y}},n=function(){return this._data?this._data.length/this._dimSize:0};Pv=(t={},t[Yt+"_"+Be]={pure:!0,appendData:a},t[Yt+"_"+$i]={pure:!0,appendData:function(){throw new Error('Do not support appendData when set seriesLayoutBy: "row".')}},t[We]={pure:!0,appendData:a},t[rr]={pure:!0,appendData:function(o){var s=this._data;T(o,function(l,u){for(var f=s[u]||(s[u]=[]),h=0;h<(l||[]).length;h++)f.push(l[h])})}},t[ge]={appendData:a},t[wr]={persistent:!1,pure:!0,appendData:function(o){this._data=o},clean:function(){this._offset+=this.count(),this._data=null}},t);function a(o){for(var s=0;s<o.length;s++)this._data.push(o[s])}}(),r}(),Ev=function(r,t,e,i){return r[i]},eb=(Hr={},Hr[Yt+"_"+Be]=function(r,t,e,i){return r[i+t]},Hr[Yt+"_"+$i]=function(r,t,e,i,n){i+=t;for(var a=n||[],o=r,s=0;s<o.length;s++){var l=o[s];a[s]=l?l[i]:null}return a},Hr[We]=Ev,Hr[rr]=function(r,t,e,i,n){for(var a=n||[],o=0;o<e.length;o++){var s=e[o].name,l=r[s];a[o]=l?l[i]:null}return a},Hr[ge]=Ev,Hr);function vy(r,t){var e=eb[Vf(r,t)];return e}var Ov=function(r,t,e){return r.length},rb=(Gr={},Gr[Yt+"_"+Be]=function(r,t,e){return Math.max(0,r.length-t)},Gr[Yt+"_"+$i]=function(r,t,e){var i=r[0];return i?Math.max(0,i.length-t):0},Gr[We]=Ov,Gr[rr]=function(r,t,e){var i=e[0].name,n=r[i];return n?n.length:0},Gr[ge]=Ov,Gr);function cy(r,t){var e=rb[Vf(r,t)];return e}var nl=function(r,t,e){return r[t]},ib=(Wr={},Wr[Yt]=nl,Wr[We]=function(r,t,e){return r[e]},Wr[rr]=nl,Wr[ge]=function(r,t,e){var i=ca(r);return i instanceof Array?i[t]:i},Wr[wr]=nl,Wr);function dy(r){var t=ib[r];return t}function Vf(r,t){return r===Yt?r+"_"+t:r}function Wi(r,t,e){if(r){var i=r.getRawDataItem(t);if(i!=null){var n=r.getStore(),a=n.getSource().sourceFormat;if(e!=null){var o=r.getDimensionIndex(e),s=n.getDimensionProperty(o);return dy(a)(i,o,s)}else{var l=i;return a===ge&&(l=ca(i)),l}}}}var nb=function(){function r(t,e){this._encode=t,this._schema=e}return r.prototype.get=function(){return{fullDimensions:this._getFullDimensionNames(),encode:this._encode}},r.prototype._getFullDimensionNames=function(){return this._cachedDimNames||(this._cachedDimNames=this._schema?this._schema.makeOutputDimensionNames():[]),this._cachedDimNames},r}();function ab(r,t){var e={},i=e.encode={},n=$(),a=[],o=[],s={};T(r.dimensions,function(c){var v=r.getDimensionInfo(c),d=v.coordDim;if(d){var y=v.coordDimIndex;al(i,d)[y]=c,v.isExtraCoord||(n.set(d,1),sb(v.type)&&(a[0]=c),al(s,d)[y]=r.getDimensionIndex(v.name)),v.defaultTooltip&&o.push(c)}iy.each(function(p,g){var m=al(i,g),_=v.otherDims[g];_!=null&&_!==!1&&(m[_]=v.name)})});var l=[],u={};n.each(function(c,v){var d=i[v];u[v]=d[0],l=l.concat(d)}),e.dataDimsOnCoord=l,e.dataDimIndicesOnCoord=H(l,function(c){return r.getDimensionInfo(c).storeDimIndex}),e.encodeFirstDimNotExtra=u;var f=i.label;f&&f.length&&(a=f.slice());var h=i.tooltip;return h&&h.length?o=h.slice():o.length||(o=a.slice()),i.defaultedLabel=a,i.defaultedTooltip=o,e.userOutput=new nb(s,t),e}function al(r,t){return r.hasOwnProperty(t)||(r[t]=[]),r[t]}function ob(r){return r==="category"?"ordinal":r==="time"?"time":"float"}function sb(r){return!(r==="ordinal"||r==="time")}var ho=function(){function r(t){this.otherDims={},t!=null&&O(this,t)}return r}();function vo(r,t){var e=t&&t.type;return e==="ordinal"?r:(e==="time"&&!ht(r)&&r!=null&&r!=="-"&&(r=+ve(r)),r==null||r===""?NaN:Number(r))}$({number:function(r){return parseFloat(r)},time:function(r){return+ve(r)},trim:function(r){return V(r)?be(r):r}});var lb=function(){function r(t,e){var i=t==="desc";this._resultLT=i?1:-1,e==null&&(e=i?"min":"max"),this._incomparable=e==="min"?-1/0:1/0}return r.prototype.evaluate=function(t,e){var i=ht(t)?t:$n(t),n=ht(e)?e:$n(e),a=isNaN(i),o=isNaN(n);if(a&&(i=this._incomparable),o&&(n=this._incomparable),a&&o){var s=V(t),l=V(e);s&&(i=l?t:0),l&&(n=s?e:0)}return i<n?this._resultLT:i>n?-this._resultLT:0},r}(),ms="undefined",ub=typeof Uint32Array===ms?Array:Uint32Array,fb=typeof Uint16Array===ms?Array:Uint16Array,py=typeof Int32Array===ms?Array:Int32Array,kv=typeof Float64Array===ms?Array:Float64Array,gy={float:kv,int:py,ordinal:Array,number:Array,time:kv},ol;function on(r){return r>65535?ub:fb}function xi(){return[1/0,-1/0]}function hb(r){var t=r.constructor;return t===Array?r.slice():new t(r)}function Bv(r,t,e,i,n){var a=gy[e||"float"];if(n){var o=r[t],s=o&&o.length;if(s!==i){for(var l=new a(i),u=0;u<s;u++)l[u]=o[u];r[t]=l}}else r[t]=new a(i)}var Tu=function(){function r(){this._chunks=[],this._rawExtent=[],this._extent=[],this._count=0,this._rawCount=0,this._calcDimNameToIdx=$()}return r.prototype.initData=function(t,e,i){this._provider=t,this._chunks=[],this._indices=null,this.getRawIndex=this._getRawIdxIdentity;var n=t.getSource(),a=this.defaultDimValueGetter=ol[n.sourceFormat];this._dimValueGetter=i||a,this._rawExtent=[],fy(n),this._dimensions=H(e,function(o){return{type:o.type,property:o.property}}),this._initDataFromProvider(0,t.count())},r.prototype.getProvider=function(){return this._provider},r.prototype.getSource=function(){return this._provider.getSource()},r.prototype.ensureCalculationDimension=function(t,e){var i=this._calcDimNameToIdx,n=this._dimensions,a=i.get(t);if(a!=null){if(n[a].type===e)return a}else a=n.length;return n[a]={type:e},i.set(t,a),this._chunks[a]=new gy[e||"float"](this._rawCount),this._rawExtent[a]=xi(),a},r.prototype.collectOrdinalMeta=function(t,e){var i=this._chunks[t],n=this._dimensions[t],a=this._rawExtent,o=n.ordinalOffset||0,s=i.length;o===0&&(a[t]=xi());for(var l=a[t],u=o;u<s;u++){var f=i[u]=e.parseAndCollect(i[u]);isNaN(f)||(l[0]=Math.min(f,l[0]),l[1]=Math.max(f,l[1]))}n.ordinalMeta=e,n.ordinalOffset=s,n.type="ordinal"},r.prototype.getOrdinalMeta=function(t){var e=this._dimensions[t],i=e.ordinalMeta;return i},r.prototype.getDimensionProperty=function(t){var e=this._dimensions[t];return e&&e.property},r.prototype.appendData=function(t){var e=this._provider,i=this.count();e.appendData(t);var n=e.count();return e.persistent||(n+=i),i<n&&this._initDataFromProvider(i,n,!0),[i,n]},r.prototype.appendValues=function(t,e){for(var i=this._chunks,n=this._dimensions,a=n.length,o=this._rawExtent,s=this.count(),l=s+Math.max(t.length,e||0),u=0;u<a;u++){var f=n[u];Bv(i,u,f.type,l,!0)}for(var h=[],c=s;c<l;c++)for(var v=c-s,d=0;d<a;d++){var f=n[d],y=ol.arrayRows.call(this,t[v]||h,f.property,v,d);i[d][c]=y;var p=o[d];y<p[0]&&(p[0]=y),y>p[1]&&(p[1]=y)}return this._rawCount=this._count=l,{start:s,end:l}},r.prototype._initDataFromProvider=function(t,e,i){for(var n=this._provider,a=this._chunks,o=this._dimensions,s=o.length,l=this._rawExtent,u=H(o,function(m){return m.property}),f=0;f<s;f++){var h=o[f];l[f]||(l[f]=xi()),Bv(a,f,h.type,e,i)}if(n.fillStorage)n.fillStorage(t,e,a,l);else for(var c=[],v=t;v<e;v++){c=n.getItem(v,c);for(var d=0;d<s;d++){var y=a[d],p=this._dimValueGetter(c,u[d],v,d);y[v]=p;var g=l[d];p<g[0]&&(g[0]=p),p>g[1]&&(g[1]=p)}}!n.persistent&&n.clean&&n.clean(),this._rawCount=this._count=e,this._extent=[]},r.prototype.count=function(){return this._count},r.prototype.get=function(t,e){if(!(e>=0&&e<this._count))return NaN;var i=this._chunks[t];return i?i[this.getRawIndex(e)]:NaN},r.prototype.getValues=function(t,e){var i=[],n=[];if(e==null){e=t,t=[];for(var a=0;a<this._dimensions.length;a++)n.push(a)}else n=t;for(var a=0,o=n.length;a<o;a++)i.push(this.get(n[a],e));return i},r.prototype.getByRawIndex=function(t,e){if(!(e>=0&&e<this._rawCount))return NaN;var i=this._chunks[t];return i?i[e]:NaN},r.prototype.getSum=function(t){var e=this._chunks[t],i=0;if(e)for(var n=0,a=this.count();n<a;n++){var o=this.get(t,n);isNaN(o)||(i+=o)}return i},r.prototype.getMedian=function(t){var e=[];this.each([t],function(a){isNaN(a)||e.push(a)});var i=e.sort(function(a,o){return a-o}),n=this.count();return n===0?0:n%2===1?i[(n-1)/2]:(i[n/2]+i[n/2-1])/2},r.prototype.indexOfRawIndex=function(t){if(t>=this._rawCount||t<0)return-1;if(!this._indices)return t;var e=this._indices,i=e[t];if(i!=null&&i<this._count&&i===t)return t;for(var n=0,a=this._count-1;n<=a;){var o=(n+a)/2|0;if(e[o]<t)n=o+1;else if(e[o]>t)a=o-1;else return o}return-1},r.prototype.indicesOfNearest=function(t,e,i){var n=this._chunks,a=n[t],o=[];if(!a)return o;i==null&&(i=1/0);for(var s=1/0,l=-1,u=0,f=0,h=this.count();f<h;f++){var c=this.getRawIndex(f),v=e-a[c],d=Math.abs(v);d<=i&&((d<s||d===s&&v>=0&&l<0)&&(s=d,l=v,u=0),v===l&&(o[u++]=f))}return o.length=u,o},r.prototype.getIndices=function(){var t,e=this._indices;if(e){var i=e.constructor,n=this._count;if(i===Array){t=new i(n);for(var a=0;a<n;a++)t[a]=e[a]}else t=new i(e.buffer,0,n)}else{var i=on(this._rawCount);t=new i(this.count());for(var a=0;a<t.length;a++)t[a]=a}return t},r.prototype.filter=function(t,e){if(!this._count)return this;for(var i=this.clone(),n=i.count(),a=on(i._rawCount),o=new a(n),s=[],l=t.length,u=0,f=t[0],h=i._chunks,c=0;c<n;c++){var v=void 0,d=i.getRawIndex(c);if(l===0)v=e(c);else if(l===1){var y=h[f][d];v=e(y,c)}else{for(var p=0;p<l;p++)s[p]=h[t[p]][d];s[p]=c,v=e.apply(null,s)}v&&(o[u++]=d)}return u<n&&(i._indices=o),i._count=u,i._extent=[],i._updateGetRawIdx(),i},r.prototype.selectRange=function(t){var e=this.clone(),i=e._count;if(!i)return this;var n=ft(t),a=n.length;if(!a)return this;var o=e.count(),s=on(e._rawCount),l=new s(o),u=0,f=n[0],h=t[f][0],c=t[f][1],v=e._chunks,d=!1;if(!e._indices){var y=0;if(a===1){for(var p=v[n[0]],g=0;g<i;g++){var m=p[g];(m>=h&&m<=c||isNaN(m))&&(l[u++]=y),y++}d=!0}else if(a===2){for(var p=v[n[0]],_=v[n[1]],S=t[n[1]][0],b=t[n[1]][1],g=0;g<i;g++){var m=p[g],w=_[g];(m>=h&&m<=c||isNaN(m))&&(w>=S&&w<=b||isNaN(w))&&(l[u++]=y),y++}d=!0}}if(!d)if(a===1)for(var g=0;g<o;g++){var x=e.getRawIndex(g),m=v[n[0]][x];(m>=h&&m<=c||isNaN(m))&&(l[u++]=x)}else for(var g=0;g<o;g++){for(var C=!0,x=e.getRawIndex(g),D=0;D<a;D++){var A=n[D],m=v[A][x];(m<t[A][0]||m>t[A][1])&&(C=!1)}C&&(l[u++]=e.getRawIndex(g))}return u<o&&(e._indices=l),e._count=u,e._extent=[],e._updateGetRawIdx(),e},r.prototype.map=function(t,e){var i=this.clone(t);return this._updateDims(i,t,e),i},r.prototype.modify=function(t,e){this._updateDims(this,t,e)},r.prototype._updateDims=function(t,e,i){for(var n=t._chunks,a=[],o=e.length,s=t.count(),l=[],u=t._rawExtent,f=0;f<e.length;f++)u[e[f]]=xi();for(var h=0;h<s;h++){for(var c=t.getRawIndex(h),v=0;v<o;v++)l[v]=n[e[v]][c];l[o]=h;var d=i&&i.apply(null,l);if(d!=null){typeof d!="object"&&(a[0]=d,d=a);for(var f=0;f<d.length;f++){var y=e[f],p=d[f],g=u[y],m=n[y];m&&(m[c]=p),p<g[0]&&(g[0]=p),p>g[1]&&(g[1]=p)}}}},r.prototype.lttbDownSample=function(t,e){var i=this.clone([t],!0),n=i._chunks,a=n[t],o=this.count(),s=0,l=Math.floor(1/e),u=this.getRawIndex(0),f,h,c,v=new(on(this._rawCount))(Math.min((Math.ceil(o/l)+2)*2,o));v[s++]=u;for(var d=1;d<o-1;d+=l){for(var y=Math.min(d+l,o-1),p=Math.min(d+l*2,o),g=(p+y)/2,m=0,_=y;_<p;_++){var S=this.getRawIndex(_),b=a[S];isNaN(b)||(m+=b)}m/=p-y;var w=d,x=Math.min(d+l,o),C=d-1,D=a[u];f=-1,c=w;for(var A=-1,M=0,_=w;_<x;_++){var S=this.getRawIndex(_),b=a[S];if(isNaN(b)){M++,A<0&&(A=S);continue}h=Math.abs((C-g)*(b-D)-(C-_)*(m-D)),h>f&&(f=h,c=S)}M>0&&M<x-w&&(v[s++]=Math.min(A,c),c=Math.max(A,c)),v[s++]=c,u=c}return v[s++]=this.getRawIndex(o-1),i._count=s,i._indices=v,i.getRawIndex=this._getRawIdx,i},r.prototype.downSample=function(t,e,i,n){for(var a=this.clone([t],!0),o=a._chunks,s=[],l=Math.floor(1/e),u=o[t],f=this.count(),h=a._rawExtent[t]=xi(),c=new(on(this._rawCount))(Math.ceil(f/l)),v=0,d=0;d<f;d+=l){l>f-d&&(l=f-d,s.length=l);for(var y=0;y<l;y++){var p=this.getRawIndex(d+y);s[y]=u[p]}var g=i(s),m=this.getRawIndex(Math.min(d+n(s,g)||0,f-1));u[m]=g,g<h[0]&&(h[0]=g),g>h[1]&&(h[1]=g),c[v++]=m}return a._count=v,a._indices=c,a._updateGetRawIdx(),a},r.prototype.each=function(t,e){if(this._count)for(var i=t.length,n=this._chunks,a=0,o=this.count();a<o;a++){var s=this.getRawIndex(a);switch(i){case 0:e(a);break;case 1:e(n[t[0]][s],a);break;case 2:e(n[t[0]][s],n[t[1]][s],a);break;default:for(var l=0,u=[];l<i;l++)u[l]=n[t[l]][s];u[l]=a,e.apply(null,u)}}},r.prototype.getDataExtent=function(t){var e=this._chunks[t],i=xi();if(!e)return i;var n=this.count(),a=!this._indices,o;if(a)return this._rawExtent[t].slice();if(o=this._extent[t],o)return o.slice();o=i;for(var s=o[0],l=o[1],u=0;u<n;u++){var f=this.getRawIndex(u),h=e[f];h<s&&(s=h),h>l&&(l=h)}return o=[s,l],this._extent[t]=o,o},r.prototype.getRawDataItem=function(t){var e=this.getRawIndex(t);if(this._provider.persistent)return this._provider.getItem(e);for(var i=[],n=this._chunks,a=0;a<n.length;a++)i.push(n[a][e]);return i},r.prototype.clone=function(t,e){var i=new r,n=this._chunks,a=t&&Ne(t,function(s,l){return s[l]=!0,s},{});if(a)for(var o=0;o<n.length;o++)i._chunks[o]=a[o]?hb(n[o]):n[o];else i._chunks=n;return this._copyCommonProps(i),e||(i._indices=this._cloneIndices()),i._updateGetRawIdx(),i},r.prototype._copyCommonProps=function(t){t._count=this._count,t._rawCount=this._rawCount,t._provider=this._provider,t._dimensions=this._dimensions,t._extent=U(this._extent),t._rawExtent=U(this._rawExtent)},r.prototype._cloneIndices=function(){if(this._indices){var t=this._indices.constructor,e=void 0;if(t===Array){var i=this._indices.length;e=new t(i);for(var n=0;n<i;n++)e[n]=this._indices[n]}else e=new t(this._indices);return e}return null},r.prototype._getRawIdxIdentity=function(t){return t},r.prototype._getRawIdx=function(t){return t<this._count&&t>=0?this._indices[t]:-1},r.prototype._updateGetRawIdx=function(){this.getRawIndex=this._indices?this._getRawIdx:this._getRawIdxIdentity},r.internalField=function(){function t(e,i,n,a){return vo(e[a],this._dimensions[a])}ol={arrayRows:t,objectRows:function(e,i,n,a){return vo(e[i],this._dimensions[a])},keyedColumns:t,original:function(e,i,n,a){var o=e&&(e.value==null?e:e.value);return vo(o instanceof Array?o[a]:o,this._dimensions[a])},typedArray:function(e,i,n,a){return e[a]}}}(),r}(),vb=_t(),cb={float:"f",int:"i",ordinal:"o",number:"n",time:"t"},yy=function(){function r(t){this.dimensions=t.dimensions,this._dimOmitted=t.dimensionOmitted,this.source=t.source,this._fullDimCount=t.fullDimensionCount,this._updateDimOmitted(t.dimensionOmitted)}return r.prototype.isDimensionOmitted=function(){return this._dimOmitted},r.prototype._updateDimOmitted=function(t){this._dimOmitted=t,t&&(this._dimNameMap||(this._dimNameMap=Sy(this.source)))},r.prototype.getSourceDimensionIndex=function(t){return Q(this._dimNameMap.get(t),-1)},r.prototype.getSourceDimension=function(t){var e=this.source.dimensionsDefine;if(e)return e[t]},r.prototype.makeStoreSchema=function(){for(var t=this._fullDimCount,e=fy(this.source),i=!wy(t),n="",a=[],o=0,s=0;o<t;o++){var l=void 0,u=void 0,f=void 0,h=this.dimensions[s];if(h&&h.storeDimIndex===o)l=e?h.name:null,u=h.type,f=h.ordinalMeta,s++;else{var c=this.getSourceDimension(o);c&&(l=e?c.name:null,u=c.type)}a.push({property:l,type:u,ordinalMeta:f}),e&&l!=null&&(!h||!h.isCalculationCoord)&&(n+=i?l.replace(/\`/g,"`1").replace(/\$/g,"`2"):l),n+="$",n+=cb[u]||"f",f&&(n+=f.uid),n+="$"}var v=this.source,d=[v.seriesLayoutBy,v.startIndex,n].join("$$");return{dimensions:a,hash:d}},r.prototype.makeOutputDimensionNames=function(){for(var t=[],e=0,i=0;e<this._fullDimCount;e++){var n=void 0,a=this.dimensions[i];if(a&&a.storeDimIndex===e)a.isCalculationCoord||(n=a.name),i++;else{var o=this.getSourceDimension(e);o&&(n=o.name)}t.push(n)}return t},r.prototype.appendCalculationDimension=function(t){this.dimensions.push(t),t.isCalculationCoord=!0,this._fullDimCount++,this._updateDimOmitted(!0)},r}();function my(r){return r instanceof yy}function _y(r){for(var t=$(),e=0;e<(r||[]).length;e++){var i=r[e],n=z(i)?i.name:i;n!=null&&t.get(n)==null&&t.set(n,e)}return t}function Sy(r){var t=vb(r);return t.dimNameMap||(t.dimNameMap=_y(r.dimensionsDefine))}function wy(r){return r>30}var sn=z,ar=H,db=typeof Int32Array>"u"?Array:Int32Array,pb="e\0\0",Nv=-1,gb=["hasItemOption","_nameList","_idList","_invertedIndicesMap","_dimSummary","userOutput","_rawData","_dimValueGetter","_nameDimIdx","_idDimIdx","_nameRepeatCount"],yb=["_approximateExtent"],Fv,ka,ln,un,sl,Ba,ll,mb=function(){function r(t,e){this.type="list",this._dimOmitted=!1,this._nameList=[],this._idList=[],this._visual={},this._layout={},this._itemVisuals=[],this._itemLayouts=[],this._graphicEls=[],this._approximateExtent={},this._calculationInfo={},this.hasItemOption=!1,this.TRANSFERABLE_METHODS=["cloneShallow","downSample","lttbDownSample","map"],this.CHANGABLE_METHODS=["filterSelf","selectRange"],this.DOWNSAMPLE_METHODS=["downSample","lttbDownSample"];var i,n=!1;my(t)?(i=t.dimensions,this._dimOmitted=t.isDimensionOmitted(),this._schema=t):(n=!0,i=t),i=i||["x","y"];for(var a={},o=[],s={},l=!1,u={},f=0;f<i.length;f++){var h=i[f],c=V(h)?new ho({name:h}):h instanceof ho?h:new ho(h),v=c.name;c.type=c.type||"float",c.coordDim||(c.coordDim=v,c.coordDimIndex=0);var d=c.otherDims=c.otherDims||{};o.push(v),a[v]=c,u[v]!=null&&(l=!0),c.createInvertedIndices&&(s[v]=[]),d.itemName===0&&(this._nameDimIdx=f),d.itemId===0&&(this._idDimIdx=f),n&&(c.storeDimIndex=f)}if(this.dimensions=o,this._dimInfos=a,this._initGetDimensionInfo(l),this.hostModel=e,this._invertedIndicesMap=s,this._dimOmitted){var y=this._dimIdxToName=$();T(o,function(p){y.set(a[p].storeDimIndex,p)})}}return r.prototype.getDimension=function(t){var e=this._recognizeDimIndex(t);if(e==null)return t;if(e=t,!this._dimOmitted)return this.dimensions[e];var i=this._dimIdxToName.get(e);if(i!=null)return i;var n=this._schema.getSourceDimension(e);if(n)return n.name},r.prototype.getDimensionIndex=function(t){var e=this._recognizeDimIndex(t);if(e!=null)return e;if(t==null)return-1;var i=this._getDimInfo(t);return i?i.storeDimIndex:this._dimOmitted?this._schema.getSourceDimensionIndex(t):-1},r.prototype._recognizeDimIndex=function(t){if(ht(t)||t!=null&&!isNaN(t)&&!this._getDimInfo(t)&&(!this._dimOmitted||this._schema.getSourceDimensionIndex(t)<0))return+t},r.prototype._getStoreDimIndex=function(t){var e=this.getDimensionIndex(t);return e},r.prototype.getDimensionInfo=function(t){return this._getDimInfo(this.getDimension(t))},r.prototype._initGetDimensionInfo=function(t){var e=this._dimInfos;this._getDimInfo=t?function(i){return e.hasOwnProperty(i)?e[i]:void 0}:function(i){return e[i]}},r.prototype.getDimensionsOnCoord=function(){return this._dimSummary.dataDimsOnCoord.slice()},r.prototype.mapDimension=function(t,e){var i=this._dimSummary;if(e==null)return i.encodeFirstDimNotExtra[t];var n=i.encode[t];return n?n[e]:null},r.prototype.mapDimensionsAll=function(t){var e=this._dimSummary,i=e.encode[t];return(i||[]).slice()},r.prototype.getStore=function(){return this._store},r.prototype.initData=function(t,e,i){var n=this,a;if(t instanceof Tu&&(a=t),!a){var o=this.dimensions,s=zf(t)||Nt(t)?new hy(t,o.length):t;a=new Tu;var l=ar(o,function(u){return{type:n._dimInfos[u].type,property:u}});a.initData(s,l,i)}this._store=a,this._nameList=(e||[]).slice(),this._idList=[],this._nameRepeatCount={},this._doInit(0,a.count()),this._dimSummary=ab(this,this._schema),this.userOutput=this._dimSummary.userOutput},r.prototype.appendData=function(t){var e=this._store.appendData(t);this._doInit(e[0],e[1])},r.prototype.appendValues=function(t,e){var i=this._store.appendValues(t,e.length),n=i.start,a=i.end,o=this._shouldMakeIdFromName();if(this._updateOrdinalMeta(),e)for(var s=n;s<a;s++){var l=s-n;this._nameList[s]=e[l],o&&ll(this,s)}},r.prototype._updateOrdinalMeta=function(){for(var t=this._store,e=this.dimensions,i=0;i<e.length;i++){var n=this._dimInfos[e[i]];n.ordinalMeta&&t.collectOrdinalMeta(n.storeDimIndex,n.ordinalMeta)}},r.prototype._shouldMakeIdFromName=function(){var t=this._store.getProvider();return this._idDimIdx==null&&t.getSource().sourceFormat!==wr&&!t.fillStorage},r.prototype._doInit=function(t,e){if(!(t>=e)){var i=this._store,n=i.getProvider();this._updateOrdinalMeta();var a=this._nameList,o=this._idList,s=n.getSource().sourceFormat,l=s===ge;if(l&&!n.pure)for(var u=[],f=t;f<e;f++){var h=n.getItem(f,u);if(!this.hasItemOption&&pS(h)&&(this.hasItemOption=!0),h){var c=h.name;a[f]==null&&c!=null&&(a[f]=Me(c,null));var v=h.id;o[f]==null&&v!=null&&(o[f]=Me(v,null))}}if(this._shouldMakeIdFromName())for(var f=t;f<e;f++)ll(this,f);Fv(this)}},r.prototype.getApproximateExtent=function(t){return this._approximateExtent[t]||this._store.getDataExtent(this._getStoreDimIndex(t))},r.prototype.setApproximateExtent=function(t,e){e=this.getDimension(e),this._approximateExtent[e]=t.slice()},r.prototype.getCalculationInfo=function(t){return this._calculationInfo[t]},r.prototype.setCalculationInfo=function(t,e){sn(t)?O(this._calculationInfo,t):this._calculationInfo[t]=e},r.prototype.getName=function(t){var e=this.getRawIndex(t),i=this._nameList[e];return i==null&&this._nameDimIdx!=null&&(i=ln(this,this._nameDimIdx,e)),i==null&&(i=""),i},r.prototype._getCategory=function(t,e){var i=this._store.get(t,e),n=this._store.getOrdinalMeta(t);return n?n.categories[i]:i},r.prototype.getId=function(t){return ka(this,this.getRawIndex(t))},r.prototype.count=function(){return this._store.count()},r.prototype.get=function(t,e){var i=this._store,n=this._dimInfos[t];if(n)return i.get(n.storeDimIndex,e)},r.prototype.getByRawIndex=function(t,e){var i=this._store,n=this._dimInfos[t];if(n)return i.getByRawIndex(n.storeDimIndex,e)},r.prototype.getIndices=function(){return this._store.getIndices()},r.prototype.getDataExtent=function(t){return this._store.getDataExtent(this._getStoreDimIndex(t))},r.prototype.getSum=function(t){return this._store.getSum(this._getStoreDimIndex(t))},r.prototype.getMedian=function(t){return this._store.getMedian(this._getStoreDimIndex(t))},r.prototype.getValues=function(t,e){var i=this,n=this._store;return N(t)?n.getValues(ar(t,function(a){return i._getStoreDimIndex(a)}),e):n.getValues(t)},r.prototype.hasValue=function(t){for(var e=this._dimSummary.dataDimIndicesOnCoord,i=0,n=e.length;i<n;i++)if(isNaN(this._store.get(e[i],t)))return!1;return!0},r.prototype.indexOfName=function(t){for(var e=0,i=this._store.count();e<i;e++)if(this.getName(e)===t)return e;return-1},r.prototype.getRawIndex=function(t){return this._store.getRawIndex(t)},r.prototype.indexOfRawIndex=function(t){return this._store.indexOfRawIndex(t)},r.prototype.rawIndexOf=function(t,e){var i=t&&this._invertedIndicesMap[t],n=i[e];return n==null||isNaN(n)?Nv:n},r.prototype.indicesOfNearest=function(t,e,i){return this._store.indicesOfNearest(this._getStoreDimIndex(t),e,i)},r.prototype.each=function(t,e,i){Y(t)&&(i=e,e=t,t=[]);var n=i||this,a=ar(un(t),this._getStoreDimIndex,this);this._store.each(a,n?K(e,n):e)},r.prototype.filterSelf=function(t,e,i){Y(t)&&(i=e,e=t,t=[]);var n=i||this,a=ar(un(t),this._getStoreDimIndex,this);return this._store=this._store.filter(a,n?K(e,n):e),this},r.prototype.selectRange=function(t){var e=this,i={},n=ft(t);return T(n,function(a){var o=e._getStoreDimIndex(a);i[o]=t[a]}),this._store=this._store.selectRange(i),this},r.prototype.mapArray=function(t,e,i){Y(t)&&(i=e,e=t,t=[]),i=i||this;var n=[];return this.each(t,function(){n.push(e&&e.apply(this,arguments))},i),n},r.prototype.map=function(t,e,i,n){var a=i||n||this,o=ar(un(t),this._getStoreDimIndex,this),s=Ba(this);return s._store=this._store.map(o,a?K(e,a):e),s},r.prototype.modify=function(t,e,i,n){var a=i||n||this,o=ar(un(t),this._getStoreDimIndex,this);this._store.modify(o,a?K(e,a):e)},r.prototype.downSample=function(t,e,i,n){var a=Ba(this);return a._store=this._store.downSample(this._getStoreDimIndex(t),e,i,n),a},r.prototype.lttbDownSample=function(t,e){var i=Ba(this);return i._store=this._store.lttbDownSample(this._getStoreDimIndex(t),e),i},r.prototype.getRawDataItem=function(t){return this._store.getRawDataItem(t)},r.prototype.getItemModel=function(t){var e=this.hostModel,i=this.getRawDataItem(t);return new xt(i,e,e&&e.ecModel)},r.prototype.diff=function(t){var e=this;return new $w(t?t.getStore().getIndices():[],this.getStore().getIndices(),function(i){return ka(t,i)},function(i){return ka(e,i)})},r.prototype.getVisual=function(t){var e=this._visual;return e&&e[t]},r.prototype.setVisual=function(t,e){this._visual=this._visual||{},sn(t)?O(this._visual,t):this._visual[t]=e},r.prototype.getItemVisual=function(t,e){var i=this._itemVisuals[t],n=i&&i[e];return n??this.getVisual(e)},r.prototype.hasItemVisual=function(){return this._itemVisuals.length>0},r.prototype.ensureUniqueItemVisual=function(t,e){var i=this._itemVisuals,n=i[t];n||(n=i[t]={});var a=n[e];return a==null&&(a=this.getVisual(e),N(a)?a=a.slice():sn(a)&&(a=O({},a)),n[e]=a),a},r.prototype.setItemVisual=function(t,e,i){var n=this._itemVisuals[t]||{};this._itemVisuals[t]=n,sn(e)?O(n,e):n[e]=i},r.prototype.clearAllVisual=function(){this._visual={},this._itemVisuals=[]},r.prototype.setLayout=function(t,e){sn(t)?O(this._layout,t):this._layout[t]=e},r.prototype.getLayout=function(t){return this._layout[t]},r.prototype.getItemLayout=function(t){return this._itemLayouts[t]},r.prototype.setItemLayout=function(t,e,i){this._itemLayouts[t]=i?O(this._itemLayouts[t]||{},e):e},r.prototype.clearItemLayouts=function(){this._itemLayouts.length=0},r.prototype.setItemGraphicEl=function(t,e){var i=this.hostModel&&this.hostModel.seriesIndex;LS(i,this.dataType,t,e),this._graphicEls[t]=e},r.prototype.getItemGraphicEl=function(t){return this._graphicEls[t]},r.prototype.eachItemGraphicEl=function(t,e){T(this._graphicEls,function(i,n){i&&t&&t.call(e,i,n)})},r.prototype.cloneShallow=function(t){return t||(t=new r(this._schema?this._schema:ar(this.dimensions,this._getDimInfo,this),this.hostModel)),sl(t,this),t._store=this._store,t},r.prototype.wrapMethod=function(t,e){var i=this[t];Y(i)&&(this.__wrappedMethods=this.__wrappedMethods||[],this.__wrappedMethods.push(t),this[t]=function(){var n=i.apply(this,arguments);return e.apply(this,[n].concat(jo(arguments)))})},r.internalField=function(){Fv=function(t){var e=t._invertedIndicesMap;T(e,function(i,n){var a=t._dimInfos[n],o=a.ordinalMeta,s=t._store;if(o){i=e[n]=new db(o.categories.length);for(var l=0;l<i.length;l++)i[l]=Nv;for(var l=0;l<s.count();l++)i[s.get(a.storeDimIndex,l)]=l}})},ln=function(t,e,i){return Me(t._getCategory(e,i),null)},ka=function(t,e){var i=t._idList[e];return i==null&&t._idDimIdx!=null&&(i=ln(t,t._idDimIdx,e)),i==null&&(i=pb+e),i},un=function(t){return N(t)||(t=t!=null?[t]:[]),t},Ba=function(t){var e=new r(t._schema?t._schema:ar(t.dimensions,t._getDimInfo,t),t.hostModel);return sl(e,t),e},sl=function(t,e){T(gb.concat(e.__wrappedMethods||[]),function(i){e.hasOwnProperty(i)&&(t[i]=e[i])}),t.__wrappedMethods=e.__wrappedMethods,T(yb,function(i){t[i]=U(e[i])}),t._calculationInfo=O({},e._calculationInfo)},ll=function(t,e){var i=t._nameList,n=t._idList,a=t._nameDimIdx,o=t._idDimIdx,s=i[e],l=n[e];if(s==null&&a!=null&&(i[e]=s=ln(t,a,e)),l==null&&o!=null&&(n[e]=l=ln(t,o,e)),l==null&&s!=null){var u=t._nameRepeatCount,f=u[s]=(u[s]||0)+1;l=s,f>1&&(l+="__ec__"+f),n[e]=l}}}(),r}();const _b=mb;function Sb(r,t){return by(r,t).dimensions}function by(r,t){zf(r)||(r=ly(r)),t=t||{};var e=t.coordDimensions||[],i=t.dimensionsDefine||r.dimensionsDefine||[],n=$(),a=[],o=bb(r,e,i,t.dimensionsCount),s=t.canOmitUnusedDimensions&&wy(o),l=i===r.dimensionsDefine,u=l?Sy(r):_y(i),f=t.encodeDefine;!f&&t.encodeDefaulter&&(f=t.encodeDefaulter(r,o));for(var h=$(f),c=new py(o),v=0;v<c.length;v++)c[v]=-1;function d(D){var A=c[D];if(A<0){var M=i[D],I=z(M)?M:{name:M},L=new ho,P=I.name;P!=null&&u.get(P)!=null&&(L.name=L.displayName=P),I.type!=null&&(L.type=I.type),I.displayName!=null&&(L.displayName=I.displayName);var R=a.length;return c[D]=R,L.storeDimIndex=D,a.push(L),L}return a[A]}if(!s)for(var v=0;v<o;v++)d(v);h.each(function(D,A){var M=Tt(D).slice();if(M.length===1&&!V(M[0])&&M[0]<0){h.set(A,!1);return}var I=h.set(A,[]);T(M,function(L,P){var R=V(L)?u.get(L):L;R!=null&&R<o&&(I[P]=R,p(d(R),A,P))})});var y=0;T(e,function(D){var A,M,I,L;if(V(D))A=D,L={};else{L=D,A=L.name;var P=L.ordinalMeta;L.ordinalMeta=null,L=O({},L),L.ordinalMeta=P,M=L.dimsDef,I=L.otherDims,L.name=L.coordDim=L.coordDimIndex=L.dimsDef=L.otherDims=null}var R=h.get(A);if(R!==!1){if(R=Tt(R),!R.length)for(var E=0;E<(M&&M.length||1);E++){for(;y<o&&d(y).coordDim!=null;)y++;y<o&&R.push(y++)}T(R,function(W,B){var F=d(W);if(l&&L.type!=null&&(F.type=L.type),p(nt(F,L),A,B),F.name==null&&M){var G=M[B];!z(G)&&(G={name:G}),F.name=F.displayName=G.name,F.defaultTooltip=G.defaultTooltip}I&&nt(F.otherDims,I)})}});function p(D,A,M){iy.get(A)!=null?D.otherDims[A]=M:(D.coordDim=A,D.coordDimIndex=M,n.set(A,!0))}var g=t.generateCoord,m=t.generateCoordCount,_=m!=null;m=g?m||1:0;var S=g||"value";function b(D){D.name==null&&(D.name=D.coordDim)}if(s)T(a,function(D){b(D)}),a.sort(function(D,A){return D.storeDimIndex-A.storeDimIndex});else for(var w=0;w<o;w++){var x=d(w),C=x.coordDim;C==null&&(x.coordDim=xb(S,n,_),x.coordDimIndex=0,(!g||m<=0)&&(x.isExtraCoord=!0),m--),b(x),x.type==null&&(sy(r,w)===qt.Must||x.isExtraCoord&&(x.otherDims.itemName!=null||x.otherDims.seriesName!=null))&&(x.type="ordinal")}return wb(a),new yy({source:r,dimensions:a,fullDimensionCount:o,dimensionOmitted:s})}function wb(r){for(var t=$(),e=0;e<r.length;e++){var i=r[e],n=i.name,a=t.get(n)||0;a>0&&(i.name=n+(a-1)),a++,t.set(n,a)}}function bb(r,t,e,i){var n=Math.max(r.dimensionsDetectedCount||1,t.length,e.length,i||0);return T(t,function(a){var o;z(a)&&(o=a.dimsDef)&&(n=Math.max(n,o.length))}),n}function xb(r,t,e){if(e||t.hasKey(r)){for(var i=0;t.hasKey(r+i);)i++;r+=i}return t.set(r,!0),r}var ul={},_s=function(){function r(){this._coordinateSystems=[]}return r.prototype.create=function(t,e){var i=[];T(ul,function(n,a){var o=n.create(t,e);i=i.concat(o||[])}),this._coordinateSystems=i},r.prototype.update=function(t,e){T(this._coordinateSystems,function(i){i.update&&i.update(t,e)})},r.prototype.getCoordinateSystems=function(){return this._coordinateSystems.slice()},r.register=function(t,e){ul[t]=e},r.get=function(t){return ul[t]},r}(),Tb=function(){function r(t){this.coordSysDims=[],this.axisMap=$(),this.categoryAxisMap=$(),this.coordSysName=t}return r}();function Mb(r){var t=r.get("coordinateSystem"),e=new Tb(t),i=Cb[t];if(i)return i(r,e,e.axisMap,e.categoryAxisMap),e}var Cb={cartesian2d:function(r,t,e,i){var n=r.getReferringComponents("xAxis",xe).models[0],a=r.getReferringComponents("yAxis",xe).models[0];t.coordSysDims=["x","y"],e.set("x",n),e.set("y",a),Ti(n)&&(i.set("x",n),t.firstCategoryDimIndex=0),Ti(a)&&(i.set("y",a),t.firstCategoryDimIndex==null&&(t.firstCategoryDimIndex=1))},singleAxis:function(r,t,e,i){var n=r.getReferringComponents("singleAxis",xe).models[0];t.coordSysDims=["single"],e.set("single",n),Ti(n)&&(i.set("single",n),t.firstCategoryDimIndex=0)},polar:function(r,t,e,i){var n=r.getReferringComponents("polar",xe).models[0],a=n.findAxisModel("radiusAxis"),o=n.findAxisModel("angleAxis");t.coordSysDims=["radius","angle"],e.set("radius",a),e.set("angle",o),Ti(a)&&(i.set("radius",a),t.firstCategoryDimIndex=0),Ti(o)&&(i.set("angle",o),t.firstCategoryDimIndex==null&&(t.firstCategoryDimIndex=1))},geo:function(r,t,e,i){t.coordSysDims=["lng","lat"]},parallel:function(r,t,e,i){var n=r.ecModel,a=n.getComponent("parallel",r.get("parallelIndex")),o=t.coordSysDims=a.dimensions.slice();T(a.parallelAxisIndex,function(s,l){var u=n.getComponent("parallelAxis",s),f=o[l];e.set(f,u),Ti(u)&&(i.set(f,u),t.firstCategoryDimIndex==null&&(t.firstCategoryDimIndex=l))})}};function Ti(r){return r.get("type")==="category"}function xy(r,t,e){e=e||{};var i=e.byIndex,n=e.stackedCoordDimension,a,o,s;Db(t)?a=t:(o=t.schema,a=o.dimensions,s=t.store);var l=!!(r&&r.get("stack")),u,f,h,c;if(T(a,function(m,_){V(m)&&(a[_]=m={name:m}),l&&!m.isExtraCoord&&(!i&&!u&&m.ordinalMeta&&(u=m),!f&&m.type!=="ordinal"&&m.type!=="time"&&(!n||n===m.coordDim)&&(f=m))}),f&&!i&&!u&&(i=!0),f){h="__\0ecstackresult_"+r.id,c="__\0ecstackedover_"+r.id,u&&(u.createInvertedIndices=!0);var v=f.coordDim,d=f.type,y=0;T(a,function(m){m.coordDim===v&&y++});var p={name:h,coordDim:v,coordDimIndex:y,type:d,isExtraCoord:!0,isCalculationCoord:!0,storeDimIndex:a.length},g={name:c,coordDim:c,coordDimIndex:y+1,type:d,isExtraCoord:!0,isCalculationCoord:!0,storeDimIndex:a.length+1};o?(s&&(p.storeDimIndex=s.ensureCalculationDimension(c,d),g.storeDimIndex=s.ensureCalculationDimension(h,d)),o.appendCalculationDimension(p),o.appendCalculationDimension(g)):(a.push(p),a.push(g))}return{stackedDimension:f&&f.name,stackedByDimension:u&&u.name,isStackedByIndex:i,stackedOverDimension:c,stackResultDimension:h}}function Db(r){return!my(r.schema)}function fi(r,t){return!!t&&t===r.getCalculationInfo("stackedDimension")}function Ty(r,t){return fi(r,t)?r.getCalculationInfo("stackResultDimension"):t}function Ab(r,t){var e=r.get("coordinateSystem"),i=_s.get(e),n;return t&&t.coordSysDims&&(n=H(t.coordSysDims,function(a){var o={name:a},s=t.axisMap.get(a);if(s){var l=s.get("type");o.type=ob(l)}return o})),n||(n=i&&(i.getDimensionsInfo?i.getDimensionsInfo():i.dimensions.slice())||["x","y"]),n}function Ib(r,t,e){var i,n;return e&&T(r,function(a,o){var s=a.coordDim,l=e.categoryAxisMap.get(s);l&&(i==null&&(i=o),a.ordinalMeta=l.getOrdinalMeta(),t&&(a.createInvertedIndices=!0)),a.otherDims.itemName!=null&&(n=!0)}),!n&&i!=null&&(r[i].otherDims.itemName=0),i}function Ss(r,t,e){e=e||{};var i=t.getSourceManager(),n,a=!1;n=i.getSource(),a=n.sourceFormat===ge;var o=Mb(t),s=Ab(t,o),l=e.useEncodeDefaulter,u=Y(l)?l:l?kt(qw,s,t):null,f={coordDimensions:s,generateCoord:e.generateCoord,encodeDefine:t.getEncode(),encodeDefaulter:u,canOmitUnusedDimensions:!a},h=by(n,f),c=Ib(h.dimensions,e.createInvertedIndices,o),v=a?null:i.getSharedDataStore(h),d=xy(t,{schema:h,store:v}),y=new _b(h,t);y.setCalculationInfo(d);var p=c!=null&&Lb(n)?function(g,m,_,S){return S===c?_:this.defaultDimValueGetter(g,m,_,S)}:null;return y.hasItemOption=!1,y.initData(a?n:v,null,p),y}function Lb(r){if(r.sourceFormat===ge){var t=Pb(r.data||[]);return!N(ca(t))}}function Pb(r){for(var t=0;t<r.length&&r[t]==null;)t++;return r[t]}var Rb=Math.round(Math.random()*10);function ws(r){return[r||"",Rb++].join("_")}function Eb(r){var t={};r.registerSubTypeDefaulter=function(e,i){var n=Ee(e);t[n.main]=i},r.determineSubType=function(e,i){var n=i.type;if(!n){var a=Ee(e).main;r.hasSubTypes(e)&&t[a]&&(n=t[a](i))}return n}}function Ob(r,t){r.topologicalTravel=function(a,o,s,l){if(!a.length)return;var u=e(o),f=u.graph,h=u.noEntryList,c={};for(T(a,function(m){c[m]=!0});h.length;){var v=h.pop(),d=f[v],y=!!c[v];y&&(s.call(l,v,d.originalDeps.slice()),delete c[v]),T(d.successor,y?g:p)}T(c,function(){var m="";throw new Error(m)});function p(m){f[m].entryCount--,f[m].entryCount===0&&h.push(m)}function g(m){c[m]=!0,p(m)}};function e(a){var o={},s=[];return T(a,function(l){var u=i(o,l),f=u.originalDeps=t(l),h=n(f,a);u.entryCount=h.length,u.entryCount===0&&s.push(l),T(h,function(c){ot(u.predecessor,c)<0&&u.predecessor.push(c);var v=i(o,c);ot(v.successor,c)<0&&v.successor.push(l)})}),{graph:o,noEntryList:s}}function i(a,o){return a[o]||(a[o]={predecessor:[],successor:[]}),a[o]}function n(a,o){var s=[];return T(a,function(l){ot(o,l)>=0&&s.push(l)}),s}}function Hf(r,t){return J(J({},r,!0),t,!0)}var kb=Math.log(2);function Mu(r,t,e,i,n,a){var o=i+"-"+n,s=r.length;if(a.hasOwnProperty(o))return a[o];if(t===1){var l=Math.round(Math.log((1<<s)-1&~n)/kb);return r[e][l]}for(var u=i|1<<e,f=e+1;i&1<<f;)f++;for(var h=0,c=0,v=0;c<s;c++){var d=1<<c;d&n||(h+=(v%2?-1:1)*r[e][c]*Mu(r,t-1,f,u,n|d,a),v++)}return a[o]=h,h}function zv(r,t){var e=[[r[0],r[1],1,0,0,0,-t[0]*r[0],-t[0]*r[1]],[0,0,0,r[0],r[1],1,-t[1]*r[0],-t[1]*r[1]],[r[2],r[3],1,0,0,0,-t[2]*r[2],-t[2]*r[3]],[0,0,0,r[2],r[3],1,-t[3]*r[2],-t[3]*r[3]],[r[4],r[5],1,0,0,0,-t[4]*r[4],-t[4]*r[5]],[0,0,0,r[4],r[5],1,-t[5]*r[4],-t[5]*r[5]],[r[6],r[7],1,0,0,0,-t[6]*r[6],-t[6]*r[7]],[0,0,0,r[6],r[7],1,-t[7]*r[6],-t[7]*r[7]]],i={},n=Mu(e,8,0,0,0,i);if(n!==0){for(var a=[],o=0;o<8;o++)for(var s=0;s<8;s++)a[s]==null&&(a[s]=0),a[s]+=((o+s)%2?-1:1)*Mu(e,7,o===0?1:0,1<<o,1<<s,i)/n*t[o];return function(l,u,f){var h=u*a[6]+f*a[7]+1;l[0]=(u*a[0]+f*a[1]+a[2])/h,l[1]=(u*a[3]+f*a[4]+a[5])/h}}}var Vv="___zrEVENTSAVED",fl=[];function Bb(r,t,e,i,n){return Cu(fl,t,i,n,!0)&&Cu(r,e,fl[0],fl[1])}function Cu(r,t,e,i,n){if(t.getBoundingClientRect&&Z.domSupported&&!My(t)){var a=t[Vv]||(t[Vv]={}),o=Nb(t,a),s=Fb(o,a,n);if(s)return s(r,e,i),!0}return!1}function Nb(r,t){var e=t.markers;if(e)return e;e=t.markers=[];for(var i=["left","right"],n=["top","bottom"],a=0;a<4;a++){var o=document.createElement("div"),s=o.style,l=a%2,u=(a>>1)%2;s.cssText=["position: absolute","visibility: hidden","padding: 0","margin: 0","border-width: 0","user-select: none","width:0","height:0",i[l]+":0",n[u]+":0",i[1-l]+":auto",n[1-u]+":auto",""].join("!important;"),r.appendChild(o),e.push(o)}return e}function Fb(r,t,e){for(var i=e?"invTrans":"trans",n=t[i],a=t.srcCoords,o=[],s=[],l=!0,u=0;u<4;u++){var f=r[u].getBoundingClientRect(),h=2*u,c=f.left,v=f.top;o.push(c,v),l=l&&a&&c===a[h]&&v===a[h+1],s.push(r[u].offsetLeft,r[u].offsetTop)}return l&&n?n:(t.srcCoords=o,t[i]=e?zv(s,o):zv(o,s))}function My(r){return r.nodeName.toUpperCase()==="CANVAS"}var zb=/([&<>"'])/g,Vb={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"};function Wt(r){return r==null?"":(r+"").replace(zb,function(t,e){return Vb[e]})}const Hb={time:{month:["January","February","March","April","May","June","July","August","September","October","November","December"],monthAbbr:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayOfWeek:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayOfWeekAbbr:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"]},legend:{selector:{all:"All",inverse:"Inv"}},toolbox:{brush:{title:{rect:"Box Select",polygon:"Lasso Select",lineX:"Horizontally Select",lineY:"Vertically Select",keep:"Keep Selections",clear:"Clear Selections"}},dataView:{title:"Data View",lang:["Data View","Close","Refresh"]},dataZoom:{title:{zoom:"Zoom",back:"Zoom Reset"}},magicType:{title:{line:"Switch to Line Chart",bar:"Switch to Bar Chart",stack:"Stack",tiled:"Tile"}},restore:{title:"Restore"},saveAsImage:{title:"Save as Image",lang:["Right Click to Save Image"]}},series:{typeNames:{pie:"Pie chart",bar:"Bar chart",line:"Line chart",scatter:"Scatter plot",effectScatter:"Ripple scatter plot",radar:"Radar chart",tree:"Tree",treemap:"Treemap",boxplot:"Boxplot",candlestick:"Candlestick",k:"K line chart",heatmap:"Heat map",map:"Map",parallel:"Parallel coordinate map",lines:"Line graph",graph:"Relationship graph",sankey:"Sankey diagram",funnel:"Funnel chart",gauge:"Gauge",pictorialBar:"Pictorial bar",themeRiver:"Theme River Map",sunburst:"Sunburst",custom:"Custom chart",chart:"Chart"}},aria:{general:{withTitle:'This is a chart about "{title}"',withoutTitle:"This is a chart"},series:{single:{prefix:"",withName:" with type {seriesType} named {seriesName}.",withoutName:" with type {seriesType}."},multiple:{prefix:". It consists of {seriesCount} series count.",withName:" The {seriesId} series is a {seriesType} representing {seriesName}.",withoutName:" The {seriesId} series is a {seriesType}.",separator:{middle:"",end:""}}},data:{allData:"The data is as follows: ",partialData:"The first {displayCnt} items are: ",withName:"the data for {name} is {value}",withoutName:"{value}",separator:{middle:", ",end:". "}}}},Gb={time:{month:["一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"],monthAbbr:["1月","2月","3月","4月","5月","6月","7月","8月","9月","10月","11月","12月"],dayOfWeek:["星期日","星期一","星期二","星期三","星期四","星期五","星期六"],dayOfWeekAbbr:["日","一","二","三","四","五","六"]},legend:{selector:{all:"全选",inverse:"反选"}},toolbox:{brush:{title:{rect:"矩形选择",polygon:"圈选",lineX:"横向选择",lineY:"纵向选择",keep:"保持选择",clear:"清除选择"}},dataView:{title:"数据视图",lang:["数据视图","关闭","刷新"]},dataZoom:{title:{zoom:"区域缩放",back:"区域缩放还原"}},magicType:{title:{line:"切换为折线图",bar:"切换为柱状图",stack:"切换为堆叠",tiled:"切换为平铺"}},restore:{title:"还原"},saveAsImage:{title:"保存为图片",lang:["右键另存为图片"]}},series:{typeNames:{pie:"饼图",bar:"柱状图",line:"折线图",scatter:"散点图",effectScatter:"涟漪散点图",radar:"雷达图",tree:"树图",treemap:"矩形树图",boxplot:"箱型图",candlestick:"K线图",k:"K线图",heatmap:"热力图",map:"地图",parallel:"平行坐标图",lines:"线图",graph:"关系图",sankey:"桑基图",funnel:"漏斗图",gauge:"仪表盘图",pictorialBar:"象形柱图",themeRiver:"主题河流图",sunburst:"旭日图",custom:"自定义图表",chart:"图表"}},aria:{general:{withTitle:"这是一个关于“{title}”的图表。",withoutTitle:"这是一个图表，"},series:{single:{prefix:"",withName:"图表类型是{seriesType}，表示{seriesName}。",withoutName:"图表类型是{seriesType}。"},multiple:{prefix:"它由{seriesCount}个图表系列组成。",withName:"第{seriesId}个系列是一个表示{seriesName}的{seriesType}，",withoutName:"第{seriesId}个系列是一个{seriesType}，",separator:{middle:"；",end:"。"}}},data:{allData:"其数据是——",partialData:"其中，前{displayCnt}项是——",withName:"{name}的数据是{value}",withoutName:"{value}",separator:{middle:"，",end:""}}}};var Fo="ZH",Gf="EN",zi=Gf,co={},Wf={},Cy=Z.domSupported?function(){var r=(document.documentElement.lang||navigator.language||navigator.browserLanguage||zi).toUpperCase();return r.indexOf(Fo)>-1?Fo:zi}():zi;function Dy(r,t){r=r.toUpperCase(),Wf[r]=new xt(t),co[r]=t}function Wb(r){if(V(r)){var t=co[r.toUpperCase()]||{};return r===Fo||r===Gf?U(t):J(U(t),U(co[zi]),!1)}else return J(U(r),U(co[zi]),!1)}function Ub(r){return Wf[r]}function Yb(){return Wf[zi]}Dy(Gf,Hb);Dy(Fo,Gb);var Uf=1e3,Yf=Uf*60,kn=Yf*60,fe=kn*24,Hv=fe*365,xn={year:"{yyyy}",month:"{MMM}",day:"{d}",hour:"{HH}:{mm}",minute:"{HH}:{mm}",second:"{HH}:{mm}:{ss}",millisecond:"{HH}:{mm}:{ss} {SSS}",none:"{yyyy}-{MM}-{dd} {HH}:{mm}:{ss} {SSS}"},Na="{yyyy}-{MM}-{dd}",Gv={year:"{yyyy}",month:"{yyyy}-{MM}",day:Na,hour:Na+" "+xn.hour,minute:Na+" "+xn.minute,second:Na+" "+xn.second,millisecond:xn.none},hl=["year","month","day","hour","minute","second","millisecond"],Ay=["year","half-year","quarter","month","week","half-week","day","half-day","quarter-day","hour","minute","second","millisecond"];function Et(r,t){return r+="","0000".substr(0,t-r.length)+r}function Vi(r){switch(r){case"half-year":case"quarter":return"month";case"week":case"half-week":return"day";case"half-day":case"quarter-day":return"hour";default:return r}}function Xb(r){return r===Vi(r)}function $b(r){switch(r){case"year":case"month":return"day";case"millisecond":return"millisecond";default:return"second"}}function pa(r,t,e,i){var n=ve(r),a=n[Xf(e)](),o=n[Hi(e)]()+1,s=Math.floor((o-1)/3)+1,l=n[bs(e)](),u=n["get"+(e?"UTC":"")+"Day"](),f=n[Kn(e)](),h=(f-1)%12+1,c=n[xs(e)](),v=n[Ts(e)](),d=n[Ms(e)](),y=f>=12?"pm":"am",p=y.toUpperCase(),g=i instanceof xt?i:Ub(i||Cy)||Yb(),m=g.getModel("time"),_=m.get("month"),S=m.get("monthAbbr"),b=m.get("dayOfWeek"),w=m.get("dayOfWeekAbbr");return(t||"").replace(/{a}/g,y+"").replace(/{A}/g,p+"").replace(/{yyyy}/g,a+"").replace(/{yy}/g,Et(a%100+"",2)).replace(/{Q}/g,s+"").replace(/{MMMM}/g,_[o-1]).replace(/{MMM}/g,S[o-1]).replace(/{MM}/g,Et(o,2)).replace(/{M}/g,o+"").replace(/{dd}/g,Et(l,2)).replace(/{d}/g,l+"").replace(/{eeee}/g,b[u]).replace(/{ee}/g,w[u]).replace(/{e}/g,u+"").replace(/{HH}/g,Et(f,2)).replace(/{H}/g,f+"").replace(/{hh}/g,Et(h+"",2)).replace(/{h}/g,h+"").replace(/{mm}/g,Et(c,2)).replace(/{m}/g,c+"").replace(/{ss}/g,Et(v,2)).replace(/{s}/g,v+"").replace(/{SSS}/g,Et(d,3)).replace(/{S}/g,d+"")}function Zb(r,t,e,i,n){var a=null;if(V(e))a=e;else if(Y(e))a=e(r.value,t,{level:r.level});else{var o=O({},xn);if(r.level>0)for(var s=0;s<hl.length;++s)o[hl[s]]="{primary|"+o[hl[s]]+"}";var l=e?e.inherit===!1?e:nt(e,o):o,u=Iy(r.value,n);if(l[u])a=l[u];else if(l.inherit){for(var f=Ay.indexOf(u),s=f-1;s>=0;--s)if(l[u]){a=l[u];break}a=a||o.none}if(N(a)){var h=r.level==null?0:r.level>=0?r.level:a.length+r.level;h=Math.min(h,a.length-1),a=a[h]}}return pa(new Date(r.value),a,n,i)}function Iy(r,t){var e=ve(r),i=e[Hi(t)]()+1,n=e[bs(t)](),a=e[Kn(t)](),o=e[xs(t)](),s=e[Ts(t)](),l=e[Ms(t)](),u=l===0,f=u&&s===0,h=f&&o===0,c=h&&a===0,v=c&&n===1,d=v&&i===1;return d?"year":v?"month":c?"day":h?"hour":f?"minute":u?"second":"millisecond"}function Wv(r,t,e){var i=ht(r)?ve(r):r;switch(t=t||Iy(r,e),t){case"year":return i[Xf(e)]();case"half-year":return i[Hi(e)]()>=6?1:0;case"quarter":return Math.floor((i[Hi(e)]()+1)/4);case"month":return i[Hi(e)]();case"day":return i[bs(e)]();case"half-day":return i[Kn(e)]()/24;case"hour":return i[Kn(e)]();case"minute":return i[xs(e)]();case"second":return i[Ts(e)]();case"millisecond":return i[Ms(e)]()}}function Xf(r){return r?"getUTCFullYear":"getFullYear"}function Hi(r){return r?"getUTCMonth":"getMonth"}function bs(r){return r?"getUTCDate":"getDate"}function Kn(r){return r?"getUTCHours":"getHours"}function xs(r){return r?"getUTCMinutes":"getMinutes"}function Ts(r){return r?"getUTCSeconds":"getSeconds"}function Ms(r){return r?"getUTCMilliseconds":"getMilliseconds"}function qb(r){return r?"setUTCFullYear":"setFullYear"}function Ly(r){return r?"setUTCMonth":"setMonth"}function Py(r){return r?"setUTCDate":"setDate"}function Ry(r){return r?"setUTCHours":"setHours"}function Ey(r){return r?"setUTCMinutes":"setMinutes"}function Oy(r){return r?"setUTCSeconds":"setSeconds"}function ky(r){return r?"setUTCMilliseconds":"setMilliseconds"}function Kb(r,t,e,i,n,a,o,s){var l=new yt({style:{text:r,font:t,align:e,verticalAlign:i,padding:n,rich:a,overflow:o?"truncate":null,lineHeight:s}});return l.getBoundingRect()}function $f(r){if(!sg(r))return V(r)?r:"-";var t=(r+"").split(".");return t[0].replace(/(\d{1,3})(?=(?:\d{3})+(?!\d))/g,"$1,")+(t.length>1?"."+t[1]:"")}function Zf(r,t){return r=(r||"").toLowerCase().replace(/-(.)/g,function(e,i){return i.toUpperCase()}),t&&r&&(r=r.charAt(0).toUpperCase()+r.slice(1)),r}var ga=ff;function Du(r,t,e){var i="{yyyy}-{MM}-{dd} {HH}:{mm}:{ss}";function n(f){return f&&be(f)?f:"-"}function a(f){return!!(f!=null&&!isNaN(f)&&isFinite(f))}var o=t==="time",s=r instanceof Date;if(o||s){var l=o?ve(r):r;if(isNaN(+l)){if(s)return"-"}else return pa(l,i,e)}if(t==="ordinal")return wo(r)?n(r):ht(r)&&a(r)?r+"":"-";var u=$n(r);return a(u)?$f(u):wo(r)?n(r):typeof r=="boolean"?r+"":"-"}var Uv=["a","b","c","d","e","f","g"],vl=function(r,t){return"{"+r+(t??"")+"}"};function qf(r,t,e){N(t)||(t=[t]);var i=t.length;if(!i)return"";for(var n=t[0].$vars||[],a=0;a<n.length;a++){var o=Uv[a];r=r.replace(vl(o),vl(o,0))}for(var s=0;s<i;s++)for(var l=0;l<n.length;l++){var u=t[s][n[l]];r=r.replace(vl(Uv[l],s),e?Wt(u):u)}return r}function By(r,t){var e=V(r)?{color:r,extraCssText:t}:r||{},i=e.color,n=e.type;t=e.extraCssText;var a=e.renderMode||"html";if(!i)return"";if(a==="html")return n==="subItem"?'<span style="display:inline-block;vertical-align:middle;margin-right:8px;margin-left:3px;border-radius:4px;width:4px;height:4px;background-color:'+Wt(i)+";"+(t||"")+'"></span>':'<span style="display:inline-block;margin-right:4px;border-radius:10px;width:10px;height:10px;background-color:'+Wt(i)+";"+(t||"")+'"></span>';var o=e.markerId||"markerX";return{renderMode:a,content:"{"+o+"|}  ",style:n==="subItem"?{width:4,height:4,borderRadius:2,backgroundColor:i}:{width:10,height:10,borderRadius:5,backgroundColor:i}}}function Qb(r,t,e){(r==="week"||r==="month"||r==="quarter"||r==="half-year"||r==="year")&&(r=`MM-dd
yyyy`);var i=ve(t),n=e?"getUTC":"get",a=i[n+"FullYear"](),o=i[n+"Month"]()+1,s=i[n+"Date"](),l=i[n+"Hours"](),u=i[n+"Minutes"](),f=i[n+"Seconds"](),h=i[n+"Milliseconds"]();return r=r.replace("MM",Et(o,2)).replace("M",o).replace("yyyy",a).replace("yy",Et(a%100+"",2)).replace("dd",Et(s,2)).replace("d",s).replace("hh",Et(l,2)).replace("h",l).replace("mm",Et(u,2)).replace("m",u).replace("ss",Et(f,2)).replace("s",f).replace("SSS",Et(h,3)),r}function jb(r){return r&&r.charAt(0).toUpperCase()+r.substr(1)}function hi(r,t){return t=t||"transparent",V(r)?r:z(r)&&r.colorStops&&(r.colorStops[0]||{}).color||t}function Yv(r,t){if(t==="_blank"||t==="blank"){var e=window.open();e.opener=null,e.location.href=r}else window.open(r,t)}var po=T,Jb=["left","right","top","bottom","width","height"],Fa=[["width","left","right"],["height","top","bottom"]];function Kf(r,t,e,i,n){var a=0,o=0;i==null&&(i=1/0),n==null&&(n=1/0);var s=0;t.eachChild(function(l,u){var f=l.getBoundingRect(),h=t.childAt(u+1),c=h&&h.getBoundingRect(),v,d;if(r==="horizontal"){var y=f.width+(c?-c.x+f.x:0);v=a+y,v>i||l.newline?(a=0,v=y,o+=s+e,s=f.height):s=Math.max(s,f.height)}else{var p=f.height+(c?-c.y+f.y:0);d=o+p,d>n||l.newline?(a+=s+e,o=0,d=p,s=f.width):s=Math.max(s,f.width)}l.newline||(l.x=a,l.y=o,l.markRedraw(),r==="horizontal"?a=v+e:o=d+e)})}var tx=Kf;kt(Kf,"vertical");kt(Kf,"horizontal");function Zi(r,t,e){e=ga(e||0);var i=t.width,n=t.height,a=Ot(r.left,i),o=Ot(r.top,n),s=Ot(r.right,i),l=Ot(r.bottom,n),u=Ot(r.width,i),f=Ot(r.height,n),h=e[2]+e[0],c=e[1]+e[3],v=r.aspect;switch(isNaN(u)&&(u=i-s-c-a),isNaN(f)&&(f=n-l-h-o),v!=null&&(isNaN(u)&&isNaN(f)&&(v>i/n?u=i*.8:f=n*.8),isNaN(u)&&(u=v*f),isNaN(f)&&(f=u/v)),isNaN(a)&&(a=i-s-u-c),isNaN(o)&&(o=n-l-f-h),r.left||r.right){case"center":a=i/2-u/2-e[3];break;case"right":a=i-u-c;break}switch(r.top||r.bottom){case"middle":case"center":o=n/2-f/2-e[0];break;case"bottom":o=n-f-h;break}a=a||0,o=o||0,isNaN(u)&&(u=i-c-a-(s||0)),isNaN(f)&&(f=n-h-o-(l||0));var d=new et(a+e[3],o+e[0],u,f);return d.margin=e,d}function ex(r,t,e,i,n,a){a=a||r,a.x=r.x,a.y=r.y;var o;if(o=r.getBoundingRect(),r.needLocalTransform()){var s=r.getLocalTransform();o=o.clone(),o.applyTransform(s)}var l=Zi(nt({width:o.width,height:o.height},t),e,i),u=l.x-o.x,f=l.y-o.y;return a.x+=u,a.y+=f,a===r&&r.markRedraw(),!0}function Qn(r){var t=r.layoutMode||r.constructor.layoutMode;return z(t)?t:t?{type:t}:null}function jn(r,t,e){var i=e&&e.ignoreSize;!N(i)&&(i=[i,i]);var n=o(Fa[0],0),a=o(Fa[1],1);u(Fa[0],r,n),u(Fa[1],r,a);function o(f,h){var c={},v=0,d={},y=0,p=2;if(po(f,function(_){d[_]=r[_]}),po(f,function(_){s(t,_)&&(c[_]=d[_]=t[_]),l(c,_)&&v++,l(d,_)&&y++}),i[h])return l(t,f[1])?d[f[2]]=null:l(t,f[2])&&(d[f[1]]=null),d;if(y===p||!v)return d;if(v>=p)return c;for(var g=0;g<f.length;g++){var m=f[g];if(!s(c,m)&&s(r,m)){c[m]=r[m];break}}return c}function s(f,h){return f.hasOwnProperty(h)}function l(f,h){return f[h]!=null&&f[h]!=="auto"}function u(f,h,c){po(f,function(v){h[v]=c[v]})}}function Qf(r){return rx({},r)}function rx(r,t){return t&&r&&po(Jb,function(e){t.hasOwnProperty(e)&&(r[e]=t[e])}),r}var ix=_t(),rt=function(r){k(t,r);function t(e,i,n){var a=r.call(this,e,i,n)||this;return a.uid=ws("ec_cpt_model"),a}return t.prototype.init=function(e,i,n){this.mergeDefaultAndTheme(e,n)},t.prototype.mergeDefaultAndTheme=function(e,i){var n=Qn(this),a=n?Qf(e):{},o=i.getTheme();J(e,o.get(this.mainType)),J(e,this.getDefaultOption()),n&&jn(e,a,n)},t.prototype.mergeOption=function(e,i){J(this.option,e,!0);var n=Qn(this);n&&jn(this.option,e,n)},t.prototype.optionUpdated=function(e,i){},t.prototype.getDefaultOption=function(){var e=this.constructor;if(!x_(e))return e.defaultOption;var i=ix(this);if(!i.defaultOption){for(var n=[],a=e;a;){var o=a.prototype.defaultOption;o&&n.push(o),a=a.superClass}for(var s={},l=n.length-1;l>=0;l--)s=J(s,n[l],!0);i.defaultOption=s}return i.defaultOption},t.prototype.getReferringComponents=function(e,i){var n=e+"Index",a=e+"Id";return da(this.ecModel,e,{index:this.get(n,!0),id:this.get(a,!0)},i)},t.prototype.getBoxLayoutParams=function(){var e=this;return{left:e.get("left"),top:e.get("top"),right:e.get("right"),bottom:e.get("bottom"),width:e.get("width"),height:e.get("height")}},t.prototype.getZLevelKey=function(){return""},t.prototype.setZLevel=function(e){this.option.zlevel=e},t.protoInitialize=function(){var e=t.prototype;e.type="component",e.id="",e.name="",e.mainType="",e.subType="",e.componentIndex=0}(),t}(xt);Dp(rt,xt);Jo(rt);Eb(rt);Ob(rt,nx);function nx(r){var t=[];return T(rt.getClassesByMainType(r),function(e){t=t.concat(e.dependencies||e.prototype.dependencies||[])}),t=H(t,function(e){return Ee(e).main}),r!=="dataset"&&ot(t,"dataset")<=0&&t.unshift("dataset"),t}var Xv=_t();_t();var jf=function(){function r(){}return r.prototype.getColorFromPalette=function(t,e,i){var n=Tt(this.get("color",!0)),a=this.get("colorLayer",!0);return ox(this,Xv,n,a,t,e,i)},r.prototype.clearColorPalette=function(){sx(this,Xv)},r}();function ax(r,t){for(var e=r.length,i=0;i<e;i++)if(r[i].length>t)return r[i];return r[e-1]}function ox(r,t,e,i,n,a,o){a=a||r;var s=t(a),l=s.paletteIdx||0,u=s.paletteNameMap=s.paletteNameMap||{};if(u.hasOwnProperty(n))return u[n];var f=o==null||!i?e:ax(i,o);if(f=f||e,!(!f||!f.length)){var h=f[l];return n&&(u[n]=h),s.paletteIdx=(l+1)%f.length,h}}function sx(r,t){t(r).paletteIdx=0,t(r).paletteNameMap={}}var lx=/\{@(.+?)\}/g,ux=function(){function r(){}return r.prototype.getDataParams=function(t,e){var i=this.getData(e),n=this.getRawValue(t,e),a=i.getRawIndex(t),o=i.getName(t),s=i.getRawDataItem(t),l=i.getItemVisual(t,"style"),u=l&&l[i.getItemVisual(t,"drawType")||"fill"],f=l&&l.stroke,h=this.mainType,c=h==="series",v=i.userOutput&&i.userOutput.get();return{componentType:h,componentSubType:this.subType,componentIndex:this.componentIndex,seriesType:c?this.subType:null,seriesIndex:this.seriesIndex,seriesId:c?this.id:null,seriesName:c?this.name:null,name:o,dataIndex:a,data:s,dataType:e,value:n,color:u,borderColor:f,dimensionNames:v?v.fullDimensions:null,encode:v?v.encode:null,$vars:["seriesName","name","value"]}},r.prototype.getFormattedLabel=function(t,e,i,n,a,o){e=e||"normal";var s=this.getData(i),l=this.getDataParams(t,i);if(o&&(l.value=o.interpolatedValue),n!=null&&N(l.value)&&(l.value=l.value[n]),!a){var u=s.getItemModel(t);a=u.get(e==="normal"?["label","formatter"]:[e,"label","formatter"])}if(Y(a))return l.status=e,l.dimensionIndex=n,a(l);if(V(a)){var f=qf(a,l);return f.replace(lx,function(h,c){var v=c.length,d=c;d.charAt(0)==="["&&d.charAt(v-1)==="]"&&(d=+d.slice(1,v-1));var y=Wi(s,t,d);if(o&&N(o.interpolatedValue)){var p=s.getDimensionIndex(d);p>=0&&(y=o.interpolatedValue[p])}return y!=null?y+"":""})}},r.prototype.getRawValue=function(t,e){return Wi(this.getData(e),t)},r.prototype.formatTooltip=function(t,e,i){},r}();function $v(r){var t,e;return z(r)?r.type&&(e=r):t=r,{text:t,frag:e}}function Bn(r){return new fx(r)}var fx=function(){function r(t){t=t||{},this._reset=t.reset,this._plan=t.plan,this._count=t.count,this._onDirty=t.onDirty,this._dirty=!0}return r.prototype.perform=function(t){var e=this._upstream,i=t&&t.skip;if(this._dirty&&e){var n=this.context;n.data=n.outputData=e.context.outputData}this.__pipeline&&(this.__pipeline.currentTask=this);var a;this._plan&&!i&&(a=this._plan(this.context));var o=f(this._modBy),s=this._modDataCount||0,l=f(t&&t.modBy),u=t&&t.modDataCount||0;(o!==l||s!==u)&&(a="reset");function f(m){return!(m>=1)&&(m=1),m}var h;(this._dirty||a==="reset")&&(this._dirty=!1,h=this._doReset(i)),this._modBy=l,this._modDataCount=u;var c=t&&t.step;if(e?this._dueEnd=e._outputDueEnd:this._dueEnd=this._count?this._count(this.context):1/0,this._progress){var v=this._dueIndex,d=Math.min(c!=null?this._dueIndex+c:1/0,this._dueEnd);if(!i&&(h||v<d)){var y=this._progress;if(N(y))for(var p=0;p<y.length;p++)this._doProgress(y[p],v,d,l,u);else this._doProgress(y,v,d,l,u)}this._dueIndex=d;var g=this._settedOutputEnd!=null?this._settedOutputEnd:d;this._outputDueEnd=g}else this._dueIndex=this._outputDueEnd=this._settedOutputEnd!=null?this._settedOutputEnd:this._dueEnd;return this.unfinished()},r.prototype.dirty=function(){this._dirty=!0,this._onDirty&&this._onDirty(this.context)},r.prototype._doProgress=function(t,e,i,n,a){Zv.reset(e,i,n,a),this._callingProgress=t,this._callingProgress({start:e,end:i,count:i-e,next:Zv.next},this.context)},r.prototype._doReset=function(t){this._dueIndex=this._outputDueEnd=this._dueEnd=0,this._settedOutputEnd=null;var e,i;!t&&this._reset&&(e=this._reset(this.context),e&&e.progress&&(i=e.forceFirstProgress,e=e.progress),N(e)&&!e.length&&(e=null)),this._progress=e,this._modBy=this._modDataCount=null;var n=this._downstream;return n&&n.dirty(),i},r.prototype.unfinished=function(){return this._progress&&this._dueIndex<this._dueEnd},r.prototype.pipe=function(t){(this._downstream!==t||this._dirty)&&(this._downstream=t,t._upstream=this,t.dirty())},r.prototype.dispose=function(){this._disposed||(this._upstream&&(this._upstream._downstream=null),this._downstream&&(this._downstream._upstream=null),this._dirty=!1,this._disposed=!0)},r.prototype.getUpstream=function(){return this._upstream},r.prototype.getDownstream=function(){return this._downstream},r.prototype.setOutputEnd=function(t){this._outputDueEnd=this._settedOutputEnd=t},r}(),Zv=function(){var r,t,e,i,n,a={reset:function(l,u,f,h){t=l,r=u,e=f,i=h,n=Math.ceil(i/e),a.next=e>1&&i>0?s:o}};return a;function o(){return t<r?t++:null}function s(){var l=t%n*e+Math.ceil(t/n),u=t>=r?null:l<i?l:t;return t++,u}}(),hx=function(){function r(){}return r.prototype.getRawData=function(){throw new Error("not supported")},r.prototype.getRawDataItem=function(t){throw new Error("not supported")},r.prototype.cloneRawData=function(){},r.prototype.getDimensionInfo=function(t){},r.prototype.cloneAllDimensionInfo=function(){},r.prototype.count=function(){},r.prototype.retrieveValue=function(t,e){},r.prototype.retrieveValueFromItem=function(t,e){},r.prototype.convertValue=function(t,e){return vo(t,e)},r}();function vx(r,t){var e=new hx,i=r.data,n=e.sourceFormat=r.sourceFormat,a=r.startIndex,o="";r.seriesLayoutBy!==Be&&Gt(o);var s=[],l={},u=r.dimensionsDefine;if(u)T(u,function(y,p){var g=y.name,m={index:p,name:g,displayName:y.displayName};if(s.push(m),g!=null){var _="";si(l,g)&&Gt(_),l[g]=m}});else for(var f=0;f<r.dimensionsDetectedCount;f++)s.push({index:f});var h=vy(n,Be);t.__isBuiltIn&&(e.getRawDataItem=function(y){return h(i,a,s,y)},e.getRawData=K(cx,null,r)),e.cloneRawData=K(dx,null,r);var c=cy(n,Be);e.count=K(c,null,i,a,s);var v=dy(n);e.retrieveValue=function(y,p){var g=h(i,a,s,y);return d(g,p)};var d=e.retrieveValueFromItem=function(y,p){if(y!=null){var g=s[p];if(g)return v(y,p,g.name)}};return e.getDimensionInfo=K(px,null,s,l),e.cloneAllDimensionInfo=K(gx,null,s),e}function cx(r){var t=r.sourceFormat;if(!Jf(t)){var e="";Gt(e)}return r.data}function dx(r){var t=r.sourceFormat,e=r.data;if(!Jf(t)){var i="";Gt(i)}if(t===Yt){for(var n=[],a=0,o=e.length;a<o;a++)n.push(e[a].slice());return n}else if(t===We){for(var n=[],a=0,o=e.length;a<o;a++)n.push(O({},e[a]));return n}}function px(r,t,e){if(e!=null){if(ht(e)||!isNaN(e)&&!si(t,e))return r[e];if(si(t,e))return t[e]}}function gx(r){return U(r)}var Ny=$();function yx(r){r=U(r);var t=r.type,e="";t||Gt(e);var i=t.split(":");i.length!==2&&Gt(e);var n=!1;i[0]==="echarts"&&(t=i[1],n=!0),r.__isBuiltIn=n,Ny.set(t,r)}function mx(r,t,e){var i=Tt(r),n=i.length,a="";n||Gt(a);for(var o=0,s=n;o<s;o++){var l=i[o];t=_x(l,t),o!==s-1&&(t.length=Math.max(t.length,1))}return t}function _x(r,t,e,i){var n="";t.length||Gt(n),z(r)||Gt(n);var a=r.type,o=Ny.get(a);o||Gt(n);var s=H(t,function(u){return vx(u,o)}),l=Tt(o.transform({upstream:s[0],upstreamList:s,config:U(r.config)}));return H(l,function(u,f){var h="";z(u)||Gt(h),u.data||Gt(h);var c=uy(u.data);Jf(c)||Gt(h);var v,d=t[0];if(d&&f===0&&!u.dimensions){var y=d.startIndex;y&&(u.data=d.data.slice(0,y).concat(u.data)),v={seriesLayoutBy:Be,sourceHeader:y,dimensions:d.metaRawOption.dimensions}}else v={seriesLayoutBy:Be,sourceHeader:0,dimensions:u.dimensions};return xu(u.data,v,null)})}function Jf(r){return r===Yt||r===We}var Fy=function(){function r(t){this._sourceList=[],this._storeList=[],this._upstreamSignList=[],this._versionSignBase=0,this._dirty=!0,this._sourceHost=t}return r.prototype.dirty=function(){this._setLocalSource([],[]),this._storeList=[],this._dirty=!0},r.prototype._setLocalSource=function(t,e){this._sourceList=t,this._upstreamSignList=e,this._versionSignBase++,this._versionSignBase>9e10&&(this._versionSignBase=0)},r.prototype._getVersionSign=function(){return this._sourceHost.uid+"_"+this._versionSignBase},r.prototype.prepareSource=function(){this._isDirty()&&(this._createSource(),this._dirty=!1)},r.prototype._createSource=function(){this._setLocalSource([],[]);var t=this._sourceHost,e=this._getUpstreamSourceManagers(),i=!!e.length,n,a;if(za(t)){var o=t,s=void 0,l=void 0,u=void 0;if(i){var f=e[0];f.prepareSource(),u=f.getSource(),s=u.data,l=u.sourceFormat,a=[f._getVersionSign()]}else s=o.get("data",!0),l=Ft(s)?wr:ge,a=[];var h=this._getSourceMetaRawOption()||{},c=u&&u.metaRawOption||{},v=Q(h.seriesLayoutBy,c.seriesLayoutBy)||null,d=Q(h.sourceHeader,c.sourceHeader),y=Q(h.dimensions,c.dimensions),p=v!==c.seriesLayoutBy||!!d!=!!c.sourceHeader||y;n=p?[xu(s,{seriesLayoutBy:v,sourceHeader:d,dimensions:y},l)]:[]}else{var g=t;if(i){var m=this._applyTransform(e);n=m.sourceList,a=m.upstreamSignList}else{var _=g.get("source",!0);n=[xu(_,this._getSourceMetaRawOption(),null)],a=[]}}this._setLocalSource(n,a)},r.prototype._applyTransform=function(t){var e=this._sourceHost,i=e.get("transform",!0),n=e.get("fromTransformResult",!0);if(n!=null){var a="";t.length!==1&&Kv(a)}var o,s=[],l=[];return T(t,function(u){u.prepareSource();var f=u.getSource(n||0),h="";n!=null&&!f&&Kv(h),s.push(f),l.push(u._getVersionSign())}),i?o=mx(i,s,{datasetIndex:e.componentIndex}):n!=null&&(o=[jw(s[0])]),{sourceList:o,upstreamSignList:l}},r.prototype._isDirty=function(){if(this._dirty)return!0;for(var t=this._getUpstreamSourceManagers(),e=0;e<t.length;e++){var i=t[e];if(i._isDirty()||this._upstreamSignList[e]!==i._getVersionSign())return!0}},r.prototype.getSource=function(t){t=t||0;var e=this._sourceList[t];if(!e){var i=this._getUpstreamSourceManagers();return i[0]&&i[0].getSource(t)}return e},r.prototype.getSharedDataStore=function(t){var e=t.makeStoreSchema();return this._innerGetDataStore(e.dimensions,t.source,e.hash)},r.prototype._innerGetDataStore=function(t,e,i){var n=0,a=this._storeList,o=a[n];o||(o=a[n]={});var s=o[i];if(!s){var l=this._getUpstreamSourceManagers()[0];za(this._sourceHost)&&l?s=l._innerGetDataStore(t,e,i):(s=new Tu,s.initData(new hy(e,t.length),t)),o[i]=s}return s},r.prototype._getUpstreamSourceManagers=function(){var t=this._sourceHost;if(za(t)){var e=oy(t);return e?[e.getSourceManager()]:[]}else return H(Kw(t),function(i){return i.getSourceManager()})},r.prototype._getSourceMetaRawOption=function(){var t=this._sourceHost,e,i,n;if(za(t))e=t.get("seriesLayoutBy",!0),i=t.get("sourceHeader",!0),n=t.get("dimensions",!0);else if(!this._getUpstreamSourceManagers().length){var a=t;e=a.get("seriesLayoutBy",!0),i=a.get("sourceHeader",!0),n=a.get("dimensions",!0)}return{seriesLayoutBy:e,sourceHeader:i,dimensions:n}},r}();function qv(r){var t=r.option.transform;t&&Hn(r.option.transform)}function za(r){return r.mainType==="series"}function Kv(r){throw new Error(r)}var zy="line-height:1";function Vy(r,t){var e=r.color||"#6e7079",i=r.fontSize||12,n=r.fontWeight||"400",a=r.color||"#464646",o=r.fontSize||14,s=r.fontWeight||"900";return t==="html"?{nameStyle:"font-size:"+Wt(i+"")+"px;color:"+Wt(e)+";font-weight:"+Wt(n+""),valueStyle:"font-size:"+Wt(o+"")+"px;color:"+Wt(a)+";font-weight:"+Wt(s+"")}:{nameStyle:{fontSize:i,fill:e,fontWeight:n},valueStyle:{fontSize:o,fill:a,fontWeight:s}}}var Sx=[0,10,20,30],bx=["",`
`,`

`,`


`];function Jn(r,t){return t.type=r,t}function Au(r){return r.type==="section"}function Hy(r){return Au(r)?xx:Tx}function Gy(r){if(Au(r)){var t=0,e=r.blocks.length,i=e>1||e>0&&!r.noHeader;return T(r.blocks,function(n){var a=Gy(n);a>=t&&(t=a+ +(i&&(!a||Au(n)&&!n.noHeader)))}),t}return 0}function xx(r,t,e,i){var n=t.noHeader,a=Mx(Gy(t)),o=[],s=t.blocks||[];Ce(!s||N(s)),s=s||[];var l=r.orderMode;if(t.sortBlocks&&l){s=s.slice();var u={valueAsc:"asc",valueDesc:"desc"};if(si(u,l)){var f=new lb(u[l],null);s.sort(function(d,y){return f.evaluate(d.sortParam,y.sortParam)})}else l==="seriesDesc"&&s.reverse()}T(s,function(d,y){var p=t.valueFormatter,g=Hy(d)(p?O(O({},r),{valueFormatter:p}):r,d,y>0?a.html:0,i);g!=null&&o.push(g)});var h=r.renderMode==="richText"?o.join(a.richText):Iu(o.join(""),n?e:a.html);if(n)return h;var c=Du(t.header,"ordinal",r.useUTC),v=Vy(i,r.renderMode).nameStyle;return r.renderMode==="richText"?Wy(r,c,v)+a.richText+h:Iu('<div style="'+v+";"+zy+';">'+Wt(c)+"</div>"+h,e)}function Tx(r,t,e,i){var n=r.renderMode,a=t.noName,o=t.noValue,s=!t.markerType,l=t.name,u=r.useUTC,f=t.valueFormatter||r.valueFormatter||function(S){return S=N(S)?S:[S],H(S,function(b,w){return Du(b,N(v)?v[w]:v,u)})};if(!(a&&o)){var h=s?"":r.markupStyleCreator.makeTooltipMarker(t.markerType,t.markerColor||"#333",n),c=a?"":Du(l,"ordinal",u),v=t.valueType,d=o?[]:f(t.value,t.dataIndex),y=!s||!a,p=!s&&a,g=Vy(i,n),m=g.nameStyle,_=g.valueStyle;return n==="richText"?(s?"":h)+(a?"":Wy(r,c,m))+(o?"":Ax(r,d,y,p,_)):Iu((s?"":h)+(a?"":Cx(c,!s,m))+(o?"":Dx(d,y,p,_)),e)}}function Qv(r,t,e,i,n,a){if(r){var o=Hy(r),s={useUTC:n,renderMode:e,orderMode:i,markupStyleCreator:t,valueFormatter:r.valueFormatter};return o(s,r,0,a)}}function Mx(r){return{html:Sx[r],richText:bx[r]}}function Iu(r,t){var e='<div style="clear:both"></div>',i="margin: "+t+"px 0 0";return'<div style="'+i+";"+zy+';">'+r+e+"</div>"}function Cx(r,t,e){var i=t?"margin-left:2px":"";return'<span style="'+e+";"+i+'">'+Wt(r)+"</span>"}function Dx(r,t,e,i){var n=e?"10px":"20px",a=t?"float:right;margin-left:"+n:"";return r=N(r)?r:[r],'<span style="'+a+";"+i+'">'+H(r,function(o){return Wt(o)}).join("&nbsp;&nbsp;")+"</span>"}function Wy(r,t,e){return r.markupStyleCreator.wrapRichTextStyle(t,e)}function Ax(r,t,e,i,n){var a=[n],o=i?10:20;return e&&a.push({padding:[0,0,0,o],align:"right"}),r.markupStyleCreator.wrapRichTextStyle(N(t)?t.join("  "):t,a)}function Ix(r,t){var e=r.getData().getItemVisual(t,"style"),i=e[r.visualDrawType];return hi(i)}function Uy(r,t){var e=r.get("padding");return e??(t==="richText"?[8,10]:10)}var cl=function(){function r(){this.richTextStyles={},this._nextStyleNameId=lg()}return r.prototype._generateStyleName=function(){return"__EC_aUTo_"+this._nextStyleNameId++},r.prototype.makeTooltipMarker=function(t,e,i){var n=i==="richText"?this._generateStyleName():null,a=By({color:e,type:t,renderMode:i,markerId:n});return V(a)?a:(this.richTextStyles[n]=a.style,a.content)},r.prototype.wrapRichTextStyle=function(t,e){var i={};N(e)?T(e,function(a){return O(i,a)}):O(i,e);var n=this._generateStyleName();return this.richTextStyles[n]=i,"{"+n+"|"+t+"}"},r}();function Lx(r){var t=r.series,e=r.dataIndex,i=r.multipleSeries,n=t.getData(),a=n.mapDimensionsAll("defaultedTooltip"),o=a.length,s=t.getRawValue(e),l=N(s),u=Ix(t,e),f,h,c,v;if(o>1||l&&!o){var d=Px(s,t,e,a,u);f=d.inlineValues,h=d.inlineValueTypes,c=d.blocks,v=d.inlineValues[0]}else if(o){var y=n.getDimensionInfo(a[0]);v=f=Wi(n,e,a[0]),h=y.type}else v=f=l?s[0]:s;var p=vg(t),g=p&&t.name||"",m=n.getName(e),_=i?g:m;return Jn("section",{header:g,noHeader:i||!p,sortParam:v,blocks:[Jn("nameValue",{markerType:"item",markerColor:u,name:_,noName:!be(_),value:f,valueType:h,dataIndex:e})].concat(c||[])})}function Px(r,t,e,i,n){var a=t.getData(),o=Ne(r,function(h,c,v){var d=a.getDimensionInfo(v);return h=h||d&&d.tooltip!==!1&&d.displayName!=null},!1),s=[],l=[],u=[];i.length?T(i,function(h){f(Wi(a,e,h),h)}):T(r,f);function f(h,c){var v=a.getDimensionInfo(c);!v||v.otherDims.tooltip===!1||(o?u.push(Jn("nameValue",{markerType:"subItem",markerColor:n,name:v.displayName,value:h,valueType:v.type})):(s.push(h),l.push(v.type)))}return{inlineValues:s,inlineValueTypes:l,blocks:u}}var or=_t();function Va(r,t){return r.getName(t)||r.getId(t)}var Rx="__universalTransitionEnabled",ce=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e._selectedDataIndicesMap={},e}return t.prototype.init=function(e,i,n){this.seriesIndex=this.componentIndex,this.dataTask=Bn({count:Ox,reset:kx}),this.dataTask.context={model:this},this.mergeDefaultAndTheme(e,n);var a=or(this).sourceManager=new Fy(this);a.prepareSource();var o=this.getInitialData(e,n);Jv(o,this),this.dataTask.context.data=o,or(this).dataBeforeProcessed=o,jv(this),this._initSelectedMapFromData(o)},t.prototype.mergeDefaultAndTheme=function(e,i){var n=Qn(this),a=n?Qf(e):{},o=this.subType;rt.hasClass(o)&&(o+="Series"),J(e,i.getTheme().get(this.subType)),J(e,this.getDefaultOption()),ev(e,"label",["show"]),this.fillDataTextStyle(e.data),n&&jn(e,a,n)},t.prototype.mergeOption=function(e,i){e=J(this.option,e,!0),this.fillDataTextStyle(e.data);var n=Qn(this);n&&jn(this.option,e,n);var a=or(this).sourceManager;a.dirty(),a.prepareSource();var o=this.getInitialData(e,i);Jv(o,this),this.dataTask.dirty(),this.dataTask.context.data=o,or(this).dataBeforeProcessed=o,jv(this),this._initSelectedMapFromData(o)},t.prototype.fillDataTextStyle=function(e){if(e&&!Ft(e))for(var i=["show"],n=0;n<e.length;n++)e[n]&&e[n].label&&ev(e[n],"label",i)},t.prototype.getInitialData=function(e,i){},t.prototype.appendData=function(e){var i=this.getRawData();i.appendData(e.data)},t.prototype.getData=function(e){var i=Lu(this);if(i){var n=i.context.data;return e==null||!n.getLinkedData?n:n.getLinkedData(e)}else return or(this).data},t.prototype.getAllData=function(){var e=this.getData();return e&&e.getLinkedDataAll?e.getLinkedDataAll():[{data:e}]},t.prototype.setData=function(e){var i=Lu(this);if(i){var n=i.context;n.outputData=e,i!==this.dataTask&&(n.data=e)}or(this).data=e},t.prototype.getEncode=function(){var e=this.get("encode",!0);if(e)return $(e)},t.prototype.getSourceManager=function(){return or(this).sourceManager},t.prototype.getSource=function(){return this.getSourceManager().getSource()},t.prototype.getRawData=function(){return or(this).dataBeforeProcessed},t.prototype.getColorBy=function(){var e=this.get("colorBy");return e||"series"},t.prototype.isColorBySeries=function(){return this.getColorBy()==="series"},t.prototype.getBaseAxis=function(){var e=this.coordinateSystem;return e&&e.getBaseAxis&&e.getBaseAxis()},t.prototype.formatTooltip=function(e,i,n){return Lx({series:this,dataIndex:e,multipleSeries:i})},t.prototype.isAnimationEnabled=function(){var e=this.ecModel;if(Z.node&&!(e&&e.ssr))return!1;var i=this.getShallow("animation");return i&&this.getData().count()>this.getShallow("animationThreshold")&&(i=!1),!!i},t.prototype.restoreData=function(){this.dataTask.dirty()},t.prototype.getColorFromPalette=function(e,i,n){var a=this.ecModel,o=jf.prototype.getColorFromPalette.call(this,e,i,n);return o||(o=a.getColorFromPalette(e,i,n)),o},t.prototype.coordDimToDataDim=function(e){return this.getRawData().mapDimensionsAll(e)},t.prototype.getProgressive=function(){return this.get("progressive")},t.prototype.getProgressiveThreshold=function(){return this.get("progressiveThreshold")},t.prototype.select=function(e,i){this._innerSelect(this.getData(i),e)},t.prototype.unselect=function(e,i){var n=this.option.selectedMap;if(n){var a=this.option.selectedMode,o=this.getData(i);if(a==="series"||n==="all"){this.option.selectedMap={},this._selectedDataIndicesMap={};return}for(var s=0;s<e.length;s++){var l=e[s],u=Va(o,l);n[u]=!1,this._selectedDataIndicesMap[u]=-1}}},t.prototype.toggleSelect=function(e,i){for(var n=[],a=0;a<e.length;a++)n[0]=e[a],this.isSelected(e[a],i)?this.unselect(n,i):this.select(n,i)},t.prototype.getSelectedDataIndices=function(){if(this.option.selectedMap==="all")return[].slice.call(this.getData().getIndices());for(var e=this._selectedDataIndicesMap,i=ft(e),n=[],a=0;a<i.length;a++){var o=e[i[a]];o>=0&&n.push(o)}return n},t.prototype.isSelected=function(e,i){var n=this.option.selectedMap;if(!n)return!1;var a=this.getData(i);return(n==="all"||n[Va(a,e)])&&!a.getItemModel(e).get(["select","disabled"])},t.prototype.isUniversalTransitionEnabled=function(){if(this[Rx])return!0;var e=this.option.universalTransition;return e?e===!0?!0:e&&e.enabled:!1},t.prototype._innerSelect=function(e,i){var n,a,o=this.option,s=o.selectedMode,l=i.length;if(!(!s||!l)){if(s==="series")o.selectedMap="all";else if(s==="multiple"){z(o.selectedMap)||(o.selectedMap={});for(var u=o.selectedMap,f=0;f<l;f++){var h=i[f],c=Va(e,h);u[c]=!0,this._selectedDataIndicesMap[c]=e.getRawIndex(h)}}else if(s==="single"||s===!0){var v=i[l-1],c=Va(e,v);o.selectedMap=(n={},n[c]=!0,n),this._selectedDataIndicesMap=(a={},a[c]=e.getRawIndex(v),a)}}},t.prototype._initSelectedMapFromData=function(e){if(!this.option.selectedMap){var i=[];e.hasItemOption&&e.each(function(n){var a=e.getRawDataItem(n);a&&a.selected&&i.push(n)}),i.length>0&&this._innerSelect(e,i)}},t.registerClass=function(e){return rt.registerClass(e)},t.protoInitialize=function(){var e=t.prototype;e.type="series.__base__",e.seriesIndex=0,e.ignoreStyleOnData=!1,e.hasSymbolVisual=!1,e.defaultSymbol="circle",e.visualStyleAccessPath="itemStyle",e.visualDrawType="fill"}(),t}(rt);de(ce,ux);de(ce,jf);Dp(ce,rt);function jv(r){var t=r.name;vg(r)||(r.name=Ex(r)||t)}function Ex(r){var t=r.getRawData(),e=t.mapDimensionsAll("seriesName"),i=[];return T(e,function(n){var a=t.getDimensionInfo(n);a.displayName&&i.push(a.displayName)}),i.join(" ")}function Ox(r){return r.model.getRawData().count()}function kx(r){var t=r.model;return t.setData(t.getRawData().cloneShallow()),Bx}function Bx(r,t){t.outputData&&r.end>t.outputData.count()&&t.model.getRawData().cloneShallow(t.outputData)}function Jv(r,t){T(Tp(r.CHANGABLE_METHODS,r.DOWNSAMPLE_METHODS),function(e){r.wrapMethod(e,kt(Nx,t))})}function Nx(r,t){var e=Lu(r);return e&&e.setOutputEnd((t||this).count()),t}function Lu(r){var t=(r.ecModel||{}).scheduler,e=t&&t.getPipeline(r.uid);if(e){var i=e.currentTask;if(i){var n=i.agentStubMap;n&&(i=n.get(r.uid))}return i}}var Fx=st.extend({type:"triangle",shape:{cx:0,cy:0,width:0,height:0},buildPath:function(r,t){var e=t.cx,i=t.cy,n=t.width/2,a=t.height/2;r.moveTo(e,i-a),r.lineTo(e+n,i+a),r.lineTo(e-n,i+a),r.closePath()}}),zx=st.extend({type:"diamond",shape:{cx:0,cy:0,width:0,height:0},buildPath:function(r,t){var e=t.cx,i=t.cy,n=t.width/2,a=t.height/2;r.moveTo(e,i-a),r.lineTo(e+n,i),r.lineTo(e,i+a),r.lineTo(e-n,i),r.closePath()}}),Vx=st.extend({type:"pin",shape:{x:0,y:0,width:0,height:0},buildPath:function(r,t){var e=t.x,i=t.y,n=t.width/5*3,a=Math.max(n,t.height),o=n/2,s=o*o/(a-o),l=i-a+o+s,u=Math.asin(s/o),f=Math.cos(u)*o,h=Math.sin(u),c=Math.cos(u),v=o*.6,d=o*.7;r.moveTo(e-f,l+s),r.arc(e,l,o,Math.PI-u,Math.PI*2+u),r.bezierCurveTo(e+f-h*v,l+s+c*v,e,i-d,e,i),r.bezierCurveTo(e,i-d,e-f+h*v,l+s+c*v,e-f,l+s),r.closePath()}}),Hx=st.extend({type:"arrow",shape:{x:0,y:0,width:0,height:0},buildPath:function(r,t){var e=t.height,i=t.width,n=t.x,a=t.y,o=i/3*2;r.moveTo(n,a),r.lineTo(n+o,a+e),r.lineTo(n,a+e/4*3),r.lineTo(n-o,a+e),r.lineTo(n,a),r.closePath()}}),Gx={line:xr,rect:gt,roundRect:gt,square:gt,circle:ls,diamond:zx,pin:Vx,arrow:Hx,triangle:Fx},Wx={line:function(r,t,e,i,n){n.x1=r,n.y1=t+i/2,n.x2=r+e,n.y2=t+i/2},rect:function(r,t,e,i,n){n.x=r,n.y=t,n.width=e,n.height=i},roundRect:function(r,t,e,i,n){n.x=r,n.y=t,n.width=e,n.height=i,n.r=Math.min(e,i)/4},square:function(r,t,e,i,n){var a=Math.min(e,i);n.x=r,n.y=t,n.width=a,n.height=a},circle:function(r,t,e,i,n){n.cx=r+e/2,n.cy=t+i/2,n.r=Math.min(e,i)/2},diamond:function(r,t,e,i,n){n.cx=r+e/2,n.cy=t+i/2,n.width=e,n.height=i},pin:function(r,t,e,i,n){n.x=r+e/2,n.y=t+i/2,n.width=e,n.height=i},arrow:function(r,t,e,i,n){n.x=r+e/2,n.y=t+i/2,n.width=e,n.height=i},triangle:function(r,t,e,i,n){n.cx=r+e/2,n.cy=t+i/2,n.width=e,n.height=i}},Pu={};T(Gx,function(r,t){Pu[t]=new r});var Ux=st.extend({type:"symbol",shape:{symbolType:"",x:0,y:0,width:0,height:0},calculateTextPosition:function(r,t,e){var i=xo(r,t,e),n=this.shape;return n&&n.symbolType==="pin"&&t.position==="inside"&&(i.y=e.y+e.height*.4),i},buildPath:function(r,t,e){var i=t.symbolType;if(i!=="none"){var n=Pu[i];n||(i="rect",n=Pu[i]),Wx[i](t.x,t.y,t.width,t.height,n.shape),n.buildPath(r,n.shape,e)}}});function Yx(r,t){if(this.type!=="image"){var e=this.style;this.__isEmptyBrush?(e.stroke=r,e.fill=t||"#fff",e.lineWidth=2):this.shape.symbolType==="line"?e.stroke=r:e.fill=r,this.markRedraw()}}function Je(r,t,e,i,n,a,o){var s=r.indexOf("empty")===0;s&&(r=r.substr(5,1).toLowerCase()+r.substr(6));var l;return r.indexOf("image://")===0?l=kf(r.slice(8),new et(t,e,i,n),o?"center":"cover"):r.indexOf("path://")===0?l=vs(r.slice(7),{},new et(t,e,i,n),o?"center":"cover"):l=new Ux({shape:{symbolType:r,x:t,y:e,width:i,height:n}}),l.__isEmptyBrush=s,l.setColor=Yx,a&&l.setColor(a),l}function Xx(r){return N(r)||(r=[+r,+r]),[r[0]||0,r[1]||0]}function Yy(r,t){if(r!=null)return N(r)||(r=[r,r]),[Ot(r[0],t[0])||0,Ot(Q(r[1],r[0]),t[1])||0]}var $x=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e.hasSymbolVisual=!0,e}return t.prototype.getInitialData=function(e){return Ss(null,this,{useEncodeDefaulter:!0})},t.prototype.getLegendIcon=function(e){var i=new mt,n=Je("line",0,e.itemHeight/2,e.itemWidth,0,e.lineStyle.stroke,!1);i.add(n),n.setStyle(e.lineStyle);var a=this.getData().getVisual("symbol"),o=this.getData().getVisual("symbolRotate"),s=a==="none"?"circle":a,l=e.itemHeight*.8,u=Je(s,(e.itemWidth-l)/2,(e.itemHeight-l)/2,l,l,e.itemStyle.fill);i.add(u),u.setStyle(e.itemStyle);var f=e.iconRotate==="inherit"?o:e.iconRotate||0;return u.rotation=f*Math.PI/180,u.setOrigin([e.itemWidth/2,e.itemHeight/2]),s.indexOf("empty")>-1&&(u.style.stroke=u.style.fill,u.style.fill="#fff",u.style.lineWidth=2),i},t.type="series.line",t.dependencies=["grid","polar"],t.defaultOption={z:3,coordinateSystem:"cartesian2d",legendHoverLink:!0,clip:!0,label:{position:"top"},endLabel:{show:!1,valueAnimation:!0,distance:8},lineStyle:{width:2,type:"solid"},emphasis:{scale:!0},step:!1,smooth:!1,smoothMonotone:null,symbol:"emptyCircle",symbolSize:4,symbolRotate:null,showSymbol:!0,showAllSymbol:"auto",connectNulls:!1,sampling:"none",animationEasing:"linear",progressive:0,hoverLayerThreshold:1/0,universalTransition:{divideShape:"clone"},triggerLineEvent:!1},t}(ce);function th(r,t){var e=r.mapDimensionsAll("defaultedLabel"),i=e.length;if(i===1){var n=Wi(r,t,e[0]);return n!=null?n+"":null}else if(i){for(var a=[],o=0;o<e.length;o++)a.push(Wi(r,t,e[o]));return a.join(" ")}}function Xy(r,t){var e=r.mapDimensionsAll("defaultedLabel");if(!N(t))return t+"";for(var i=[],n=0;n<e.length;n++){var a=r.getDimensionIndex(e[n]);a>=0&&i.push(t[a])}return i.join(" ")}var eh=function(r){k(t,r);function t(e,i,n,a){var o=r.call(this)||this;return o.updateData(e,i,n,a),o}return t.prototype._createSymbol=function(e,i,n,a,o){this.removeAll();var s=Je(e,-1,-1,2,2,null,o);s.attr({z2:100,culling:!0,scaleX:a[0]/2,scaleY:a[1]/2}),s.drift=Zx,this._symbolType=e,this.add(s)},t.prototype.stopSymbolAnimation=function(e){this.childAt(0).stopAnimation(null,e)},t.prototype.getSymbolType=function(){return this._symbolType},t.prototype.getSymbolPath=function(){return this.childAt(0)},t.prototype.highlight=function(){Po(this.childAt(0))},t.prototype.downplay=function(){Ro(this.childAt(0))},t.prototype.setZ=function(e,i){var n=this.childAt(0);n.zlevel=e,n.z=i},t.prototype.setDraggable=function(e,i){var n=this.childAt(0);n.draggable=e,n.cursor=!i&&e?"move":n.cursor},t.prototype.updateData=function(e,i,n,a){this.silent=!1;var o=e.getItemVisual(i,"symbol")||"circle",s=e.hostModel,l=t.getSymbolSize(e,i),u=o!==this._symbolType,f=a&&a.disableAnimation;if(u){var h=e.getItemVisual(i,"symbolKeepAspect");this._createSymbol(o,e,i,l,h)}else{var c=this.childAt(0);c.silent=!1;var v={scaleX:l[0]/2,scaleY:l[1]/2};f?c.attr(v):te(c,v,s,i),Wg(c)}if(this._updateCommon(e,i,l,n,a),u){var c=this.childAt(0);if(!f){var v={scaleX:this._sizeX,scaleY:this._sizeY,style:{opacity:c.style.opacity}};c.scaleX=c.scaleY=0,c.style.opacity=0,He(c,v,s,i)}}f&&this.childAt(0).stopAnimation("leave")},t.prototype._updateCommon=function(e,i,n,a,o){var s=this.childAt(0),l=e.hostModel,u,f,h,c,v,d,y,p,g;if(a&&(u=a.emphasisItemStyle,f=a.blurItemStyle,h=a.selectItemStyle,c=a.focus,v=a.blurScope,y=a.labelStatesModels,p=a.hoverScale,g=a.cursorStyle,d=a.emphasisDisabled),!a||e.hasItemOption){var m=a&&a.itemModel?a.itemModel:e.getItemModel(i),_=m.getModel("emphasis");u=_.getModel("itemStyle").getItemStyle(),h=m.getModel(["select","itemStyle"]).getItemStyle(),f=m.getModel(["blur","itemStyle"]).getItemStyle(),c=_.get("focus"),v=_.get("blurScope"),d=_.get("disabled"),y=ps(m),p=_.getShallow("scale"),g=m.getShallow("cursor")}var S=e.getItemVisual(i,"symbolRotate");s.attr("rotation",(S||0)*Math.PI/180||0);var b=Yy(e.getItemVisual(i,"symbolOffset"),n);b&&(s.x=b[0],s.y=b[1]),g&&s.attr("cursor",g);var w=e.getItemVisual(i,"style"),x=w.fill;if(s instanceof tr){var C=s.style;s.useStyle(O({image:C.image,x:C.x,y:C.y,width:C.width,height:C.height},w))}else s.__isEmptyBrush?s.useStyle(O({},w)):s.useStyle(w),s.style.decal=null,s.setColor(x,o&&o.symbolInnerColor),s.style.strokeNoScale=!0;var D=e.getItemVisual(i,"liftZ"),A=this._z2;D!=null?A==null&&(this._z2=s.z2,s.z2+=D):A!=null&&(s.z2=A,this._z2=null);var M=o&&o.useNameLabel;Ff(s,y,{labelFetcher:l,labelDataIndex:i,defaultText:I,inheritColor:x,defaultOpacity:w.opacity});function I(R){return M?e.getName(R):th(e,R)}this._sizeX=n[0]/2,this._sizeY=n[1]/2;var L=s.ensureState("emphasis");L.style=u,s.ensureState("select").style=h,s.ensureState("blur").style=f;var P=p==null||p===!0?Math.max(1.1,3/this._sizeY):isFinite(p)&&p>0?+p:1;L.scaleX=this._sizeX*P,L.scaleY=this._sizeY*P,this.setSymbolScale(1),Eo(this,c,v,d)},t.prototype.setSymbolScale=function(e){this.scaleX=this.scaleY=e},t.prototype.fadeOut=function(e,i,n){var a=this.childAt(0),o=tt(this).dataIndex,s=n&&n.animation;if(this.silent=a.silent=!0,n&&n.fadeLabel){var l=a.getTextContent();l&&ko(l,{style:{opacity:0}},i,{dataIndex:o,removeOpt:s,cb:function(){a.removeTextContent()}})}else a.removeTextContent();ko(a,{style:{opacity:0},scaleX:0,scaleY:0},i,{dataIndex:o,cb:e,removeOpt:s})},t.getSymbolSize=function(e,i){return Xx(e.getItemVisual(i,"symbolSize"))},t}(mt);function Zx(r,t){this.parent.drift(r,t)}function dl(r,t,e,i){return t&&!isNaN(t[0])&&!isNaN(t[1])&&!(i.isIgnore&&i.isIgnore(e))&&!(i.clipShape&&!i.clipShape.contain(t[0],t[1]))&&r.getItemVisual(e,"symbol")!=="none"}function tc(r){return r!=null&&!z(r)&&(r={isIgnore:r}),r||{}}function ec(r){var t=r.hostModel,e=t.getModel("emphasis");return{emphasisItemStyle:e.getModel("itemStyle").getItemStyle(),blurItemStyle:t.getModel(["blur","itemStyle"]).getItemStyle(),selectItemStyle:t.getModel(["select","itemStyle"]).getItemStyle(),focus:e.get("focus"),blurScope:e.get("blurScope"),emphasisDisabled:e.get("disabled"),hoverScale:e.get("scale"),labelStatesModels:ps(t),cursorStyle:t.get("cursor")}}var qx=function(){function r(t){this.group=new mt,this._SymbolCtor=t||eh}return r.prototype.updateData=function(t,e){this._progressiveEls=null,e=tc(e);var i=this.group,n=t.hostModel,a=this._data,o=this._SymbolCtor,s=e.disableAnimation,l=ec(t),u={disableAnimation:s},f=e.getSymbolPoint||function(h){return t.getItemLayout(h)};a||i.removeAll(),t.diff(a).add(function(h){var c=f(h);if(dl(t,c,h,e)){var v=new o(t,h,l,u);v.setPosition(c),t.setItemGraphicEl(h,v),i.add(v)}}).update(function(h,c){var v=a.getItemGraphicEl(c),d=f(h);if(!dl(t,d,h,e)){i.remove(v);return}var y=t.getItemVisual(h,"symbol")||"circle",p=v&&v.getSymbolType&&v.getSymbolType();if(!v||p&&p!==y)i.remove(v),v=new o(t,h,l,u),v.setPosition(d);else{v.updateData(t,h,l,u);var g={x:d[0],y:d[1]};s?v.attr(g):te(v,g,n)}i.add(v),t.setItemGraphicEl(h,v)}).remove(function(h){var c=a.getItemGraphicEl(h);c&&c.fadeOut(function(){i.remove(c)},n)}).execute(),this._getSymbolPoint=f,this._data=t},r.prototype.updateLayout=function(){var t=this,e=this._data;e&&e.eachItemGraphicEl(function(i,n){var a=t._getSymbolPoint(n);i.setPosition(a),i.markRedraw()})},r.prototype.incrementalPrepareUpdate=function(t){this._seriesScope=ec(t),this._data=null,this.group.removeAll()},r.prototype.incrementalUpdate=function(t,e,i){this._progressiveEls=[],i=tc(i);function n(l){l.isGroup||(l.incremental=!0,l.ensureState("emphasis").hoverLayer=!0)}for(var a=t.start;a<t.end;a++){var o=e.getItemLayout(a);if(dl(e,o,a,i)){var s=new this._SymbolCtor(e,a,this._seriesScope);s.traverse(n),s.setPosition(o),this.group.add(s),e.setItemGraphicEl(a,s),this._progressiveEls.push(s)}}},r.prototype.eachRendered=function(t){cs(this._progressiveEls||this.group,t)},r.prototype.remove=function(t){var e=this.group,i=this._data;i&&t?i.eachItemGraphicEl(function(n){n.fadeOut(function(){e.remove(n)},i.hostModel)}):e.removeAll()},r}();function $y(r,t,e){var i=r.getBaseAxis(),n=r.getOtherAxis(i),a=Kx(n,e),o=i.dim,s=n.dim,l=t.mapDimension(s),u=t.mapDimension(o),f=s==="x"||s==="radius"?1:0,h=H(r.dimensions,function(d){return t.mapDimension(d)}),c=!1,v=t.getCalculationInfo("stackResultDimension");return fi(t,h[0])&&(c=!0,h[0]=v),fi(t,h[1])&&(c=!0,h[1]=v),{dataDimsForPoint:h,valueStart:a,valueAxisDim:s,baseAxisDim:o,stacked:!!c,valueDim:l,baseDim:u,baseDataOffset:f,stackedOverDimension:t.getCalculationInfo("stackedOverDimension")}}function Kx(r,t){var e=0,i=r.scale.getExtent();return t==="start"?e=i[0]:t==="end"?e=i[1]:ht(t)&&!isNaN(t)?e=t:i[0]>0?e=i[0]:i[1]<0&&(e=i[1]),e}function Zy(r,t,e,i){var n=NaN;r.stacked&&(n=e.get(e.getCalculationInfo("stackedOverDimension"),i)),isNaN(n)&&(n=r.valueStart);var a=r.baseDataOffset,o=[];return o[a]=e.get(r.baseDim,i),o[1-a]=n,t.dataToPoint(o)}var qy=typeof Float32Array<"u",Qx=qy?Float32Array:Array;function Ke(r){return N(r)?qy?new Float32Array(r):r:new Qx(r)}function jx(r,t){var e=[];return t.diff(r).add(function(i){e.push({cmd:"+",idx:i})}).update(function(i,n){e.push({cmd:"=",idx:n,idx1:i})}).remove(function(i){e.push({cmd:"-",idx:i})}).execute(),e}function Jx(r,t,e,i,n,a,o,s){for(var l=jx(r,t),u=[],f=[],h=[],c=[],v=[],d=[],y=[],p=$y(n,t,o),g=r.getLayout("points")||[],m=t.getLayout("points")||[],_=0;_<l.length;_++){var S=l[_],b=!0,w=void 0,x=void 0;switch(S.cmd){case"=":w=S.idx*2,x=S.idx1*2;var C=g[w],D=g[w+1],A=m[x],M=m[x+1];(isNaN(C)||isNaN(D))&&(C=A,D=M),u.push(C,D),f.push(A,M),h.push(e[w],e[w+1]),c.push(i[x],i[x+1]),y.push(t.getRawIndex(S.idx1));break;case"+":var I=S.idx,L=p.dataDimsForPoint,P=n.dataToPoint([t.get(L[0],I),t.get(L[1],I)]);x=I*2,u.push(P[0],P[1]),f.push(m[x],m[x+1]);var R=Zy(p,n,t,I);h.push(R[0],R[1]),c.push(i[x],i[x+1]),y.push(t.getRawIndex(I));break;case"-":b=!1}b&&(v.push(S),d.push(d.length))}d.sort(function($t,ct){return y[$t]-y[ct]});for(var E=u.length,W=Ke(E),B=Ke(E),F=Ke(E),G=Ke(E),q=[],_=0;_<d.length;_++){var it=d[_],lt=_*2,vt=it*2;W[lt]=u[vt],W[lt+1]=u[vt+1],B[lt]=f[vt],B[lt+1]=f[vt+1],F[lt]=h[vt],F[lt+1]=h[vt+1],G[lt]=c[vt],G[lt+1]=c[vt+1],q[_]=v[it]}return{current:W,next:B,stackedOnCurrent:F,stackedOnNext:G,status:q}}var sr=Math.min,lr=Math.max;function ii(r,t){return isNaN(r)||isNaN(t)}function Ru(r,t,e,i,n,a,o,s,l){for(var u,f,h,c,v,d,y=e,p=0;p<i;p++){var g=t[y*2],m=t[y*2+1];if(y>=n||y<0)break;if(ii(g,m)){if(l){y+=a;continue}break}if(y===e)r[a>0?"moveTo":"lineTo"](g,m),h=g,c=m;else{var _=g-u,S=m-f;if(_*_+S*S<.5){y+=a;continue}if(o>0){for(var b=y+a,w=t[b*2],x=t[b*2+1];w===g&&x===m&&p<i;)p++,b+=a,y+=a,w=t[b*2],x=t[b*2+1],g=t[y*2],m=t[y*2+1],_=g-u,S=m-f;var C=p+1;if(l)for(;ii(w,x)&&C<i;)C++,b+=a,w=t[b*2],x=t[b*2+1];var D=.5,A=0,M=0,I=void 0,L=void 0;if(C>=i||ii(w,x))v=g,d=m;else{A=w-u,M=x-f;var P=g-u,R=w-g,E=m-f,W=x-m,B=void 0,F=void 0;if(s==="x"){B=Math.abs(P),F=Math.abs(R);var G=A>0?1:-1;v=g-G*B*o,d=m,I=g+G*F*o,L=m}else if(s==="y"){B=Math.abs(E),F=Math.abs(W);var q=M>0?1:-1;v=g,d=m-q*B*o,I=g,L=m+q*F*o}else B=Math.sqrt(P*P+E*E),F=Math.sqrt(R*R+W*W),D=F/(F+B),v=g-A*o*(1-D),d=m-M*o*(1-D),I=g+A*o*D,L=m+M*o*D,I=sr(I,lr(w,g)),L=sr(L,lr(x,m)),I=lr(I,sr(w,g)),L=lr(L,sr(x,m)),A=I-g,M=L-m,v=g-A*B/F,d=m-M*B/F,v=sr(v,lr(u,g)),d=sr(d,lr(f,m)),v=lr(v,sr(u,g)),d=lr(d,sr(f,m)),A=g-v,M=m-d,I=g+A*F/B,L=m+M*F/B}r.bezierCurveTo(h,c,v,d,g,m),h=I,c=L}else r.lineTo(g,m)}u=g,f=m,y+=a}return p}var Ky=function(){function r(){this.smooth=0,this.smoothConstraint=!0}return r}(),tT=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;return i.type="ec-polyline",i}return t.prototype.getDefaultStyle=function(){return{stroke:"#000",fill:null}},t.prototype.getDefaultShape=function(){return new Ky},t.prototype.buildPath=function(e,i){var n=i.points,a=0,o=n.length/2;if(i.connectNulls){for(;o>0&&ii(n[o*2-2],n[o*2-1]);o--);for(;a<o&&ii(n[a*2],n[a*2+1]);a++);}for(;a<o;)a+=Ru(e,n,a,o,o,1,i.smooth,i.smoothMonotone,i.connectNulls)+1},t.prototype.getPointOn=function(e,i){this.path||(this.createPathProxy(),this.buildPath(this.path,this.shape));for(var n=this.path,a=n.data,o=li.CMD,s,l,u=i==="x",f=[],h=0;h<a.length;){var c=a[h++],v=void 0,d=void 0,y=void 0,p=void 0,g=void 0,m=void 0,_=void 0;switch(c){case o.M:s=a[h++],l=a[h++];break;case o.L:if(v=a[h++],d=a[h++],_=u?(e-s)/(v-s):(e-l)/(d-l),_<=1&&_>=0){var S=u?(d-l)*_+l:(v-s)*_+s;return u?[e,S]:[S,e]}s=v,l=d;break;case o.C:v=a[h++],d=a[h++],y=a[h++],p=a[h++],g=a[h++],m=a[h++];var b=u?Co(s,v,y,g,e,f):Co(l,d,p,m,e,f);if(b>0)for(var w=0;w<b;w++){var x=f[w];if(x<=1&&x>=0){var S=u?bt(l,d,p,m,x):bt(s,v,y,g,x);return u?[e,S]:[S,e]}}s=g,l=m;break}}},t}(st),eT=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t}(Ky),rT=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;return i.type="ec-polygon",i}return t.prototype.getDefaultShape=function(){return new eT},t.prototype.buildPath=function(e,i){var n=i.points,a=i.stackedOnPoints,o=0,s=n.length/2,l=i.smoothMonotone;if(i.connectNulls){for(;s>0&&ii(n[s*2-2],n[s*2-1]);s--);for(;o<s&&ii(n[o*2],n[o*2+1]);o++);}for(;o<s;){var u=Ru(e,n,o,s,s,1,i.smooth,l,i.connectNulls);Ru(e,a,o+u-1,u,s,-1,i.stackedOnSmooth,l,i.connectNulls),o+=u+1,e.closePath()}},t}(st);function rh(){var r=_t();return function(t){var e=r(t),i=t.pipelineContext,n=!!e.large,a=!!e.progressiveRender,o=e.large=!!(i&&i.large),s=e.progressiveRender=!!(i&&i.progressiveRender);return(n!==o||a!==s)&&"reset"}}var Qy=_t(),iT=rh(),Jt=function(){function r(){this.group=new mt,this.uid=ws("viewChart"),this.renderTask=Bn({plan:nT,reset:aT}),this.renderTask.context={view:this}}return r.prototype.init=function(t,e){},r.prototype.render=function(t,e,i,n){},r.prototype.highlight=function(t,e,i,n){var a=t.getData(n&&n.dataType);a&&ic(a,n,"emphasis")},r.prototype.downplay=function(t,e,i,n){var a=t.getData(n&&n.dataType);a&&ic(a,n,"normal")},r.prototype.remove=function(t,e){this.group.removeAll()},r.prototype.dispose=function(t,e){},r.prototype.updateView=function(t,e,i,n){this.render(t,e,i,n)},r.prototype.updateLayout=function(t,e,i,n){this.render(t,e,i,n)},r.prototype.updateVisual=function(t,e,i,n){this.render(t,e,i,n)},r.prototype.eachRendered=function(t){cs(this.group,t)},r.markUpdateMethod=function(t,e){Qy(t).updateMethod=e},r.protoInitialize=function(){var t=r.prototype;t.type="chart"}(),r}();function rc(r,t,e){r&&mu(r)&&(t==="emphasis"?Po:Ro)(r,e)}function ic(r,t,e){var i=ui(r,t),n=t&&t.highlightKey!=null?qS(t.highlightKey):null;i!=null?T(Tt(i),function(a){rc(r.getItemGraphicEl(a),e,n)}):r.eachItemGraphicEl(function(a){rc(a,e,n)})}vf(Jt);Jo(Jt);function nT(r){return iT(r.model)}function aT(r){var t=r.model,e=r.ecModel,i=r.api,n=r.payload,a=t.pipelineContext.progressiveRender,o=r.view,s=n&&Qy(n).updateMethod,l=a?"incrementalPrepareRender":s&&o[s]?s:"render";return l!=="render"&&o[l](t,e,i,n),oT[l]}var oT={incrementalPrepareRender:{progress:function(r,t){t.view.incrementalRender(r,t.model,t.ecModel,t.api,t.payload)}},render:{forceFirstProgress:!0,progress:function(r,t){t.view.render(t.model,t.ecModel,t.api,t.payload)}}};function jy(r,t,e,i,n){var a=r.getArea(),o=a.x,s=a.y,l=a.width,u=a.height,f=e.get(["lineStyle","width"])||2;o-=f/2,s-=f/2,l+=f,u+=f,l=Math.ceil(l),o!==Math.floor(o)&&(o=Math.floor(o),l++);var h=new gt({shape:{x:o,y:s,width:l,height:u}});if(t){var c=r.getBaseAxis(),v=c.isHorizontal(),d=c.inverse;v?(d&&(h.shape.x+=l),h.shape.width=0):(d||(h.shape.y+=u),h.shape.height=0);var y=Y(n)?function(p){n(p,h)}:null;He(h,{shape:{width:l,height:u,x:o,y:s}},e,null,i,y)}return h}function Jy(r,t,e){var i=r.getArea(),n=pt(i.r0,1),a=pt(i.r,1),o=new Xi({shape:{cx:pt(r.cx,1),cy:pt(r.cy,1),r0:n,r:a,startAngle:i.startAngle,endAngle:i.endAngle,clockwise:i.clockwise}});if(t){var s=r.getBaseAxis().dim==="angle";s?o.shape.endAngle=i.startAngle:o.shape.r=n,He(o,{shape:{endAngle:i.endAngle,r:a}},e)}return o}function sT(r,t,e,i,n){if(r){if(r.type==="polar")return Jy(r,t,e);if(r.type==="cartesian2d")return jy(r,t,e,i,n)}else return null;return null}function ih(r,t){return r.type===t}function nc(r,t){if(r.length===t.length){for(var e=0;e<r.length;e++)if(r[e]!==t[e])return;return!0}}function ac(r){for(var t=1/0,e=1/0,i=-1/0,n=-1/0,a=0;a<r.length;){var o=r[a++],s=r[a++];isNaN(o)||(t=Math.min(o,t),i=Math.max(o,i)),isNaN(s)||(e=Math.min(s,e),n=Math.max(s,n))}return[[t,e],[i,n]]}function oc(r,t){var e=ac(r),i=e[0],n=e[1],a=ac(t),o=a[0],s=a[1];return Math.max(Math.abs(i[0]-o[0]),Math.abs(i[1]-o[1]),Math.abs(n[0]-s[0]),Math.abs(n[1]-s[1]))}function sc(r){return ht(r)?r:r?.5:0}function lT(r,t,e){if(!e.valueDim)return[];for(var i=t.count(),n=Ke(i*2),a=0;a<i;a++){var o=Zy(e,r,t,a);n[a*2]=o[0],n[a*2+1]=o[1]}return n}function ur(r,t,e,i){var n=t.getBaseAxis(),a=n.dim==="x"||n.dim==="radius"?0:1,o=[],s=0,l=[],u=[],f=[],h=[];if(i){for(s=0;s<r.length;s+=2)!isNaN(r[s])&&!isNaN(r[s+1])&&h.push(r[s],r[s+1]);r=h}for(s=0;s<r.length-2;s+=2)switch(f[0]=r[s+2],f[1]=r[s+3],u[0]=r[s],u[1]=r[s+1],o.push(u[0],u[1]),e){case"end":l[a]=f[a],l[1-a]=u[1-a],o.push(l[0],l[1]);break;case"middle":var c=(u[a]+f[a])/2,v=[];l[a]=v[a]=c,l[1-a]=u[1-a],v[1-a]=f[1-a],o.push(l[0],l[1]),o.push(v[0],v[1]);break;default:l[a]=u[a],l[1-a]=f[1-a],o.push(l[0],l[1])}return o.push(r[s++],r[s++]),o}function uT(r,t){var e=[],i=r.length,n,a;function o(f,h,c){var v=f.coord,d=(c-v)/(h.coord-v),y=_f(d,[f.color,h.color]);return{coord:c,color:y}}for(var s=0;s<i;s++){var l=r[s],u=l.coord;if(u<0)n=l;else if(u>t){a?e.push(o(a,l,t)):n&&e.push(o(n,l,0),o(n,l,t));break}else n&&(e.push(o(n,l,0)),n=null),e.push(l),a=l}return e}function fT(r,t,e){var i=r.getVisual("visualMeta");if(!(!i||!i.length||!r.count())&&t.type==="cartesian2d"){for(var n,a,o=i.length-1;o>=0;o--){var s=r.getDimensionInfo(i[o].dimension);if(n=s&&s.coordDim,n==="x"||n==="y"){a=i[o];break}}if(a){var l=t.getAxis(n),u=H(a.stops,function(_){return{coord:l.toGlobalCoord(l.dataToCoord(_.value)),color:_.color}}),f=u.length,h=a.outerColors.slice();f&&u[0].coord>u[f-1].coord&&(u.reverse(),h.reverse());var c=uT(u,n==="x"?e.getWidth():e.getHeight()),v=c.length;if(!v&&f)return u[0].coord<0?h[1]?h[1]:u[f-1].color:h[0]?h[0]:u[0].color;var d=10,y=c[0].coord-d,p=c[v-1].coord+d,g=p-y;if(g<.001)return"transparent";T(c,function(_){_.offset=(_.coord-y)/g}),c.push({offset:v?c[v-1].offset:.5,color:h[1]||"transparent"}),c.unshift({offset:v?c[0].offset:.5,color:h[0]||"transparent"});var m=new hs(0,0,0,0,c,!0);return m[n]=y,m[n+"2"]=p,m}}}function hT(r,t,e){var i=r.get("showAllSymbol"),n=i==="auto";if(!(i&&!n)){var a=e.getAxesByScale("ordinal")[0];if(a&&!(n&&vT(a,t))){var o=t.mapDimension(a.dim),s={};return T(a.getViewLabels(),function(l){var u=a.scale.getRawOrdinalNumber(l.tickValue);s[u]=1}),function(l){return!s.hasOwnProperty(t.get(o,l))}}}}function vT(r,t){var e=r.getExtent(),i=Math.abs(e[1]-e[0])/r.scale.count();isNaN(i)&&(i=0);for(var n=t.count(),a=Math.max(1,Math.round(n/5)),o=0;o<n;o+=a)if(eh.getSymbolSize(t,o)[r.isHorizontal()?1:0]*1.5>i)return!1;return!0}function cT(r,t){return isNaN(r)||isNaN(t)}function dT(r){for(var t=r.length/2;t>0&&cT(r[t*2-2],r[t*2-1]);t--);return t-1}function lc(r,t){return[r[t*2],r[t*2+1]]}function pT(r,t,e){for(var i=r.length/2,n=e==="x"?0:1,a,o,s=0,l=-1,u=0;u<i;u++)if(o=r[u*2+n],!(isNaN(o)||isNaN(r[u*2+1-n]))){if(u===0){a=o;continue}if(a<=t&&o>=t||a>=t&&o<=t){l=u;break}s=u,a=o}return{range:[s,l],t:(t-a)/(o-a)}}function tm(r){if(r.get(["endLabel","show"]))return!0;for(var t=0;t<Ve.length;t++)if(r.get([Ve[t],"endLabel","show"]))return!0;return!1}function pl(r,t,e,i){if(ih(t,"cartesian2d")){var n=i.getModel("endLabel"),a=n.get("valueAnimation"),o=i.getData(),s={lastFrameIndex:0},l=tm(i)?function(v,d){r._endLabelOnDuring(v,d,o,s,a,n,t)}:null,u=t.getBaseAxis().isHorizontal(),f=jy(t,e,i,function(){var v=r._endLabel;v&&e&&s.originalX!=null&&v.attr({x:s.originalX,y:s.originalY})},l);if(!i.get("clip",!0)){var h=f.shape,c=Math.max(h.width,h.height);u?(h.y-=c,h.height+=c*2):(h.x-=c,h.width+=c*2)}return l&&l(1,f),f}else return Jy(t,e,i)}function gT(r,t){var e=t.getBaseAxis(),i=e.isHorizontal(),n=e.inverse,a=i?n?"right":"left":"center",o=i?"middle":n?"top":"bottom";return{normal:{align:r.get("align")||a,verticalAlign:r.get("verticalAlign")||o}}}var yT=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.init=function(){var e=new mt,i=new qx;this.group.add(i.group),this._symbolDraw=i,this._lineGroup=e},t.prototype.render=function(e,i,n){var a=this,o=e.coordinateSystem,s=this.group,l=e.getData(),u=e.getModel("lineStyle"),f=e.getModel("areaStyle"),h=l.getLayout("points")||[],c=o.type==="polar",v=this._coordSys,d=this._symbolDraw,y=this._polyline,p=this._polygon,g=this._lineGroup,m=!i.ssr&&e.get("animation"),_=!f.isEmpty(),S=f.get("origin"),b=$y(o,l,S),w=_&&lT(o,l,b),x=e.get("showSymbol"),C=e.get("connectNulls"),D=x&&!c&&hT(e,l,o),A=this._data;A&&A.eachItemGraphicEl(function(ct,Ye){ct.__temp&&(s.remove(ct),A.setItemGraphicEl(Ye,null))}),x||d.remove(),s.add(g);var M=c?!1:e.get("step"),I;o&&o.getArea&&e.get("clip",!0)&&(I=o.getArea(),I.width!=null?(I.x-=.1,I.y-=.1,I.width+=.2,I.height+=.2):I.r0&&(I.r0-=.5,I.r+=.5)),this._clipShapeForSymbol=I;var L=fT(l,o,n)||l.getVisual("style")[l.getVisual("drawType")];if(!(y&&v.type===o.type&&M===this._step))x&&d.updateData(l,{isIgnore:D,clipShape:I,disableAnimation:!0,getSymbolPoint:function(ct){return[h[ct*2],h[ct*2+1]]}}),m&&this._initSymbolLabelAnimation(l,o,I),M&&(h=ur(h,o,M,C),w&&(w=ur(w,o,M,C))),y=this._newPolyline(h),_?p=this._newPolygon(h,w):p&&(g.remove(p),p=this._polygon=null),c||this._initOrUpdateEndLabel(e,o,hi(L)),g.setClipPath(pl(this,o,!0,e));else{_&&!p?p=this._newPolygon(h,w):p&&!_&&(g.remove(p),p=this._polygon=null),c||this._initOrUpdateEndLabel(e,o,hi(L));var P=g.getClipPath();if(P){var R=pl(this,o,!1,e);He(P,{shape:R.shape},e)}else g.setClipPath(pl(this,o,!0,e));x&&d.updateData(l,{isIgnore:D,clipShape:I,disableAnimation:!0,getSymbolPoint:function(ct){return[h[ct*2],h[ct*2+1]]}}),(!nc(this._stackedOnPoints,w)||!nc(this._points,h))&&(m?this._doUpdateAnimation(l,w,o,n,M,S,C):(M&&(h=ur(h,o,M,C),w&&(w=ur(w,o,M,C))),y.setShape({points:h}),p&&p.setShape({points:h,stackedOnPoints:w})))}var E=e.getModel("emphasis"),W=E.get("focus"),B=E.get("blurScope"),F=E.get("disabled");if(y.useStyle(nt(u.getLineStyle(),{fill:"none",stroke:L,lineJoin:"bevel"})),yu(y,e,"lineStyle"),y.style.lineWidth>0&&e.get(["emphasis","lineStyle","width"])==="bolder"){var G=y.getState("emphasis").style;G.lineWidth=+y.style.lineWidth+1}tt(y).seriesIndex=e.seriesIndex,Eo(y,W,B,F);var q=sc(e.get("smooth")),it=e.get("smoothMonotone");if(y.setShape({smooth:q,smoothMonotone:it,connectNulls:C}),p){var lt=l.getCalculationInfo("stackedOnSeries"),vt=0;p.useStyle(nt(f.getAreaStyle(),{fill:L,opacity:.7,lineJoin:"bevel",decal:l.getVisual("style").decal})),lt&&(vt=sc(lt.get("smooth"))),p.setShape({smooth:q,stackedOnSmooth:vt,smoothMonotone:it,connectNulls:C}),yu(p,e,"areaStyle"),tt(p).seriesIndex=e.seriesIndex,Eo(p,W,B,F)}var $t=function(ct){a._changePolyState(ct)};l.eachItemGraphicEl(function(ct){ct&&(ct.onHoverStateChange=$t)}),this._polyline.onHoverStateChange=$t,this._data=l,this._coordSys=o,this._stackedOnPoints=w,this._points=h,this._step=M,this._valueOrigin=S,e.get("triggerLineEvent")&&(this.packEventData(e,y),p&&this.packEventData(e,p))},t.prototype.packEventData=function(e,i){tt(i).eventData={componentType:"series",componentSubType:"line",componentIndex:e.componentIndex,seriesIndex:e.seriesIndex,seriesName:e.name,seriesType:"line"}},t.prototype.highlight=function(e,i,n,a){var o=e.getData(),s=ui(o,a);if(this._changePolyState("emphasis"),!(s instanceof Array)&&s!=null&&s>=0){var l=o.getLayout("points"),u=o.getItemGraphicEl(s);if(!u){var f=l[s*2],h=l[s*2+1];if(isNaN(f)||isNaN(h)||this._clipShapeForSymbol&&!this._clipShapeForSymbol.contain(f,h))return;var c=e.get("zlevel")||0,v=e.get("z")||0;u=new eh(o,s),u.x=f,u.y=h,u.setZ(c,v);var d=u.getSymbolPath().getTextContent();d&&(d.zlevel=c,d.z=v,d.z2=this._polyline.z2+1),u.__temp=!0,o.setItemGraphicEl(s,u),u.stopSymbolAnimation(!0),this.group.add(u)}u.highlight()}else Jt.prototype.highlight.call(this,e,i,n,a)},t.prototype.downplay=function(e,i,n,a){var o=e.getData(),s=ui(o,a);if(this._changePolyState("normal"),s!=null&&s>=0){var l=o.getItemGraphicEl(s);l&&(l.__temp?(o.setItemGraphicEl(s,null),this.group.remove(l)):l.downplay())}else Jt.prototype.downplay.call(this,e,i,n,a)},t.prototype._changePolyState=function(e){var i=this._polygon;sv(this._polyline,e),i&&sv(i,e)},t.prototype._newPolyline=function(e){var i=this._polyline;return i&&this._lineGroup.remove(i),i=new tT({shape:{points:e},segmentIgnoreThreshold:2,z2:10}),this._lineGroup.add(i),this._polyline=i,i},t.prototype._newPolygon=function(e,i){var n=this._polygon;return n&&this._lineGroup.remove(n),n=new rT({shape:{points:e,stackedOnPoints:i},segmentIgnoreThreshold:2}),this._lineGroup.add(n),this._polygon=n,n},t.prototype._initSymbolLabelAnimation=function(e,i,n){var a,o,s=i.getBaseAxis(),l=s.inverse;i.type==="cartesian2d"?(a=s.isHorizontal(),o=!1):i.type==="polar"&&(a=s.dim==="angle",o=!0);var u=e.hostModel,f=u.get("animationDuration");Y(f)&&(f=f(null));var h=u.get("animationDelay")||0,c=Y(h)?h(null):h;e.eachItemGraphicEl(function(v,d){var y=v;if(y){var p=[v.x,v.y],g=void 0,m=void 0,_=void 0;if(n)if(o){var S=n,b=i.pointToCoord(p);a?(g=S.startAngle,m=S.endAngle,_=-b[1]/180*Math.PI):(g=S.r0,m=S.r,_=b[0])}else{var w=n;a?(g=w.x,m=w.x+w.width,_=v.x):(g=w.y+w.height,m=w.y,_=v.y)}var x=m===g?0:(_-g)/(m-g);l&&(x=1-x);var C=Y(h)?h(d):f*x+c,D=y.getSymbolPath(),A=D.getTextContent();y.attr({scaleX:0,scaleY:0}),y.animateTo({scaleX:1,scaleY:1},{duration:200,setToFinal:!0,delay:C}),A&&A.animateFrom({style:{opacity:0}},{duration:300,delay:C}),D.disableLabelAnimation=!0}})},t.prototype._initOrUpdateEndLabel=function(e,i,n){var a=e.getModel("endLabel");if(tm(e)){var o=e.getData(),s=this._polyline,l=o.getLayout("points");if(!l){s.removeTextContent(),this._endLabel=null;return}var u=this._endLabel;u||(u=this._endLabel=new yt({z2:200}),u.ignoreClip=!0,s.setTextContent(this._endLabel),s.disableLabelAnimation=!0);var f=dT(l);f>=0&&(Ff(s,ps(e,"endLabel"),{inheritColor:n,labelFetcher:e,labelDataIndex:f,defaultText:function(h,c,v){return v!=null?Xy(o,v):th(o,h)},enableTextSetter:!0},gT(a,i)),s.textConfig.position=null)}else this._endLabel&&(this._polyline.removeTextContent(),this._endLabel=null)},t.prototype._endLabelOnDuring=function(e,i,n,a,o,s,l){var u=this._endLabel,f=this._polyline;if(u){e<1&&a.originalX==null&&(a.originalX=u.x,a.originalY=u.y);var h=n.getLayout("points"),c=n.hostModel,v=c.get("connectNulls"),d=s.get("precision"),y=s.get("distance")||0,p=l.getBaseAxis(),g=p.isHorizontal(),m=p.inverse,_=i.shape,S=m?g?_.x:_.y+_.height:g?_.x+_.width:_.y,b=(g?y:0)*(m?-1:1),w=(g?0:-y)*(m?-1:1),x=g?"x":"y",C=pT(h,S,x),D=C.range,A=D[1]-D[0],M=void 0;if(A>=1){if(A>1&&!v){var I=lc(h,D[0]);u.attr({x:I[0]+b,y:I[1]+w}),o&&(M=c.getRawValue(D[0]))}else{var I=f.getPointOn(S,x);I&&u.attr({x:I[0]+b,y:I[1]+w});var L=c.getRawValue(D[0]),P=c.getRawValue(D[1]);o&&(M=IS(n,d,L,P,C.t))}a.lastFrameIndex=D[0]}else{var R=e===1||a.lastFrameIndex>0?D[0]:0,I=lc(h,R);o&&(M=c.getRawValue(R)),u.attr({x:I[0]+b,y:I[1]+w})}if(o){var E=gs(u);typeof E.setLabelText=="function"&&E.setLabelText(M)}}},t.prototype._doUpdateAnimation=function(e,i,n,a,o,s,l){var u=this._polyline,f=this._polygon,h=e.hostModel,c=Jx(this._data,e,this._stackedOnPoints,i,this._coordSys,n,this._valueOrigin),v=c.current,d=c.stackedOnCurrent,y=c.next,p=c.stackedOnNext;if(o&&(v=ur(c.current,n,o,l),d=ur(c.stackedOnCurrent,n,o,l),y=ur(c.next,n,o,l),p=ur(c.stackedOnNext,n,o,l)),oc(v,y)>3e3||f&&oc(d,p)>3e3){u.stopAnimation(),u.setShape({points:y}),f&&(f.stopAnimation(),f.setShape({points:y,stackedOnPoints:p}));return}u.shape.__points=c.current,u.shape.points=v;var g={shape:{points:y}};c.current!==v&&(g.shape.__points=c.next),u.stopAnimation(),te(u,g,h),f&&(f.setShape({points:v,stackedOnPoints:d}),f.stopAnimation(),te(f,{shape:{stackedOnPoints:p}},h),u.shape.points!==f.shape.points&&(f.shape.points=u.shape.points));for(var m=[],_=c.status,S=0;S<_.length;S++){var b=_[S].cmd;if(b==="="){var w=e.getItemGraphicEl(_[S].idx1);w&&m.push({el:w,ptIdx:S})}}u.animators&&u.animators.length&&u.animators[0].during(function(){f&&f.dirtyShape();for(var x=u.shape.__points,C=0;C<m.length;C++){var D=m[C].el,A=m[C].ptIdx*2;D.x=x[A],D.y=x[A+1],D.markRedraw()}})},t.prototype.remove=function(e){var i=this.group,n=this._data;this._lineGroup.removeAll(),this._symbolDraw.remove(!0),n&&n.eachItemGraphicEl(function(a,o){a.__temp&&(i.remove(a),n.setItemGraphicEl(o,null))}),this._polyline=this._polygon=this._coordSys=this._points=this._stackedOnPoints=this._endLabel=this._data=null},t.type="line",t}(Jt);function mT(r,t){return{seriesType:r,plan:rh(),reset:function(e){var i=e.getData(),n=e.coordinateSystem;if(e.pipelineContext,!!n){var a=H(n.dimensions,function(h){return i.mapDimension(h)}).slice(0,2),o=a.length,s=i.getCalculationInfo("stackResultDimension");fi(i,a[0])&&(a[0]=s),fi(i,a[1])&&(a[1]=s);var l=i.getStore(),u=i.getDimensionIndex(a[0]),f=i.getDimensionIndex(a[1]);return o&&{progress:function(h,c){for(var v=h.end-h.start,d=Ke(v*o),y=[],p=[],g=h.start,m=0;g<h.end;g++){var _=void 0;if(o===1){var S=l.get(u,g);_=n.dataToPoint(S,null,p)}else y[0]=l.get(u,g),y[1]=l.get(f,g),_=n.dataToPoint(y,null,p);d[m++]=_[0],d[m++]=_[1]}c.setLayout("points",d)}}}}}}var _T={average:function(r){for(var t=0,e=0,i=0;i<r.length;i++)isNaN(r[i])||(t+=r[i],e++);return e===0?NaN:t/e},sum:function(r){for(var t=0,e=0;e<r.length;e++)t+=r[e]||0;return t},max:function(r){for(var t=-1/0,e=0;e<r.length;e++)r[e]>t&&(t=r[e]);return isFinite(t)?t:NaN},min:function(r){for(var t=1/0,e=0;e<r.length;e++)r[e]<t&&(t=r[e]);return isFinite(t)?t:NaN},minmax:function(r){for(var t=-1/0,e=-1/0,i=0;i<r.length;i++){var n=r[i],a=Math.abs(n);a>t&&(t=a,e=n)}return isFinite(e)?e:NaN},nearest:function(r){return r[0]}},ST=function(r){return Math.round(r.length/2)};function em(r){return{seriesType:r,reset:function(t,e,i){var n=t.getData(),a=t.get("sampling"),o=t.coordinateSystem,s=n.count();if(s>10&&o.type==="cartesian2d"&&a){var l=o.getBaseAxis(),u=o.getOtherAxis(l),f=l.getExtent(),h=i.getDevicePixelRatio(),c=Math.abs(f[1]-f[0])*(h||1),v=Math.round(s/c);if(isFinite(v)&&v>1){a==="lttb"&&t.setData(n.lttbDownSample(n.mapDimension(u.dim),1/v));var d=void 0;V(a)?d=_T[a]:Y(a)&&(d=a),d&&t.setData(n.downSample(n.mapDimension(u.dim),1/v,d,ST))}}}}}function eL(r){r.registerChartView(yT),r.registerSeriesModel($x),r.registerLayout(mT("line")),r.registerVisual({seriesType:"line",reset:function(t){var e=t.getData(),i=t.getModel("lineStyle").getLineStyle();i&&!i.stroke&&(i.stroke=e.getVisual("style").fill),e.setVisual("legendLineStyle",i)}}),r.registerProcessor(r.PRIORITY.PROCESSOR.STATISTIC,em("line"))}var wT="__ec_stack_";function nh(r){return r.get("stack")||wT+r.seriesIndex}function ah(r){return r.dim+r.index}function rm(r,t){var e=[];return t.eachSeriesByType(r,function(i){nm(i)&&e.push(i)}),e}function bT(r){var t={};T(r,function(l){var u=l.coordinateSystem,f=u.getBaseAxis();if(!(f.type!=="time"&&f.type!=="value"))for(var h=l.getData(),c=f.dim+"_"+f.index,v=h.getDimensionIndex(h.mapDimension(f.dim)),d=h.getStore(),y=0,p=d.count();y<p;++y){var g=d.get(v,y);t[c]?t[c].push(g):t[c]=[g]}});var e={};for(var i in t)if(t.hasOwnProperty(i)){var n=t[i];if(n){n.sort(function(l,u){return l-u});for(var a=null,o=1;o<n.length;++o){var s=n[o]-n[o-1];s>0&&(a=a===null?s:Math.min(a,s))}e[i]=a}}return e}function im(r){var t=bT(r),e=[];return T(r,function(i){var n=i.coordinateSystem,a=n.getBaseAxis(),o=a.getExtent(),s;if(a.type==="category")s=a.getBandWidth();else if(a.type==="value"||a.type==="time"){var l=a.dim+"_"+a.index,u=t[l],f=Math.abs(o[1]-o[0]),h=a.scale.getExtent(),c=Math.abs(h[1]-h[0]);s=u?f/c*u:f}else{var v=i.getData();s=Math.abs(o[1]-o[0])/v.count()}var d=Ot(i.get("barWidth"),s),y=Ot(i.get("barMaxWidth"),s),p=Ot(i.get("barMinWidth")||(am(i)?.5:1),s),g=i.get("barGap"),m=i.get("barCategoryGap");e.push({bandWidth:s,barWidth:d,barMaxWidth:y,barMinWidth:p,barGap:g,barCategoryGap:m,axisKey:ah(a),stackId:nh(i)})}),xT(e)}function xT(r){var t={};T(r,function(i,n){var a=i.axisKey,o=i.bandWidth,s=t[a]||{bandWidth:o,remainedWidth:o,autoWidthCount:0,categoryGap:null,gap:"20%",stacks:{}},l=s.stacks;t[a]=s;var u=i.stackId;l[u]||s.autoWidthCount++,l[u]=l[u]||{width:0,maxWidth:0};var f=i.barWidth;f&&!l[u].width&&(l[u].width=f,f=Math.min(s.remainedWidth,f),s.remainedWidth-=f);var h=i.barMaxWidth;h&&(l[u].maxWidth=h);var c=i.barMinWidth;c&&(l[u].minWidth=c);var v=i.barGap;v!=null&&(s.gap=v);var d=i.barCategoryGap;d!=null&&(s.categoryGap=d)});var e={};return T(t,function(i,n){e[n]={};var a=i.stacks,o=i.bandWidth,s=i.categoryGap;if(s==null){var l=ft(a).length;s=Math.max(35-l*4,15)+"%"}var u=Ot(s,o),f=Ot(i.gap,1),h=i.remainedWidth,c=i.autoWidthCount,v=(h-u)/(c+(c-1)*f);v=Math.max(v,0),T(a,function(g){var m=g.maxWidth,_=g.minWidth;if(g.width){var S=g.width;m&&(S=Math.min(S,m)),_&&(S=Math.max(S,_)),g.width=S,h-=S+f*S,c--}else{var S=v;m&&m<S&&(S=Math.min(m,h)),_&&_>S&&(S=_),S!==v&&(g.width=S,h-=S+f*S,c--)}}),v=(h-u)/(c+(c-1)*f),v=Math.max(v,0);var d=0,y;T(a,function(g,m){g.width||(g.width=v),y=g,d+=g.width*(1+f)}),y&&(d-=y.width*f);var p=-d/2;T(a,function(g,m){e[n][m]=e[n][m]||{bandWidth:o,offset:p,width:g.width},p+=g.width*(1+f)})}),e}function TT(r,t,e){if(r&&t){var i=r[ah(t)];return i!=null&&e!=null?i[nh(e)]:i}}function MT(r,t){var e=rm(r,t),i=im(e);T(e,function(n){var a=n.getData(),o=n.coordinateSystem,s=o.getBaseAxis(),l=nh(n),u=i[ah(s)][l],f=u.offset,h=u.width;a.setLayout({bandWidth:u.bandWidth,offset:f,size:h})})}function CT(r){return{seriesType:r,plan:rh(),reset:function(t){if(nm(t)){var e=t.getData(),i=t.coordinateSystem,n=i.getBaseAxis(),a=i.getOtherAxis(n),o=e.getDimensionIndex(e.mapDimension(a.dim)),s=e.getDimensionIndex(e.mapDimension(n.dim)),l=t.get("showBackground",!0),u=e.mapDimension(a.dim),f=e.getCalculationInfo("stackResultDimension"),h=fi(e,u)&&!!e.getCalculationInfo("stackedOnSeries"),c=a.isHorizontal(),v=DT(n,a),d=am(t),y=t.get("barMinHeight")||0,p=f&&e.getDimensionIndex(f),g=e.getLayout("size"),m=e.getLayout("offset");return{progress:function(_,S){for(var b=_.count,w=d&&Ke(b*3),x=d&&l&&Ke(b*3),C=d&&Ke(b),D=i.master.getRect(),A=c?D.width:D.height,M,I=S.getStore(),L=0;(M=_.next())!=null;){var P=I.get(h?p:o,M),R=I.get(s,M),E=v,W=void 0;h&&(W=+P-I.get(o,M));var B=void 0,F=void 0,G=void 0,q=void 0;if(c){var it=i.dataToPoint([P,R]);if(h){var lt=i.dataToPoint([W,R]);E=lt[0]}B=E,F=it[1]+m,G=it[0]-E,q=g,Math.abs(G)<y&&(G=(G<0?-1:1)*y)}else{var it=i.dataToPoint([R,P]);if(h){var lt=i.dataToPoint([R,W]);E=lt[1]}B=it[0]+m,F=E,G=g,q=it[1]-E,Math.abs(q)<y&&(q=(q<=0?-1:1)*y)}d?(w[L]=B,w[L+1]=F,w[L+2]=c?G:q,x&&(x[L]=c?D.x:B,x[L+1]=c?F:D.y,x[L+2]=A),C[M]=M):S.setItemLayout(M,{x:B,y:F,width:G,height:q}),L+=3}d&&S.setLayout({largePoints:w,largeDataIndices:C,largeBackgroundPoints:x,valueAxisHorizontal:c})}}}}}}function nm(r){return r.coordinateSystem&&r.coordinateSystem.type==="cartesian2d"}function am(r){return r.pipelineContext&&r.pipelineContext.large}function DT(r,t){var e=t.model.get("startValue");return e||(e=0),t.toGlobalCoord(t.dataToCoord(t.type==="log"?e>0?e:1:e))}var Eu=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.getInitialData=function(e,i){return Ss(null,this,{useEncodeDefaulter:!0})},t.prototype.getMarkerPosition=function(e,i,n){var a=this.coordinateSystem;if(a&&a.clampData){var o=a.clampData(e),s=a.dataToPoint(o);if(n)T(a.getAxes(),function(c,v){if(c.type==="category"&&i!=null){var d=c.getTicksCoords(),y=c.getTickModel().get("alignWithLabel"),p=o[v],g=i[v]==="x1"||i[v]==="y1";if(g&&!y&&(p+=1),d.length<2)return;if(d.length===2){s[v]=c.toGlobalCoord(c.getExtent()[g?1:0]);return}for(var m=void 0,_=void 0,S=1,b=0;b<d.length;b++){var w=d[b].coord,x=b===d.length-1?d[b-1].tickValue+S:d[b].tickValue;if(x===p){_=w;break}else if(x<p)m=w;else if(m!=null&&x>p){_=(w+m)/2;break}b===1&&(S=x-d[0].tickValue)}_==null&&(m?m&&(_=d[d.length-1].coord):_=d[0].coord),s[v]=c.toGlobalCoord(_)}});else{var l=this.getData(),u=l.getLayout("offset"),f=l.getLayout("size"),h=a.getBaseAxis().isHorizontal()?0:1;s[h]+=u+f/2}return s}return[NaN,NaN]},t.type="series.__base_bar__",t.defaultOption={z:2,coordinateSystem:"cartesian2d",legendHoverLink:!0,barMinHeight:0,barMinAngle:0,large:!1,largeThreshold:400,progressive:3e3,progressiveChunkMode:"mod"},t}(ce);ce.registerClass(Eu);var AT=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.getInitialData=function(){return Ss(null,this,{useEncodeDefaulter:!0,createInvertedIndices:!!this.get("realtimeSort",!0)||null})},t.prototype.getProgressive=function(){return this.get("large")?this.get("progressive"):!1},t.prototype.getProgressiveThreshold=function(){var e=this.get("progressiveThreshold"),i=this.get("largeThreshold");return i>e&&(e=i),e},t.prototype.brushSelector=function(e,i,n){return n.rect(i.getItemLayout(e))},t.type="series.bar",t.dependencies=["grid","polar"],t.defaultOption=Hf(Eu.defaultOption,{clip:!0,roundCap:!1,showBackground:!1,backgroundStyle:{color:"rgba(180, 180, 180, 0.2)",borderColor:null,borderWidth:0,borderType:"solid",borderRadius:0,shadowBlur:0,shadowColor:null,shadowOffsetX:0,shadowOffsetY:0,opacity:1},select:{itemStyle:{borderColor:"#212121"}},realtimeSort:!1}),t}(Eu),zo="\0__throttleOriginMethod",uc="\0__throttleRate",fc="\0__throttleType";function oh(r,t,e){var i,n=0,a=0,o=null,s,l,u,f;t=t||0;function h(){a=new Date().getTime(),o=null,r.apply(l,u||[])}var c=function(){for(var v=[],d=0;d<arguments.length;d++)v[d]=arguments[d];i=new Date().getTime(),l=this,u=v;var y=f||t,p=f||e;f=null,s=i-(p?n:a)-y,clearTimeout(o),p?o=setTimeout(h,y):s>=0?h():o=setTimeout(h,-s),n=i};return c.clear=function(){o&&(clearTimeout(o),o=null)},c.debounceNextCall=function(v){f=v},c}function om(r,t,e,i){var n=r[t];if(n){var a=n[zo]||n,o=n[fc],s=n[uc];if(s!==e||o!==i){if(e==null||!i)return r[t]=a;n=r[t]=oh(a,e,i==="debounce"),n[zo]=a,n[fc]=i,n[uc]=e}return n}}function Ou(r,t){var e=r[t];e&&e[zo]&&(e.clear&&e.clear(),r[t]=e[zo])}var IT=function(){function r(){this.cx=0,this.cy=0,this.r0=0,this.r=0,this.startAngle=0,this.endAngle=Math.PI*2,this.clockwise=!0}return r}(),hc=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;return i.type="sausage",i}return t.prototype.getDefaultShape=function(){return new IT},t.prototype.buildPath=function(e,i){var n=i.cx,a=i.cy,o=Math.max(i.r0||0,0),s=Math.max(i.r,0),l=(s-o)*.5,u=o+l,f=i.startAngle,h=i.endAngle,c=i.clockwise,v=Math.PI*2,d=c?h-f<v:f-h<v;d||(f=h-(c?v:-v));var y=Math.cos(f),p=Math.sin(f),g=Math.cos(h),m=Math.sin(h);d?(e.moveTo(y*o+n,p*o+a),e.arc(y*u+n,p*u+a,l,-Math.PI+f,f,!c)):e.moveTo(y*s+n,p*s+a),e.arc(n,a,s,f,h,!c),e.arc(g*u+n,m*u+a,l,h-Math.PI*2,h-Math.PI,!c),o!==0&&e.arc(n,a,o,h,f,c)},t}(st);function LT(r,t){t=t||{};var e=t.isRoundCap;return function(i,n,a){var o=n.position;if(!o||o instanceof Array)return xo(i,n,a);var s=r(o),l=n.distance!=null?n.distance:5,u=this.shape,f=u.cx,h=u.cy,c=u.r,v=u.r0,d=(c+v)/2,y=u.startAngle,p=u.endAngle,g=(y+p)/2,m=e?Math.abs(c-v)/2:0,_=Math.cos,S=Math.sin,b=f+c*_(y),w=h+c*S(y),x="left",C="top";switch(s){case"startArc":b=f+(v-l)*_(g),w=h+(v-l)*S(g),x="center",C="top";break;case"insideStartArc":b=f+(v+l)*_(g),w=h+(v+l)*S(g),x="center",C="bottom";break;case"startAngle":b=f+d*_(y)+Ha(y,l+m,!1),w=h+d*S(y)+Ga(y,l+m,!1),x="right",C="middle";break;case"insideStartAngle":b=f+d*_(y)+Ha(y,-l+m,!1),w=h+d*S(y)+Ga(y,-l+m,!1),x="left",C="middle";break;case"middle":b=f+d*_(g),w=h+d*S(g),x="center",C="middle";break;case"endArc":b=f+(c+l)*_(g),w=h+(c+l)*S(g),x="center",C="bottom";break;case"insideEndArc":b=f+(c-l)*_(g),w=h+(c-l)*S(g),x="center",C="top";break;case"endAngle":b=f+d*_(p)+Ha(p,l+m,!0),w=h+d*S(p)+Ga(p,l+m,!0),x="left",C="middle";break;case"insideEndAngle":b=f+d*_(p)+Ha(p,-l+m,!0),w=h+d*S(p)+Ga(p,-l+m,!0),x="right",C="middle";break;default:return xo(i,n,a)}return i=i||{},i.x=b,i.y=w,i.align=x,i.verticalAlign=C,i}}function PT(r,t,e,i){if(ht(i)){r.setTextConfig({rotation:i});return}else if(N(t)){r.setTextConfig({rotation:0});return}var n=r.shape,a=n.clockwise?n.startAngle:n.endAngle,o=n.clockwise?n.endAngle:n.startAngle,s=(a+o)/2,l,u=e(t);switch(u){case"startArc":case"insideStartArc":case"middle":case"insideEndArc":case"endArc":l=s;break;case"startAngle":case"insideStartAngle":l=a;break;case"endAngle":case"insideEndAngle":l=o;break;default:r.setTextConfig({rotation:0});return}var f=Math.PI*1.5-l;u==="middle"&&f>Math.PI/2&&f<Math.PI*1.5&&(f-=Math.PI),r.setTextConfig({rotation:f})}function Ha(r,t,e){return t*Math.sin(r)*(e?-1:1)}function Ga(r,t,e){return t*Math.cos(r)*(e?1:-1)}function RT(r,t,e){var i=r.get("borderRadius");if(i==null)return{cornerRadius:0};N(i)||(i=[i,i,i,i]);var n=Math.abs(t.r||0-t.r0||0);return{cornerRadius:H(i,function(a){return Fe(a,n)})}}var gl=Math.max,yl=Math.min;function ET(r,t){var e=r.getArea&&r.getArea();if(ih(r,"cartesian2d")){var i=r.getBaseAxis();if(i.type!=="category"||!i.onBand){var n=t.getLayout("bandWidth");i.isHorizontal()?(e.x-=n,e.width+=n*2):(e.y-=n,e.height+=n*2)}}return e}var OT=function(r){k(t,r);function t(){var e=r.call(this)||this;return e.type=t.type,e._isFirstFrame=!0,e}return t.prototype.render=function(e,i,n,a){this._model=e,this._removeOnRenderedListener(n),this._updateDrawMode(e);var o=e.get("coordinateSystem");(o==="cartesian2d"||o==="polar")&&(this._progressiveEls=null,this._isLargeDraw?this._renderLarge(e,i,n):this._renderNormal(e,i,n,a))},t.prototype.incrementalPrepareRender=function(e){this._clear(),this._updateDrawMode(e),this._updateLargeClip(e)},t.prototype.incrementalRender=function(e,i){this._progressiveEls=[],this._incrementalRenderLarge(e,i)},t.prototype.eachRendered=function(e){cs(this._progressiveEls||this.group,e)},t.prototype._updateDrawMode=function(e){var i=e.pipelineContext.large;(this._isLargeDraw==null||i!==this._isLargeDraw)&&(this._isLargeDraw=i,this._clear())},t.prototype._renderNormal=function(e,i,n,a){var o=this.group,s=e.getData(),l=this._data,u=e.coordinateSystem,f=u.getBaseAxis(),h;u.type==="cartesian2d"?h=f.isHorizontal():u.type==="polar"&&(h=f.dim==="angle");var c=e.isAnimationEnabled()?e:null,v=kT(e,u);v&&this._enableRealtimeSort(v,s,n);var d=e.get("clip",!0)||v,y=ET(u,s);o.removeClipPath();var p=e.get("roundCap",!0),g=e.get("showBackground",!0),m=e.getModel("backgroundStyle"),_=m.get("borderRadius")||0,S=[],b=this._backgroundEls,w=a&&a.isInitSort,x=a&&a.type==="changeAxisOrder";function C(M){var I=Wa[u.type](s,M),L=GT(u,h,I);return L.useStyle(m.getItemStyle()),u.type==="cartesian2d"?L.setShape("r",_):L.setShape("cornerRadius",_),S[M]=L,L}s.diff(l).add(function(M){var I=s.getItemModel(M),L=Wa[u.type](s,M,I);if(g&&C(M),!(!s.hasValue(M)||!gc[u.type](L))){var P=!1;d&&(P=vc[u.type](y,L));var R=cc[u.type](e,s,M,L,h,c,f.model,!1,p);v&&(R.forceLabelAnimation=!0),yc(R,s,M,I,L,e,h,u.type==="polar"),w?R.attr({shape:L}):v?dc(v,c,R,L,M,h,!1,!1):He(R,{shape:L},e,M),s.setItemGraphicEl(M,R),o.add(R),R.ignore=P}}).update(function(M,I){var L=s.getItemModel(M),P=Wa[u.type](s,M,L);if(g){var R=void 0;b.length===0?R=C(I):(R=b[I],R.useStyle(m.getItemStyle()),u.type==="cartesian2d"?R.setShape("r",_):R.setShape("cornerRadius",_),S[M]=R);var E=Wa[u.type](s,M),W=lm(h,E,u);te(R,{shape:W},c,M)}var B=l.getItemGraphicEl(I);if(!s.hasValue(M)||!gc[u.type](P)){o.remove(B);return}var F=!1;if(d&&(F=vc[u.type](y,P),F&&o.remove(B)),B?Wg(B):B=cc[u.type](e,s,M,P,h,c,f.model,!!B,p),v&&(B.forceLabelAnimation=!0),x){var G=B.getTextContent();if(G){var q=gs(G);q.prevValue!=null&&(q.prevValue=q.value)}}else yc(B,s,M,L,P,e,h,u.type==="polar");w?B.attr({shape:P}):v?dc(v,c,B,P,M,h,!0,x):te(B,{shape:P},e,M,null),s.setItemGraphicEl(M,B),B.ignore=F,o.add(B)}).remove(function(M){var I=l.getItemGraphicEl(M);I&&wu(I,e,M)}).execute();var D=this._backgroundGroup||(this._backgroundGroup=new mt);D.removeAll();for(var A=0;A<S.length;++A)D.add(S[A]);o.add(D),this._backgroundEls=S,this._data=s},t.prototype._renderLarge=function(e,i,n){this._clear(),_c(e,this.group),this._updateLargeClip(e)},t.prototype._incrementalRenderLarge=function(e,i){this._removeBackground(),_c(i,this.group,this._progressiveEls,!0)},t.prototype._updateLargeClip=function(e){var i=e.get("clip",!0)&&sT(e.coordinateSystem,!1,e),n=this.group;i?n.setClipPath(i):n.removeClipPath()},t.prototype._enableRealtimeSort=function(e,i,n){var a=this;if(i.count()){var o=e.baseAxis;if(this._isFirstFrame)this._dispatchInitSort(i,e,n),this._isFirstFrame=!1;else{var s=function(l){var u=i.getItemGraphicEl(l),f=u&&u.shape;return f&&Math.abs(o.isHorizontal()?f.height:f.width)||0};this._onRendered=function(){a._updateSortWithinSameData(i,s,o,n)},n.getZr().on("rendered",this._onRendered)}}},t.prototype._dataSort=function(e,i,n){var a=[];return e.each(e.mapDimension(i.dim),function(o,s){var l=n(s);l=l??NaN,a.push({dataIndex:s,mappedValue:l,ordinalNumber:o})}),a.sort(function(o,s){return s.mappedValue-o.mappedValue}),{ordinalNumbers:H(a,function(o){return o.ordinalNumber})}},t.prototype._isOrderChangedWithinSameData=function(e,i,n){for(var a=n.scale,o=e.mapDimension(n.dim),s=Number.MAX_VALUE,l=0,u=a.getOrdinalMeta().categories.length;l<u;++l){var f=e.rawIndexOf(o,a.getRawOrdinalNumber(l)),h=f<0?Number.MIN_VALUE:i(e.indexOfRawIndex(f));if(h>s)return!0;s=h}return!1},t.prototype._isOrderDifferentInView=function(e,i){for(var n=i.scale,a=n.getExtent(),o=Math.max(0,a[0]),s=Math.min(a[1],n.getOrdinalMeta().categories.length-1);o<=s;++o)if(e.ordinalNumbers[o]!==n.getRawOrdinalNumber(o))return!0},t.prototype._updateSortWithinSameData=function(e,i,n,a){if(this._isOrderChangedWithinSameData(e,i,n)){var o=this._dataSort(e,n,i);this._isOrderDifferentInView(o,n)&&(this._removeOnRenderedListener(a),a.dispatchAction({type:"changeAxisOrder",componentType:n.dim+"Axis",axisId:n.index,sortInfo:o}))}},t.prototype._dispatchInitSort=function(e,i,n){var a=i.baseAxis,o=this._dataSort(e,a,function(s){return e.get(e.mapDimension(i.otherAxis.dim),s)});n.dispatchAction({type:"changeAxisOrder",componentType:a.dim+"Axis",isInitSort:!0,axisId:a.index,sortInfo:o})},t.prototype.remove=function(e,i){this._clear(this._model),this._removeOnRenderedListener(i)},t.prototype.dispose=function(e,i){this._removeOnRenderedListener(i)},t.prototype._removeOnRenderedListener=function(e){this._onRendered&&(e.getZr().off("rendered",this._onRendered),this._onRendered=null)},t.prototype._clear=function(e){var i=this.group,n=this._data;e&&e.isAnimationEnabled()&&n&&!this._isLargeDraw?(this._removeBackground(),this._backgroundEls=[],n.eachItemGraphicEl(function(a){wu(a,e,tt(a).dataIndex)})):i.removeAll(),this._data=null,this._isFirstFrame=!0},t.prototype._removeBackground=function(){this.group.remove(this._backgroundGroup),this._backgroundGroup=null},t.type="bar",t}(Jt),vc={cartesian2d:function(r,t){var e=t.width<0?-1:1,i=t.height<0?-1:1;e<0&&(t.x+=t.width,t.width=-t.width),i<0&&(t.y+=t.height,t.height=-t.height);var n=r.x+r.width,a=r.y+r.height,o=gl(t.x,r.x),s=yl(t.x+t.width,n),l=gl(t.y,r.y),u=yl(t.y+t.height,a),f=s<o,h=u<l;return t.x=f&&o>n?s:o,t.y=h&&l>a?u:l,t.width=f?0:s-o,t.height=h?0:u-l,e<0&&(t.x+=t.width,t.width=-t.width),i<0&&(t.y+=t.height,t.height=-t.height),f||h},polar:function(r,t){var e=t.r0<=t.r?1:-1;if(e<0){var i=t.r;t.r=t.r0,t.r0=i}var n=yl(t.r,r.r),a=gl(t.r0,r.r0);t.r=n,t.r0=a;var o=n-a<0;if(e<0){var i=t.r;t.r=t.r0,t.r0=i}return o}},cc={cartesian2d:function(r,t,e,i,n,a,o,s,l){var u=new gt({shape:O({},i),z2:1});if(u.__dataIndex=e,u.name="item",a){var f=u.shape,h=n?"height":"width";f[h]=0}return u},polar:function(r,t,e,i,n,a,o,s,l){var u=!n&&l?hc:Xi,f=new u({shape:i,z2:1});f.name="item";var h=sm(n);if(f.calculateTextPosition=LT(h,{isRoundCap:u===hc}),a){var c=f.shape,v=n?"r":"endAngle",d={};c[v]=n?i.r0:i.startAngle,d[v]=i[v],(s?te:He)(f,{shape:d},a)}return f}};function kT(r,t){var e=r.get("realtimeSort",!0),i=t.getBaseAxis();if(e&&i.type==="category"&&t.type==="cartesian2d")return{baseAxis:i,otherAxis:t.getOtherAxis(i)}}function dc(r,t,e,i,n,a,o,s){var l,u;a?(u={x:i.x,width:i.width},l={y:i.y,height:i.height}):(u={y:i.y,height:i.height},l={x:i.x,width:i.width}),s||(o?te:He)(e,{shape:l},t,n,null);var f=t?r.baseAxis.model:null;(o?te:He)(e,{shape:u},f,n)}function pc(r,t){for(var e=0;e<t.length;e++)if(!isFinite(r[t[e]]))return!0;return!1}var BT=["x","y","width","height"],NT=["cx","cy","r","startAngle","endAngle"],gc={cartesian2d:function(r){return!pc(r,BT)},polar:function(r){return!pc(r,NT)}},Wa={cartesian2d:function(r,t,e){var i=r.getItemLayout(t),n=e?zT(e,i):0,a=i.width>0?1:-1,o=i.height>0?1:-1;return{x:i.x+a*n/2,y:i.y+o*n/2,width:i.width-a*n,height:i.height-o*n}},polar:function(r,t,e){var i=r.getItemLayout(t);return{cx:i.cx,cy:i.cy,r0:i.r0,r:i.r,startAngle:i.startAngle,endAngle:i.endAngle,clockwise:i.clockwise}}};function FT(r){return r.startAngle!=null&&r.endAngle!=null&&r.startAngle===r.endAngle}function sm(r){return function(t){var e=t?"Arc":"Angle";return function(i){switch(i){case"start":case"insideStart":case"end":case"insideEnd":return i+e;default:return i}}}(r)}function yc(r,t,e,i,n,a,o,s){var l=t.getItemVisual(e,"style");if(s){if(!a.get("roundCap")){var f=r.shape,h=RT(i.getModel("itemStyle"),f);O(f,h),r.setShape(f)}}else{var u=i.get(["itemStyle","borderRadius"])||0;r.setShape("r",u)}r.useStyle(l);var c=i.getShallow("cursor");c&&r.attr("cursor",c);var v=s?o?n.r>=n.r0?"endArc":"startArc":n.endAngle>=n.startAngle?"endAngle":"startAngle":o?n.height>=0?"bottom":"top":n.width>=0?"right":"left",d=ps(i);Ff(r,d,{labelFetcher:a,labelDataIndex:e,defaultText:th(a.getData(),e),inheritColor:l.fill,defaultOpacity:l.opacity,defaultOutsidePosition:v});var y=r.getTextContent();if(s&&y){var p=i.get(["label","position"]);r.textConfig.inside=p==="middle"?!0:null,PT(r,p==="outside"?v:p,sm(o),i.get(["label","rotate"]))}Vw(y,d,a.getRawValue(e),function(m){return Xy(t,m)});var g=i.getModel(["emphasis"]);Eo(r,g.get("focus"),g.get("blurScope"),g.get("disabled")),yu(r,i),FT(n)&&(r.style.fill="none",r.style.stroke="none",T(r.states,function(m){m.style&&(m.style.fill=m.style.stroke="none")}))}function zT(r,t){var e=r.get(["itemStyle","borderColor"]);if(!e||e==="none")return 0;var i=r.get(["itemStyle","borderWidth"])||0,n=isNaN(t.width)?Number.MAX_VALUE:Math.abs(t.width),a=isNaN(t.height)?Number.MAX_VALUE:Math.abs(t.height);return Math.min(i,n,a)}var VT=function(){function r(){}return r}(),mc=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;return i.type="largeBar",i}return t.prototype.getDefaultShape=function(){return new VT},t.prototype.buildPath=function(e,i){for(var n=i.points,a=this.baseDimIdx,o=1-this.baseDimIdx,s=[],l=[],u=this.barWidth,f=0;f<n.length;f+=3)l[a]=u,l[o]=n[f+2],s[a]=n[f+a],s[o]=n[f+o],e.rect(s[0],s[1],l[0],l[1])},t}(st);function _c(r,t,e,i){var n=r.getData(),a=n.getLayout("valueAxisHorizontal")?1:0,o=n.getLayout("largeDataIndices"),s=n.getLayout("size"),l=r.getModel("backgroundStyle"),u=n.getLayout("largeBackgroundPoints");if(u){var f=new mc({shape:{points:u},incremental:!!i,silent:!0,z2:0});f.baseDimIdx=a,f.largeDataIndices=o,f.barWidth=s,f.useStyle(l.getItemStyle()),t.add(f),e&&e.push(f)}var h=new mc({shape:{points:n.getLayout("largePoints")},incremental:!!i,ignoreCoarsePointer:!0,z2:1});h.baseDimIdx=a,h.largeDataIndices=o,h.barWidth=s,t.add(h),h.useStyle(n.getVisual("style")),tt(h).seriesIndex=r.seriesIndex,r.get("silent")||(h.on("mousedown",Sc),h.on("mousemove",Sc)),e&&e.push(h)}var Sc=oh(function(r){var t=this,e=HT(t,r.offsetX,r.offsetY);tt(t).dataIndex=e>=0?e:null},30,!1);function HT(r,t,e){for(var i=r.baseDimIdx,n=1-i,a=r.shape.points,o=r.largeDataIndices,s=[],l=[],u=r.barWidth,f=0,h=a.length/3;f<h;f++){var c=f*3;if(l[i]=u,l[n]=a[c+2],s[i]=a[c+i],s[n]=a[c+n],l[n]<0&&(s[n]+=l[n],l[n]=-l[n]),t>=s[0]&&t<=s[0]+l[0]&&e>=s[1]&&e<=s[1]+l[1])return o[f]}return-1}function lm(r,t,e){if(ih(e,"cartesian2d")){var i=t,n=e.getArea();return{x:r?i.x:n.x,y:r?n.y:i.y,width:r?i.width:n.width,height:r?n.height:i.height}}else{var n=e.getArea(),a=t;return{cx:n.cx,cy:n.cy,r0:r?n.r0:a.r0,r:r?n.r:a.r,startAngle:r?a.startAngle:0,endAngle:r?a.endAngle:Math.PI*2}}}function GT(r,t,e){var i=r.type==="polar"?Xi:gt;return new i({shape:lm(t,e,r),silent:!0,z2:0})}function rL(r){r.registerChartView(OT),r.registerSeriesModel(AT),r.registerLayout(r.PRIORITY.VISUAL.LAYOUT,kt(MT,"bar")),r.registerLayout(r.PRIORITY.VISUAL.PROGRESSIVE_LAYOUT,CT("bar")),r.registerProcessor(r.PRIORITY.PROCESSOR.STATISTIC,em("bar")),r.registerAction({type:"changeAxisOrder",event:"changeAxisOrder",update:"update"},function(t,e){var i=t.componentType||"series";e.eachComponent({mainType:i,query:t},function(n){t.sortInfo&&n.axis.setCategorySortInfo(t.sortInfo)})})}function Mi(r,t,e,i,n){var a=r+t;e.isSilent(a)||i.eachComponent({mainType:"series",subType:"pie"},function(o){for(var s=o.seriesIndex,l=o.option.selectedMap,u=n.selected,f=0;f<u.length;f++)if(u[f].seriesIndex===s){var h=o.getData(),c=ui(h,n.fromActionPayload);e.trigger(a,{type:a,seriesId:o.id,name:N(c)?h.getName(c[0]):h.getName(c),selected:V(l)?l:O({},l)})}})}function WT(r,t,e){r.on("selectchanged",function(i){var n=e.getModel();i.isFromClick?(Mi("map","selectchanged",t,n,i),Mi("pie","selectchanged",t,n,i)):i.fromAction==="select"?(Mi("map","selected",t,n,i),Mi("pie","selected",t,n,i)):i.fromAction==="unselect"&&(Mi("map","unselected",t,n,i),Mi("pie","unselected",t,n,i))})}function UT(r){for(var t=[],e=0;e<r.length;e++){var i=r[e];if(!i.defaultAttr.ignore){var n=i.label,a=n.getComputedTransform(),o=n.getBoundingRect(),s=!a||a[1]<1e-5&&a[2]<1e-5,l=n.style.margin||0,u=o.clone();u.applyTransform(a),u.x-=l/2,u.y-=l/2,u.width+=l,u.height+=l;var f=s?new Oo(o,a):null;t.push({label:n,labelLine:i.labelLine,rect:u,localRect:o,obb:f,priority:i.priority,defaultAttr:i.defaultAttr,layoutOption:i.computedLayoutOption,axisAligned:s,transform:a})}}return t}function YT(r){var t=[];r.sort(function(y,p){return p.priority-y.priority});var e=new et(0,0,0,0);function i(y){if(!y.ignore){var p=y.ensureState("emphasis");p.ignore==null&&(p.ignore=!1)}y.ignore=!0}for(var n=0;n<r.length;n++){var a=r[n],o=a.axisAligned,s=a.localRect,l=a.transform,u=a.label,f=a.labelLine;e.copy(a.rect),e.width-=.1,e.height-=.1,e.x+=.05,e.y+=.05;for(var h=a.obb,c=!1,v=0;v<t.length;v++){var d=t[v];if(e.intersect(d.rect)){if(o&&d.axisAligned){c=!0;break}if(d.obb||(d.obb=new Oo(d.localRect,d.transform)),h||(h=new Oo(s,l)),h.intersect(d.obb)){c=!0;break}}}c?(i(u),f&&i(f)):(u.attr("ignore",a.defaultAttr.ignore),f&&f.attr("ignore",a.defaultAttr.labelGuideIgnore),t.push(a))}}var Ci=function(){function r(t,e){this.target=t,this.topTarget=e&&e.topTarget}return r}(),XT=function(){function r(t){this.handler=t,t.on("mousedown",this._dragStart,this),t.on("mousemove",this._drag,this),t.on("mouseup",this._dragEnd,this)}return r.prototype._dragStart=function(t){for(var e=t.target;e&&!e.draggable;)e=e.parent||e.__hostTarget;e&&(this._draggingTarget=e,e.dragging=!0,this._x=t.offsetX,this._y=t.offsetY,this.handler.dispatchToElement(new Ci(e,t),"dragstart",t.event))},r.prototype._drag=function(t){var e=this._draggingTarget;if(e){var i=t.offsetX,n=t.offsetY,a=i-this._x,o=n-this._y;this._x=i,this._y=n,e.drift(a,o,t),this.handler.dispatchToElement(new Ci(e,t),"drag",t.event);var s=this.handler.findHover(i,n,e).target,l=this._dropTarget;this._dropTarget=s,e!==s&&(l&&s!==l&&this.handler.dispatchToElement(new Ci(l,t),"dragleave",t.event),s&&s!==l&&this.handler.dispatchToElement(new Ci(s,t),"dragenter",t.event))}},r.prototype._dragEnd=function(t){var e=this._draggingTarget;e&&(e.dragging=!1),this.handler.dispatchToElement(new Ci(e,t),"dragend",t.event),this._dropTarget&&this.handler.dispatchToElement(new Ci(this._dropTarget,t),"drop",t.event),this._draggingTarget=null,this._dropTarget=null},r}(),$T=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,ml=[],ZT=Z.browser.firefox&&+Z.browser.version.split(".")[0]<39;function ku(r,t,e,i){return e=e||{},i?wc(r,t,e):ZT&&t.layerX!=null&&t.layerX!==t.offsetX?(e.zrX=t.layerX,e.zrY=t.layerY):t.offsetX!=null?(e.zrX=t.offsetX,e.zrY=t.offsetY):wc(r,t,e),e}function wc(r,t,e){if(Z.domSupported&&r.getBoundingClientRect){var i=t.clientX,n=t.clientY;if(My(r)){var a=r.getBoundingClientRect();e.zrX=i-a.left,e.zrY=n-a.top;return}else if(Cu(ml,r,i,n)){e.zrX=ml[0],e.zrY=ml[1];return}}e.zrX=e.zrY=0}function sh(r){return r||window.event}function ne(r,t,e){if(t=sh(t),t.zrX!=null)return t;var i=t.type,n=i&&i.indexOf("touch")>=0;if(n){var o=i!=="touchend"?t.targetTouches[0]:t.changedTouches[0];o&&ku(r,o,t,e)}else{ku(r,t,t,e);var a=qT(t);t.zrDelta=a?a/120:-(t.detail||0)/3}var s=t.button;return t.which==null&&s!==void 0&&$T.test(t.type)&&(t.which=s&1?1:s&2?3:s&4?2:0),t}function qT(r){var t=r.wheelDelta;if(t)return t;var e=r.deltaX,i=r.deltaY;if(e==null||i==null)return t;var n=Math.abs(i!==0?i:e),a=i>0?-1:i<0?1:e>0?-1:1;return 3*n*a}function KT(r,t,e,i){r.addEventListener(t,e,i)}function QT(r,t,e,i){r.removeEventListener(t,e,i)}var ta=function(r){r.preventDefault(),r.stopPropagation(),r.cancelBubble=!0},jT=function(){function r(){this._track=[]}return r.prototype.recognize=function(t,e,i){return this._doTrack(t,e,i),this._recognize(t)},r.prototype.clear=function(){return this._track.length=0,this},r.prototype._doTrack=function(t,e,i){var n=t.touches;if(n){for(var a={points:[],touches:[],target:e,event:t},o=0,s=n.length;o<s;o++){var l=n[o],u=ku(i,l,{});a.points.push([u.zrX,u.zrY]),a.touches.push(l)}this._track.push(a)}},r.prototype._recognize=function(t){for(var e in _l)if(_l.hasOwnProperty(e)){var i=_l[e](this._track,t);if(i)return i}},r}();function bc(r){var t=r[1][0]-r[0][0],e=r[1][1]-r[0][1];return Math.sqrt(t*t+e*e)}function JT(r){return[(r[0][0]+r[1][0])/2,(r[0][1]+r[1][1])/2]}var _l={pinch:function(r,t){var e=r.length;if(e){var i=(r[e-1]||{}).points,n=(r[e-2]||{}).points||i;if(n&&n.length>1&&i&&i.length>1){var a=bc(i)/bc(n);!isFinite(a)&&(a=1),t.pinchScale=a;var o=JT(i);return t.pinchX=o[0],t.pinchY=o[1],{type:"pinch",target:r[0].target,event:t}}}}},um="silent";function tM(r,t,e){return{type:r,event:e,target:t.target,topTarget:t.topTarget,cancelBubble:!1,offsetX:e.zrX,offsetY:e.zrY,gestureEvent:e.gestureEvent,pinchX:e.pinchX,pinchY:e.pinchY,pinchScale:e.pinchScale,wheelDelta:e.zrDelta,zrByTouch:e.zrByTouch,which:e.which,stop:eM}}function eM(){ta(this.event)}var rM=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.handler=null,e}return t.prototype.dispose=function(){},t.prototype.setCursor=function(){},t}(Ge),fn=function(){function r(t,e){this.x=t,this.y=e}return r}(),iM=["click","dblclick","mousewheel","mouseout","mouseup","mousedown","mousemove","contextmenu"],Sl=new et(0,0,0,0),fm=function(r){k(t,r);function t(e,i,n,a,o){var s=r.call(this)||this;return s._hovered=new fn(0,0),s.storage=e,s.painter=i,s.painterRoot=a,s._pointerSize=o,n=n||new rM,s.proxy=null,s.setHandlerProxy(n),s._draggingMgr=new XT(s),s}return t.prototype.setHandlerProxy=function(e){this.proxy&&this.proxy.dispose(),e&&(T(iM,function(i){e.on&&e.on(i,this[i],this)},this),e.handler=this),this.proxy=e},t.prototype.mousemove=function(e){var i=e.zrX,n=e.zrY,a=hm(this,i,n),o=this._hovered,s=o.target;s&&!s.__zr&&(o=this.findHover(o.x,o.y),s=o.target);var l=this._hovered=a?new fn(i,n):this.findHover(i,n),u=l.target,f=this.proxy;f.setCursor&&f.setCursor(u?u.cursor:"default"),s&&u!==s&&this.dispatchToElement(o,"mouseout",e),this.dispatchToElement(l,"mousemove",e),u&&u!==s&&this.dispatchToElement(l,"mouseover",e)},t.prototype.mouseout=function(e){var i=e.zrEventControl;i!=="only_globalout"&&this.dispatchToElement(this._hovered,"mouseout",e),i!=="no_globalout"&&this.trigger("globalout",{type:"globalout",event:e})},t.prototype.resize=function(){this._hovered=new fn(0,0)},t.prototype.dispatch=function(e,i){var n=this[e];n&&n.call(this,i)},t.prototype.dispose=function(){this.proxy.dispose(),this.storage=null,this.proxy=null,this.painter=null},t.prototype.setCursorStyle=function(e){var i=this.proxy;i.setCursor&&i.setCursor(e)},t.prototype.dispatchToElement=function(e,i,n){e=e||{};var a=e.target;if(!(a&&a.silent)){for(var o="on"+i,s=tM(i,e,n);a&&(a[o]&&(s.cancelBubble=!!a[o].call(a,s)),a.trigger(i,s),a=a.__hostTarget?a.__hostTarget:a.parent,!s.cancelBubble););s.cancelBubble||(this.trigger(i,s),this.painter&&this.painter.eachOtherLayer&&this.painter.eachOtherLayer(function(l){typeof l[o]=="function"&&l[o].call(l,s),l.trigger&&l.trigger(i,s)}))}},t.prototype.findHover=function(e,i,n){var a=this.storage.getDisplayList(),o=new fn(e,i);if(xc(a,o,e,i,n),this._pointerSize&&!o.target){for(var s=[],l=this._pointerSize,u=l/2,f=new et(e-u,i-u,l,l),h=a.length-1;h>=0;h--){var c=a[h];c!==n&&!c.ignore&&!c.ignoreCoarsePointer&&(!c.parent||!c.parent.ignoreCoarsePointer)&&(Sl.copy(c.getBoundingRect()),c.transform&&Sl.applyTransform(c.transform),Sl.intersect(f)&&s.push(c))}if(s.length)for(var v=4,d=Math.PI/12,y=Math.PI*2,p=0;p<u;p+=v)for(var g=0;g<y;g+=d){var m=e+p*Math.cos(g),_=i+p*Math.sin(g);if(xc(s,o,m,_,n),o.target)return o}}return o},t.prototype.processGesture=function(e,i){this._gestureMgr||(this._gestureMgr=new jT);var n=this._gestureMgr;i==="start"&&n.clear();var a=n.recognize(e,this.findHover(e.zrX,e.zrY,null).target,this.proxy.dom);if(i==="end"&&n.clear(),a){var o=a.type;e.gestureEvent=o;var s=new fn;s.target=a.target,this.dispatchToElement(s,o,a.event)}},t}(Ge);T(["click","mousedown","mouseup","mousewheel","dblclick","contextmenu"],function(r){fm.prototype[r]=function(t){var e=t.zrX,i=t.zrY,n=hm(this,e,i),a,o;if((r!=="mouseup"||!n)&&(a=this.findHover(e,i),o=a.target),r==="mousedown")this._downEl=o,this._downPoint=[t.zrX,t.zrY],this._upEl=o;else if(r==="mouseup")this._upEl=o;else if(r==="click"){if(this._downEl!==this._upEl||!this._downPoint||Fp(this._downPoint,[t.zrX,t.zrY])>4)return;this._downPoint=null}this.dispatchToElement(a,r,t)}});function nM(r,t,e){if(r[r.rectHover?"rectContain":"contain"](t,e)){for(var i=r,n=void 0,a=!1;i;){if(i.ignoreClip&&(a=!0),!a){var o=i.getClipPath();if(o&&!o.contain(t,e))return!1}i.silent&&(n=!0);var s=i.__hostTarget;i=s||i.parent}return n?um:!0}return!1}function xc(r,t,e,i,n){for(var a=r.length-1;a>=0;a--){var o=r[a],s=void 0;if(o!==n&&!o.ignore&&(s=nM(o,e,i))&&(!t.topTarget&&(t.topTarget=o),s!==um)){t.target=o;break}}}function hm(r,t,e){var i=r.painter;return t<0||t>i.getWidth()||e<0||e>i.getHeight()}var vm=32,hn=7;function aM(r){for(var t=0;r>=vm;)t|=r&1,r>>=1;return r+t}function Tc(r,t,e,i){var n=t+1;if(n===e)return 1;if(i(r[n++],r[t])<0){for(;n<e&&i(r[n],r[n-1])<0;)n++;oM(r,t,n)}else for(;n<e&&i(r[n],r[n-1])>=0;)n++;return n-t}function oM(r,t,e){for(e--;t<e;){var i=r[t];r[t++]=r[e],r[e--]=i}}function Mc(r,t,e,i,n){for(i===t&&i++;i<e;i++){for(var a=r[i],o=t,s=i,l;o<s;)l=o+s>>>1,n(a,r[l])<0?s=l:o=l+1;var u=i-o;switch(u){case 3:r[o+3]=r[o+2];case 2:r[o+2]=r[o+1];case 1:r[o+1]=r[o];break;default:for(;u>0;)r[o+u]=r[o+u-1],u--}r[o]=a}}function wl(r,t,e,i,n,a){var o=0,s=0,l=1;if(a(r,t[e+n])>0){for(s=i-n;l<s&&a(r,t[e+n+l])>0;)o=l,l=(l<<1)+1,l<=0&&(l=s);l>s&&(l=s),o+=n,l+=n}else{for(s=n+1;l<s&&a(r,t[e+n-l])<=0;)o=l,l=(l<<1)+1,l<=0&&(l=s);l>s&&(l=s);var u=o;o=n-l,l=n-u}for(o++;o<l;){var f=o+(l-o>>>1);a(r,t[e+f])>0?o=f+1:l=f}return l}function bl(r,t,e,i,n,a){var o=0,s=0,l=1;if(a(r,t[e+n])<0){for(s=n+1;l<s&&a(r,t[e+n-l])<0;)o=l,l=(l<<1)+1,l<=0&&(l=s);l>s&&(l=s);var u=o;o=n-l,l=n-u}else{for(s=i-n;l<s&&a(r,t[e+n+l])>=0;)o=l,l=(l<<1)+1,l<=0&&(l=s);l>s&&(l=s),o+=n,l+=n}for(o++;o<l;){var f=o+(l-o>>>1);a(r,t[e+f])<0?l=f:o=f+1}return l}function sM(r,t){var e=hn,i,n,a=0,o=[];i=[],n=[];function s(v,d){i[a]=v,n[a]=d,a+=1}function l(){for(;a>1;){var v=a-2;if(v>=1&&n[v-1]<=n[v]+n[v+1]||v>=2&&n[v-2]<=n[v]+n[v-1])n[v-1]<n[v+1]&&v--;else if(n[v]>n[v+1])break;f(v)}}function u(){for(;a>1;){var v=a-2;v>0&&n[v-1]<n[v+1]&&v--,f(v)}}function f(v){var d=i[v],y=n[v],p=i[v+1],g=n[v+1];n[v]=y+g,v===a-3&&(i[v+1]=i[v+2],n[v+1]=n[v+2]),a--;var m=bl(r[p],r,d,y,0,t);d+=m,y-=m,y!==0&&(g=wl(r[d+y-1],r,p,g,g-1,t),g!==0&&(y<=g?h(d,y,p,g):c(d,y,p,g)))}function h(v,d,y,p){var g=0;for(g=0;g<d;g++)o[g]=r[v+g];var m=0,_=y,S=v;if(r[S++]=r[_++],--p===0){for(g=0;g<d;g++)r[S+g]=o[m+g];return}if(d===1){for(g=0;g<p;g++)r[S+g]=r[_+g];r[S+p]=o[m];return}for(var b=e,w,x,C;;){w=0,x=0,C=!1;do if(t(r[_],o[m])<0){if(r[S++]=r[_++],x++,w=0,--p===0){C=!0;break}}else if(r[S++]=o[m++],w++,x=0,--d===1){C=!0;break}while((w|x)<b);if(C)break;do{if(w=bl(r[_],o,m,d,0,t),w!==0){for(g=0;g<w;g++)r[S+g]=o[m+g];if(S+=w,m+=w,d-=w,d<=1){C=!0;break}}if(r[S++]=r[_++],--p===0){C=!0;break}if(x=wl(o[m],r,_,p,0,t),x!==0){for(g=0;g<x;g++)r[S+g]=r[_+g];if(S+=x,_+=x,p-=x,p===0){C=!0;break}}if(r[S++]=o[m++],--d===1){C=!0;break}b--}while(w>=hn||x>=hn);if(C)break;b<0&&(b=0),b+=2}if(e=b,e<1&&(e=1),d===1){for(g=0;g<p;g++)r[S+g]=r[_+g];r[S+p]=o[m]}else{if(d===0)throw new Error;for(g=0;g<d;g++)r[S+g]=o[m+g]}}function c(v,d,y,p){var g=0;for(g=0;g<p;g++)o[g]=r[y+g];var m=v+d-1,_=p-1,S=y+p-1,b=0,w=0;if(r[S--]=r[m--],--d===0){for(b=S-(p-1),g=0;g<p;g++)r[b+g]=o[g];return}if(p===1){for(S-=d,m-=d,w=S+1,b=m+1,g=d-1;g>=0;g--)r[w+g]=r[b+g];r[S]=o[_];return}for(var x=e;;){var C=0,D=0,A=!1;do if(t(o[_],r[m])<0){if(r[S--]=r[m--],C++,D=0,--d===0){A=!0;break}}else if(r[S--]=o[_--],D++,C=0,--p===1){A=!0;break}while((C|D)<x);if(A)break;do{if(C=d-bl(o[_],r,v,d,d-1,t),C!==0){for(S-=C,m-=C,d-=C,w=S+1,b=m+1,g=C-1;g>=0;g--)r[w+g]=r[b+g];if(d===0){A=!0;break}}if(r[S--]=o[_--],--p===1){A=!0;break}if(D=p-wl(r[m],o,0,p,p-1,t),D!==0){for(S-=D,_-=D,p-=D,w=S+1,b=_+1,g=0;g<D;g++)r[w+g]=o[b+g];if(p<=1){A=!0;break}}if(r[S--]=r[m--],--d===0){A=!0;break}x--}while(C>=hn||D>=hn);if(A)break;x<0&&(x=0),x+=2}if(e=x,e<1&&(e=1),p===1){for(S-=d,m-=d,w=S+1,b=m+1,g=d-1;g>=0;g--)r[w+g]=r[b+g];r[S]=o[_]}else{if(p===0)throw new Error;for(b=S-(p-1),g=0;g<p;g++)r[b+g]=o[g]}}return{mergeRuns:l,forceMergeRuns:u,pushRun:s}}function go(r,t,e,i){e||(e=0),i||(i=r.length);var n=i-e;if(!(n<2)){var a=0;if(n<vm){a=Tc(r,e,i,t),Mc(r,e,i,e+a,t);return}var o=sM(r,t),s=aM(n);do{if(a=Tc(r,e,i,t),a<s){var l=n;l>s&&(l=s),Mc(r,e,e+l,e+a,t),a=l}o.pushRun(e,a),o.mergeRuns(),n-=a,e+=a}while(n!==0);o.forceMergeRuns()}}var Cc=!1;function xl(){Cc||(Cc=!0,console.warn("z / z2 / zlevel of displayable is invalid, which may cause unexpected errors"))}function Dc(r,t){return r.zlevel===t.zlevel?r.z===t.z?r.z2-t.z2:r.z-t.z:r.zlevel-t.zlevel}var lM=function(){function r(){this._roots=[],this._displayList=[],this._displayListLen=0,this.displayableSortFunc=Dc}return r.prototype.traverse=function(t,e){for(var i=0;i<this._roots.length;i++)this._roots[i].traverse(t,e)},r.prototype.getDisplayList=function(t,e){e=e||!1;var i=this._displayList;return(t||!i.length)&&this.updateDisplayList(e),i},r.prototype.updateDisplayList=function(t){this._displayListLen=0;for(var e=this._roots,i=this._displayList,n=0,a=e.length;n<a;n++)this._updateAndAddDisplayable(e[n],null,t);i.length=this._displayListLen,go(i,Dc)},r.prototype._updateAndAddDisplayable=function(t,e,i){if(!(t.ignore&&!i)){t.beforeUpdate(),t.update(),t.afterUpdate();var n=t.getClipPath();if(t.ignoreClip)e=null;else if(n){e?e=e.slice():e=[];for(var a=n,o=t;a;)a.parent=o,a.updateTransform(),e.push(a),o=a,a=a.getClipPath()}if(t.childrenRef){for(var s=t.childrenRef(),l=0;l<s.length;l++){var u=s[l];t.__dirty&&(u.__dirty|=Kt),this._updateAndAddDisplayable(u,e,i)}t.__dirty=0}else{var f=t;e&&e.length?f.__clipPaths=e:f.__clipPaths&&f.__clipPaths.length>0&&(f.__clipPaths=[]),isNaN(f.z)&&(xl(),f.z=0),isNaN(f.z2)&&(xl(),f.z2=0),isNaN(f.zlevel)&&(xl(),f.zlevel=0),this._displayList[this._displayListLen++]=f}var h=t.getDecalElement&&t.getDecalElement();h&&this._updateAndAddDisplayable(h,e,i);var c=t.getTextGuideLine();c&&this._updateAndAddDisplayable(c,e,i);var v=t.getTextContent();v&&this._updateAndAddDisplayable(v,e,i)}},r.prototype.addRoot=function(t){t.__zr&&t.__zr.storage===this||this._roots.push(t)},r.prototype.delRoot=function(t){if(t instanceof Array){for(var e=0,i=t.length;e<i;e++)this.delRoot(t[e]);return}var n=ot(this._roots,t);n>=0&&this._roots.splice(n,1)},r.prototype.delAllRoots=function(){this._roots=[],this._displayList=[],this._displayListLen=0},r.prototype.getRoots=function(){return this._roots},r.prototype.dispose=function(){this._displayList=null,this._roots=null},r}(),Vo;Vo=Z.hasGlobalWindow&&(window.requestAnimationFrame&&window.requestAnimationFrame.bind(window)||window.msRequestAnimationFrame&&window.msRequestAnimationFrame.bind(window)||window.mozRequestAnimationFrame||window.webkitRequestAnimationFrame)||function(r){return setTimeout(r,16)};function Oi(){return new Date().getTime()}var uM=function(r){k(t,r);function t(e){var i=r.call(this)||this;return i._running=!1,i._time=0,i._pausedTime=0,i._pauseStart=0,i._paused=!1,e=e||{},i.stage=e.stage||{},i}return t.prototype.addClip=function(e){e.animation&&this.removeClip(e),this._head?(this._tail.next=e,e.prev=this._tail,e.next=null,this._tail=e):this._head=this._tail=e,e.animation=this},t.prototype.addAnimator=function(e){e.animation=this;var i=e.getClip();i&&this.addClip(i)},t.prototype.removeClip=function(e){if(e.animation){var i=e.prev,n=e.next;i?i.next=n:this._head=n,n?n.prev=i:this._tail=i,e.next=e.prev=e.animation=null}},t.prototype.removeAnimator=function(e){var i=e.getClip();i&&this.removeClip(i),e.animation=null},t.prototype.update=function(e){for(var i=Oi()-this._pausedTime,n=i-this._time,a=this._head;a;){var o=a.next,s=a.step(i,n);s&&(a.ondestroy(),this.removeClip(a)),a=o}this._time=i,e||(this.trigger("frame",n),this.stage.update&&this.stage.update())},t.prototype._startLoop=function(){var e=this;this._running=!0;function i(){e._running&&(Vo(i),!e._paused&&e.update())}Vo(i)},t.prototype.start=function(){this._running||(this._time=Oi(),this._pausedTime=0,this._startLoop())},t.prototype.stop=function(){this._running=!1},t.prototype.pause=function(){this._paused||(this._pauseStart=Oi(),this._paused=!0)},t.prototype.resume=function(){this._paused&&(this._pausedTime+=Oi()-this._pauseStart,this._paused=!1)},t.prototype.clear=function(){for(var e=this._head;e;){var i=e.next;e.prev=e.next=e.animation=null,e=i}this._head=this._tail=null},t.prototype.isFinished=function(){return this._head==null},t.prototype.animate=function(e,i){i=i||{},this.start();var n=new Sf(e,i.loop);return this.addAnimator(n),n},t}(Ge),fM=300,Tl=Z.domSupported,Ml=function(){var r=["click","dblclick","mousewheel","wheel","mouseout","mouseup","mousedown","mousemove","contextmenu"],t=["touchstart","touchend","touchmove"],e={pointerdown:1,pointerup:1,pointermove:1,pointerout:1},i=H(r,function(n){var a=n.replace("mouse","pointer");return e.hasOwnProperty(a)?a:n});return{mouse:r,touch:t,pointer:i}}(),Ac={mouse:["mousemove","mouseup"],pointer:["pointermove","pointerup"]},Ic=!1;function Bu(r){var t=r.pointerType;return t==="pen"||t==="touch"}function hM(r){r.touching=!0,r.touchTimer!=null&&(clearTimeout(r.touchTimer),r.touchTimer=null),r.touchTimer=setTimeout(function(){r.touching=!1,r.touchTimer=null},700)}function Cl(r){r&&(r.zrByTouch=!0)}function vM(r,t){return ne(r.dom,new cM(r,t),!0)}function cm(r,t){for(var e=t,i=!1;e&&e.nodeType!==9&&!(i=e.domBelongToZr||e!==t&&e===r.painterRoot);)e=e.parentNode;return i}var cM=function(){function r(t,e){this.stopPropagation=At,this.stopImmediatePropagation=At,this.preventDefault=At,this.type=e.type,this.target=this.currentTarget=t.dom,this.pointerType=e.pointerType,this.clientX=e.clientX,this.clientY=e.clientY}return r}(),Se={mousedown:function(r){r=ne(this.dom,r),this.__mayPointerCapture=[r.zrX,r.zrY],this.trigger("mousedown",r)},mousemove:function(r){r=ne(this.dom,r);var t=this.__mayPointerCapture;t&&(r.zrX!==t[0]||r.zrY!==t[1])&&this.__togglePointerCapture(!0),this.trigger("mousemove",r)},mouseup:function(r){r=ne(this.dom,r),this.__togglePointerCapture(!1),this.trigger("mouseup",r)},mouseout:function(r){r=ne(this.dom,r);var t=r.toElement||r.relatedTarget;cm(this,t)||(this.__pointerCapturing&&(r.zrEventControl="no_globalout"),this.trigger("mouseout",r))},wheel:function(r){Ic=!0,r=ne(this.dom,r),this.trigger("mousewheel",r)},mousewheel:function(r){Ic||(r=ne(this.dom,r),this.trigger("mousewheel",r))},touchstart:function(r){r=ne(this.dom,r),Cl(r),this.__lastTouchMoment=new Date,this.handler.processGesture(r,"start"),Se.mousemove.call(this,r),Se.mousedown.call(this,r)},touchmove:function(r){r=ne(this.dom,r),Cl(r),this.handler.processGesture(r,"change"),Se.mousemove.call(this,r)},touchend:function(r){r=ne(this.dom,r),Cl(r),this.handler.processGesture(r,"end"),Se.mouseup.call(this,r),+new Date-+this.__lastTouchMoment<fM&&Se.click.call(this,r)},pointerdown:function(r){Se.mousedown.call(this,r)},pointermove:function(r){Bu(r)||Se.mousemove.call(this,r)},pointerup:function(r){Se.mouseup.call(this,r)},pointerout:function(r){Bu(r)||Se.mouseout.call(this,r)}};T(["click","dblclick","contextmenu"],function(r){Se[r]=function(t){t=ne(this.dom,t),this.trigger(r,t)}});var Nu={pointermove:function(r){Bu(r)||Nu.mousemove.call(this,r)},pointerup:function(r){Nu.mouseup.call(this,r)},mousemove:function(r){this.trigger("mousemove",r)},mouseup:function(r){var t=this.__pointerCapturing;this.__togglePointerCapture(!1),this.trigger("mouseup",r),t&&(r.zrEventControl="only_globalout",this.trigger("mouseout",r))}};function dM(r,t){var e=t.domHandlers;Z.pointerEventsSupported?T(Ml.pointer,function(i){yo(t,i,function(n){e[i].call(r,n)})}):(Z.touchEventsSupported&&T(Ml.touch,function(i){yo(t,i,function(n){e[i].call(r,n),hM(t)})}),T(Ml.mouse,function(i){yo(t,i,function(n){n=sh(n),t.touching||e[i].call(r,n)})}))}function pM(r,t){Z.pointerEventsSupported?T(Ac.pointer,e):Z.touchEventsSupported||T(Ac.mouse,e);function e(i){function n(a){a=sh(a),cm(r,a.target)||(a=vM(r,a),t.domHandlers[i].call(r,a))}yo(t,i,n,{capture:!0})}}function yo(r,t,e,i){r.mounted[t]=e,r.listenerOpts[t]=i,KT(r.domTarget,t,e,i)}function Dl(r){var t=r.mounted;for(var e in t)t.hasOwnProperty(e)&&QT(r.domTarget,e,t[e],r.listenerOpts[e]);r.mounted={}}var Lc=function(){function r(t,e){this.mounted={},this.listenerOpts={},this.touching=!1,this.domTarget=t,this.domHandlers=e}return r}(),gM=function(r){k(t,r);function t(e,i){var n=r.call(this)||this;return n.__pointerCapturing=!1,n.dom=e,n.painterRoot=i,n._localHandlerScope=new Lc(e,Se),Tl&&(n._globalHandlerScope=new Lc(document,Nu)),dM(n,n._localHandlerScope),n}return t.prototype.dispose=function(){Dl(this._localHandlerScope),Tl&&Dl(this._globalHandlerScope)},t.prototype.setCursor=function(e){this.dom.style&&(this.dom.style.cursor=e||"default")},t.prototype.__togglePointerCapture=function(e){if(this.__mayPointerCapture=null,Tl&&+this.__pointerCapturing^+e){this.__pointerCapturing=e;var i=this._globalHandlerScope;e?pM(this,i):Dl(i)}},t}(Ge);/*!
* ZRender, a high performance 2d drawing library.
*
* Copyright (c) 2013, Baidu Inc.
* All rights reserved.
*
* LICENSE
* https://github.com/ecomfe/zrender/blob/master/LICENSE.txt
*/var mo={},Kr={};function yM(r){delete Kr[r]}function mM(r){if(!r)return!1;if(typeof r=="string")return Yn(r,1)<uu;if(r.colorStops){for(var t=r.colorStops,e=0,i=t.length,n=0;n<i;n++)e+=Yn(t[n].color,1);return e/=i,e<uu}return!1}var _M=function(){function r(t,e,i){var n=this;this._sleepAfterStill=10,this._stillFrameAccum=0,this._needsRefresh=!0,this._needsRefreshHover=!0,this._darkMode=!1,i=i||{},this.dom=e,this.id=t;var a=new lM,o=i.renderer||"canvas";mo[o]||(o=ft(mo)[0]),i.useDirtyRect=i.useDirtyRect==null?!1:i.useDirtyRect;var s=new mo[o](e,a,i,t),l=i.ssr||s.ssrOnly;this.storage=a,this.painter=s;var u=!Z.node&&!Z.worker&&!l?new gM(s.getViewportRoot(),s.root):null,f=i.useCoarsePointer,h=f==null||f==="auto"?Z.touchEventsSupported:!!f,c=44,v;h&&(v=Q(i.pointerSize,c)),this.handler=new fm(a,s,u,s.root,v),this.animation=new uM({stage:{update:l?null:function(){return n._flush(!0)}}}),l||this.animation.start()}return r.prototype.add=function(t){this._disposed||!t||(this.storage.addRoot(t),t.addSelfToZr(this),this.refresh())},r.prototype.remove=function(t){this._disposed||!t||(this.storage.delRoot(t),t.removeSelfFromZr(this),this.refresh())},r.prototype.configLayer=function(t,e){this._disposed||(this.painter.configLayer&&this.painter.configLayer(t,e),this.refresh())},r.prototype.setBackgroundColor=function(t){this._disposed||(this.painter.setBackgroundColor&&this.painter.setBackgroundColor(t),this.refresh(),this._backgroundColor=t,this._darkMode=mM(t))},r.prototype.getBackgroundColor=function(){return this._backgroundColor},r.prototype.setDarkMode=function(t){this._darkMode=t},r.prototype.isDarkMode=function(){return this._darkMode},r.prototype.refreshImmediately=function(t){this._disposed||(t||this.animation.update(!0),this._needsRefresh=!1,this.painter.refresh(),this._needsRefresh=!1)},r.prototype.refresh=function(){this._disposed||(this._needsRefresh=!0,this.animation.start())},r.prototype.flush=function(){this._disposed||this._flush(!1)},r.prototype._flush=function(t){var e,i=Oi();this._needsRefresh&&(e=!0,this.refreshImmediately(t)),this._needsRefreshHover&&(e=!0,this.refreshHoverImmediately());var n=Oi();e?(this._stillFrameAccum=0,this.trigger("rendered",{elapsedTime:n-i})):this._sleepAfterStill>0&&(this._stillFrameAccum++,this._stillFrameAccum>this._sleepAfterStill&&this.animation.stop())},r.prototype.setSleepAfterStill=function(t){this._sleepAfterStill=t},r.prototype.wakeUp=function(){this._disposed||(this.animation.start(),this._stillFrameAccum=0)},r.prototype.refreshHover=function(){this._needsRefreshHover=!0},r.prototype.refreshHoverImmediately=function(){this._disposed||(this._needsRefreshHover=!1,this.painter.refreshHover&&this.painter.getType()==="canvas"&&this.painter.refreshHover())},r.prototype.resize=function(t){this._disposed||(t=t||{},this.painter.resize(t.width,t.height),this.handler.resize())},r.prototype.clearAnimation=function(){this._disposed||this.animation.clear()},r.prototype.getWidth=function(){if(!this._disposed)return this.painter.getWidth()},r.prototype.getHeight=function(){if(!this._disposed)return this.painter.getHeight()},r.prototype.setCursorStyle=function(t){this._disposed||this.handler.setCursorStyle(t)},r.prototype.findHover=function(t,e){if(!this._disposed)return this.handler.findHover(t,e)},r.prototype.on=function(t,e,i){return this._disposed||this.handler.on(t,e,i),this},r.prototype.off=function(t,e){this._disposed||this.handler.off(t,e)},r.prototype.trigger=function(t,e){this._disposed||this.handler.trigger(t,e)},r.prototype.clear=function(){if(!this._disposed){for(var t=this.storage.getRoots(),e=0;e<t.length;e++)t[e]instanceof mt&&t[e].removeSelfFromZr(this);this.storage.delAllRoots(),this.painter.clear()}},r.prototype.dispose=function(){this._disposed||(this.animation.stop(),this.clear(),this.storage.dispose(),this.painter.dispose(),this.handler.dispose(),this.animation=this.storage=this.painter=this.handler=null,this._disposed=!0,yM(this.id))},r}();function Fu(r,t){var e=new _M(lf(),r,t);return Kr[e.id]=e,e}function SM(r){r.dispose()}function wM(){for(var r in Kr)Kr.hasOwnProperty(r)&&Kr[r].dispose();Kr={}}function bM(r){return Kr[r]}function dm(r,t){mo[r]=t}var zu;function xM(r){if(typeof zu=="function")return zu(r)}function pm(r){zu=r}var TM="5.6.0";const iL=Object.freeze(Object.defineProperty({__proto__:null,dispose:SM,disposeAll:wM,getElementSSRData:xM,getInstance:bM,init:Fu,registerPainter:dm,registerSSRDataGetter:pm,version:TM},Symbol.toStringTag,{value:"Module"}));var gm="";typeof navigator<"u"&&(gm=navigator.platform||"");var Di="rgba(0, 0, 0, 0.2)";const MM={darkMode:"auto",colorBy:"series",color:["#5470c6","#91cc75","#fac858","#ee6666","#73c0de","#3ba272","#fc8452","#9a60b4","#ea7ccc"],gradientColor:["#f6efa6","#d88273","#bf444c"],aria:{decal:{decals:[{color:Di,dashArrayX:[1,0],dashArrayY:[2,5],symbolSize:1,rotation:Math.PI/6},{color:Di,symbol:"circle",dashArrayX:[[8,8],[0,8,8,0]],dashArrayY:[6,0],symbolSize:.8},{color:Di,dashArrayX:[1,0],dashArrayY:[4,3],rotation:-Math.PI/4},{color:Di,dashArrayX:[[6,6],[0,6,6,0]],dashArrayY:[6,0]},{color:Di,dashArrayX:[[1,0],[1,6]],dashArrayY:[1,0,6,0],rotation:Math.PI/4},{color:Di,symbol:"triangle",dashArrayX:[[9,9],[0,9,9,0]],dashArrayY:[7,2],symbolSize:.75}]}},textStyle:{fontFamily:gm.match(/^Win/)?"Microsoft YaHei":"sans-serif",fontSize:12,fontStyle:"normal",fontWeight:"normal"},blendMode:null,stateAnimation:{duration:300,easing:"cubicOut"},animation:"auto",animationDuration:1e3,animationDurationUpdate:500,animationEasing:"cubicInOut",animationEasingUpdate:"cubicInOut",animationThreshold:2e3,progressiveThreshold:3e3,progressive:400,hoverLayerThreshold:3e3,useUTC:!1};var CM=$();function DM(r,t,e){var i=CM.get(t);if(!i)return e;var n=i(r);return n?e.concat(n):e}var Ua,vn,Pc,Rc="\0_ec_inner",AM=1,lh=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.init=function(e,i,n,a,o,s){a=a||{},this.option=null,this._theme=new xt(a),this._locale=new xt(o),this._optionManager=s},t.prototype.setOption=function(e,i,n){var a=kc(i);this._optionManager.setOption(e,n,a),this._resetOption(null,a)},t.prototype.resetOption=function(e,i){return this._resetOption(e,kc(i))},t.prototype._resetOption=function(e,i){var n=!1,a=this._optionManager;if(!e||e==="recreate"){var o=a.mountOption(e==="recreate");!this.option||e==="recreate"?Pc(this,o):(this.restoreData(),this._mergeOption(o,i)),n=!0}if((e==="timeline"||e==="media")&&this.restoreData(),!e||e==="recreate"||e==="timeline"){var s=a.getTimelineOption(this);s&&(n=!0,this._mergeOption(s,i))}if(!e||e==="recreate"||e==="media"){var l=a.getMediaOption(this);l.length&&T(l,function(u){n=!0,this._mergeOption(u,i)},this)}return n},t.prototype.mergeOption=function(e){this._mergeOption(e,null)},t.prototype._mergeOption=function(e,i){var n=this.option,a=this._componentsMap,o=this._componentsCount,s=[],l=$(),u=i&&i.replaceMergeMainTypeMap;Zw(this),T(e,function(h,c){h!=null&&(rt.hasClass(c)?c&&(s.push(c),l.set(c,!0)):n[c]=n[c]==null?U(h):J(n[c],h,!0))}),u&&u.each(function(h,c){rt.hasClass(c)&&!l.get(c)&&(s.push(c),l.set(c,!0))}),rt.topologicalTravel(s,rt.getAllClassMainTypes(),f,this);function f(h){var c=DM(this,h,Tt(e[h])),v=a.get(h),d=v?u&&u.get(h)?"replaceMerge":"normalMerge":"replaceAll",y=gS(v,c,d);xS(y,h,rt),n[h]=null,a.set(h,null),o.set(h,0);var p=[],g=[],m=0,_;T(y,function(S,b){var w=S.existing,x=S.newOption;if(!x)w&&(w.mergeOption({},this),w.optionUpdated({},!1));else{var C=h==="series",D=rt.getClass(h,S.keyInfo.subType,!C);if(!D)return;if(h==="tooltip"){if(_)return;_=!0}if(w&&w.constructor===D)w.name=S.keyInfo.name,w.mergeOption(x,this),w.optionUpdated(x,!1);else{var A=O({componentIndex:b},S.keyInfo);w=new D(x,this,this,A),O(w,A),S.brandNew&&(w.__requireNewView=!0),w.init(x,this,this),w.optionUpdated(null,!0)}}w?(p.push(w.option),g.push(w),m++):(p.push(void 0),g.push(void 0))},this),n[h]=p,a.set(h,g),o.set(h,m),h==="series"&&Ua(this)}this._seriesIndices||Ua(this)},t.prototype.getOption=function(){var e=U(this.option);return T(e,function(i,n){if(rt.hasClass(n)){for(var a=Tt(i),o=a.length,s=!1,l=o-1;l>=0;l--)a[l]&&!Zn(a[l])?s=!0:(a[l]=null,!s&&o--);a.length=o,e[n]=a}}),delete e[Rc],e},t.prototype.getTheme=function(){return this._theme},t.prototype.getLocaleModel=function(){return this._locale},t.prototype.setUpdatePayload=function(e){this._payload=e},t.prototype.getUpdatePayload=function(){return this._payload},t.prototype.getComponent=function(e,i){var n=this._componentsMap.get(e);if(n){var a=n[i||0];if(a)return a;if(i==null){for(var o=0;o<n.length;o++)if(n[o])return n[o]}}},t.prototype.queryComponents=function(e){var i=e.mainType;if(!i)return[];var n=e.index,a=e.id,o=e.name,s=this._componentsMap.get(i);if(!s||!s.length)return[];var l;return n!=null?(l=[],T(Tt(n),function(u){s[u]&&l.push(s[u])})):a!=null?l=Ec("id",a,s):o!=null?l=Ec("name",o,s):l=wt(s,function(u){return!!u}),Oc(l,e)},t.prototype.findComponents=function(e){var i=e.query,n=e.mainType,a=s(i),o=a?this.queryComponents(a):wt(this._componentsMap.get(n),function(u){return!!u});return l(Oc(o,e));function s(u){var f=n+"Index",h=n+"Id",c=n+"Name";return u&&(u[f]!=null||u[h]!=null||u[c]!=null)?{mainType:n,index:u[f],id:u[h],name:u[c]}:null}function l(u){return e.filter?wt(u,e.filter):u}},t.prototype.eachComponent=function(e,i,n){var a=this._componentsMap;if(Y(e)){var o=i,s=e;a.each(function(h,c){for(var v=0;h&&v<h.length;v++){var d=h[v];d&&s.call(o,c,d,d.componentIndex)}})}else for(var l=V(e)?a.get(e):z(e)?this.findComponents(e):null,u=0;l&&u<l.length;u++){var f=l[u];f&&i.call(n,f,f.componentIndex)}},t.prototype.getSeriesByName=function(e){var i=Me(e,null);return wt(this._componentsMap.get("series"),function(n){return!!n&&i!=null&&n.name===i})},t.prototype.getSeriesByIndex=function(e){return this._componentsMap.get("series")[e]},t.prototype.getSeriesByType=function(e){return wt(this._componentsMap.get("series"),function(i){return!!i&&i.subType===e})},t.prototype.getSeries=function(){return wt(this._componentsMap.get("series"),function(e){return!!e})},t.prototype.getSeriesCount=function(){return this._componentsCount.get("series")},t.prototype.eachSeries=function(e,i){vn(this),T(this._seriesIndices,function(n){var a=this._componentsMap.get("series")[n];e.call(i,a,n)},this)},t.prototype.eachRawSeries=function(e,i){T(this._componentsMap.get("series"),function(n){n&&e.call(i,n,n.componentIndex)})},t.prototype.eachSeriesByType=function(e,i,n){vn(this),T(this._seriesIndices,function(a){var o=this._componentsMap.get("series")[a];o.subType===e&&i.call(n,o,a)},this)},t.prototype.eachRawSeriesByType=function(e,i,n){return T(this.getSeriesByType(e),i,n)},t.prototype.isSeriesFiltered=function(e){return vn(this),this._seriesIndicesMap.get(e.componentIndex)==null},t.prototype.getCurrentSeriesIndices=function(){return(this._seriesIndices||[]).slice()},t.prototype.filterSeries=function(e,i){vn(this);var n=[];T(this._seriesIndices,function(a){var o=this._componentsMap.get("series")[a];e.call(i,o,a)&&n.push(a)},this),this._seriesIndices=n,this._seriesIndicesMap=$(n)},t.prototype.restoreData=function(e){Ua(this);var i=this._componentsMap,n=[];i.each(function(a,o){rt.hasClass(o)&&n.push(o)}),rt.topologicalTravel(n,rt.getAllClassMainTypes(),function(a){T(i.get(a),function(o){o&&(a!=="series"||!IM(o,e))&&o.restoreData()})})},t.internalField=function(){Ua=function(e){var i=e._seriesIndices=[];T(e._componentsMap.get("series"),function(n){n&&i.push(n.componentIndex)}),e._seriesIndicesMap=$(i)},vn=function(e){},Pc=function(e,i){e.option={},e.option[Rc]=AM,e._componentsMap=$({series:[]}),e._componentsCount=$();var n=i.aria;z(n)&&n.enabled==null&&(n.enabled=!0),LM(i,e._theme.option),J(i,MM,!1),e._mergeOption(i,null)}}(),t}(xt);function IM(r,t){if(t){var e=t.seriesIndex,i=t.seriesId,n=t.seriesName;return e!=null&&r.componentIndex!==e||i!=null&&r.id!==i||n!=null&&r.name!==n}}function LM(r,t){var e=r.color&&!r.colorLayer;T(t,function(i,n){n==="colorLayer"&&e||rt.hasClass(n)||(typeof i=="object"?r[n]=r[n]?J(r[n],i,!1):U(i):r[n]==null&&(r[n]=i))})}function Ec(r,t,e){if(N(t)){var i=$();return T(t,function(a){if(a!=null){var o=Me(a,null);o!=null&&i.set(a,!0)}}),wt(e,function(a){return a&&i.get(a[r])})}else{var n=Me(t,null);return wt(e,function(a){return a&&n!=null&&a[r]===n})}}function Oc(r,t){return t.hasOwnProperty("subType")?wt(r,function(e){return e&&e.subType===t.subType}):r}function kc(r){var t=$();return r&&T(Tt(r.replaceMerge),function(e){t.set(e,!0)}),{replaceMergeMainTypeMap:t}}de(lh,jf);var PM=["getDom","getZr","getWidth","getHeight","getDevicePixelRatio","dispatchAction","isSSR","isDisposed","on","off","getDataURL","getConnectedDataURL","getOption","getId","updateLabelLayout"],ym=function(){function r(t){T(PM,function(e){this[e]=K(t[e],t)},this)}return r}(),RM=/^(min|max)?(.+)$/,EM=function(){function r(t){this._timelineOptions=[],this._mediaList=[],this._currentMediaIndices=[],this._api=t}return r.prototype.setOption=function(t,e,i){t&&(T(Tt(t.series),function(o){o&&o.data&&Ft(o.data)&&Hn(o.data)}),T(Tt(t.dataset),function(o){o&&o.source&&Ft(o.source)&&Hn(o.source)})),t=U(t);var n=this._optionBackup,a=OM(t,e,!n);this._newBaseOption=a.baseOption,n?(a.timelineOptions.length&&(n.timelineOptions=a.timelineOptions),a.mediaList.length&&(n.mediaList=a.mediaList),a.mediaDefault&&(n.mediaDefault=a.mediaDefault)):this._optionBackup=a},r.prototype.mountOption=function(t){var e=this._optionBackup;return this._timelineOptions=e.timelineOptions,this._mediaList=e.mediaList,this._mediaDefault=e.mediaDefault,this._currentMediaIndices=[],U(t?e.baseOption:this._newBaseOption)},r.prototype.getTimelineOption=function(t){var e,i=this._timelineOptions;if(i.length){var n=t.getComponent("timeline");n&&(e=U(i[n.getCurrentIndex()]))}return e},r.prototype.getMediaOption=function(t){var e=this._api.getWidth(),i=this._api.getHeight(),n=this._mediaList,a=this._mediaDefault,o=[],s=[];if(!n.length&&!a)return s;for(var l=0,u=n.length;l<u;l++)kM(n[l].query,e,i)&&o.push(l);return!o.length&&a&&(o=[-1]),o.length&&!NM(o,this._currentMediaIndices)&&(s=H(o,function(f){return U(f===-1?a.option:n[f].option)})),this._currentMediaIndices=o,s},r}();function OM(r,t,e){var i=[],n,a,o=r.baseOption,s=r.timeline,l=r.options,u=r.media,f=!!r.media,h=!!(l||s||o&&o.timeline);o?(a=o,a.timeline||(a.timeline=s)):((h||f)&&(r.options=r.media=null),a=r),f&&N(u)&&T(u,function(v){v&&v.option&&(v.query?i.push(v):n||(n=v))}),c(a),T(l,function(v){return c(v)}),T(i,function(v){return c(v.option)});function c(v){T(t,function(d){d(v,e)})}return{baseOption:a,timelineOptions:l||[],mediaDefault:n,mediaList:i}}function kM(r,t,e){var i={width:t,height:e,aspectratio:t/e},n=!0;return T(r,function(a,o){var s=o.match(RM);if(!(!s||!s[1]||!s[2])){var l=s[1],u=s[2].toLowerCase();BM(i[u],a,l)||(n=!1)}}),n}function BM(r,t,e){return e==="min"?r>=t:e==="max"?r<=t:r===t}function NM(r,t){return r.join(",")===t.join(",")}var ye=T,ea=z,Bc=["areaStyle","lineStyle","nodeStyle","linkStyle","chordStyle","label","labelLine"];function Al(r){var t=r&&r.itemStyle;if(t)for(var e=0,i=Bc.length;e<i;e++){var n=Bc[e],a=t.normal,o=t.emphasis;a&&a[n]&&(r[n]=r[n]||{},r[n].normal?J(r[n].normal,a[n]):r[n].normal=a[n],a[n]=null),o&&o[n]&&(r[n]=r[n]||{},r[n].emphasis?J(r[n].emphasis,o[n]):r[n].emphasis=o[n],o[n]=null)}}function Dt(r,t,e){if(r&&r[t]&&(r[t].normal||r[t].emphasis)){var i=r[t].normal,n=r[t].emphasis;i&&(e?(r[t].normal=r[t].emphasis=null,nt(r[t],i)):r[t]=i),n&&(r.emphasis=r.emphasis||{},r.emphasis[t]=n,n.focus&&(r.emphasis.focus=n.focus),n.blurScope&&(r.emphasis.blurScope=n.blurScope))}}function Tn(r){Dt(r,"itemStyle"),Dt(r,"lineStyle"),Dt(r,"areaStyle"),Dt(r,"label"),Dt(r,"labelLine"),Dt(r,"upperLabel"),Dt(r,"edgeLabel")}function dt(r,t){var e=ea(r)&&r[t],i=ea(e)&&e.textStyle;if(i)for(var n=0,a=rv.length;n<a;n++){var o=rv[n];i.hasOwnProperty(o)&&(e[o]=i[o])}}function ae(r){r&&(Tn(r),dt(r,"label"),r.emphasis&&dt(r.emphasis,"label"))}function FM(r){if(ea(r)){Al(r),Tn(r),dt(r,"label"),dt(r,"upperLabel"),dt(r,"edgeLabel"),r.emphasis&&(dt(r.emphasis,"label"),dt(r.emphasis,"upperLabel"),dt(r.emphasis,"edgeLabel"));var t=r.markPoint;t&&(Al(t),ae(t));var e=r.markLine;e&&(Al(e),ae(e));var i=r.markArea;i&&ae(i);var n=r.data;if(r.type==="graph"){n=n||r.nodes;var a=r.links||r.edges;if(a&&!Ft(a))for(var o=0;o<a.length;o++)ae(a[o]);T(r.categories,function(u){Tn(u)})}if(n&&!Ft(n))for(var o=0;o<n.length;o++)ae(n[o]);if(t=r.markPoint,t&&t.data)for(var s=t.data,o=0;o<s.length;o++)ae(s[o]);if(e=r.markLine,e&&e.data)for(var l=e.data,o=0;o<l.length;o++)N(l[o])?(ae(l[o][0]),ae(l[o][1])):ae(l[o]);r.type==="gauge"?(dt(r,"axisLabel"),dt(r,"title"),dt(r,"detail")):r.type==="treemap"?(Dt(r.breadcrumb,"itemStyle"),T(r.levels,function(u){Tn(u)})):r.type==="tree"&&Tn(r.leaves)}}function $e(r){return N(r)?r:r?[r]:[]}function Nc(r){return(N(r)?r[0]:r)||{}}function zM(r,t){ye($e(r.series),function(i){ea(i)&&FM(i)});var e=["xAxis","yAxis","radiusAxis","angleAxis","singleAxis","parallelAxis","radar"];t&&e.push("valueAxis","categoryAxis","logAxis","timeAxis"),ye(e,function(i){ye($e(r[i]),function(n){n&&(dt(n,"axisLabel"),dt(n.axisPointer,"label"))})}),ye($e(r.parallel),function(i){var n=i&&i.parallelAxisDefault;dt(n,"axisLabel"),dt(n&&n.axisPointer,"label")}),ye($e(r.calendar),function(i){Dt(i,"itemStyle"),dt(i,"dayLabel"),dt(i,"monthLabel"),dt(i,"yearLabel")}),ye($e(r.radar),function(i){dt(i,"name"),i.name&&i.axisName==null&&(i.axisName=i.name,delete i.name),i.nameGap!=null&&i.axisNameGap==null&&(i.axisNameGap=i.nameGap,delete i.nameGap)}),ye($e(r.geo),function(i){ea(i)&&(ae(i),ye($e(i.regions),function(n){ae(n)}))}),ye($e(r.timeline),function(i){ae(i),Dt(i,"label"),Dt(i,"itemStyle"),Dt(i,"controlStyle",!0);var n=i.data;N(n)&&T(n,function(a){z(a)&&(Dt(a,"label"),Dt(a,"itemStyle"))})}),ye($e(r.toolbox),function(i){Dt(i,"iconStyle"),ye(i.feature,function(n){Dt(n,"iconStyle")})}),dt(Nc(r.axisPointer),"label"),dt(Nc(r.tooltip).axisPointer,"label")}function VM(r,t){for(var e=t.split(","),i=r,n=0;n<e.length&&(i=i&&i[e[n]],i!=null);n++);return i}function HM(r,t,e,i){for(var n=t.split(","),a=r,o,s=0;s<n.length-1;s++)o=n[s],a[o]==null&&(a[o]={}),a=a[o];a[n[s]]==null&&(a[n[s]]=e)}function Fc(r){r&&T(GM,function(t){t[0]in r&&!(t[1]in r)&&(r[t[1]]=r[t[0]])})}var GM=[["x","left"],["y","top"],["x2","right"],["y2","bottom"]],WM=["grid","geo","parallel","legend","toolbox","title","visualMap","dataZoom","timeline"],Il=[["borderRadius","barBorderRadius"],["borderColor","barBorderColor"],["borderWidth","barBorderWidth"]];function cn(r){var t=r&&r.itemStyle;if(t)for(var e=0;e<Il.length;e++){var i=Il[e][1],n=Il[e][0];t[i]!=null&&(t[n]=t[i])}}function zc(r){r&&r.alignTo==="edge"&&r.margin!=null&&r.edgeDistance==null&&(r.edgeDistance=r.margin)}function Vc(r){r&&r.downplay&&!r.blur&&(r.blur=r.downplay)}function UM(r){r&&r.focusNodeAdjacency!=null&&(r.emphasis=r.emphasis||{},r.emphasis.focus==null&&(r.emphasis.focus="adjacency"))}function mm(r,t){if(r)for(var e=0;e<r.length;e++)t(r[e]),r[e]&&mm(r[e].children,t)}function _m(r,t){zM(r,t),r.series=Tt(r.series),T(r.series,function(e){if(z(e)){var i=e.type;if(i==="line")e.clipOverflow!=null&&(e.clip=e.clipOverflow);else if(i==="pie"||i==="gauge"){e.clockWise!=null&&(e.clockwise=e.clockWise),zc(e.label);var n=e.data;if(n&&!Ft(n))for(var a=0;a<n.length;a++)zc(n[a]);e.hoverOffset!=null&&(e.emphasis=e.emphasis||{},(e.emphasis.scaleSize=null)&&(e.emphasis.scaleSize=e.hoverOffset))}else if(i==="gauge"){var o=VM(e,"pointer.color");o!=null&&HM(e,"itemStyle.color",o)}else if(i==="bar"){cn(e),cn(e.backgroundStyle),cn(e.emphasis);var n=e.data;if(n&&!Ft(n))for(var a=0;a<n.length;a++)typeof n[a]=="object"&&(cn(n[a]),cn(n[a]&&n[a].emphasis))}else if(i==="sunburst"){var s=e.highlightPolicy;s&&(e.emphasis=e.emphasis||{},e.emphasis.focus||(e.emphasis.focus=s)),Vc(e),mm(e.data,Vc)}else i==="graph"||i==="sankey"?UM(e):i==="map"&&(e.mapType&&!e.map&&(e.map=e.mapType),e.mapLocation&&nt(e,e.mapLocation));e.hoverAnimation!=null&&(e.emphasis=e.emphasis||{},e.emphasis&&e.emphasis.scale==null&&(e.emphasis.scale=e.hoverAnimation)),Fc(e)}}),r.dataRange&&(r.visualMap=r.dataRange),T(WM,function(e){var i=r[e];i&&(N(i)||(i=[i]),T(i,function(n){Fc(n)}))})}function YM(r){var t=$();r.eachSeries(function(e){var i=e.get("stack");if(i){var n=t.get(i)||t.set(i,[]),a=e.getData(),o={stackResultDimension:a.getCalculationInfo("stackResultDimension"),stackedOverDimension:a.getCalculationInfo("stackedOverDimension"),stackedDimension:a.getCalculationInfo("stackedDimension"),stackedByDimension:a.getCalculationInfo("stackedByDimension"),isStackedByIndex:a.getCalculationInfo("isStackedByIndex"),data:a,seriesModel:e};if(!o.stackedDimension||!(o.isStackedByIndex||o.stackedByDimension))return;n.length&&a.setCalculationInfo("stackedOnSeries",n[n.length-1].seriesModel),n.push(o)}}),t.each(XM)}function XM(r){T(r,function(t,e){var i=[],n=[NaN,NaN],a=[t.stackResultDimension,t.stackedOverDimension],o=t.data,s=t.isStackedByIndex,l=t.seriesModel.get("stackStrategy")||"samesign";o.modify(a,function(u,f,h){var c=o.get(t.stackedDimension,h);if(isNaN(c))return n;var v,d;s?d=o.getRawIndex(h):v=o.get(t.stackedByDimension,h);for(var y=NaN,p=e-1;p>=0;p--){var g=r[p];if(s||(d=g.data.rawIndexOf(g.stackedByDimension,v)),d>=0){var m=g.data.getByRawIndex(g.stackResultDimension,d);if(l==="all"||l==="positive"&&m>0||l==="negative"&&m<0||l==="samesign"&&c>=0&&m>0||l==="samesign"&&c<=0&&m<0){c=fS(c,m),y=m;break}}}return i[0]=c,i[1]=y,i})})}var Xt=function(){function r(){this.group=new mt,this.uid=ws("viewComponent")}return r.prototype.init=function(t,e){},r.prototype.render=function(t,e,i,n){},r.prototype.dispose=function(t,e){},r.prototype.updateView=function(t,e,i,n){},r.prototype.updateLayout=function(t,e,i,n){},r.prototype.updateVisual=function(t,e,i,n){},r.prototype.toggleBlurSeries=function(t,e,i){},r.prototype.eachRendered=function(t){var e=this.group;e&&e.traverse(t)},r}();vf(Xt);Jo(Xt);var Hc=_t(),Gc={itemStyle:Gn(ry,!0),lineStyle:Gn(ey,!0)},$M={lineStyle:"stroke",itemStyle:"fill"};function Sm(r,t){var e=r.visualStyleMapper||Gc[t];return e||(console.warn("Unknown style type '"+t+"'."),Gc.itemStyle)}function wm(r,t){var e=r.visualDrawType||$M[t];return e||(console.warn("Unknown style type '"+t+"'."),"fill")}var ZM={createOnAllSeries:!0,performRawSeries:!0,reset:function(r,t){var e=r.getData(),i=r.visualStyleAccessPath||"itemStyle",n=r.getModel(i),a=Sm(r,i),o=a(n),s=n.getShallow("decal");s&&(e.setVisual("decal",s),s.dirty=!0);var l=wm(r,i),u=o[l],f=Y(u)?u:null,h=o.fill==="auto"||o.stroke==="auto";if(!o[l]||f||h){var c=r.getColorFromPalette(r.name,null,t.getSeriesCount());o[l]||(o[l]=c,e.setVisual("colorFromPalette",!0)),o.fill=o.fill==="auto"||Y(o.fill)?c:o.fill,o.stroke=o.stroke==="auto"||Y(o.stroke)?c:o.stroke}if(e.setVisual("style",o),e.setVisual("drawType",l),!t.isSeriesFiltered(r)&&f)return e.setVisual("colorFromPalette",!1),{dataEach:function(v,d){var y=r.getDataParams(d),p=O({},o);p[l]=f(y),v.setItemVisual(d,"style",p)}}}},dn=new xt,qM={createOnAllSeries:!0,performRawSeries:!0,reset:function(r,t){if(!(r.ignoreStyleOnData||t.isSeriesFiltered(r))){var e=r.getData(),i=r.visualStyleAccessPath||"itemStyle",n=Sm(r,i),a=e.getVisual("drawType");return{dataEach:e.hasItemOption?function(o,s){var l=o.getRawDataItem(s);if(l&&l[i]){dn.option=l[i];var u=n(dn),f=o.ensureUniqueItemVisual(s,"style");O(f,u),dn.option.decal&&(o.setItemVisual(s,"decal",dn.option.decal),dn.option.decal.dirty=!0),a in u&&o.setItemVisual(s,"colorFromPalette",!1)}}:null}}}},KM={performRawSeries:!0,overallReset:function(r){var t=$();r.eachSeries(function(e){var i=e.getColorBy();if(!e.isColorBySeries()){var n=e.type+"-"+i,a=t.get(n);a||(a={},t.set(n,a)),Hc(e).scope=a}}),r.eachSeries(function(e){if(!(e.isColorBySeries()||r.isSeriesFiltered(e))){var i=e.getRawData(),n={},a=e.getData(),o=Hc(e).scope,s=e.visualStyleAccessPath||"itemStyle",l=wm(e,s);a.each(function(u){var f=a.getRawIndex(u);n[f]=u}),i.each(function(u){var f=n[u],h=a.getItemVisual(f,"colorFromPalette");if(h){var c=a.ensureUniqueItemVisual(f,"style"),v=i.getName(u)||u+"",d=i.count();c[l]=e.getColorFromPalette(v,o,d)}})}})}},Ya=Math.PI;function QM(r,t){t=t||{},nt(t,{text:"loading",textColor:"#000",fontSize:12,fontWeight:"normal",fontStyle:"normal",fontFamily:"sans-serif",maskColor:"rgba(255, 255, 255, 0.8)",showSpinner:!0,color:"#5470c6",spinnerRadius:10,lineWidth:5,zlevel:0});var e=new mt,i=new gt({style:{fill:t.maskColor},zlevel:t.zlevel,z:1e4});e.add(i);var n=new yt({style:{text:t.text,fill:t.textColor,fontSize:t.fontSize,fontWeight:t.fontWeight,fontStyle:t.fontStyle,fontFamily:t.fontFamily},zlevel:t.zlevel,z:10001}),a=new gt({style:{fill:"none"},textContent:n,textConfig:{position:"right",distance:10},zlevel:t.zlevel,z:10001});e.add(a);var o;return t.showSpinner&&(o=new fs({shape:{startAngle:-Ya/2,endAngle:-Ya/2+.1,r:t.spinnerRadius},style:{stroke:t.color,lineCap:"round",lineWidth:t.lineWidth},zlevel:t.zlevel,z:10001}),o.animateShape(!0).when(1e3,{endAngle:Ya*3/2}).start("circularInOut"),o.animateShape(!0).when(1e3,{startAngle:Ya*3/2}).delay(300).start("circularInOut"),e.add(o)),e.resize=function(){var s=n.getBoundingRect().width,l=t.showSpinner?t.spinnerRadius:0,u=(r.getWidth()-l*2-(t.showSpinner&&s?10:0)-s)/2-(t.showSpinner&&s?0:5+s/2)+(t.showSpinner?0:s/2)+(s?0:l),f=r.getHeight()/2;t.showSpinner&&o.setShape({cx:u,cy:f}),a.setShape({x:u-l,y:f-l,width:l*2,height:l*2}),i.setShape({x:0,y:0,width:r.getWidth(),height:r.getHeight()})},e.resize(),e}var bm=function(){function r(t,e,i,n){this._stageTaskMap=$(),this.ecInstance=t,this.api=e,i=this._dataProcessorHandlers=i.slice(),n=this._visualHandlers=n.slice(),this._allHandlers=i.concat(n)}return r.prototype.restoreData=function(t,e){t.restoreData(e),this._stageTaskMap.each(function(i){var n=i.overallTask;n&&n.dirty()})},r.prototype.getPerformArgs=function(t,e){if(t.__pipeline){var i=this._pipelineMap.get(t.__pipeline.id),n=i.context,a=!e&&i.progressiveEnabled&&(!n||n.progressiveRender)&&t.__idxInPipeline>i.blockIndex,o=a?i.step:null,s=n&&n.modDataCount,l=s!=null?Math.ceil(s/o):null;return{step:o,modBy:l,modDataCount:s}}},r.prototype.getPipeline=function(t){return this._pipelineMap.get(t)},r.prototype.updateStreamModes=function(t,e){var i=this._pipelineMap.get(t.uid),n=t.getData(),a=n.count(),o=i.progressiveEnabled&&e.incrementalPrepareRender&&a>=i.threshold,s=t.get("large")&&a>=t.get("largeThreshold"),l=t.get("progressiveChunkMode")==="mod"?a:null;t.pipelineContext=i.context={progressiveRender:o,modDataCount:l,large:s}},r.prototype.restorePipelines=function(t){var e=this,i=e._pipelineMap=$();t.eachSeries(function(n){var a=n.getProgressive(),o=n.uid;i.set(o,{id:o,head:null,tail:null,threshold:n.getProgressiveThreshold(),progressiveEnabled:a&&!(n.preventIncremental&&n.preventIncremental()),blockIndex:-1,step:Math.round(a||700),count:0}),e._pipe(n,n.dataTask)})},r.prototype.prepareStageTasks=function(){var t=this._stageTaskMap,e=this.api.getModel(),i=this.api;T(this._allHandlers,function(n){var a=t.get(n.uid)||t.set(n.uid,{}),o="";Ce(!(n.reset&&n.overallReset),o),n.reset&&this._createSeriesStageTask(n,a,e,i),n.overallReset&&this._createOverallStageTask(n,a,e,i)},this)},r.prototype.prepareView=function(t,e,i,n){var a=t.renderTask,o=a.context;o.model=e,o.ecModel=i,o.api=n,a.__block=!t.incrementalPrepareRender,this._pipe(e,a)},r.prototype.performDataProcessorTasks=function(t,e){this._performStageTasks(this._dataProcessorHandlers,t,e,{block:!0})},r.prototype.performVisualTasks=function(t,e,i){this._performStageTasks(this._visualHandlers,t,e,i)},r.prototype._performStageTasks=function(t,e,i,n){n=n||{};var a=!1,o=this;T(t,function(l,u){if(!(n.visualType&&n.visualType!==l.visualType)){var f=o._stageTaskMap.get(l.uid),h=f.seriesTaskMap,c=f.overallTask;if(c){var v,d=c.agentStubMap;d.each(function(p){s(n,p)&&(p.dirty(),v=!0)}),v&&c.dirty(),o.updatePayload(c,i);var y=o.getPerformArgs(c,n.block);d.each(function(p){p.perform(y)}),c.perform(y)&&(a=!0)}else h&&h.each(function(p,g){s(n,p)&&p.dirty();var m=o.getPerformArgs(p,n.block);m.skip=!l.performRawSeries&&e.isSeriesFiltered(p.context.model),o.updatePayload(p,i),p.perform(m)&&(a=!0)})}});function s(l,u){return l.setDirty&&(!l.dirtyMap||l.dirtyMap.get(u.__pipeline.id))}this.unfinished=a||this.unfinished},r.prototype.performSeriesTasks=function(t){var e;t.eachSeries(function(i){e=i.dataTask.perform()||e}),this.unfinished=e||this.unfinished},r.prototype.plan=function(){this._pipelineMap.each(function(t){var e=t.tail;do{if(e.__block){t.blockIndex=e.__idxInPipeline;break}e=e.getUpstream()}while(e)})},r.prototype.updatePayload=function(t,e){e!=="remain"&&(t.context.payload=e)},r.prototype._createSeriesStageTask=function(t,e,i,n){var a=this,o=e.seriesTaskMap,s=e.seriesTaskMap=$(),l=t.seriesType,u=t.getTargetSeries;t.createOnAllSeries?i.eachRawSeries(f):l?i.eachRawSeriesByType(l,f):u&&u(i,n).each(f);function f(h){var c=h.uid,v=s.set(c,o&&o.get(c)||Bn({plan:rC,reset:iC,count:aC}));v.context={model:h,ecModel:i,api:n,useClearVisual:t.isVisual&&!t.isLayout,plan:t.plan,reset:t.reset,scheduler:a},a._pipe(h,v)}},r.prototype._createOverallStageTask=function(t,e,i,n){var a=this,o=e.overallTask=e.overallTask||Bn({reset:jM});o.context={ecModel:i,api:n,overallReset:t.overallReset,scheduler:a};var s=o.agentStubMap,l=o.agentStubMap=$(),u=t.seriesType,f=t.getTargetSeries,h=!0,c=!1,v="";Ce(!t.createOnAllSeries,v),u?i.eachRawSeriesByType(u,d):f?f(i,n).each(d):(h=!1,T(i.getSeries(),d));function d(y){var p=y.uid,g=l.set(p,s&&s.get(p)||(c=!0,Bn({reset:JM,onDirty:eC})));g.context={model:y,overallProgress:h},g.agent=o,g.__block=h,a._pipe(y,g)}c&&o.dirty()},r.prototype._pipe=function(t,e){var i=t.uid,n=this._pipelineMap.get(i);!n.head&&(n.head=e),n.tail&&n.tail.pipe(e),n.tail=e,e.__idxInPipeline=n.count++,e.__pipeline=n},r.wrapStageHandler=function(t,e){return Y(t)&&(t={overallReset:t,seriesType:oC(t)}),t.uid=ws("stageHandler"),e&&(t.visualType=e),t},r}();function jM(r){r.overallReset(r.ecModel,r.api,r.payload)}function JM(r){return r.overallProgress&&tC}function tC(){this.agent.dirty(),this.getDownstream().dirty()}function eC(){this.agent&&this.agent.dirty()}function rC(r){return r.plan?r.plan(r.model,r.ecModel,r.api,r.payload):null}function iC(r){r.useClearVisual&&r.data.clearAllVisual();var t=r.resetDefines=Tt(r.reset(r.model,r.ecModel,r.api,r.payload));return t.length>1?H(t,function(e,i){return xm(i)}):nC}var nC=xm(0);function xm(r){return function(t,e){var i=e.data,n=e.resetDefines[r];if(n&&n.dataEach)for(var a=t.start;a<t.end;a++)n.dataEach(i,a);else n&&n.progress&&n.progress(t,i)}}function aC(r){return r.data.count()}function oC(r){Ho=null;try{r(ra,Tm)}catch{}return Ho}var ra={},Tm={},Ho;Mm(ra,lh);Mm(Tm,ym);ra.eachSeriesByType=ra.eachRawSeriesByType=function(r){Ho=r};ra.eachComponent=function(r){r.mainType==="series"&&r.subType&&(Ho=r.subType)};function Mm(r,t){for(var e in t.prototype)r[e]=At}var Wc=["#37A2DA","#32C5E9","#67E0E3","#9FE6B8","#FFDB5C","#ff9f7f","#fb7293","#E062AE","#E690D1","#e7bcf3","#9d96f5","#8378EA","#96BFFF"];const sC={color:Wc,colorLayer:[["#37A2DA","#ffd85c","#fd7b5f"],["#37A2DA","#67E0E3","#FFDB5C","#ff9f7f","#E062AE","#9d96f5"],["#37A2DA","#32C5E9","#9FE6B8","#FFDB5C","#ff9f7f","#fb7293","#e7bcf3","#8378EA","#96BFFF"],Wc]};var Pt="#B9B8CE",Uc="#100C2A",Xa=function(){return{axisLine:{lineStyle:{color:Pt}},splitLine:{lineStyle:{color:"#484753"}},splitArea:{areaStyle:{color:["rgba(255,255,255,0.02)","rgba(255,255,255,0.05)"]}},minorSplitLine:{lineStyle:{color:"#20203B"}}}},Yc=["#4992ff","#7cffb2","#fddd60","#ff6e76","#58d9f9","#05c091","#ff8a45","#8d48e3","#dd79ff"],Cm={darkMode:!0,color:Yc,backgroundColor:Uc,axisPointer:{lineStyle:{color:"#817f91"},crossStyle:{color:"#817f91"},label:{color:"#fff"}},legend:{textStyle:{color:Pt}},textStyle:{color:Pt},title:{textStyle:{color:"#EEF1FA"},subtextStyle:{color:"#B9B8CE"}},toolbox:{iconStyle:{borderColor:Pt}},dataZoom:{borderColor:"#71708A",textStyle:{color:Pt},brushStyle:{color:"rgba(135,163,206,0.3)"},handleStyle:{color:"#353450",borderColor:"#C5CBE3"},moveHandleStyle:{color:"#B0B6C3",opacity:.3},fillerColor:"rgba(135,163,206,0.2)",emphasis:{handleStyle:{borderColor:"#91B7F2",color:"#4D587D"},moveHandleStyle:{color:"#636D9A",opacity:.7}},dataBackground:{lineStyle:{color:"#71708A",width:1},areaStyle:{color:"#71708A"}},selectedDataBackground:{lineStyle:{color:"#87A3CE"},areaStyle:{color:"#87A3CE"}}},visualMap:{textStyle:{color:Pt}},timeline:{lineStyle:{color:Pt},label:{color:Pt},controlStyle:{color:Pt,borderColor:Pt}},calendar:{itemStyle:{color:Uc},dayLabel:{color:Pt},monthLabel:{color:Pt},yearLabel:{color:Pt}},timeAxis:Xa(),logAxis:Xa(),valueAxis:Xa(),categoryAxis:Xa(),line:{symbol:"circle"},graph:{color:Yc},gauge:{title:{color:Pt},axisLine:{lineStyle:{color:[[1,"rgba(207,212,219,0.2)"]]}},axisLabel:{color:Pt},detail:{color:"#EEF1FA"}},candlestick:{itemStyle:{color:"#f64e56",color0:"#54ea92",borderColor:"#f64e56",borderColor0:"#54ea92"}}};Cm.categoryAxis.splitLine.show=!1;var lC=function(){function r(){}return r.prototype.normalizeQuery=function(t){var e={},i={},n={};if(V(t)){var a=Ee(t);e.mainType=a.main||null,e.subType=a.sub||null}else{var o=["Index","Name","Id"],s={name:1,dataIndex:1,dataType:1};T(t,function(l,u){for(var f=!1,h=0;h<o.length;h++){var c=o[h],v=u.lastIndexOf(c);if(v>0&&v===u.length-c.length){var d=u.slice(0,v);d!=="data"&&(e.mainType=d,e[c.toLowerCase()]=l,f=!0)}}s.hasOwnProperty(u)&&(i[u]=l,f=!0),f||(n[u]=l)})}return{cptQuery:e,dataQuery:i,otherQuery:n}},r.prototype.filter=function(t,e){var i=this.eventInfo;if(!i)return!0;var n=i.targetEl,a=i.packedEvent,o=i.model,s=i.view;if(!o||!s)return!0;var l=e.cptQuery,u=e.dataQuery;return f(l,o,"mainType")&&f(l,o,"subType")&&f(l,o,"index","componentIndex")&&f(l,o,"name")&&f(l,o,"id")&&f(u,a,"name")&&f(u,a,"dataIndex")&&f(u,a,"dataType")&&(!s.filterForExposedEvent||s.filterForExposedEvent(t,e.otherQuery,n,a));function f(h,c,v,d){return h[v]==null||c[d||v]===h[v]}},r.prototype.afterTrigger=function(){this.eventInfo=null},r}(),Vu=["symbol","symbolSize","symbolRotate","symbolOffset"],Xc=Vu.concat(["symbolKeepAspect"]),uC={createOnAllSeries:!0,performRawSeries:!0,reset:function(r,t){var e=r.getData();if(r.legendIcon&&e.setVisual("legendIcon",r.legendIcon),!r.hasSymbolVisual)return;for(var i={},n={},a=!1,o=0;o<Vu.length;o++){var s=Vu[o],l=r.get(s);Y(l)?(a=!0,n[s]=l):i[s]=l}if(i.symbol=i.symbol||r.defaultSymbol,e.setVisual(O({legendIcon:r.legendIcon||i.symbol,symbolKeepAspect:r.get("symbolKeepAspect")},i)),t.isSeriesFiltered(r))return;var u=ft(n);function f(h,c){for(var v=r.getRawValue(c),d=r.getDataParams(c),y=0;y<u.length;y++){var p=u[y];h.setItemVisual(c,p,n[p](v,d))}}return{dataEach:a?f:null}}},fC={createOnAllSeries:!0,performRawSeries:!0,reset:function(r,t){if(!r.hasSymbolVisual||t.isSeriesFiltered(r))return;var e=r.getData();function i(n,a){for(var o=n.getItemModel(a),s=0;s<Xc.length;s++){var l=Xc[s],u=o.getShallow(l,!0);u!=null&&n.setItemVisual(a,l,u)}}return{dataEach:e.hasItemOption?i:null}}};function Dm(r,t,e){switch(e){case"color":var i=r.getItemVisual(t,"style");return i[r.getVisual("drawType")];case"opacity":return r.getItemVisual(t,"style").opacity;case"symbol":case"symbolSize":case"liftZ":return r.getItemVisual(t,e)}}function Am(r,t){switch(t){case"color":var e=r.getVisual("style");return e[r.getVisual("drawType")];case"opacity":return r.getVisual("style").opacity;case"symbol":case"symbolSize":case"liftZ":return r.getVisual(t)}}function hC(r,t,e,i){switch(e){case"color":var n=r.ensureUniqueItemVisual(t,"style");n[r.getVisual("drawType")]=i,r.setItemVisual(t,"colorFromPalette",!1);break;case"opacity":r.ensureUniqueItemVisual(t,"style").opacity=i;break;case"symbol":case"symbolSize":case"liftZ":r.setItemVisual(t,e,i);break}}function ki(r,t,e){for(var i;r&&!(t(r)&&(i=r,e));)r=r.__hostTarget||r.parent;return i}var vC=Math.round(Math.random()*9),cC=typeof Object.defineProperty=="function",dC=function(){function r(){this._id="__ec_inner_"+vC++}return r.prototype.get=function(t){return this._guard(t)[this._id]},r.prototype.set=function(t,e){var i=this._guard(t);return cC?Object.defineProperty(i,this._id,{value:e,enumerable:!1,configurable:!0}):i[this._id]=e,this},r.prototype.delete=function(t){return this.has(t)?(delete this._guard(t)[this._id],!0):!1},r.prototype.has=function(t){return!!this._guard(t)[this._id]},r.prototype._guard=function(t){if(t!==Object(t))throw TypeError("Value of WeakMap is not a non-null object.");return t},r}();function Qr(r){return isFinite(r)}function pC(r,t,e){var i=t.x==null?0:t.x,n=t.x2==null?1:t.x2,a=t.y==null?0:t.y,o=t.y2==null?0:t.y2;t.global||(i=i*e.width+e.x,n=n*e.width+e.x,a=a*e.height+e.y,o=o*e.height+e.y),i=Qr(i)?i:0,n=Qr(n)?n:1,a=Qr(a)?a:0,o=Qr(o)?o:0;var s=r.createLinearGradient(i,a,n,o);return s}function gC(r,t,e){var i=e.width,n=e.height,a=Math.min(i,n),o=t.x==null?.5:t.x,s=t.y==null?.5:t.y,l=t.r==null?.5:t.r;t.global||(o=o*i+e.x,s=s*n+e.y,l=l*a),o=Qr(o)?o:.5,s=Qr(s)?s:.5,l=l>=0&&Qr(l)?l:.5;var u=r.createRadialGradient(o,s,0,o,s,l);return u}function Hu(r,t,e){for(var i=t.type==="radial"?gC(r,t,e):pC(r,t,e),n=t.colorStops,a=0;a<n.length;a++)i.addColorStop(n[a].offset,n[a].color);return i}function yC(r,t){if(r===t||!r&&!t)return!1;if(!r||!t||r.length!==t.length)return!0;for(var e=0;e<r.length;e++)if(r[e]!==t[e])return!0;return!1}function $a(r){return parseInt(r,10)}function Za(r,t,e){var i=["width","height"][t],n=["clientWidth","clientHeight"][t],a=["paddingLeft","paddingTop"][t],o=["paddingRight","paddingBottom"][t];if(e[i]!=null&&e[i]!=="auto")return parseFloat(e[i]);var s=document.defaultView.getComputedStyle(r);return(r[n]||$a(s[i])||$a(r.style[i]))-($a(s[a])||0)-($a(s[o])||0)|0}function mC(r,t){return!r||r==="solid"||!(t>0)?null:r==="dashed"?[4*t,2*t]:r==="dotted"?[t]:ht(r)?[r]:N(r)?r:null}function Im(r){var t=r.style,e=t.lineDash&&t.lineWidth>0&&mC(t.lineDash,t.lineWidth),i=t.lineDashOffset;if(e){var n=t.strokeNoScale&&r.getLineScale?r.getLineScale():1;n&&n!==1&&(e=H(e,function(a){return a/n}),i/=n)}return[e,i]}var _C=new li(!0);function Go(r){var t=r.stroke;return!(t==null||t==="none"||!(r.lineWidth>0))}function $c(r){return typeof r=="string"&&r!=="none"}function Wo(r){var t=r.fill;return t!=null&&t!=="none"}function Zc(r,t){if(t.fillOpacity!=null&&t.fillOpacity!==1){var e=r.globalAlpha;r.globalAlpha=t.fillOpacity*t.opacity,r.fill(),r.globalAlpha=e}else r.fill()}function qc(r,t){if(t.strokeOpacity!=null&&t.strokeOpacity!==1){var e=r.globalAlpha;r.globalAlpha=t.strokeOpacity*t.opacity,r.stroke(),r.globalAlpha=e}else r.stroke()}function Gu(r,t,e){var i=Ip(t.image,t.__image,e);if(ts(i)){var n=r.createPattern(i,t.repeat||"repeat");if(typeof DOMMatrix=="function"&&n&&n.setTransform){var a=new DOMMatrix;a.translateSelf(t.x||0,t.y||0),a.rotateSelf(0,0,(t.rotation||0)*Mp),a.scaleSelf(t.scaleX||1,t.scaleY||1),n.setTransform(a)}return n}}function SC(r,t,e,i){var n,a=Go(e),o=Wo(e),s=e.strokePercent,l=s<1,u=!t.path;(!t.silent||l)&&u&&t.createPathProxy();var f=t.path||_C,h=t.__dirty;if(!i){var c=e.fill,v=e.stroke,d=o&&!!c.colorStops,y=a&&!!v.colorStops,p=o&&!!c.image,g=a&&!!v.image,m=void 0,_=void 0,S=void 0,b=void 0,w=void 0;(d||y)&&(w=t.getBoundingRect()),d&&(m=h?Hu(r,c,w):t.__canvasFillGradient,t.__canvasFillGradient=m),y&&(_=h?Hu(r,v,w):t.__canvasStrokeGradient,t.__canvasStrokeGradient=_),p&&(S=h||!t.__canvasFillPattern?Gu(r,c,t):t.__canvasFillPattern,t.__canvasFillPattern=S),g&&(b=h||!t.__canvasStrokePattern?Gu(r,v,t):t.__canvasStrokePattern,t.__canvasStrokePattern=S),d?r.fillStyle=m:p&&(S?r.fillStyle=S:o=!1),y?r.strokeStyle=_:g&&(b?r.strokeStyle=b:a=!1)}var x=t.getGlobalScale();f.setScale(x[0],x[1],t.segmentIgnoreThreshold);var C,D;r.setLineDash&&e.lineDash&&(n=Im(t),C=n[0],D=n[1]);var A=!0;(u||h&Ri)&&(f.setDPR(r.dpr),l?f.setContext(null):(f.setContext(r),A=!1),f.reset(),t.buildPath(f,t.shape,i),f.toStatic(),t.pathUpdated()),A&&f.rebuildPath(r,l?s:1),C&&(r.setLineDash(C),r.lineDashOffset=D),i||(e.strokeFirst?(a&&qc(r,e),o&&Zc(r,e)):(o&&Zc(r,e),a&&qc(r,e))),C&&r.setLineDash([])}function wC(r,t,e){var i=t.__image=Ip(e.image,t.__image,t,t.onload);if(!(!i||!ts(i))){var n=e.x||0,a=e.y||0,o=t.getWidth(),s=t.getHeight(),l=i.width/i.height;if(o==null&&s!=null?o=s*l:s==null&&o!=null?s=o/l:o==null&&s==null&&(o=i.width,s=i.height),e.sWidth&&e.sHeight){var u=e.sx||0,f=e.sy||0;r.drawImage(i,u,f,e.sWidth,e.sHeight,n,a,o,s)}else if(e.sx&&e.sy){var u=e.sx,f=e.sy,h=o-u,c=s-f;r.drawImage(i,u,f,h,c,n,a,o,s)}else r.drawImage(i,n,a,o,s)}}function bC(r,t,e){var i,n=e.text;if(n!=null&&(n+=""),n){r.font=e.font||ai,r.textAlign=e.textAlign,r.textBaseline=e.textBaseline;var a=void 0,o=void 0;r.setLineDash&&e.lineDash&&(i=Im(t),a=i[0],o=i[1]),a&&(r.setLineDash(a),r.lineDashOffset=o),e.strokeFirst?(Go(e)&&r.strokeText(n,e.x,e.y),Wo(e)&&r.fillText(n,e.x,e.y)):(Wo(e)&&r.fillText(n,e.x,e.y),Go(e)&&r.strokeText(n,e.x,e.y)),a&&r.setLineDash([])}}var Kc=["shadowBlur","shadowOffsetX","shadowOffsetY"],Qc=[["lineCap","butt"],["lineJoin","miter"],["miterLimit",10]];function Lm(r,t,e,i,n){var a=!1;if(!i&&(e=e||{},t===e))return!1;if(i||t.opacity!==e.opacity){Ut(r,n),a=!0;var o=Math.max(Math.min(t.opacity,1),0);r.globalAlpha=isNaN(o)?ei.opacity:o}(i||t.blend!==e.blend)&&(a||(Ut(r,n),a=!0),r.globalCompositeOperation=t.blend||ei.blend);for(var s=0;s<Kc.length;s++){var l=Kc[s];(i||t[l]!==e[l])&&(a||(Ut(r,n),a=!0),r[l]=r.dpr*(t[l]||0))}return(i||t.shadowColor!==e.shadowColor)&&(a||(Ut(r,n),a=!0),r.shadowColor=t.shadowColor||ei.shadowColor),a}function jc(r,t,e,i,n){var a=ia(t,n.inHover),o=i?null:e&&ia(e,n.inHover)||{};if(a===o)return!1;var s=Lm(r,a,o,i,n);if((i||a.fill!==o.fill)&&(s||(Ut(r,n),s=!0),$c(a.fill)&&(r.fillStyle=a.fill)),(i||a.stroke!==o.stroke)&&(s||(Ut(r,n),s=!0),$c(a.stroke)&&(r.strokeStyle=a.stroke)),(i||a.opacity!==o.opacity)&&(s||(Ut(r,n),s=!0),r.globalAlpha=a.opacity==null?1:a.opacity),t.hasStroke()){var l=a.lineWidth,u=l/(a.strokeNoScale&&t.getLineScale?t.getLineScale():1);r.lineWidth!==u&&(s||(Ut(r,n),s=!0),r.lineWidth=u)}for(var f=0;f<Qc.length;f++){var h=Qc[f],c=h[0];(i||a[c]!==o[c])&&(s||(Ut(r,n),s=!0),r[c]=a[c]||h[1])}return s}function xC(r,t,e,i,n){return Lm(r,ia(t,n.inHover),e&&ia(e,n.inHover),i,n)}function Pm(r,t){var e=t.transform,i=r.dpr||1;e?r.setTransform(i*e[0],i*e[1],i*e[2],i*e[3],i*e[4],i*e[5]):r.setTransform(i,0,0,i,0,0)}function TC(r,t,e){for(var i=!1,n=0;n<r.length;n++){var a=r[n];i=i||a.isZeroArea(),Pm(t,a),t.beginPath(),a.buildPath(t,a.shape),t.clip()}e.allClipped=i}function MC(r,t){return r&&t?r[0]!==t[0]||r[1]!==t[1]||r[2]!==t[2]||r[3]!==t[3]||r[4]!==t[4]||r[5]!==t[5]:!(!r&&!t)}var Jc=1,td=2,ed=3,rd=4;function CC(r){var t=Wo(r),e=Go(r);return!(r.lineDash||!(+t^+e)||t&&typeof r.fill!="string"||e&&typeof r.stroke!="string"||r.strokePercent<1||r.strokeOpacity<1||r.fillOpacity<1)}function Ut(r,t){t.batchFill&&r.fill(),t.batchStroke&&r.stroke(),t.batchFill="",t.batchStroke=""}function ia(r,t){return t&&r.__hoverStyle||r.style}function Rm(r,t){jr(r,t,{inHover:!1,viewWidth:0,viewHeight:0},!0)}function jr(r,t,e,i){var n=t.transform;if(!t.shouldBePainted(e.viewWidth,e.viewHeight,!1,!1)){t.__dirty&=~Kt,t.__isRendered=!1;return}var a=t.__clipPaths,o=e.prevElClipPaths,s=!1,l=!1;if((!o||yC(a,o))&&(o&&o.length&&(Ut(r,e),r.restore(),l=s=!0,e.prevElClipPaths=null,e.allClipped=!1,e.prevEl=null),a&&a.length&&(Ut(r,e),r.save(),TC(a,r,e),s=!0),e.prevElClipPaths=a),e.allClipped){t.__isRendered=!1;return}t.beforeBrush&&t.beforeBrush(),t.innerBeforeBrush();var u=e.prevEl;u||(l=s=!0);var f=t instanceof st&&t.autoBatch&&CC(t.style);s||MC(n,u.transform)?(Ut(r,e),Pm(r,t)):f||Ut(r,e);var h=ia(t,e.inHover);t instanceof st?(e.lastDrawType!==Jc&&(l=!0,e.lastDrawType=Jc),jc(r,t,u,l,e),(!f||!e.batchFill&&!e.batchStroke)&&r.beginPath(),SC(r,t,h,f),f&&(e.batchFill=h.fill||"",e.batchStroke=h.stroke||"")):t instanceof Lo?(e.lastDrawType!==ed&&(l=!0,e.lastDrawType=ed),jc(r,t,u,l,e),bC(r,t,h)):t instanceof tr?(e.lastDrawType!==td&&(l=!0,e.lastDrawType=td),xC(r,t,u,l,e),wC(r,t,h)):t.getTemporalDisplayables&&(e.lastDrawType!==rd&&(l=!0,e.lastDrawType=rd),DC(r,t,e)),f&&i&&Ut(r,e),t.innerAfterBrush(),t.afterBrush&&t.afterBrush(),e.prevEl=t,t.__dirty=0,t.__isRendered=!0}function DC(r,t,e){var i=t.getDisplayables(),n=t.getTemporalDisplayables();r.save();var a={prevElClipPaths:null,prevEl:null,allClipped:!1,viewWidth:e.viewWidth,viewHeight:e.viewHeight,inHover:e.inHover},o,s;for(o=t.getCursor(),s=i.length;o<s;o++){var l=i[o];l.beforeBrush&&l.beforeBrush(),l.innerBeforeBrush(),jr(r,l,a,o===s-1),l.innerAfterBrush(),l.afterBrush&&l.afterBrush(),a.prevEl=l}for(var u=0,f=n.length;u<f;u++){var l=n[u];l.beforeBrush&&l.beforeBrush(),l.innerBeforeBrush(),jr(r,l,a,u===f-1),l.innerAfterBrush(),l.afterBrush&&l.afterBrush(),a.prevEl=l}t.clearTemporalDisplayables(),t.notClear=!0,r.restore()}var Ll=new dC,id=new ua(100),nd=["symbol","symbolSize","symbolKeepAspect","color","backgroundColor","dashArrayX","dashArrayY","maxTileWidth","maxTileHeight"];function ad(r,t){if(r==="none")return null;var e=t.getDevicePixelRatio(),i=t.getZr(),n=i.painter.type==="svg";r.dirty&&Ll.delete(r);var a=Ll.get(r);if(a)return a;var o=nt(r,{symbol:"rect",symbolSize:1,symbolKeepAspect:!0,color:"rgba(0, 0, 0, 0.2)",backgroundColor:null,dashArrayX:5,dashArrayY:5,rotation:0,maxTileWidth:512,maxTileHeight:512});o.backgroundColor==="none"&&(o.backgroundColor=null);var s={repeat:"repeat"};return l(s),s.rotation=o.rotation,s.scaleX=s.scaleY=n?1:1/e,Ll.set(r,s),r.dirty=!1,s;function l(u){for(var f=[e],h=!0,c=0;c<nd.length;++c){var v=o[nd[c]];if(v!=null&&!N(v)&&!V(v)&&!ht(v)&&typeof v!="boolean"){h=!1;break}f.push(v)}var d;if(h){d=f.join(",")+(n?"-svg":"");var y=id.get(d);y&&(n?u.svgElement=y:u.image=y)}var p=Om(o.dashArrayX),g=AC(o.dashArrayY),m=Em(o.symbol),_=IC(p),S=km(g),b=!n&&je.createCanvas(),w=n&&{tag:"g",attrs:{},key:"dcl",children:[]},x=D(),C;b&&(b.width=x.width*e,b.height=x.height*e,C=b.getContext("2d")),A(),h&&id.put(d,b||w),u.image=b,u.svgElement=w,u.svgWidth=x.width,u.svgHeight=x.height;function D(){for(var M=1,I=0,L=_.length;I<L;++I)M=Jh(M,_[I]);for(var P=1,I=0,L=m.length;I<L;++I)P=Jh(P,m[I].length);M*=P;var R=S*_.length*m.length;return{width:Math.max(1,Math.min(M,o.maxTileWidth)),height:Math.max(1,Math.min(R,o.maxTileHeight))}}function A(){C&&(C.clearRect(0,0,b.width,b.height),o.backgroundColor&&(C.fillStyle=o.backgroundColor,C.fillRect(0,0,b.width,b.height)));for(var M=0,I=0;I<g.length;++I)M+=g[I];if(M<=0)return;for(var L=-S,P=0,R=0,E=0;L<x.height;){if(P%2===0){for(var W=R/2%m.length,B=0,F=0,G=0;B<x.width*2;){for(var q=0,I=0;I<p[E].length;++I)q+=p[E][I];if(q<=0)break;if(F%2===0){var it=(1-o.symbolSize)*.5,lt=B+p[E][F]*it,vt=L+g[P]*it,$t=p[E][F]*o.symbolSize,ct=g[P]*o.symbolSize,Ye=G/2%m[W].length;pi(lt,vt,$t,ct,m[W][Ye])}B+=p[E][F],++G,++F,F===p[E].length&&(F=0)}++E,E===p.length&&(E=0)}L+=g[P],++R,++P,P===g.length&&(P=0)}function pi(Zt,St,X,j,Tr){var Lt=n?1:e,wh=Je(Tr,Zt*Lt,St*Lt,X*Lt,j*Lt,o.color,o.symbolKeepAspect);if(n){var bh=i.painter.renderOneToVNode(wh);bh&&w.children.push(bh)}else Rm(C,wh)}}}}function Em(r){if(!r||r.length===0)return[["rect"]];if(V(r))return[[r]];for(var t=!0,e=0;e<r.length;++e)if(!V(r[e])){t=!1;break}if(t)return Em([r]);for(var i=[],e=0;e<r.length;++e)V(r[e])?i.push([r[e]]):i.push(r[e]);return i}function Om(r){if(!r||r.length===0)return[[0,0]];if(ht(r)){var t=Math.ceil(r);return[[t,t]]}for(var e=!0,i=0;i<r.length;++i)if(!ht(r[i])){e=!1;break}if(e)return Om([r]);for(var n=[],i=0;i<r.length;++i)if(ht(r[i])){var t=Math.ceil(r[i]);n.push([t,t])}else{var t=H(r[i],function(s){return Math.ceil(s)});t.length%2===1?n.push(t.concat(t)):n.push(t)}return n}function AC(r){if(!r||typeof r=="object"&&r.length===0)return[0,0];if(ht(r)){var t=Math.ceil(r);return[t,t]}var e=H(r,function(i){return Math.ceil(i)});return r.length%2?e.concat(e):e}function IC(r){return H(r,function(t){return km(t)})}function km(r){for(var t=0,e=0;e<r.length;++e)t+=r[e];return r.length%2===1?t*2:t}function LC(r,t){r.eachRawSeries(function(e){if(!r.isSeriesFiltered(e)){var i=e.getData();i.hasItemVisual()&&i.each(function(o){var s=i.getItemVisual(o,"decal");if(s){var l=i.ensureUniqueItemVisual(o,"style");l.decal=ad(s,t)}});var n=i.getVisual("decal");if(n){var a=i.getVisual("style");a.decal=ad(n,t)}}})}var we=new Ge,Bm={};function PC(r,t){Bm[r]=t}function Nm(r){return Bm[r]}var nL="5.5.1",aL={zrender:"5.6.0"},RC=1,EC=800,OC=900,kC=1e3,BC=2e3,NC=5e3,Fm=1e3,FC=1100,uh=2e3,zm=3e3,zC=4e3,Cs=4500,VC=4600,HC=5e3,GC=6e3,Vm=7e3,WC={PROCESSOR:{FILTER:kC,SERIES_FILTER:EC,STATISTIC:NC},VISUAL:{LAYOUT:Fm,PROGRESSIVE_LAYOUT:FC,GLOBAL:uh,CHART:zm,POST_CHART_LAYOUT:VC,COMPONENT:zC,BRUSH:HC,CHART_ITEM:Cs,ARIA:GC,DECAL:Vm}},Ct="__flagInMainProcess",Vt="__pendingUpdate",Pl="__needsUpdateStatus",od=/^[a-zA-Z0-9_]+$/,Rl="__connectUpdateStatus",sd=0,UC=1,YC=2;function Hm(r){return function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];if(this.isDisposed()){this.id;return}return Wm(this,r,t)}}function Gm(r){return function(){for(var t=[],e=0;e<arguments.length;e++)t[e]=arguments[e];return Wm(this,r,t)}}function Wm(r,t,e){return e[0]=e[0]&&e[0].toLowerCase(),Ge.prototype[t].apply(r,e)}var Um=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t}(Ge),Ym=Um.prototype;Ym.on=Gm("on");Ym.off=Gm("off");var Ai,El,qa,fr,Ol,kl,Bl,pn,gn,ld,ud,Nl,fd,Ka,hd,Xm,ee,vd,Uo=function(r){k(t,r);function t(e,i,n){var a=r.call(this,new lC)||this;a._chartsViews=[],a._chartsMap={},a._componentsViews=[],a._componentsMap={},a._pendingActions=[],n=n||{},V(i)&&(i=$m[i]),a._dom=e;var o="canvas",s="auto",l=!1;n.ssr&&pm(function(c){var v=tt(c),d=v.dataIndex;if(d!=null){var y=$();return y.set("series_index",v.seriesIndex),y.set("data_index",d),v.ssrType&&y.set("ssr_type",v.ssrType),y}});var u=a._zr=Fu(e,{renderer:n.renderer||o,devicePixelRatio:n.devicePixelRatio,width:n.width,height:n.height,ssr:n.ssr,useDirtyRect:Q(n.useDirtyRect,l),useCoarsePointer:Q(n.useCoarsePointer,s),pointerSize:n.pointerSize});a._ssr=n.ssr,a._throttledZrFlush=oh(K(u.flush,u),17),i=U(i),i&&_m(i,!0),a._theme=i,a._locale=Wb(n.locale||Cy),a._coordSysMgr=new _s;var f=a._api=hd(a);function h(c,v){return c.__prio-v.__prio}return go(Xo,h),go(Wu,h),a._scheduler=new bm(a,f,Wu,Xo),a._messageCenter=new Um,a._initEvents(),a.resize=K(a.resize,a),u.animation.on("frame",a._onframe,a),ld(u,a),ud(u,a),Hn(a),a}return t.prototype._onframe=function(){if(!this._disposed){vd(this);var e=this._scheduler;if(this[Vt]){var i=this[Vt].silent;this[Ct]=!0;try{Ai(this),fr.update.call(this,null,this[Vt].updateParams)}catch(l){throw this[Ct]=!1,this[Vt]=null,l}this._zr.flush(),this[Ct]=!1,this[Vt]=null,pn.call(this,i),gn.call(this,i)}else if(e.unfinished){var n=RC,a=this._model,o=this._api;e.unfinished=!1;do{var s=+new Date;e.performSeriesTasks(a),e.performDataProcessorTasks(a),kl(this,a),e.performVisualTasks(a),Ka(this,this._model,o,"remain",{}),n-=+new Date-s}while(n>0&&e.unfinished);e.unfinished||this._zr.flush()}}},t.prototype.getDom=function(){return this._dom},t.prototype.getId=function(){return this.id},t.prototype.getZr=function(){return this._zr},t.prototype.isSSR=function(){return this._ssr},t.prototype.setOption=function(e,i,n){if(!this[Ct]){if(this._disposed){this.id;return}var a,o,s;if(z(i)&&(n=i.lazyUpdate,a=i.silent,o=i.replaceMerge,s=i.transition,i=i.notMerge),this[Ct]=!0,!this._model||i){var l=new EM(this._api),u=this._theme,f=this._model=new lh;f.scheduler=this._scheduler,f.ssr=this._ssr,f.init(null,null,null,u,this._locale,l)}this._model.setOption(e,{replaceMerge:o},Uu);var h={seriesTransition:s,optionChanged:!0};if(n)this[Vt]={silent:a,updateParams:h},this[Ct]=!1,this.getZr().wakeUp();else{try{Ai(this),fr.update.call(this,null,h)}catch(c){throw this[Vt]=null,this[Ct]=!1,c}this._ssr||this._zr.flush(),this[Vt]=null,this[Ct]=!1,pn.call(this,a),gn.call(this,a)}}},t.prototype.setTheme=function(){},t.prototype.getModel=function(){return this._model},t.prototype.getOption=function(){return this._model&&this._model.getOption()},t.prototype.getWidth=function(){return this._zr.getWidth()},t.prototype.getHeight=function(){return this._zr.getHeight()},t.prototype.getDevicePixelRatio=function(){return this._zr.painter.dpr||Z.hasGlobalWindow&&window.devicePixelRatio||1},t.prototype.getRenderedCanvas=function(e){return this.renderToCanvas(e)},t.prototype.renderToCanvas=function(e){e=e||{};var i=this._zr.painter;return i.getRenderedCanvas({backgroundColor:e.backgroundColor||this._model.get("backgroundColor"),pixelRatio:e.pixelRatio||this.getDevicePixelRatio()})},t.prototype.renderToSVGString=function(e){e=e||{};var i=this._zr.painter;return i.renderToString({useViewBox:e.useViewBox})},t.prototype.getSvgDataURL=function(){if(Z.svgSupported){var e=this._zr,i=e.storage.getDisplayList();return T(i,function(n){n.stopAnimation(null,!0)}),e.painter.toDataURL()}},t.prototype.getDataURL=function(e){if(this._disposed){this.id;return}e=e||{};var i=e.excludeComponents,n=this._model,a=[],o=this;T(i,function(l){n.eachComponent({mainType:l},function(u){var f=o._componentsMap[u.__viewId];f.group.ignore||(a.push(f),f.group.ignore=!0)})});var s=this._zr.painter.getType()==="svg"?this.getSvgDataURL():this.renderToCanvas(e).toDataURL("image/"+(e&&e.type||"png"));return T(a,function(l){l.group.ignore=!1}),s},t.prototype.getConnectedDataURL=function(e){if(this._disposed){this.id;return}var i=e.type==="svg",n=this.group,a=Math.min,o=Math.max,s=1/0;if($o[n]){var l=s,u=s,f=-s,h=-s,c=[],v=e&&e.pixelRatio||this.getDevicePixelRatio();T(ni,function(_,S){if(_.group===n){var b=i?_.getZr().painter.getSvgDom().innerHTML:_.renderToCanvas(U(e)),w=_.getDom().getBoundingClientRect();l=a(w.left,l),u=a(w.top,u),f=o(w.right,f),h=o(w.bottom,h),c.push({dom:b,left:w.left,top:w.top})}}),l*=v,u*=v,f*=v,h*=v;var d=f-l,y=h-u,p=je.createCanvas(),g=Fu(p,{renderer:i?"svg":"canvas"});if(g.resize({width:d,height:y}),i){var m="";return T(c,function(_){var S=_.left-l,b=_.top-u;m+='<g transform="translate('+S+","+b+')">'+_.dom+"</g>"}),g.painter.getSvgRoot().innerHTML=m,e.connectedBackgroundColor&&g.painter.setBackgroundColor(e.connectedBackgroundColor),g.refreshImmediately(),g.painter.toDataURL()}else return e.connectedBackgroundColor&&g.add(new gt({shape:{x:0,y:0,width:d,height:y},style:{fill:e.connectedBackgroundColor}})),T(c,function(_){var S=new tr({style:{x:_.left*v-l,y:_.top*v-u,image:_.dom}});g.add(S)}),g.refreshImmediately(),p.toDataURL("image/"+(e&&e.type||"png"))}else return this.getDataURL(e)},t.prototype.convertToPixel=function(e,i){return Ol(this,"convertToPixel",e,i)},t.prototype.convertFromPixel=function(e,i){return Ol(this,"convertFromPixel",e,i)},t.prototype.containPixel=function(e,i){if(this._disposed){this.id;return}var n=this._model,a,o=Qs(n,e);return T(o,function(s,l){l.indexOf("Models")>=0&&T(s,function(u){var f=u.coordinateSystem;if(f&&f.containPoint)a=a||!!f.containPoint(i);else if(l==="seriesModels"){var h=this._chartsMap[u.__viewId];h&&h.containPoint&&(a=a||h.containPoint(i,u))}},this)},this),!!a},t.prototype.getVisual=function(e,i){var n=this._model,a=Qs(n,e,{defaultMainType:"series"}),o=a.seriesModel,s=o.getData(),l=a.hasOwnProperty("dataIndexInside")?a.dataIndexInside:a.hasOwnProperty("dataIndex")?s.indexOfRawIndex(a.dataIndex):null;return l!=null?Dm(s,l,i):Am(s,i)},t.prototype.getViewOfComponentModel=function(e){return this._componentsMap[e.__viewId]},t.prototype.getViewOfSeriesModel=function(e){return this._chartsMap[e.__viewId]},t.prototype._initEvents=function(){var e=this;T(XC,function(i){var n=function(a){var o=e.getModel(),s=a.target,l,u=i==="globalout";if(u?l={}:s&&ki(s,function(d){var y=tt(d);if(y&&y.dataIndex!=null){var p=y.dataModel||o.getSeriesByIndex(y.seriesIndex);return l=p&&p.getDataParams(y.dataIndex,y.dataType,s)||{},!0}else if(y.eventData)return l=O({},y.eventData),!0},!0),l){var f=l.componentType,h=l.componentIndex;(f==="markLine"||f==="markPoint"||f==="markArea")&&(f="series",h=l.seriesIndex);var c=f&&h!=null&&o.getComponent(f,h),v=c&&e[c.mainType==="series"?"_chartsMap":"_componentsMap"][c.__viewId];l.event=a,l.type=i,e._$eventProcessor.eventInfo={targetEl:s,packedEvent:l,model:c,view:v},e.trigger(i,l)}};n.zrEventfulCallAtLast=!0,e._zr.on(i,n,e)}),T(Nn,function(i,n){e._messageCenter.on(n,function(a){this.trigger(n,a)},e)}),T(["selectchanged"],function(i){e._messageCenter.on(i,function(n){this.trigger(i,n)},e)}),WT(this._messageCenter,this,this._api)},t.prototype.isDisposed=function(){return this._disposed},t.prototype.clear=function(){if(this._disposed){this.id;return}this.setOption({series:[]},!0)},t.prototype.dispose=function(){if(this._disposed){this.id;return}this._disposed=!0;var e=this.getDom();e&&cg(this.getDom(),hh,"");var i=this,n=i._api,a=i._model;T(i._componentsViews,function(o){o.dispose(a,n)}),T(i._chartsViews,function(o){o.dispose(a,n)}),i._zr.dispose(),i._dom=i._model=i._chartsMap=i._componentsMap=i._chartsViews=i._componentsViews=i._scheduler=i._api=i._zr=i._throttledZrFlush=i._theme=i._coordSysMgr=i._messageCenter=null,delete ni[i.id]},t.prototype.resize=function(e){if(!this[Ct]){if(this._disposed){this.id;return}this._zr.resize(e);var i=this._model;if(this._loadingFX&&this._loadingFX.resize(),!!i){var n=i.resetOption("media"),a=e&&e.silent;this[Vt]&&(a==null&&(a=this[Vt].silent),n=!0,this[Vt]=null),this[Ct]=!0;try{n&&Ai(this),fr.update.call(this,{type:"resize",animation:O({duration:0},e&&e.animation)})}catch(o){throw this[Ct]=!1,o}this[Ct]=!1,pn.call(this,a),gn.call(this,a)}}},t.prototype.showLoading=function(e,i){if(this._disposed){this.id;return}if(z(e)&&(i=e,e=""),e=e||"default",this.hideLoading(),!!Yu[e]){var n=Yu[e](this._api,i),a=this._zr;this._loadingFX=n,a.add(n)}},t.prototype.hideLoading=function(){if(this._disposed){this.id;return}this._loadingFX&&this._zr.remove(this._loadingFX),this._loadingFX=null},t.prototype.makeActionFromEvent=function(e){var i=O({},e);return i.type=Nn[e.type],i},t.prototype.dispatchAction=function(e,i){if(this._disposed){this.id;return}if(z(i)||(i={silent:!!i}),!!Yo[e.type]&&this._model){if(this[Ct]){this._pendingActions.push(e);return}var n=i.silent;Bl.call(this,e,n);var a=i.flush;a?this._zr.flush():a!==!1&&Z.browser.weChat&&this._throttledZrFlush(),pn.call(this,n),gn.call(this,n)}},t.prototype.updateLabelLayout=function(){we.trigger("series:layoutlabels",this._model,this._api,{updatedSeries:[]})},t.prototype.appendData=function(e){if(this._disposed){this.id;return}var i=e.seriesIndex,n=this.getModel(),a=n.getSeriesByIndex(i);a.appendData(e),this._scheduler.unfinished=!0,this.getZr().wakeUp()},t.internalField=function(){Ai=function(h){var c=h._scheduler;c.restorePipelines(h._model),c.prepareStageTasks(),El(h,!0),El(h,!1),c.plan()},El=function(h,c){for(var v=h._model,d=h._scheduler,y=c?h._componentsViews:h._chartsViews,p=c?h._componentsMap:h._chartsMap,g=h._zr,m=h._api,_=0;_<y.length;_++)y[_].__alive=!1;c?v.eachComponent(function(w,x){w!=="series"&&S(x)}):v.eachSeries(S);function S(w){var x=w.__requireNewView;w.__requireNewView=!1;var C="_ec_"+w.id+"_"+w.type,D=!x&&p[C];if(!D){var A=Ee(w.type),M=c?Xt.getClass(A.main,A.sub):Jt.getClass(A.sub);D=new M,D.init(v,m),p[C]=D,y.push(D),g.add(D.group)}w.__viewId=D.__id=C,D.__alive=!0,D.__model=w,D.group.__ecComponentInfo={mainType:w.mainType,index:w.componentIndex},!c&&d.prepareView(D,w,v,m)}for(var _=0;_<y.length;){var b=y[_];b.__alive?_++:(!c&&b.renderTask.dispose(),g.remove(b.group),b.dispose(v,m),y.splice(_,1),p[b.__id]===b&&delete p[b.__id],b.__id=b.group.__ecComponentInfo=null)}},qa=function(h,c,v,d,y){var p=h._model;if(p.setUpdatePayload(v),!d){T([].concat(h._componentsViews).concat(h._chartsViews),b);return}var g={};g[d+"Id"]=v[d+"Id"],g[d+"Index"]=v[d+"Index"],g[d+"Name"]=v[d+"Name"];var m={mainType:d,query:g};y&&(m.subType=y);var _=v.excludeSeriesId,S;_!=null&&(S=$(),T(Tt(_),function(w){var x=Me(w,null);x!=null&&S.set(x,!0)})),p&&p.eachComponent(m,function(w){var x=S&&S.get(w.id)!=null;if(!x)if(vv(v))if(w instanceof ce)v.type===ri&&!v.notBlur&&!w.get(["emphasis","disabled"])&&HS(w,v,h._api);else{var C=Af(w.mainType,w.componentIndex,v.name,h._api),D=C.focusSelf,A=C.dispatchers;v.type===ri&&D&&!v.notBlur&&gu(w.mainType,w.componentIndex,h._api),A&&T(A,function(M){v.type===ri?Po(M):Ro(M)})}else _u(v)&&w instanceof ce&&(US(w,v,h._api),fv(w),ee(h))},h),p&&p.eachComponent(m,function(w){var x=S&&S.get(w.id)!=null;x||b(h[d==="series"?"_chartsMap":"_componentsMap"][w.__viewId])},h);function b(w){w&&w.__alive&&w[c]&&w[c](w.__model,p,h._api,v)}},fr={prepareAndUpdate:function(h){Ai(this),fr.update.call(this,h,{optionChanged:h.newOption!=null})},update:function(h,c){var v=this._model,d=this._api,y=this._zr,p=this._coordSysMgr,g=this._scheduler;if(v){v.setUpdatePayload(h),g.restoreData(v,h),g.performSeriesTasks(v),p.create(v,d),g.performDataProcessorTasks(v,h),kl(this,v),p.update(v,d),e(v),g.performVisualTasks(v,h),Nl(this,v,d,h,c);var m=v.get("backgroundColor")||"transparent",_=v.get("darkMode");y.setBackgroundColor(m),_!=null&&_!=="auto"&&y.setDarkMode(_),we.trigger("afterupdate",v,d)}},updateTransform:function(h){var c=this,v=this._model,d=this._api;if(v){v.setUpdatePayload(h);var y=[];v.eachComponent(function(g,m){if(g!=="series"){var _=c.getViewOfComponentModel(m);if(_&&_.__alive)if(_.updateTransform){var S=_.updateTransform(m,v,d,h);S&&S.update&&y.push(_)}else y.push(_)}});var p=$();v.eachSeries(function(g){var m=c._chartsMap[g.__viewId];if(m.updateTransform){var _=m.updateTransform(g,v,d,h);_&&_.update&&p.set(g.uid,1)}else p.set(g.uid,1)}),e(v),this._scheduler.performVisualTasks(v,h,{setDirty:!0,dirtyMap:p}),Ka(this,v,d,h,{},p),we.trigger("afterupdate",v,d)}},updateView:function(h){var c=this._model;c&&(c.setUpdatePayload(h),Jt.markUpdateMethod(h,"updateView"),e(c),this._scheduler.performVisualTasks(c,h,{setDirty:!0}),Nl(this,c,this._api,h,{}),we.trigger("afterupdate",c,this._api))},updateVisual:function(h){var c=this,v=this._model;v&&(v.setUpdatePayload(h),v.eachSeries(function(d){d.getData().clearAllVisual()}),Jt.markUpdateMethod(h,"updateVisual"),e(v),this._scheduler.performVisualTasks(v,h,{visualType:"visual",setDirty:!0}),v.eachComponent(function(d,y){if(d!=="series"){var p=c.getViewOfComponentModel(y);p&&p.__alive&&p.updateVisual(y,v,c._api,h)}}),v.eachSeries(function(d){var y=c._chartsMap[d.__viewId];y.updateVisual(d,v,c._api,h)}),we.trigger("afterupdate",v,this._api))},updateLayout:function(h){fr.update.call(this,h)}},Ol=function(h,c,v,d){if(h._disposed){h.id;return}for(var y=h._model,p=h._coordSysMgr.getCoordinateSystems(),g,m=Qs(y,v),_=0;_<p.length;_++){var S=p[_];if(S[c]&&(g=S[c](y,m,d))!=null)return g}},kl=function(h,c){var v=h._chartsMap,d=h._scheduler;c.eachSeries(function(y){d.updateStreamModes(y,v[y.__viewId])})},Bl=function(h,c){var v=this,d=this.getModel(),y=h.type,p=h.escapeConnect,g=Yo[y],m=g.actionInfo,_=(m.update||"update").split(":"),S=_.pop(),b=_[0]!=null&&Ee(_[0]);this[Ct]=!0;var w=[h],x=!1;h.batch&&(x=!0,w=H(h.batch,function(P){return P=nt(O({},P),h),P.batch=null,P}));var C=[],D,A=_u(h),M=vv(h);if(M&&bg(this._api),T(w,function(P){if(D=g.action(P,v._model,v._api),D=D||O({},P),D.type=m.event||D.type,C.push(D),M){var R=Tf(h),E=R.queryOptionMap,W=R.mainTypeSpecified,B=W?E.keys()[0]:"series";qa(v,S,P,B),ee(v)}else A?(qa(v,S,P,"series"),ee(v)):b&&qa(v,S,P,b.main,b.sub)}),S!=="none"&&!M&&!A&&!b)try{this[Vt]?(Ai(this),fr.update.call(this,h),this[Vt]=null):fr[S].call(this,h)}catch(P){throw this[Ct]=!1,P}if(x?D={type:m.event||y,escapeConnect:p,batch:C}:D=C[0],this[Ct]=!1,!c){var I=this._messageCenter;if(I.trigger(D.type,D),A){var L={type:"selectchanged",escapeConnect:p,selected:YS(d),isFromClick:h.isFromClick||!1,fromAction:h.type,fromActionPayload:h};I.trigger(L.type,L)}}},pn=function(h){for(var c=this._pendingActions;c.length;){var v=c.shift();Bl.call(this,v,h)}},gn=function(h){!h&&this.trigger("updated")},ld=function(h,c){h.on("rendered",function(v){c.trigger("rendered",v),h.animation.isFinished()&&!c[Vt]&&!c._scheduler.unfinished&&!c._pendingActions.length&&c.trigger("finished")})},ud=function(h,c){h.on("mouseover",function(v){var d=v.target,y=ki(d,mu);y&&(GS(y,v,c._api),ee(c))}).on("mouseout",function(v){var d=v.target,y=ki(d,mu);y&&(WS(y,v,c._api),ee(c))}).on("click",function(v){var d=v.target,y=ki(d,function(m){return tt(m).dataIndex!=null},!0);if(y){var p=y.selected?"unselect":"select",g=tt(y);c._api.dispatchAction({type:p,dataType:g.dataType,dataIndexInside:g.dataIndex,seriesIndex:g.seriesIndex,isFromClick:!0})}})};function e(h){h.clearColorPalette(),h.eachSeries(function(c){c.clearColorPalette()})}function i(h){var c=[],v=[],d=!1;if(h.eachComponent(function(m,_){var S=_.get("zlevel")||0,b=_.get("z")||0,w=_.getZLevelKey();d=d||!!w,(m==="series"?v:c).push({zlevel:S,z:b,idx:_.componentIndex,type:m,key:w})}),d){var y=c.concat(v),p,g;go(y,function(m,_){return m.zlevel===_.zlevel?m.z-_.z:m.zlevel-_.zlevel}),T(y,function(m){var _=h.getComponent(m.type,m.idx),S=m.zlevel,b=m.key;p!=null&&(S=Math.max(p,S)),b?(S===p&&b!==g&&S++,g=b):g&&(S===p&&S++,g=""),p=S,_.setZLevel(S)})}}Nl=function(h,c,v,d,y){i(c),fd(h,c,v,d,y),T(h._chartsViews,function(p){p.__alive=!1}),Ka(h,c,v,d,y),T(h._chartsViews,function(p){p.__alive||p.remove(c,v)})},fd=function(h,c,v,d,y,p){T(p||h._componentsViews,function(g){var m=g.__model;u(m,g),g.render(m,c,v,d),s(m,g),f(m,g)})},Ka=function(h,c,v,d,y,p){var g=h._scheduler;y=O(y||{},{updatedSeries:c.getSeries()}),we.trigger("series:beforeupdate",c,v,y);var m=!1;c.eachSeries(function(_){var S=h._chartsMap[_.__viewId];S.__alive=!0;var b=S.renderTask;g.updatePayload(b,d),u(_,S),p&&p.get(_.uid)&&b.dirty(),b.perform(g.getPerformArgs(b))&&(m=!0),S.group.silent=!!_.get("silent"),o(_,S),fv(_)}),g.unfinished=m||g.unfinished,we.trigger("series:layoutlabels",c,v,y),we.trigger("series:transition",c,v,y),c.eachSeries(function(_){var S=h._chartsMap[_.__viewId];s(_,S),f(_,S)}),a(h,c),we.trigger("series:afterupdate",c,v,y)},ee=function(h){h[Pl]=!0,h.getZr().wakeUp()},vd=function(h){h[Pl]&&(h.getZr().storage.traverse(function(c){En(c)||n(c)}),h[Pl]=!1)};function n(h){for(var c=[],v=h.currentStates,d=0;d<v.length;d++){var y=v[d];y==="emphasis"||y==="blur"||y==="select"||c.push(y)}h.selected&&h.states.select&&c.push("select"),h.hoverState===os&&h.states.emphasis?c.push("emphasis"):h.hoverState===as&&h.states.blur&&c.push("blur"),h.useStates(c)}function a(h,c){var v=h._zr,d=v.storage,y=0;d.traverse(function(p){p.isGroup||y++}),y>c.get("hoverLayerThreshold")&&!Z.node&&!Z.worker&&c.eachSeries(function(p){if(!p.preventUsingHoverLayer){var g=h._chartsMap[p.__viewId];g.__alive&&g.eachRendered(function(m){m.states.emphasis&&(m.states.emphasis.hoverLayer=!0)})}})}function o(h,c){var v=h.get("blendMode")||null;c.eachRendered(function(d){d.isGroup||(d.style.blend=v)})}function s(h,c){if(!h.preventAutoZ){var v=h.get("z")||0,d=h.get("zlevel")||0;c.eachRendered(function(y){return l(y,v,d,-1/0),!0})}}function l(h,c,v,d){var y=h.getTextContent(),p=h.getTextGuideLine(),g=h.isGroup;if(g)for(var m=h.childrenRef(),_=0;_<m.length;_++)d=Math.max(l(m[_],c,v,d),d);else h.z=c,h.zlevel=v,d=Math.max(h.z2,d);if(y&&(y.z=c,y.zlevel=v,isFinite(d)&&(y.z2=d+2)),p){var S=h.textGuideLineConfig;p.z=c,p.zlevel=v,isFinite(d)&&(p.z2=d+(S&&S.showAbove?1:-1))}return d}function u(h,c){c.eachRendered(function(v){if(!En(v)){var d=v.getTextContent(),y=v.getTextGuideLine();v.stateTransition&&(v.stateTransition=null),d&&d.stateTransition&&(d.stateTransition=null),y&&y.stateTransition&&(y.stateTransition=null),v.hasState()?(v.prevStates=v.currentStates,v.clearStates()):v.prevStates&&(v.prevStates=null)}})}function f(h,c){var v=h.getModel("stateAnimation"),d=h.isAnimationEnabled(),y=v.get("duration"),p=y>0?{duration:y,delay:v.get("delay"),easing:v.get("easing")}:null;c.eachRendered(function(g){if(g.states&&g.states.emphasis){if(En(g))return;if(g instanceof st&&KS(g),g.__dirty){var m=g.prevStates;m&&g.useStates(m)}if(d){g.stateTransition=p;var _=g.getTextContent(),S=g.getTextGuideLine();_&&(_.stateTransition=p),S&&(S.stateTransition=p)}g.__dirty&&n(g)}})}hd=function(h){return new(function(c){k(v,c);function v(){return c!==null&&c.apply(this,arguments)||this}return v.prototype.getCoordinateSystems=function(){return h._coordSysMgr.getCoordinateSystems()},v.prototype.getComponentByElement=function(d){for(;d;){var y=d.__ecComponentInfo;if(y!=null)return h._model.getComponent(y.mainType,y.index);d=d.parent}},v.prototype.enterEmphasis=function(d,y){Po(d,y),ee(h)},v.prototype.leaveEmphasis=function(d,y){Ro(d,y),ee(h)},v.prototype.enterBlur=function(d){VS(d),ee(h)},v.prototype.leaveBlur=function(d){mg(d),ee(h)},v.prototype.enterSelect=function(d){_g(d),ee(h)},v.prototype.leaveSelect=function(d){Sg(d),ee(h)},v.prototype.getModel=function(){return h.getModel()},v.prototype.getViewOfComponentModel=function(d){return h.getViewOfComponentModel(d)},v.prototype.getViewOfSeriesModel=function(d){return h.getViewOfSeriesModel(d)},v}(ym))(h)},Xm=function(h){function c(v,d){for(var y=0;y<v.length;y++){var p=v[y];p[Rl]=d}}T(Nn,function(v,d){h._messageCenter.on(d,function(y){if($o[h.group]&&h[Rl]!==sd){if(y&&y.escapeConnect)return;var p=h.makeActionFromEvent(y),g=[];T(ni,function(m){m!==h&&m.group===h.group&&g.push(m)}),c(g,sd),T(g,function(m){m[Rl]!==UC&&m.dispatchAction(p)}),c(g,YC)}})})}}(),t}(Ge),fh=Uo.prototype;fh.on=Hm("on");fh.off=Hm("off");fh.one=function(r,t,e){var i=this;function n(){for(var a=[],o=0;o<arguments.length;o++)a[o]=arguments[o];t&&t.apply&&t.apply(this,a),i.off(r,n)}this.on.call(this,r,n,e)};var XC=["click","dblclick","mouseover","mouseout","mousemove","mousedown","mouseup","globalout","contextmenu"];var Yo={},Nn={},Wu=[],Uu=[],Xo=[],$m={},Yu={},ni={},$o={},$C=+new Date-0,ZC=+new Date-0,hh="_echarts_instance_";function oL(r,t,e){var i=!(e&&e.ssr);if(i){var n=Zm(r);if(n)return n}var a=new Uo(r,t,e);return a.id="ec_"+$C++,ni[a.id]=a,i&&cg(r,hh,a.id),Xm(a),we.trigger("afterinit",a),a}function sL(r){if(N(r)){var t=r;r=null,T(t,function(e){e.group!=null&&(r=e.group)}),r=r||"g_"+ZC++,T(t,function(e){e.group=r})}return $o[r]=!0,r}function qC(r){$o[r]=!1}var lL=qC;function uL(r){V(r)?r=ni[r]:r instanceof Uo||(r=Zm(r)),r instanceof Uo&&!r.isDisposed()&&r.dispose()}function Zm(r){return ni[DS(r,hh)]}function fL(r){return ni[r]}function qm(r,t){$m[r]=t}function Km(r){ot(Uu,r)<0&&Uu.push(r)}function Qm(r,t){ch(Wu,r,t,BC)}function KC(r){vh("afterinit",r)}function QC(r){vh("afterupdate",r)}function vh(r,t){we.on(r,t)}function qi(r,t,e){Y(t)&&(e=t,t="");var i=z(r)?r.type:[r,r={event:t}][0];r.event=(r.event||i).toLowerCase(),t=r.event,!Nn[t]&&(Ce(od.test(i)&&od.test(t)),Yo[i]||(Yo[i]={action:e,actionInfo:r}),Nn[t]=i)}function jC(r,t){_s.register(r,t)}function hL(r){var t=_s.get(r);if(t)return t.getDimensionsInfo?t.getDimensionsInfo():t.dimensions.slice()}function JC(r,t){ch(Xo,r,t,Fm,"layout")}function di(r,t){ch(Xo,r,t,zm,"visual")}var cd=[];function ch(r,t,e,i,n){if((Y(t)||z(t))&&(e=t,t=i),!(ot(cd,e)>=0)){cd.push(e);var a=bm.wrapStageHandler(e,n);a.__prio=t,a.__raw=e,r.push(a)}}function jm(r,t){Yu[r]=t}function vL(r){o_({createCanvas:r})}function tD(r,t,e){var i=Nm("registerMap");i&&i(r,t,e)}function cL(r){var t=Nm("getMap");return t&&t(r)}var eD=yx;di(uh,ZM);di(Cs,qM);di(Cs,KM);di(uh,uC);di(Cs,fC);di(Vm,LC);Km(_m);Qm(OC,YM);jm("default",QM);qi({type:ri,event:ri,update:ri},At);qi({type:uo,event:uo,update:uo},At);qi({type:Ln,event:Ln,update:Ln},At);qi({type:fo,event:fo,update:fo},At);qi({type:Pn,event:Pn,update:Pn},At);qm("light",sC);qm("dark",Cm);var dL={},dd=[],rD={registerPreprocessor:Km,registerProcessor:Qm,registerPostInit:KC,registerPostUpdate:QC,registerUpdateLifecycle:vh,registerAction:qi,registerCoordinateSystem:jC,registerLayout:JC,registerVisual:di,registerTransform:eD,registerLoading:jm,registerMap:tD,registerImpl:PC,PRIORITY:WC,ComponentModel:rt,ComponentView:Xt,SeriesModel:ce,ChartView:Jt,registerComponentModel:function(r){rt.registerClass(r)},registerComponentView:function(r){Xt.registerClass(r)},registerSeriesModel:function(r){ce.registerClass(r)},registerChartView:function(r){Jt.registerClass(r)},registerSubTypeDefaulter:function(r,t){rt.registerSubTypeDefaulter(r,t)},registerPainter:function(r,t){dm(r,t)}};function Ui(r){if(N(r)){T(r,function(t){Ui(t)});return}ot(dd,r)>=0||(dd.push(r),Y(r)&&(r={install:r}),r.install(rD))}var iD=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.type="grid",t.dependencies=["xAxis","yAxis"],t.layoutMode="box",t.defaultOption={show:!1,z:0,left:"10%",top:60,right:"10%",bottom:70,containLabel:!1,backgroundColor:"rgba(0,0,0,0)",borderWidth:1,borderColor:"#ccc"},t}(rt),Jm=function(){function r(){}return r.prototype.getNeedCrossZero=function(){var t=this.option;return!t.scale},r.prototype.getCoordSysModel=function(){},r}(),Xu=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.getCoordSysModel=function(){return this.getReferringComponents("grid",xe).models[0]},t.type="cartesian2dAxis",t}(rt);de(Xu,Jm);var t0={show:!0,z:0,inverse:!1,name:"",nameLocation:"end",nameRotate:null,nameTruncate:{maxWidth:null,ellipsis:"...",placeholder:"."},nameTextStyle:{},nameGap:15,silent:!1,triggerEvent:!1,tooltip:{show:!1},axisPointer:{},axisLine:{show:!0,onZero:!0,onZeroAxisIndex:null,lineStyle:{color:"#6E7079",width:1,type:"solid"},symbol:["none","none"],symbolSize:[10,15]},axisTick:{show:!0,inside:!1,length:5,lineStyle:{width:1}},axisLabel:{show:!0,inside:!1,rotate:0,showMinLabel:null,showMaxLabel:null,margin:8,fontSize:12},splitLine:{show:!0,lineStyle:{color:["#E0E6F1"],width:1,type:"solid"}},splitArea:{show:!1,areaStyle:{color:["rgba(250,250,250,0.2)","rgba(210,219,238,0.2)"]}}},nD=J({boundaryGap:!0,deduplication:null,splitLine:{show:!1},axisTick:{alignWithLabel:!1,interval:"auto"},axisLabel:{interval:"auto"}},t0),dh=J({boundaryGap:[0,0],axisLine:{show:"auto"},axisTick:{show:"auto"},splitNumber:5,minorTick:{show:!1,splitNumber:5,length:3,lineStyle:{}},minorSplitLine:{show:!1,lineStyle:{color:"#F4F7FD",width:1}}},t0),aD=J({splitNumber:6,axisLabel:{showMinLabel:!1,showMaxLabel:!1,rich:{primary:{fontWeight:"bold"}}},splitLine:{show:!1}},dh),oD=nt({logBase:10},dh);const sD={category:nD,value:dh,time:aD,log:oD};var lD=0,$u=function(){function r(t){this.categories=t.categories||[],this._needCollect=t.needCollect,this._deduplication=t.deduplication,this.uid=++lD}return r.createByAxisModel=function(t){var e=t.option,i=e.data,n=i&&H(i,uD);return new r({categories:n,needCollect:!n,deduplication:e.dedplication!==!1})},r.prototype.getOrdinal=function(t){return this._getOrCreateMap().get(t)},r.prototype.parseAndCollect=function(t){var e,i=this._needCollect;if(!V(t)&&!i)return t;if(i&&!this._deduplication)return e=this.categories.length,this.categories[e]=t,e;var n=this._getOrCreateMap();return e=n.get(t),e==null&&(i?(e=this.categories.length,this.categories[e]=t,n.set(t,e)):e=NaN),e},r.prototype._getOrCreateMap=function(){return this._map||(this._map=$(this.categories))},r}();function uD(r){return z(r)&&r.value!=null?r.value:r+""}var fD={value:1,category:1,time:1,log:1};function pd(r,t,e,i){T(fD,function(n,a){var o=J(J({},sD[a],!0),i,!0),s=function(l){k(u,l);function u(){var f=l!==null&&l.apply(this,arguments)||this;return f.type=t+"Axis."+a,f}return u.prototype.mergeDefaultAndTheme=function(f,h){var c=Qn(this),v=c?Qf(f):{},d=h.getTheme();J(f,d.get(a+"Axis")),J(f,this.getDefaultOption()),f.type=gd(f),c&&jn(f,v,c)},u.prototype.optionUpdated=function(){var f=this.option;f.type==="category"&&(this.__ordinalMeta=$u.createByAxisModel(this))},u.prototype.getCategories=function(f){var h=this.option;if(h.type==="category")return f?h.data:this.__ordinalMeta.categories},u.prototype.getOrdinalMeta=function(){return this.__ordinalMeta},u.type=t+"Axis."+a,u.defaultOption=o,u}(e);r.registerComponentModel(s)}),r.registerSubTypeDefaulter(t+"Axis",gd)}function gd(r){return r.type||(r.data?"category":"value")}var Ue=function(){function r(t){this._setting=t||{},this._extent=[1/0,-1/0]}return r.prototype.getSetting=function(t){return this._setting[t]},r.prototype.unionExtent=function(t){var e=this._extent;t[0]<e[0]&&(e[0]=t[0]),t[1]>e[1]&&(e[1]=t[1])},r.prototype.unionExtentFromData=function(t,e){this.unionExtent(t.getApproximateExtent(e))},r.prototype.getExtent=function(){return this._extent.slice()},r.prototype.setExtent=function(t,e){var i=this._extent;isNaN(t)||(i[0]=t),isNaN(e)||(i[1]=e)},r.prototype.isInExtentRange=function(t){return this._extent[0]<=t&&this._extent[1]>=t},r.prototype.isBlank=function(){return this._isBlank},r.prototype.setBlank=function(t){this._isBlank=t},r}();Jo(Ue);function Zu(r){return r.type==="interval"||r.type==="log"}function hD(r,t,e,i){var n={},a=r[1]-r[0],o=n.interval=xf(a/t,!0);e!=null&&o<e&&(o=n.interval=e),i!=null&&o>i&&(o=n.interval=i);var s=n.intervalPrecision=e0(o),l=n.niceTickExtent=[pt(Math.ceil(r[0]/o)*o,s),pt(Math.floor(r[1]/o)*o,s)];return vD(l,r),n}function Fl(r){var t=Math.pow(10,ns(r)),e=r/t;return e?e===2?e=3:e===3?e=5:e*=2:e=1,pt(e*t)}function e0(r){return Oe(r)+2}function yd(r,t,e){r[t]=Math.max(Math.min(r[t],e[1]),e[0])}function vD(r,t){!isFinite(r[0])&&(r[0]=t[0]),!isFinite(r[1])&&(r[1]=t[1]),yd(r,0,t),yd(r,1,t),r[0]>r[1]&&(r[0]=r[1])}function Ds(r,t){return r>=t[0]&&r<=t[1]}function As(r,t){return t[1]===t[0]?.5:(r-t[0])/(t[1]-t[0])}function Is(r,t){return r*(t[1]-t[0])+t[0]}var ph=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;i.type="ordinal";var n=i.getSetting("ordinalMeta");return n||(n=new $u({})),N(n)&&(n=new $u({categories:H(n,function(a){return z(a)?a.value:a})})),i._ordinalMeta=n,i._extent=i.getSetting("extent")||[0,n.categories.length-1],i}return t.prototype.parse=function(e){return e==null?NaN:V(e)?this._ordinalMeta.getOrdinal(e):Math.round(e)},t.prototype.contain=function(e){return e=this.parse(e),Ds(e,this._extent)&&this._ordinalMeta.categories[e]!=null},t.prototype.normalize=function(e){return e=this._getTickNumber(this.parse(e)),As(e,this._extent)},t.prototype.scale=function(e){return e=Math.round(Is(e,this._extent)),this.getRawOrdinalNumber(e)},t.prototype.getTicks=function(){for(var e=[],i=this._extent,n=i[0];n<=i[1];)e.push({value:n}),n++;return e},t.prototype.getMinorTicks=function(e){},t.prototype.setSortInfo=function(e){if(e==null){this._ordinalNumbersByTick=this._ticksByOrdinalNumber=null;return}for(var i=e.ordinalNumbers,n=this._ordinalNumbersByTick=[],a=this._ticksByOrdinalNumber=[],o=0,s=this._ordinalMeta.categories.length,l=Math.min(s,i.length);o<l;++o){var u=i[o];n[o]=u,a[u]=o}for(var f=0;o<s;++o){for(;a[f]!=null;)f++;n.push(f),a[f]=o}},t.prototype._getTickNumber=function(e){var i=this._ticksByOrdinalNumber;return i&&e>=0&&e<i.length?i[e]:e},t.prototype.getRawOrdinalNumber=function(e){var i=this._ordinalNumbersByTick;return i&&e>=0&&e<i.length?i[e]:e},t.prototype.getLabel=function(e){if(!this.isBlank()){var i=this.getRawOrdinalNumber(e.value),n=this._ordinalMeta.categories[i];return n==null?"":n+""}},t.prototype.count=function(){return this._extent[1]-this._extent[0]+1},t.prototype.unionExtentFromData=function(e,i){this.unionExtent(e.getApproximateExtent(i))},t.prototype.isInExtentRange=function(e){return e=this._getTickNumber(e),this._extent[0]<=e&&this._extent[1]>=e},t.prototype.getOrdinalMeta=function(){return this._ordinalMeta},t.prototype.calcNiceTicks=function(){},t.prototype.calcNiceExtent=function(){},t.type="ordinal",t}(Ue);Ue.registerClass(ph);var Ur=pt,Ki=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="interval",e._interval=0,e._intervalPrecision=2,e}return t.prototype.parse=function(e){return e},t.prototype.contain=function(e){return Ds(e,this._extent)},t.prototype.normalize=function(e){return As(e,this._extent)},t.prototype.scale=function(e){return Is(e,this._extent)},t.prototype.setExtent=function(e,i){var n=this._extent;isNaN(e)||(n[0]=parseFloat(e)),isNaN(i)||(n[1]=parseFloat(i))},t.prototype.unionExtent=function(e){var i=this._extent;e[0]<i[0]&&(i[0]=e[0]),e[1]>i[1]&&(i[1]=e[1]),this.setExtent(i[0],i[1])},t.prototype.getInterval=function(){return this._interval},t.prototype.setInterval=function(e){this._interval=e,this._niceExtent=this._extent.slice(),this._intervalPrecision=e0(e)},t.prototype.getTicks=function(e){var i=this._interval,n=this._extent,a=this._niceExtent,o=this._intervalPrecision,s=[];if(!i)return s;var l=1e4;n[0]<a[0]&&(e?s.push({value:Ur(a[0]-i,o)}):s.push({value:n[0]}));for(var u=a[0];u<=a[1]&&(s.push({value:u}),u=Ur(u+i,o),u!==s[s.length-1].value);)if(s.length>l)return[];var f=s.length?s[s.length-1].value:a[1];return n[1]>f&&(e?s.push({value:Ur(f+i,o)}):s.push({value:n[1]})),s},t.prototype.getMinorTicks=function(e){for(var i=this.getTicks(!0),n=[],a=this.getExtent(),o=1;o<i.length;o++){for(var s=i[o],l=i[o-1],u=0,f=[],h=s.value-l.value,c=h/e;u<e-1;){var v=Ur(l.value+(u+1)*c);v>a[0]&&v<a[1]&&f.push(v),u++}n.push(f)}return n},t.prototype.getLabel=function(e,i){if(e==null)return"";var n=i&&i.precision;n==null?n=Oe(e.value)||0:n==="auto"&&(n=this._intervalPrecision);var a=Ur(e.value,n,!0);return $f(a)},t.prototype.calcNiceTicks=function(e,i,n){e=e||5;var a=this._extent,o=a[1]-a[0];if(isFinite(o)){o<0&&(o=-o,a.reverse());var s=hD(a,e,i,n);this._intervalPrecision=s.intervalPrecision,this._interval=s.interval,this._niceExtent=s.niceTickExtent}},t.prototype.calcNiceExtent=function(e){var i=this._extent;if(i[0]===i[1])if(i[0]!==0){var n=Math.abs(i[0]);e.fixMax||(i[1]+=n/2),i[0]-=n/2}else i[1]=1;var a=i[1]-i[0];isFinite(a)||(i[0]=0,i[1]=1),this.calcNiceTicks(e.splitNumber,e.minInterval,e.maxInterval);var o=this._interval;e.fixMin||(i[0]=Ur(Math.floor(i[0]/o)*o)),e.fixMax||(i[1]=Ur(Math.ceil(i[1]/o)*o))},t.prototype.setNiceExtent=function(e,i){this._niceExtent=[e,i]},t.type="interval",t}(Ue);Ue.registerClass(Ki);var cD=function(r,t,e,i){for(;e<i;){var n=e+i>>>1;r[n][1]<t?e=n+1:i=n}return e},r0=function(r){k(t,r);function t(e){var i=r.call(this,e)||this;return i.type="time",i}return t.prototype.getLabel=function(e){var i=this.getSetting("useUTC");return pa(e.value,Gv[$b(Vi(this._minLevelUnit))]||Gv.second,i,this.getSetting("locale"))},t.prototype.getFormattedLabel=function(e,i,n){var a=this.getSetting("useUTC"),o=this.getSetting("locale");return Zb(e,i,n,o,a)},t.prototype.getTicks=function(){var e=this._interval,i=this._extent,n=[];if(!e)return n;n.push({value:i[0],level:0});var a=this.getSetting("useUTC"),o=SD(this._minLevelUnit,this._approxInterval,a,i);return n=n.concat(o),n.push({value:i[1],level:0}),n},t.prototype.calcNiceExtent=function(e){var i=this._extent;if(i[0]===i[1]&&(i[0]-=fe,i[1]+=fe),i[1]===-1/0&&i[0]===1/0){var n=new Date;i[1]=+new Date(n.getFullYear(),n.getMonth(),n.getDate()),i[0]=i[1]-fe}this.calcNiceTicks(e.splitNumber,e.minInterval,e.maxInterval)},t.prototype.calcNiceTicks=function(e,i,n){e=e||10;var a=this._extent,o=a[1]-a[0];this._approxInterval=o/e,i!=null&&this._approxInterval<i&&(this._approxInterval=i),n!=null&&this._approxInterval>n&&(this._approxInterval=n);var s=Qa.length,l=Math.min(cD(Qa,this._approxInterval,0,s),s-1);this._interval=Qa[l][1],this._minLevelUnit=Qa[Math.max(l-1,0)][0]},t.prototype.parse=function(e){return ht(e)?e:+ve(e)},t.prototype.contain=function(e){return Ds(this.parse(e),this._extent)},t.prototype.normalize=function(e){return As(this.parse(e),this._extent)},t.prototype.scale=function(e){return Is(e,this._extent)},t.type="time",t}(Ki),Qa=[["second",Uf],["minute",Yf],["hour",kn],["quarter-day",kn*6],["half-day",kn*12],["day",fe*1.2],["half-week",fe*3.5],["week",fe*7],["month",fe*31],["quarter",fe*95],["half-year",Hv/2],["year",Hv]];function dD(r,t,e,i){var n=ve(t),a=ve(e),o=function(d){return Wv(n,d,i)===Wv(a,d,i)},s=function(){return o("year")},l=function(){return s()&&o("month")},u=function(){return l()&&o("day")},f=function(){return u()&&o("hour")},h=function(){return f()&&o("minute")},c=function(){return h()&&o("second")},v=function(){return c()&&o("millisecond")};switch(r){case"year":return s();case"month":return l();case"day":return u();case"hour":return f();case"minute":return h();case"second":return c();case"millisecond":return v()}}function pD(r,t){return r/=fe,r>16?16:r>7.5?7:r>3.5?4:r>1.5?2:1}function gD(r){var t=30*fe;return r/=t,r>6?6:r>3?3:r>2?2:1}function yD(r){return r/=kn,r>12?12:r>6?6:r>3.5?4:r>2?2:1}function md(r,t){return r/=t?Yf:Uf,r>30?30:r>20?20:r>15?15:r>10?10:r>5?5:r>2?2:1}function mD(r){return xf(r,!0)}function _D(r,t,e){var i=new Date(r);switch(Vi(t)){case"year":case"month":i[Ly(e)](0);case"day":i[Py(e)](1);case"hour":i[Ry(e)](0);case"minute":i[Ey(e)](0);case"second":i[Oy(e)](0),i[ky(e)](0)}return i.getTime()}function SD(r,t,e,i){var n=1e4,a=Ay,o=0;function s(A,M,I,L,P,R,E){for(var W=new Date(M),B=M,F=W[L]();B<I&&B<=i[1];)E.push({value:B}),F+=A,W[P](F),B=W.getTime();E.push({value:B,notAdd:!0})}function l(A,M,I){var L=[],P=!M.length;if(!dD(Vi(A),i[0],i[1],e)){P&&(M=[{value:_D(new Date(i[0]),A,e)},{value:i[1]}]);for(var R=0;R<M.length-1;R++){var E=M[R].value,W=M[R+1].value;if(E!==W){var B=void 0,F=void 0,G=void 0,q=!1;switch(A){case"year":B=Math.max(1,Math.round(t/fe/365)),F=Xf(e),G=qb(e);break;case"half-year":case"quarter":case"month":B=gD(t),F=Hi(e),G=Ly(e);break;case"week":case"half-week":case"day":B=pD(t),F=bs(e),G=Py(e),q=!0;break;case"half-day":case"quarter-day":case"hour":B=yD(t),F=Kn(e),G=Ry(e);break;case"minute":B=md(t,!0),F=xs(e),G=Ey(e);break;case"second":B=md(t,!1),F=Ts(e),G=Oy(e);break;case"millisecond":B=mD(t),F=Ms(e),G=ky(e);break}s(B,E,W,F,G,q,L),A==="year"&&I.length>1&&R===0&&I.unshift({value:I[0].value-B})}}for(var R=0;R<L.length;R++)I.push(L[R]);return L}}for(var u=[],f=[],h=0,c=0,v=0;v<a.length&&o++<n;++v){var d=Vi(a[v]);if(Xb(a[v])){l(a[v],u[u.length-1]||[],f);var y=a[v+1]?Vi(a[v+1]):null;if(d!==y){if(f.length){c=h,f.sort(function(A,M){return A.value-M.value});for(var p=[],g=0;g<f.length;++g){var m=f[g].value;(g===0||f[g-1].value!==m)&&(p.push(f[g]),m>=i[0]&&m<=i[1]&&h++)}var _=(i[1]-i[0])/t;if(h>_*1.5&&c>_/1.5||(u.push(p),h>_||r===a[v]))break}f=[]}}}for(var S=wt(H(u,function(A){return wt(A,function(M){return M.value>=i[0]&&M.value<=i[1]&&!M.notAdd})}),function(A){return A.length>0}),b=[],w=S.length-1,v=0;v<S.length;++v)for(var x=S[v],C=0;C<x.length;++C)b.push({value:x[C].value,level:w-v});b.sort(function(A,M){return A.value-M.value});for(var D=[],v=0;v<b.length;++v)(v===0||b[v].value!==b[v-1].value)&&D.push(b[v]);return D}Ue.registerClass(r0);var _d=Ue.prototype,Fn=Ki.prototype,wD=pt,bD=Math.floor,xD=Math.ceil,ja=Math.pow,me=Math.log,gh=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="log",e.base=10,e._originalScale=new Ki,e._interval=0,e}return t.prototype.getTicks=function(e){var i=this._originalScale,n=this._extent,a=i.getExtent(),o=Fn.getTicks.call(this,e);return H(o,function(s){var l=s.value,u=pt(ja(this.base,l));return u=l===n[0]&&this._fixMin?Ja(u,a[0]):u,u=l===n[1]&&this._fixMax?Ja(u,a[1]):u,{value:u}},this)},t.prototype.setExtent=function(e,i){var n=me(this.base);e=me(Math.max(0,e))/n,i=me(Math.max(0,i))/n,Fn.setExtent.call(this,e,i)},t.prototype.getExtent=function(){var e=this.base,i=_d.getExtent.call(this);i[0]=ja(e,i[0]),i[1]=ja(e,i[1]);var n=this._originalScale,a=n.getExtent();return this._fixMin&&(i[0]=Ja(i[0],a[0])),this._fixMax&&(i[1]=Ja(i[1],a[1])),i},t.prototype.unionExtent=function(e){this._originalScale.unionExtent(e);var i=this.base;e[0]=me(e[0])/me(i),e[1]=me(e[1])/me(i),_d.unionExtent.call(this,e)},t.prototype.unionExtentFromData=function(e,i){this.unionExtent(e.getApproximateExtent(i))},t.prototype.calcNiceTicks=function(e){e=e||10;var i=this._extent,n=i[1]-i[0];if(!(n===1/0||n<=0)){var a=og(n),o=e/n*a;for(o<=.5&&(a*=10);!isNaN(a)&&Math.abs(a)<1&&Math.abs(a)>0;)a*=10;var s=[pt(xD(i[0]/a)*a),pt(bD(i[1]/a)*a)];this._interval=a,this._niceExtent=s}},t.prototype.calcNiceExtent=function(e){Fn.calcNiceExtent.call(this,e),this._fixMin=e.fixMin,this._fixMax=e.fixMax},t.prototype.parse=function(e){return e},t.prototype.contain=function(e){return e=me(e)/me(this.base),Ds(e,this._extent)},t.prototype.normalize=function(e){return e=me(e)/me(this.base),As(e,this._extent)},t.prototype.scale=function(e){return e=Is(e,this._extent),ja(this.base,e)},t.type="log",t}(Ue),i0=gh.prototype;i0.getMinorTicks=Fn.getMinorTicks;i0.getLabel=Fn.getLabel;function Ja(r,t){return wD(r,Oe(t))}Ue.registerClass(gh);var TD=function(){function r(t,e,i){this._prepareParams(t,e,i)}return r.prototype._prepareParams=function(t,e,i){i[1]<i[0]&&(i=[NaN,NaN]),this._dataMin=i[0],this._dataMax=i[1];var n=this._isOrdinal=t.type==="ordinal";this._needCrossZero=t.type==="interval"&&e.getNeedCrossZero&&e.getNeedCrossZero();var a=e.get("min",!0);a==null&&(a=e.get("startValue",!0));var o=this._modelMinRaw=a;Y(o)?this._modelMinNum=to(t,o({min:i[0],max:i[1]})):o!=="dataMin"&&(this._modelMinNum=to(t,o));var s=this._modelMaxRaw=e.get("max",!0);if(Y(s)?this._modelMaxNum=to(t,s({min:i[0],max:i[1]})):s!=="dataMax"&&(this._modelMaxNum=to(t,s)),n)this._axisDataLen=e.getCategories().length;else{var l=e.get("boundaryGap"),u=N(l)?l:[l||0,l||0];typeof u[0]=="boolean"||typeof u[1]=="boolean"?this._boundaryGapInner=[0,0]:this._boundaryGapInner=[Fe(u[0],1),Fe(u[1],1)]}},r.prototype.calculate=function(){var t=this._isOrdinal,e=this._dataMin,i=this._dataMax,n=this._axisDataLen,a=this._boundaryGapInner,o=t?null:i-e||Math.abs(e),s=this._modelMinRaw==="dataMin"?e:this._modelMinNum,l=this._modelMaxRaw==="dataMax"?i:this._modelMaxNum,u=s!=null,f=l!=null;s==null&&(s=t?n?0:NaN:e-a[0]*o),l==null&&(l=t?n?n-1:NaN:i+a[1]*o),(s==null||!isFinite(s))&&(s=NaN),(l==null||!isFinite(l))&&(l=NaN);var h=Vn(s)||Vn(l)||t&&!n;this._needCrossZero&&(s>0&&l>0&&!u&&(s=0),s<0&&l<0&&!f&&(l=0));var c=this._determinedMin,v=this._determinedMax;return c!=null&&(s=c,u=!0),v!=null&&(l=v,f=!0),{min:s,max:l,minFixed:u,maxFixed:f,isBlank:h}},r.prototype.modifyDataMinMax=function(t,e){this[CD[t]]=e},r.prototype.setDeterminedMinMax=function(t,e){var i=MD[t];this[i]=e},r.prototype.freeze=function(){this.frozen=!0},r}(),MD={min:"_determinedMin",max:"_determinedMax"},CD={min:"_dataMin",max:"_dataMax"};function DD(r,t,e){var i=r.rawExtentInfo;return i||(i=new TD(r,t,e),r.rawExtentInfo=i,i)}function to(r,t){return t==null?null:Vn(t)?NaN:r.parse(t)}function n0(r,t){var e=r.type,i=DD(r,t,r.getExtent()).calculate();r.setBlank(i.isBlank);var n=i.min,a=i.max,o=t.ecModel;if(o&&e==="time"){var s=rm("bar",o),l=!1;if(T(s,function(h){l=l||h.getBaseAxis()===t.axis}),l){var u=im(s),f=AD(n,a,t,u);n=f.min,a=f.max}}return{extent:[n,a],fixMin:i.minFixed,fixMax:i.maxFixed}}function AD(r,t,e,i){var n=e.axis.getExtent(),a=n[1]-n[0],o=TT(i,e.axis);if(o===void 0)return{min:r,max:t};var s=1/0;T(o,function(v){s=Math.min(v.offset,s)});var l=-1/0;T(o,function(v){l=Math.max(v.offset+v.width,l)}),s=Math.abs(s),l=Math.abs(l);var u=s+l,f=t-r,h=1-(s+l)/a,c=f/h-f;return t+=c*(l/u),r-=c*(s/u),{min:r,max:t}}function qu(r,t){var e=t,i=n0(r,e),n=i.extent,a=e.get("splitNumber");r instanceof gh&&(r.base=e.get("logBase"));var o=r.type,s=e.get("interval"),l=o==="interval"||o==="time";r.setExtent(n[0],n[1]),r.calcNiceExtent({splitNumber:a,fixMin:i.fixMin,fixMax:i.fixMax,minInterval:l?e.get("minInterval"):null,maxInterval:l?e.get("maxInterval"):null}),s!=null&&r.setInterval&&r.setInterval(s)}function a0(r,t){if(t=t||r.get("type"),t)switch(t){case"category":return new ph({ordinalMeta:r.getOrdinalMeta?r.getOrdinalMeta():r.getCategories(),extent:[1/0,-1/0]});case"time":return new r0({locale:r.ecModel.getLocaleModel(),useUTC:r.ecModel.get("useUTC")});default:return new(Ue.getClass(t)||Ki)}}function ID(r){var t=r.scale.getExtent(),e=t[0],i=t[1];return!(e>0&&i>0||e<0&&i<0)}function Qi(r){var t=r.getLabelModel().get("formatter"),e=r.type==="category"?r.scale.getExtent()[0]:null;return r.scale.type==="time"?function(i){return function(n,a){return r.scale.getFormattedLabel(n,a,i)}}(t):V(t)?function(i){return function(n){var a=r.scale.getLabel(n),o=i.replace("{value}",a??"");return o}}(t):Y(t)?function(i){return function(n,a){return e!=null&&(a=n.value-e),i(yh(r,n),a,n.level!=null?{level:n.level}:null)}}(t):function(i){return r.scale.getLabel(i)}}function yh(r,t){return r.type==="category"?r.scale.getLabel(t):t.value}function LD(r){var t=r.model,e=r.scale;if(!(!t.get(["axisLabel","show"])||e.isBlank())){var i,n,a=e.getExtent();e instanceof ph?n=e.count():(i=e.getTicks(),n=i.length);var o=r.getLabelModel(),s=Qi(r),l,u=1;n>40&&(u=Math.ceil(n/40));for(var f=0;f<n;f+=u){var h=i?i[f]:{value:a[0]+f},c=s(h,f),v=o.getTextRect(c),d=PD(v,o.get("rotate")||0);l?l.union(d):l=d}return l}}function PD(r,t){var e=t*Math.PI/180,i=r.width,n=r.height,a=i*Math.abs(Math.cos(e))+Math.abs(n*Math.sin(e)),o=i*Math.abs(Math.sin(e))+Math.abs(n*Math.cos(e)),s=new et(r.x,r.y,a,o);return s}function mh(r){var t=r.get("interval");return t??"auto"}function o0(r){return r.type==="category"&&mh(r.getLabelModel())===0}function RD(r,t){var e={};return T(r.mapDimensionsAll(t),function(i){e[Ty(r,i)]=!0}),ft(e)}var ED=function(){function r(t){this.type="cartesian",this._dimList=[],this._axes={},this.name=t||""}return r.prototype.getAxis=function(t){return this._axes[t]},r.prototype.getAxes=function(){return H(this._dimList,function(t){return this._axes[t]},this)},r.prototype.getAxesByScale=function(t){return t=t.toLowerCase(),wt(this.getAxes(),function(e){return e.scale.type===t})},r.prototype.addAxis=function(t){var e=t.dim;this._axes[e]=t,this._dimList.push(e)},r}(),Ku=["x","y"];function Sd(r){return r.type==="interval"||r.type==="time"}var OD=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="cartesian2d",e.dimensions=Ku,e}return t.prototype.calcAffineTransform=function(){this._transform=this._invTransform=null;var e=this.getAxis("x").scale,i=this.getAxis("y").scale;if(!(!Sd(e)||!Sd(i))){var n=e.getExtent(),a=i.getExtent(),o=this.dataToPoint([n[0],a[0]]),s=this.dataToPoint([n[1],a[1]]),l=n[1]-n[0],u=a[1]-a[0];if(!(!l||!u)){var f=(s[0]-o[0])/l,h=(s[1]-o[1])/u,c=o[0]-n[0]*f,v=o[1]-a[0]*h,d=this._transform=[f,0,0,h,c,v];this._invTransform=ha([],d)}}},t.prototype.getBaseAxis=function(){return this.getAxesByScale("ordinal")[0]||this.getAxesByScale("time")[0]||this.getAxis("x")},t.prototype.containPoint=function(e){var i=this.getAxis("x"),n=this.getAxis("y");return i.contain(i.toLocalCoord(e[0]))&&n.contain(n.toLocalCoord(e[1]))},t.prototype.containData=function(e){return this.getAxis("x").containData(e[0])&&this.getAxis("y").containData(e[1])},t.prototype.containZone=function(e,i){var n=this.dataToPoint(e),a=this.dataToPoint(i),o=this.getArea(),s=new et(n[0],n[1],a[0]-n[0],a[1]-n[1]);return o.intersect(s)},t.prototype.dataToPoint=function(e,i,n){n=n||[];var a=e[0],o=e[1];if(this._transform&&a!=null&&isFinite(a)&&o!=null&&isFinite(o))return Bt(n,e,this._transform);var s=this.getAxis("x"),l=this.getAxis("y");return n[0]=s.toGlobalCoord(s.dataToCoord(a,i)),n[1]=l.toGlobalCoord(l.dataToCoord(o,i)),n},t.prototype.clampData=function(e,i){var n=this.getAxis("x").scale,a=this.getAxis("y").scale,o=n.getExtent(),s=a.getExtent(),l=n.parse(e[0]),u=a.parse(e[1]);return i=i||[],i[0]=Math.min(Math.max(Math.min(o[0],o[1]),l),Math.max(o[0],o[1])),i[1]=Math.min(Math.max(Math.min(s[0],s[1]),u),Math.max(s[0],s[1])),i},t.prototype.pointToData=function(e,i){var n=[];if(this._invTransform)return Bt(n,e,this._invTransform);var a=this.getAxis("x"),o=this.getAxis("y");return n[0]=a.coordToData(a.toLocalCoord(e[0]),i),n[1]=o.coordToData(o.toLocalCoord(e[1]),i),n},t.prototype.getOtherAxis=function(e){return this.getAxis(e.dim==="x"?"y":"x")},t.prototype.getArea=function(e){e=e||0;var i=this.getAxis("x").getGlobalExtent(),n=this.getAxis("y").getGlobalExtent(),a=Math.min(i[0],i[1])-e,o=Math.min(n[0],n[1])-e,s=Math.max(i[0],i[1])-a+e,l=Math.max(n[0],n[1])-o+e;return new et(a,o,s,l)},t}(ED),na=_t();function s0(r,t){var e=H(t,function(i){return r.scale.parse(i)});return r.type==="time"&&e.length>0&&(e.sort(),e.unshift(e[0]),e.push(e[e.length-1])),e}function kD(r){var t=r.getLabelModel().get("customValues");if(t){var e=Qi(r);return{labels:s0(r,t).map(function(i){var n={value:i};return{formattedLabel:e(n),rawLabel:r.scale.getLabel(n),tickValue:i}})}}return r.type==="category"?ND(r):zD(r)}function BD(r,t){var e=r.getTickModel().get("customValues");return e?{ticks:s0(r,e)}:r.type==="category"?FD(r,t):{ticks:H(r.scale.getTicks(),function(i){return i.value})}}function ND(r){var t=r.getLabelModel(),e=l0(r,t);return!t.get("show")||r.scale.isBlank()?{labels:[],labelCategoryInterval:e.labelCategoryInterval}:e}function l0(r,t){var e=u0(r,"labels"),i=mh(t),n=f0(e,i);if(n)return n;var a,o;return Y(i)?a=c0(r,i):(o=i==="auto"?VD(r):i,a=v0(r,o)),h0(e,i,{labels:a,labelCategoryInterval:o})}function FD(r,t){var e=u0(r,"ticks"),i=mh(t),n=f0(e,i);if(n)return n;var a,o;if((!t.get("show")||r.scale.isBlank())&&(a=[]),Y(i))a=c0(r,i,!0);else if(i==="auto"){var s=l0(r,r.getLabelModel());o=s.labelCategoryInterval,a=H(s.labels,function(l){return l.tickValue})}else o=i,a=v0(r,o,!0);return h0(e,i,{ticks:a,tickCategoryInterval:o})}function zD(r){var t=r.scale.getTicks(),e=Qi(r);return{labels:H(t,function(i,n){return{level:i.level,formattedLabel:e(i,n),rawLabel:r.scale.getLabel(i),tickValue:i.value}})}}function u0(r,t){return na(r)[t]||(na(r)[t]=[])}function f0(r,t){for(var e=0;e<r.length;e++)if(r[e].key===t)return r[e].value}function h0(r,t,e){return r.push({key:t,value:e}),e}function VD(r){var t=na(r).autoInterval;return t??(na(r).autoInterval=r.calculateCategoryInterval())}function HD(r){var t=GD(r),e=Qi(r),i=(t.axisRotate-t.labelRotate)/180*Math.PI,n=r.scale,a=n.getExtent(),o=n.count();if(a[1]-a[0]<1)return 0;var s=1;o>40&&(s=Math.max(1,Math.floor(o/40)));for(var l=a[0],u=r.dataToCoord(l+1)-r.dataToCoord(l),f=Math.abs(u*Math.cos(i)),h=Math.abs(u*Math.sin(i)),c=0,v=0;l<=a[1];l+=s){var d=0,y=0,p=df(e({value:l}),t.font,"center","top");d=p.width*1.3,y=p.height*1.3,c=Math.max(c,d,7),v=Math.max(v,y,7)}var g=c/f,m=v/h;isNaN(g)&&(g=1/0),isNaN(m)&&(m=1/0);var _=Math.max(0,Math.floor(Math.min(g,m))),S=na(r.model),b=r.getExtent(),w=S.lastAutoInterval,x=S.lastTickCount;return w!=null&&x!=null&&Math.abs(w-_)<=1&&Math.abs(x-o)<=1&&w>_&&S.axisExtent0===b[0]&&S.axisExtent1===b[1]?_=w:(S.lastTickCount=o,S.lastAutoInterval=_,S.axisExtent0=b[0],S.axisExtent1=b[1]),_}function GD(r){var t=r.getLabelModel();return{axisRotate:r.getRotate?r.getRotate():r.isHorizontal&&!r.isHorizontal()?90:0,labelRotate:t.get("rotate")||0,font:t.getFont()}}function v0(r,t,e){var i=Qi(r),n=r.scale,a=n.getExtent(),o=r.getLabelModel(),s=[],l=Math.max((t||0)+1,1),u=a[0],f=n.count();u!==0&&l>1&&f/l>2&&(u=Math.round(Math.ceil(u/l)*l));var h=o0(r),c=o.get("showMinLabel")||h,v=o.get("showMaxLabel")||h;c&&u!==a[0]&&y(a[0]);for(var d=u;d<=a[1];d+=l)y(d);v&&d-l!==a[1]&&y(a[1]);function y(p){var g={value:p};s.push(e?p:{formattedLabel:i(g),rawLabel:n.getLabel(g),tickValue:p})}return s}function c0(r,t,e){var i=r.scale,n=Qi(r),a=[];return T(i.getTicks(),function(o){var s=i.getLabel(o),l=o.value;t(o.value,s)&&a.push(e?l:{formattedLabel:n(o),rawLabel:s,tickValue:l})}),a}var wd=[0,1],WD=function(){function r(t,e,i){this.onBand=!1,this.inverse=!1,this.dim=t,this.scale=e,this._extent=i||[0,0]}return r.prototype.contain=function(t){var e=this._extent,i=Math.min(e[0],e[1]),n=Math.max(e[0],e[1]);return t>=i&&t<=n},r.prototype.containData=function(t){return this.scale.contain(t)},r.prototype.getExtent=function(){return this._extent.slice()},r.prototype.getPixelPrecision=function(t){return ag(t||this.scale.getExtent(),this._extent)},r.prototype.setExtent=function(t,e){var i=this._extent;i[0]=t,i[1]=e},r.prototype.dataToCoord=function(t,e){var i=this._extent,n=this.scale;return t=n.normalize(t),this.onBand&&n.type==="ordinal"&&(i=i.slice(),bd(i,n.count())),ze(t,wd,i,e)},r.prototype.coordToData=function(t,e){var i=this._extent,n=this.scale;this.onBand&&n.type==="ordinal"&&(i=i.slice(),bd(i,n.count()));var a=ze(t,i,wd,e);return this.scale.scale(a)},r.prototype.pointToData=function(t,e){},r.prototype.getTicksCoords=function(t){t=t||{};var e=t.tickModel||this.getTickModel(),i=BD(this,e),n=i.ticks,a=H(n,function(s){return{coord:this.dataToCoord(this.scale.type==="ordinal"?this.scale.getRawOrdinalNumber(s):s),tickValue:s}},this),o=e.get("alignWithLabel");return UD(this,a,o,t.clamp),a},r.prototype.getMinorTicksCoords=function(){if(this.scale.type==="ordinal")return[];var t=this.model.getModel("minorTick"),e=t.get("splitNumber");e>0&&e<100||(e=5);var i=this.scale.getMinorTicks(e),n=H(i,function(a){return H(a,function(o){return{coord:this.dataToCoord(o),tickValue:o}},this)},this);return n},r.prototype.getViewLabels=function(){return kD(this).labels},r.prototype.getLabelModel=function(){return this.model.getModel("axisLabel")},r.prototype.getTickModel=function(){return this.model.getModel("axisTick")},r.prototype.getBandWidth=function(){var t=this._extent,e=this.scale.getExtent(),i=e[1]-e[0]+(this.onBand?1:0);i===0&&(i=1);var n=Math.abs(t[1]-t[0]);return Math.abs(n)/i},r.prototype.calculateCategoryInterval=function(){return HD(this)},r}();function bd(r,t){var e=r[1]-r[0],i=t,n=e/i/2;r[0]+=n,r[1]-=n}function UD(r,t,e,i){var n=t.length;if(!r.onBand||e||!n)return;var a=r.getExtent(),o,s;if(n===1)t[0].coord=a[0],o=t[1]={coord:a[1]};else{var l=t[n-1].tickValue-t[0].tickValue,u=(t[n-1].coord-t[0].coord)/l;T(t,function(v){v.coord-=u/2});var f=r.scale.getExtent();s=1+f[1]-t[n-1].tickValue,o={coord:t[n-1].coord+u*s},t.push(o)}var h=a[0]>a[1];c(t[0].coord,a[0])&&(i?t[0].coord=a[0]:t.shift()),i&&c(a[0],t[0].coord)&&t.unshift({coord:a[0]}),c(a[1],o.coord)&&(i?o.coord=a[1]:t.pop()),i&&c(o.coord,a[1])&&t.push({coord:a[1]});function c(v,d){return v=pt(v),d=pt(d),h?v>d:v<d}}const YD=WD;var XD=function(r){k(t,r);function t(e,i,n,a,o){var s=r.call(this,e,i,n)||this;return s.index=0,s.type=a||"value",s.position=o||"bottom",s}return t.prototype.isHorizontal=function(){var e=this.position;return e==="top"||e==="bottom"},t.prototype.getGlobalExtent=function(e){var i=this.getExtent();return i[0]=this.toGlobalCoord(i[0]),i[1]=this.toGlobalCoord(i[1]),e&&i[0]>i[1]&&i.reverse(),i},t.prototype.pointToData=function(e,i){return this.coordToData(this.toLocalCoord(e[this.dim==="x"?0:1]),i)},t.prototype.setCategorySortInfo=function(e){if(this.type!=="category")return!1;this.model.option.categorySortInfo=e,this.scale.setSortInfo(e)},t}(YD);function Qu(r,t,e){e=e||{};var i=r.coordinateSystem,n=t.axis,a={},o=n.getAxesOnZeroOf()[0],s=n.position,l=o?"onZero":s,u=n.dim,f=i.getRect(),h=[f.x,f.x+f.width,f.y,f.y+f.height],c={left:0,right:1,top:0,bottom:1,onZero:2},v=t.get("offset")||0,d=u==="x"?[h[2]-v,h[3]+v]:[h[0]-v,h[1]+v];if(o){var y=o.toGlobalCoord(o.dataToCoord(0));d[c.onZero]=Math.max(Math.min(y,d[1]),d[0])}a.position=[u==="y"?d[c[l]]:h[0],u==="x"?d[c[l]]:h[3]],a.rotation=Math.PI/2*(u==="x"?0:1);var p={top:-1,bottom:1,left:-1,right:1};a.labelDirection=a.tickDirection=a.nameDirection=p[s],a.labelOffset=o?d[c[s]]-d[c.onZero]:0,t.get(["axisTick","inside"])&&(a.tickDirection=-a.tickDirection),oi(e.labelInside,t.get(["axisLabel","inside"]))&&(a.labelDirection=-a.labelDirection);var g=t.get(["axisLabel","rotate"]);return a.labelRotate=l==="top"?-g:g,a.z2=1,a}function xd(r){return r.get("coordinateSystem")==="cartesian2d"}function Td(r){var t={xAxisModel:null,yAxisModel:null};return T(t,function(e,i){var n=i.replace(/Model$/,""),a=r.getReferringComponents(n,xe).models[0];t[i]=a}),t}var zl=Math.log;function $D(r,t,e){var i=Ki.prototype,n=i.getTicks.call(e),a=i.getTicks.call(e,!0),o=n.length-1,s=i.getInterval.call(e),l=n0(r,t),u=l.extent,f=l.fixMin,h=l.fixMax;if(r.type==="log"){var c=zl(r.base);u=[zl(u[0])/c,zl(u[1])/c]}r.setExtent(u[0],u[1]),r.calcNiceExtent({splitNumber:o,fixMin:f,fixMax:h});var v=i.getExtent.call(r);f&&(u[0]=v[0]),h&&(u[1]=v[1]);var d=i.getInterval.call(r),y=u[0],p=u[1];if(f&&h)d=(p-y)/o;else if(f)for(p=u[0]+d*o;p<u[1]&&isFinite(p)&&isFinite(u[1]);)d=Fl(d),p=u[0]+d*o;else if(h)for(y=u[1]-d*o;y>u[0]&&isFinite(y)&&isFinite(u[0]);)d=Fl(d),y=u[1]-d*o;else{var g=r.getTicks().length-1;g>o&&(d=Fl(d));var m=d*o;p=Math.ceil(u[1]/d)*d,y=pt(p-m),y<0&&u[0]>=0?(y=0,p=pt(m)):p>0&&u[1]<=0&&(p=0,y=-pt(m))}var _=(n[0].value-a[0].value)/s,S=(n[o].value-a[o].value)/s;i.setExtent.call(r,y+d*_,p+d*S),i.setInterval.call(r,d),(_||S)&&i.setNiceExtent.call(r,y+d,p-d)}var ZD=function(){function r(t,e,i){this.type="grid",this._coordsMap={},this._coordsList=[],this._axesMap={},this._axesList=[],this.axisPointerEnabled=!0,this.dimensions=Ku,this._initCartesian(t,e,i),this.model=t}return r.prototype.getRect=function(){return this._rect},r.prototype.update=function(t,e){var i=this._axesMap;this._updateScale(t,this.model);function n(o){var s,l=ft(o),u=l.length;if(u){for(var f=[],h=u-1;h>=0;h--){var c=+l[h],v=o[c],d=v.model,y=v.scale;Zu(y)&&d.get("alignTicks")&&d.get("interval")==null?f.push(v):(qu(y,d),Zu(y)&&(s=v))}f.length&&(s||(s=f.pop(),qu(s.scale,s.model)),T(f,function(p){$D(p.scale,p.model,s.scale)}))}}n(i.x),n(i.y);var a={};T(i.x,function(o){Md(i,"y",o,a)}),T(i.y,function(o){Md(i,"x",o,a)}),this.resize(this.model,e)},r.prototype.resize=function(t,e,i){var n=t.getBoxLayoutParams(),a=!i&&t.get("containLabel"),o=Zi(n,{width:e.getWidth(),height:e.getHeight()});this._rect=o;var s=this._axesList;l(),a&&(T(s,function(u){if(!u.model.get(["axisLabel","inside"])){var f=LD(u);if(f){var h=u.isHorizontal()?"height":"width",c=u.model.get(["axisLabel","margin"]);o[h]-=f[h]+c,u.position==="top"?o.y+=f.height+c:u.position==="left"&&(o.x+=f.width+c)}}}),l()),T(this._coordsList,function(u){u.calcAffineTransform()});function l(){T(s,function(u){var f=u.isHorizontal(),h=f?[0,o.width]:[0,o.height],c=u.inverse?1:0;u.setExtent(h[c],h[1-c]),qD(u,f?o.x:o.y)})}},r.prototype.getAxis=function(t,e){var i=this._axesMap[t];if(i!=null)return i[e||0]},r.prototype.getAxes=function(){return this._axesList.slice()},r.prototype.getCartesian=function(t,e){if(t!=null&&e!=null){var i="x"+t+"y"+e;return this._coordsMap[i]}z(t)&&(e=t.yAxisIndex,t=t.xAxisIndex);for(var n=0,a=this._coordsList;n<a.length;n++)if(a[n].getAxis("x").index===t||a[n].getAxis("y").index===e)return a[n]},r.prototype.getCartesians=function(){return this._coordsList.slice()},r.prototype.convertToPixel=function(t,e,i){var n=this._findConvertTarget(e);return n.cartesian?n.cartesian.dataToPoint(i):n.axis?n.axis.toGlobalCoord(n.axis.dataToCoord(i)):null},r.prototype.convertFromPixel=function(t,e,i){var n=this._findConvertTarget(e);return n.cartesian?n.cartesian.pointToData(i):n.axis?n.axis.coordToData(n.axis.toLocalCoord(i)):null},r.prototype._findConvertTarget=function(t){var e=t.seriesModel,i=t.xAxisModel||e&&e.getReferringComponents("xAxis",xe).models[0],n=t.yAxisModel||e&&e.getReferringComponents("yAxis",xe).models[0],a=t.gridModel,o=this._coordsList,s,l;if(e)s=e.coordinateSystem,ot(o,s)<0&&(s=null);else if(i&&n)s=this.getCartesian(i.componentIndex,n.componentIndex);else if(i)l=this.getAxis("x",i.componentIndex);else if(n)l=this.getAxis("y",n.componentIndex);else if(a){var u=a.coordinateSystem;u===this&&(s=this._coordsList[0])}return{cartesian:s,axis:l}},r.prototype.containPoint=function(t){var e=this._coordsList[0];if(e)return e.containPoint(t)},r.prototype._initCartesian=function(t,e,i){var n=this,a=this,o={left:!1,right:!1,top:!1,bottom:!1},s={x:{},y:{}},l={x:0,y:0};if(e.eachComponent("xAxis",u("x"),this),e.eachComponent("yAxis",u("y"),this),!l.x||!l.y){this._axesMap={},this._axesList=[];return}this._axesMap=s,T(s.x,function(f,h){T(s.y,function(c,v){var d="x"+h+"y"+v,y=new OD(d);y.master=n,y.model=t,n._coordsMap[d]=y,n._coordsList.push(y),y.addAxis(f),y.addAxis(c)})});function u(f){return function(h,c){if(Vl(h,t)){var v=h.get("position");f==="x"?v!=="top"&&v!=="bottom"&&(v=o.bottom?"top":"bottom"):v!=="left"&&v!=="right"&&(v=o.left?"right":"left"),o[v]=!0;var d=new XD(f,a0(h),[0,0],h.get("type"),v),y=d.type==="category";d.onBand=y&&h.get("boundaryGap"),d.inverse=h.get("inverse"),h.axis=d,d.model=h,d.grid=a,d.index=c,a._axesList.push(d),s[f][c]=d,l[f]++}}}},r.prototype._updateScale=function(t,e){T(this._axesList,function(n){if(n.scale.setExtent(1/0,-1/0),n.type==="category"){var a=n.model.get("categorySortInfo");n.scale.setSortInfo(a)}}),t.eachSeries(function(n){if(xd(n)){var a=Td(n),o=a.xAxisModel,s=a.yAxisModel;if(!Vl(o,e)||!Vl(s,e))return;var l=this.getCartesian(o.componentIndex,s.componentIndex),u=n.getData(),f=l.getAxis("x"),h=l.getAxis("y");i(u,f),i(u,h)}},this);function i(n,a){T(RD(n,a.dim),function(o){a.scale.unionExtentFromData(n,o)})}},r.prototype.getTooltipAxes=function(t){var e=[],i=[];return T(this.getCartesians(),function(n){var a=t!=null&&t!=="auto"?n.getAxis(t):n.getBaseAxis(),o=n.getOtherAxis(a);ot(e,a)<0&&e.push(a),ot(i,o)<0&&i.push(o)}),{baseAxes:e,otherAxes:i}},r.create=function(t,e){var i=[];return t.eachComponent("grid",function(n,a){var o=new r(n,t,e);o.name="grid_"+a,o.resize(n,e,!0),n.coordinateSystem=o,i.push(o)}),t.eachSeries(function(n){if(xd(n)){var a=Td(n),o=a.xAxisModel,s=a.yAxisModel,l=o.getCoordSysModel(),u=l.coordinateSystem;n.coordinateSystem=u.getCartesian(o.componentIndex,s.componentIndex)}}),i},r.dimensions=Ku,r}();function Vl(r,t){return r.getCoordSysModel()===t}function Md(r,t,e,i){e.getAxesOnZeroOf=function(){return a?[a]:[]};var n=r[t],a,o=e.model,s=o.get(["axisLine","onZero"]),l=o.get(["axisLine","onZeroAxisIndex"]);if(!s)return;if(l!=null)Cd(n[l])&&(a=n[l]);else for(var u in n)if(n.hasOwnProperty(u)&&Cd(n[u])&&!i[f(n[u])]){a=n[u];break}a&&(i[f(a)]=!0);function f(h){return h.dim+"_"+h.index}}function Cd(r){return r&&r.type!=="category"&&r.type!=="time"&&ID(r)}function qD(r,t){var e=r.getExtent(),i=e[0]+e[1];r.toGlobalCoord=r.dim==="x"?function(n){return n+t}:function(n){return i-n+t},r.toLocalCoord=r.dim==="x"?function(n){return n-t}:function(n){return i-n+t}}var yr=Math.PI,br=function(){function r(t,e){this.group=new mt,this.opt=e,this.axisModel=t,nt(e,{labelOffset:0,nameDirection:1,tickDirection:1,labelDirection:1,silent:!0,handleAutoShown:function(){return!0}});var i=new mt({x:e.position[0],y:e.position[1],rotation:e.rotation});i.updateTransform(),this._transformGroup=i}return r.prototype.hasBuilder=function(t){return!!Dd[t]},r.prototype.add=function(t){Dd[t](this.opt,this.axisModel,this.group,this._transformGroup)},r.prototype.getGroup=function(){return this.group},r.innerTextLayout=function(t,e,i){var n=bf(e-t),a,o;return Xn(n)?(o=i>0?"top":"bottom",a="center"):Xn(n-yr)?(o=i>0?"bottom":"top",a="center"):(o="middle",n>0&&n<yr?a=i>0?"right":"left":a=i>0?"left":"right"),{rotation:n,textAlign:a,textVerticalAlign:o}},r.makeAxisEventDataBase=function(t){var e={componentType:t.mainType,componentIndex:t.componentIndex};return e[t.mainType+"Index"]=t.componentIndex,e},r.isLabelSilent=function(t){var e=t.get("tooltip");return t.get("silent")||!(t.get("triggerEvent")||e&&e.show)},r}(),Dd={axisLine:function(r,t,e,i){var n=t.get(["axisLine","show"]);if(n==="auto"&&r.handleAutoShown&&(n=r.handleAutoShown("axisLine")),!!n){var a=t.axis.getExtent(),o=i.transform,s=[a[0],0],l=[a[1],0],u=s[0]>l[0];o&&(Bt(s,s,o),Bt(l,l,o));var f=O({lineCap:"round"},t.getModel(["axisLine","lineStyle"]).getLineStyle()),h=new xr({shape:{x1:s[0],y1:s[1],x2:l[0],y2:l[1]},style:f,strokeContainThreshold:r.strokeContainThreshold||5,silent:!0,z2:1});qn(h.shape,h.style.lineWidth),h.anid="line",e.add(h);var c=t.get(["axisLine","symbol"]);if(c!=null){var v=t.get(["axisLine","symbolSize"]);V(c)&&(c=[c,c]),(V(v)||ht(v))&&(v=[v,v]);var d=Yy(t.get(["axisLine","symbolOffset"])||0,v),y=v[0],p=v[1];T([{rotate:r.rotation+Math.PI/2,offset:d[0],r:0},{rotate:r.rotation-Math.PI/2,offset:d[1],r:Math.sqrt((s[0]-l[0])*(s[0]-l[0])+(s[1]-l[1])*(s[1]-l[1]))}],function(g,m){if(c[m]!=="none"&&c[m]!=null){var _=Je(c[m],-y/2,-p/2,y,p,f.stroke,!0),S=g.r+g.offset,b=u?l:s;_.attr({rotation:g.rotate,x:b[0]+S*Math.cos(r.rotation),y:b[1]-S*Math.sin(r.rotation),silent:!0,z2:11}),e.add(_)}})}}},axisTickLabel:function(r,t,e,i){var n=jD(e,i,t,r),a=tA(e,i,t,r);if(QD(t,a,n),JD(e,i,t,r.tickDirection),t.get(["axisLabel","hideOverlap"])){var o=UT(H(a,function(s){return{label:s,priority:s.z2,defaultAttr:{ignore:s.ignore}}}));YT(o)}},axisName:function(r,t,e,i){var n=oi(r.axisName,t.get("name"));if(n){var a=t.get("nameLocation"),o=r.nameDirection,s=t.getModel("nameTextStyle"),l=t.get("nameGap")||0,u=t.axis.getExtent(),f=u[0]>u[1]?-1:1,h=[a==="start"?u[0]-f*l:a==="end"?u[1]+f*l:(u[0]+u[1])/2,Id(a)?r.labelOffset+o*l:0],c,v=t.get("nameRotate");v!=null&&(v=v*yr/180);var d;Id(a)?c=br.innerTextLayout(r.rotation,v??r.rotation,o):(c=KD(r.rotation,a,v||0,u),d=r.axisNameAvailableWidth,d!=null&&(d=Math.abs(d/Math.sin(c.rotation)),!isFinite(d)&&(d=null)));var y=s.getFont(),p=t.get("nameTruncate",!0)||{},g=p.ellipsis,m=oi(r.nameTruncateMaxWidth,p.maxWidth,d),_=new yt({x:h[0],y:h[1],rotation:c.rotation,silent:br.isLabelSilent(t),style:he(s,{text:n,font:y,overflow:"truncate",width:m,ellipsis:g,fill:s.getTextColor()||t.get(["axisLine","lineStyle","color"]),align:s.get("align")||c.textAlign,verticalAlign:s.get("verticalAlign")||c.textVerticalAlign}),z2:1});if(ty({el:_,componentModel:t,itemName:n}),_.__fullText=n,_.anid="name",t.get("triggerEvent")){var S=br.makeAxisEventDataBase(t);S.targetType="axisName",S.name=n,tt(_).eventData=S}i.add(_),_.updateTransform(),e.add(_),_.decomposeTransform()}}};function KD(r,t,e,i){var n=bf(e-r),a,o,s=i[0]>i[1],l=t==="start"&&!s||t!=="start"&&s;return Xn(n-yr/2)?(o=l?"bottom":"top",a="center"):Xn(n-yr*1.5)?(o=l?"top":"bottom",a="center"):(o="middle",n<yr*1.5&&n>yr/2?a=l?"left":"right":a=l?"right":"left"),{rotation:n,textAlign:a,textVerticalAlign:o}}function QD(r,t,e){if(!o0(r.axis)){var i=r.get(["axisLabel","showMinLabel"]),n=r.get(["axisLabel","showMaxLabel"]);t=t||[],e=e||[];var a=t[0],o=t[1],s=t[t.length-1],l=t[t.length-2],u=e[0],f=e[1],h=e[e.length-1],c=e[e.length-2];i===!1?(re(a),re(u)):Ad(a,o)&&(i?(re(o),re(f)):(re(a),re(u))),n===!1?(re(s),re(h)):Ad(l,s)&&(n?(re(l),re(c)):(re(s),re(h)))}}function re(r){r&&(r.ignore=!0)}function Ad(r,t){var e=r&&r.getBoundingRect().clone(),i=t&&t.getBoundingRect().clone();if(!(!e||!i)){var n=fa([]);return es(n,n,-r.rotation),e.applyTransform(_r([],n,r.getLocalTransform())),i.applyTransform(_r([],n,t.getLocalTransform())),e.intersect(i)}}function Id(r){return r==="middle"||r==="center"}function d0(r,t,e,i,n){for(var a=[],o=[],s=[],l=0;l<r.length;l++){var u=r[l].coord;o[0]=u,o[1]=0,s[0]=u,s[1]=e,t&&(Bt(o,o,t),Bt(s,s,t));var f=new xr({shape:{x1:o[0],y1:o[1],x2:s[0],y2:s[1]},style:i,z2:2,autoBatch:!0,silent:!0});qn(f.shape,f.style.lineWidth),f.anid=n+"_"+r[l].tickValue,a.push(f)}return a}function jD(r,t,e,i){var n=e.axis,a=e.getModel("axisTick"),o=a.get("show");if(o==="auto"&&i.handleAutoShown&&(o=i.handleAutoShown("axisTick")),!(!o||n.scale.isBlank())){for(var s=a.getModel("lineStyle"),l=i.tickDirection*a.get("length"),u=n.getTicksCoords(),f=d0(u,t.transform,l,nt(s.getLineStyle(),{stroke:e.get(["axisLine","lineStyle","color"])}),"ticks"),h=0;h<f.length;h++)r.add(f[h]);return f}}function JD(r,t,e,i){var n=e.axis,a=e.getModel("minorTick");if(!(!a.get("show")||n.scale.isBlank())){var o=n.getMinorTicksCoords();if(o.length)for(var s=a.getModel("lineStyle"),l=i*a.get("length"),u=nt(s.getLineStyle(),nt(e.getModel("axisTick").getLineStyle(),{stroke:e.get(["axisLine","lineStyle","color"])})),f=0;f<o.length;f++)for(var h=d0(o[f],t.transform,l,u,"minorticks_"+f),c=0;c<h.length;c++)r.add(h[c])}}function tA(r,t,e,i){var n=e.axis,a=oi(i.axisLabelShow,e.get(["axisLabel","show"]));if(!(!a||n.scale.isBlank())){var o=e.getModel("axisLabel"),s=o.get("margin"),l=n.getViewLabels(),u=(oi(i.labelRotate,o.get("rotate"))||0)*yr/180,f=br.innerTextLayout(i.rotation,u,i.labelDirection),h=e.getCategories&&e.getCategories(!0),c=[],v=br.isLabelSilent(e),d=e.get("triggerEvent");return T(l,function(y,p){var g=n.scale.type==="ordinal"?n.scale.getRawOrdinalNumber(y.tickValue):y.tickValue,m=y.formattedLabel,_=y.rawLabel,S=o;if(h&&h[g]){var b=h[g];z(b)&&b.textStyle&&(S=new xt(b.textStyle,o,e.ecModel))}var w=S.getTextColor()||e.get(["axisLine","lineStyle","color"]),x=n.dataToCoord(g),C=S.getShallow("align",!0)||f.textAlign,D=Q(S.getShallow("alignMinLabel",!0),C),A=Q(S.getShallow("alignMaxLabel",!0),C),M=S.getShallow("verticalAlign",!0)||S.getShallow("baseline",!0)||f.textVerticalAlign,I=Q(S.getShallow("verticalAlignMinLabel",!0),M),L=Q(S.getShallow("verticalAlignMaxLabel",!0),M),P=new yt({x,y:i.labelOffset+i.labelDirection*s,rotation:f.rotation,silent:v,z2:10+(y.level||0),style:he(S,{text:m,align:p===0?D:p===l.length-1?A:C,verticalAlign:p===0?I:p===l.length-1?L:M,fill:Y(w)?w(n.type==="category"?_:n.type==="value"?g+"":g,p):w})});if(P.anid="label_"+g,d){var R=br.makeAxisEventDataBase(e);R.targetType="axisLabel",R.value=_,R.tickIndex=p,n.type==="category"&&(R.dataIndex=g),tt(P).eventData=R}t.add(P),P.updateTransform(),c.push(P),r.add(P),P.decomposeTransform()}),c}}function eA(r,t){var e={axesInfo:{},seriesInvolved:!1,coordSysAxesInfo:{},coordSysMap:{}};return rA(e,r,t),e.seriesInvolved&&nA(e,r),e}function rA(r,t,e){var i=t.getComponent("tooltip"),n=t.getComponent("axisPointer"),a=n.get("link",!0)||[],o=[];T(e.getCoordinateSystems(),function(s){if(!s.axisPointerEnabled)return;var l=aa(s.model),u=r.coordSysAxesInfo[l]={};r.coordSysMap[l]=s;var f=s.model,h=f.getModel("tooltip",i);if(T(s.getAxes(),kt(y,!1,null)),s.getTooltipAxes&&i&&h.get("show")){var c=h.get("trigger")==="axis",v=h.get(["axisPointer","type"])==="cross",d=s.getTooltipAxes(h.get(["axisPointer","axis"]));(c||v)&&T(d.baseAxes,kt(y,v?"cross":!0,c)),v&&T(d.otherAxes,kt(y,"cross",!1))}function y(p,g,m){var _=m.model.getModel("axisPointer",n),S=_.get("show");if(!(!S||S==="auto"&&!p&&!ju(_))){g==null&&(g=_.get("triggerTooltip")),_=p?iA(m,h,n,t,p,g):_;var b=_.get("snap"),w=_.get("triggerEmphasis"),x=aa(m.model),C=g||b||m.type==="category",D=r.axesInfo[x]={key:x,axis:m,coordSys:s,axisPointerModel:_,triggerTooltip:g,triggerEmphasis:w,involveSeries:C,snap:b,useHandle:ju(_),seriesModels:[],linkGroup:null};u[x]=D,r.seriesInvolved=r.seriesInvolved||C;var A=aA(a,m);if(A!=null){var M=o[A]||(o[A]={axesInfo:{}});M.axesInfo[x]=D,M.mapper=a[A].mapper,D.linkGroup=M}}}})}function iA(r,t,e,i,n,a){var o=t.getModel("axisPointer"),s=["type","snap","lineStyle","shadowStyle","label","animation","animationDurationUpdate","animationEasingUpdate","z"],l={};T(s,function(c){l[c]=U(o.get(c))}),l.snap=r.type!=="category"&&!!a,o.get("type")==="cross"&&(l.type="line");var u=l.label||(l.label={});if(u.show==null&&(u.show=!1),n==="cross"){var f=o.get(["label","show"]);if(u.show=f??!0,!a){var h=l.lineStyle=o.get("crossStyle");h&&nt(u,h.textStyle)}}return r.model.getModel("axisPointer",new xt(l,e,i))}function nA(r,t){t.eachSeries(function(e){var i=e.coordinateSystem,n=e.get(["tooltip","trigger"],!0),a=e.get(["tooltip","show"],!0);!i||n==="none"||n===!1||n==="item"||a===!1||e.get(["axisPointer","show"],!0)===!1||T(r.coordSysAxesInfo[aa(i.model)],function(o){var s=o.axis;i.getAxis(s.dim)===s&&(o.seriesModels.push(e),o.seriesDataCount==null&&(o.seriesDataCount=0),o.seriesDataCount+=e.getData().count())})})}function aA(r,t){for(var e=t.model,i=t.dim,n=0;n<r.length;n++){var a=r[n]||{};if(Hl(a[i+"AxisId"],e.id)||Hl(a[i+"AxisIndex"],e.componentIndex)||Hl(a[i+"AxisName"],e.name))return n}}function Hl(r,t){return r==="all"||N(r)&&ot(r,t)>=0||r===t}function oA(r){var t=_h(r);if(t){var e=t.axisPointerModel,i=t.axis.scale,n=e.option,a=e.get("status"),o=e.get("value");o!=null&&(o=i.parse(o));var s=ju(e);a==null&&(n.status=s?"show":"hide");var l=i.getExtent().slice();l[0]>l[1]&&l.reverse(),(o==null||o>l[1])&&(o=l[1]),o<l[0]&&(o=l[0]),n.value=o,s&&(n.status=t.axis.scale.isBlank()?"hide":"show")}}function _h(r){var t=(r.ecModel.getComponent("axisPointer")||{}).coordSysAxesInfo;return t&&t.axesInfo[aa(r)]}function sA(r){var t=_h(r);return t&&t.axisPointerModel}function ju(r){return!!r.get(["handle","show"])}function aa(r){return r.type+"||"+r.id}var Ld={},p0=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.render=function(e,i,n,a){this.axisPointerClass&&oA(e),r.prototype.render.apply(this,arguments),this._doUpdateAxisPointerClass(e,n,!0)},t.prototype.updateAxisPointer=function(e,i,n,a){this._doUpdateAxisPointerClass(e,n,!1)},t.prototype.remove=function(e,i){var n=this._axisPointer;n&&n.remove(i)},t.prototype.dispose=function(e,i){this._disposeAxisPointer(i),r.prototype.dispose.apply(this,arguments)},t.prototype._doUpdateAxisPointerClass=function(e,i,n){var a=t.getAxisPointerClass(this.axisPointerClass);if(a){var o=sA(e);o?(this._axisPointer||(this._axisPointer=new a)).render(e,o,i,n):this._disposeAxisPointer(i)}},t.prototype._disposeAxisPointer=function(e){this._axisPointer&&this._axisPointer.dispose(e),this._axisPointer=null},t.registerAxisPointerClass=function(e,i){Ld[e]=i},t.getAxisPointerClass=function(e){return e&&Ld[e]},t.type="axis",t}(Xt),Ju=_t();function lA(r,t,e,i){var n=e.axis;if(!n.scale.isBlank()){var a=e.getModel("splitArea"),o=a.getModel("areaStyle"),s=o.get("color"),l=i.coordinateSystem.getRect(),u=n.getTicksCoords({tickModel:a,clamp:!0});if(u.length){var f=s.length,h=Ju(r).splitAreaColors,c=$(),v=0;if(h)for(var d=0;d<u.length;d++){var y=h.get(u[d].tickValue);if(y!=null){v=(y+(f-1)*d)%f;break}}var p=n.toGlobalCoord(u[0].coord),g=o.getAreaStyle();s=N(s)?s:[s];for(var d=1;d<u.length;d++){var m=n.toGlobalCoord(u[d].coord),_=void 0,S=void 0,b=void 0,w=void 0;n.isHorizontal()?(_=p,S=l.y,b=m-_,w=l.height,p=_+b):(_=l.x,S=p,b=l.width,w=m-S,p=S+w);var x=u[d-1].tickValue;x!=null&&c.set(x,v),t.add(new gt({anid:x!=null?"area_"+x:null,shape:{x:_,y:S,width:b,height:w},style:nt({fill:s[v]},g),autoBatch:!0,silent:!0})),v=(v+1)%f}Ju(r).splitAreaColors=c}}}function uA(r){Ju(r).splitAreaColors=null}var fA=["axisLine","axisTickLabel","axisName"],hA=["splitArea","splitLine","minorSplitLine"],g0=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e.axisPointerClass="CartesianAxisPointer",e}return t.prototype.render=function(e,i,n,a){this.group.removeAll();var o=this._axisGroup;if(this._axisGroup=new mt,this.group.add(this._axisGroup),!!e.get("show")){var s=e.getCoordSysModel(),l=Qu(s,e),u=new br(e,O({handleAutoShown:function(h){for(var c=s.coordinateSystem.getCartesians(),v=0;v<c.length;v++)if(Zu(c[v].getOtherAxis(e.axis).scale))return!0;return!1}},l));T(fA,u.add,u),this._axisGroup.add(u.getGroup()),T(hA,function(h){e.get([h,"show"])&&vA[h](this,this._axisGroup,e,s)},this);var f=a&&a.type==="changeAxisOrder"&&a.isInitSort;f||Kg(o,this._axisGroup,e),r.prototype.render.call(this,e,i,n,a)}},t.prototype.remove=function(){uA(this)},t.type="cartesianAxis",t}(p0),vA={splitLine:function(r,t,e,i){var n=e.axis;if(!n.scale.isBlank()){var a=e.getModel("splitLine"),o=a.getModel("lineStyle"),s=o.get("color");s=N(s)?s:[s];for(var l=i.coordinateSystem.getRect(),u=n.isHorizontal(),f=0,h=n.getTicksCoords({tickModel:a}),c=[],v=[],d=o.getLineStyle(),y=0;y<h.length;y++){var p=n.toGlobalCoord(h[y].coord);u?(c[0]=p,c[1]=l.y,v[0]=p,v[1]=l.y+l.height):(c[0]=l.x,c[1]=p,v[0]=l.x+l.width,v[1]=p);var g=f++%s.length,m=h[y].tickValue,_=new xr({anid:m!=null?"line_"+h[y].tickValue:null,autoBatch:!0,shape:{x1:c[0],y1:c[1],x2:v[0],y2:v[1]},style:nt({stroke:s[g]},d),silent:!0});qn(_.shape,d.lineWidth),t.add(_)}}},minorSplitLine:function(r,t,e,i){var n=e.axis,a=e.getModel("minorSplitLine"),o=a.getModel("lineStyle"),s=i.coordinateSystem.getRect(),l=n.isHorizontal(),u=n.getMinorTicksCoords();if(u.length)for(var f=[],h=[],c=o.getLineStyle(),v=0;v<u.length;v++)for(var d=0;d<u[v].length;d++){var y=n.toGlobalCoord(u[v][d].coord);l?(f[0]=y,f[1]=s.y,h[0]=y,h[1]=s.y+s.height):(f[0]=s.x,f[1]=y,h[0]=s.x+s.width,h[1]=y);var p=new xr({anid:"minor_line_"+u[v][d].tickValue,autoBatch:!0,shape:{x1:f[0],y1:f[1],x2:h[0],y2:h[1]},style:c,silent:!0});qn(p.shape,c.lineWidth),t.add(p)}},splitArea:function(r,t,e,i){lA(r,t,e,i)}},y0=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.type="xAxis",t}(g0),cA=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=y0.type,e}return t.type="yAxis",t}(g0),dA=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="grid",e}return t.prototype.render=function(e,i){this.group.removeAll(),e.get("show")&&this.group.add(new gt({shape:e.coordinateSystem.getRect(),style:nt({fill:e.get("backgroundColor")},e.getItemStyle()),silent:!0,z2:-1}))},t.type="grid",t}(Xt),Pd={offset:0};function pA(r){r.registerComponentView(dA),r.registerComponentModel(iD),r.registerCoordinateSystem("cartesian2d",ZD),pd(r,"x",Xu,Pd),pd(r,"y",Xu,Pd),r.registerComponentView(y0),r.registerComponentView(cA),r.registerPreprocessor(function(t){t.xAxis&&t.yAxis&&!t.grid&&(t.grid={})})}var gA=1e-8;function Rd(r,t){return Math.abs(r-t)<gA}function Ed(r,t,e){var i=0,n=r[0];if(!n)return!1;for(var a=1;a<r.length;a++){var o=r[a];i+=qe(n[0],n[1],o[0],o[1],t,e),n=o}var s=r[0];return(!Rd(n[0],s[0])||!Rd(n[1],s[1]))&&(i+=qe(n[0],n[1],s[0],s[1],t,e)),i!==0}var yA=[];function Gl(r,t){for(var e=0;e<r.length;e++)Bt(r[e],r[e],t)}function Od(r,t,e,i){for(var n=0;n<r.length;n++){var a=r[n];i&&(a=i.project(a)),a&&isFinite(a[0])&&isFinite(a[1])&&(cr(t,t,a),dr(e,e,a))}}function mA(r){for(var t=0,e=0,i=0,n=r.length,a=r[n-1][0],o=r[n-1][1],s=0;s<n;s++){var l=r[s][0],u=r[s][1],f=a*u-l*o;t+=f,e+=(a+l)*f,i+=(o+u)*f,a=l,o=u}return t?[e/t/3,i/t/3,t]:[r[0][0]||0,r[0][1]||0]}var m0=function(){function r(t){this.name=t}return r.prototype.setCenter=function(t){this._center=t},r.prototype.getCenter=function(){var t=this._center;return t||(t=this._center=this.calcCenter()),t},r}(),kd=function(){function r(t,e){this.type="polygon",this.exterior=t,this.interiors=e}return r}(),Bd=function(){function r(t){this.type="linestring",this.points=t}return r}(),_A=function(r){k(t,r);function t(e,i,n){var a=r.call(this,e)||this;return a.type="geoJSON",a.geometries=i,a._center=n&&[n[0],n[1]],a}return t.prototype.calcCenter=function(){for(var e=this.geometries,i,n=0,a=0;a<e.length;a++){var o=e[a],s=o.exterior,l=s&&s.length;l>n&&(i=o,n=l)}if(i)return mA(i.exterior);var u=this.getBoundingRect();return[u.x+u.width/2,u.y+u.height/2]},t.prototype.getBoundingRect=function(e){var i=this._rect;if(i&&!e)return i;var n=[1/0,1/0],a=[-1/0,-1/0],o=this.geometries;return T(o,function(s){s.type==="polygon"?Od(s.exterior,n,a,e):T(s.points,function(l){Od(l,n,a,e)})}),isFinite(n[0])&&isFinite(n[1])&&isFinite(a[0])&&isFinite(a[1])||(n[0]=n[1]=a[0]=a[1]=0),i=new et(n[0],n[1],a[0]-n[0],a[1]-n[1]),e||(this._rect=i),i},t.prototype.contain=function(e){var i=this.getBoundingRect(),n=this.geometries;if(!i.contain(e[0],e[1]))return!1;t:for(var a=0,o=n.length;a<o;a++){var s=n[a];if(s.type==="polygon"){var l=s.exterior,u=s.interiors;if(Ed(l,e[0],e[1])){for(var f=0;f<(u?u.length:0);f++)if(Ed(u[f],e[0],e[1]))continue t;return!0}}}return!1},t.prototype.transformTo=function(e,i,n,a){var o=this.getBoundingRect(),s=o.width/o.height;n?a||(a=n/s):n=s*a;for(var l=new et(e,i,n,a),u=o.calculateTransform(l),f=this.geometries,h=0;h<f.length;h++){var c=f[h];c.type==="polygon"?(Gl(c.exterior,u),T(c.interiors,function(v){Gl(v,u)})):T(c.points,function(v){Gl(v,u)})}o=this._rect,o.copy(l),this._center=[o.x+o.width/2,o.y+o.height/2]},t.prototype.cloneShallow=function(e){e==null&&(e=this.name);var i=new t(e,this.geometries,this._center);return i._rect=this._rect,i.transformTo=null,i},t}(m0);(function(r){k(t,r);function t(e,i){var n=r.call(this,e)||this;return n.type="geoSVG",n._elOnlyForCalculate=i,n}return t.prototype.calcCenter=function(){for(var e=this._elOnlyForCalculate,i=e.getBoundingRect(),n=[i.x+i.width/2,i.y+i.height/2],a=fa(yA),o=e;o&&!o.isGeoSVGGraphicRoot;)_r(a,o.getLocalTransform(),a),o=o.parent;return ha(a,a),Bt(n,n,a),n},t})(m0);function SA(r){if(!r.UTF8Encoding)return r;var t=r,e=t.UTF8Scale;e==null&&(e=1024);var i=t.features;return T(i,function(n){var a=n.geometry,o=a.encodeOffsets,s=a.coordinates;if(o)switch(a.type){case"LineString":a.coordinates=_0(s,o,e);break;case"Polygon":Wl(s,o,e);break;case"MultiLineString":Wl(s,o,e);break;case"MultiPolygon":T(s,function(l,u){return Wl(l,o[u],e)})}}),t.UTF8Encoding=!1,t}function Wl(r,t,e){for(var i=0;i<r.length;i++)r[i]=_0(r[i],t[i],e)}function _0(r,t,e){for(var i=[],n=t[0],a=t[1],o=0;o<r.length;o+=2){var s=r.charCodeAt(o)-64,l=r.charCodeAt(o+1)-64;s=s>>1^-(s&1),l=l>>1^-(l&1),s+=n,l+=a,n=s,a=l,i.push([s/e,l/e])}return i}function pL(r,t){return r=SA(r),H(wt(r.features,function(e){return e.geometry&&e.properties&&e.geometry.coordinates.length>0}),function(e){var i=e.properties,n=e.geometry,a=[];switch(n.type){case"Polygon":var o=n.coordinates;a.push(new kd(o[0],o.slice(1)));break;case"MultiPolygon":T(n.coordinates,function(l){l[0]&&a.push(new kd(l[0],l.slice(1)))});break;case"LineString":a.push(new Bd([n.coordinates]));break;case"MultiLineString":a.push(new Bd(n.coordinates))}var s=new _A(i[t||"name"],a,i.cp);return s.properties=i,s})}var oa=T,wA=z,Zo=-1,It=function(){function r(t){var e=t.mappingMethod,i=t.type,n=this.option=U(t);this.type=i,this.mappingMethod=e,this._normalizeData=TA[e];var a=r.visualHandlers[i];this.applyVisual=a.applyVisual,this.getColorMapper=a.getColorMapper,this._normalizedToVisual=a._normalizedToVisual[e],e==="piecewise"?(Ul(n),bA(n)):e==="category"?n.categories?xA(n):Ul(n,!0):(Ce(e!=="linear"||n.dataExtent),Ul(n))}return r.prototype.mapValueToVisual=function(t){var e=this._normalizeData(t);return this._normalizedToVisual(e,t)},r.prototype.getNormalizer=function(){return K(this._normalizeData,this)},r.listVisualTypes=function(){return ft(r.visualHandlers)},r.isValidType=function(t){return r.visualHandlers.hasOwnProperty(t)},r.eachVisual=function(t,e,i){z(t)?T(t,e,i):e.call(i,t)},r.mapVisual=function(t,e,i){var n,a=N(t)?[]:z(t)?{}:(n=!0,null);return r.eachVisual(t,function(o,s){var l=e.call(i,o,s);n?a=l:a[s]=l}),a},r.retrieveVisuals=function(t){var e={},i;return t&&oa(r.visualHandlers,function(n,a){t.hasOwnProperty(a)&&(e[a]=t[a],i=!0)}),i?e:null},r.prepareVisualTypes=function(t){if(N(t))t=t.slice();else if(wA(t)){var e=[];oa(t,function(i,n){e.push(n)}),t=e}else return[];return t.sort(function(i,n){return n==="color"&&i!=="color"&&i.indexOf("color")===0?1:-1}),t},r.dependsOn=function(t,e){return e==="color"?!!(t&&t.indexOf(e)===0):t===e},r.findPieceIndex=function(t,e,i){for(var n,a=1/0,o=0,s=e.length;o<s;o++){var l=e[o].value;if(l!=null){if(l===t||V(l)&&l===t+"")return o;i&&c(l,o)}}for(var o=0,s=e.length;o<s;o++){var u=e[o],f=u.interval,h=u.close;if(f){if(f[0]===-1/0){if(ro(h[1],t,f[1]))return o}else if(f[1]===1/0){if(ro(h[0],f[0],t))return o}else if(ro(h[0],f[0],t)&&ro(h[1],t,f[1]))return o;i&&c(f[0],o),i&&c(f[1],o)}}if(i)return t===1/0?e.length-1:t===-1/0?0:n;function c(v,d){var y=Math.abs(v-t);y<a&&(a=y,n=d)}},r.visualHandlers={color:{applyVisual:yn("color"),getColorMapper:function(){var t=this.option;return K(t.mappingMethod==="category"?function(e,i){return!i&&(e=this._normalizeData(e)),Mn.call(this,e)}:function(e,i,n){var a=!!n;return!i&&(e=this._normalizeData(e)),n=An(e,t.parsedVisual,n),a?n:ke(n,"rgba")},this)},_normalizedToVisual:{linear:function(t){return ke(An(t,this.option.parsedVisual),"rgba")},category:Mn,piecewise:function(t,e){var i=ef.call(this,e);return i==null&&(i=ke(An(t,this.option.parsedVisual),"rgba")),i},fixed:$r}},colorHue:eo(function(t,e){return ao(t,e)}),colorSaturation:eo(function(t,e){return ao(t,null,e)}),colorLightness:eo(function(t,e){return ao(t,null,null,e)}),colorAlpha:eo(function(t,e){return Xp(t,e)}),decal:{applyVisual:yn("decal"),_normalizedToVisual:{linear:null,category:Mn,piecewise:null,fixed:null}},opacity:{applyVisual:yn("opacity"),_normalizedToVisual:tf([0,1])},liftZ:{applyVisual:yn("liftZ"),_normalizedToVisual:{linear:$r,category:$r,piecewise:$r,fixed:$r}},symbol:{applyVisual:function(t,e,i){var n=this.mapValueToVisual(t);i("symbol",n)},_normalizedToVisual:{linear:Nd,category:Mn,piecewise:function(t,e){var i=ef.call(this,e);return i==null&&(i=Nd.call(this,t)),i},fixed:$r}},symbolSize:{applyVisual:yn("symbolSize"),_normalizedToVisual:tf([0,1])}},r}();function bA(r){var t=r.pieceList;r.hasSpecialVisual=!1,T(t,function(e,i){e.originIndex=i,e.visual!=null&&(r.hasSpecialVisual=!0)})}function xA(r){var t=r.categories,e=r.categoryMap={},i=r.visual;if(oa(t,function(o,s){e[o]=s}),!N(i)){var n=[];z(i)?oa(i,function(o,s){var l=e[s];n[l??Zo]=o}):n[Zo]=i,i=S0(r,n)}for(var a=t.length-1;a>=0;a--)i[a]==null&&(delete e[t[a]],t.pop())}function Ul(r,t){var e=r.visual,i=[];z(e)?oa(e,function(a){i.push(a)}):e!=null&&i.push(e);var n={color:1,symbol:1};!t&&i.length===1&&!n.hasOwnProperty(r.type)&&(i[1]=i[0]),S0(r,i)}function eo(r){return{applyVisual:function(t,e,i){var n=this.mapValueToVisual(t);i("color",r(e("color"),n))},_normalizedToVisual:tf([0,1])}}function Nd(r){var t=this.option.visual;return t[Math.round(ze(r,[0,1],[0,t.length-1],!0))]||{}}function yn(r){return function(t,e,i){i(r,this.mapValueToVisual(t))}}function Mn(r){var t=this.option.visual;return t[this.option.loop&&r!==Zo?r%t.length:r]}function $r(){return this.option.visual[0]}function tf(r){return{linear:function(t){return ze(t,r,this.option.visual,!0)},category:Mn,piecewise:function(t,e){var i=ef.call(this,e);return i==null&&(i=ze(t,r,this.option.visual,!0)),i},fixed:$r}}function ef(r){var t=this.option,e=t.pieceList;if(t.hasSpecialVisual){var i=It.findPieceIndex(r,e),n=e[i];if(n&&n.visual)return n.visual[this.type]}}function S0(r,t){return r.visual=t,r.type==="color"&&(r.parsedVisual=H(t,function(e){var i=jt(e);return i||[0,0,0,1]})),t}var TA={linear:function(r){return ze(r,this.option.dataExtent,[0,1],!0)},piecewise:function(r){var t=this.option.pieceList,e=It.findPieceIndex(r,t,!0);if(e!=null)return ze(e,[0,t.length-1],[0,1],!0)},category:function(r){var t=this.option.categories?this.option.categoryMap[r]:r;return t??Zo},fixed:At};function ro(r,t,e){return r?t<=e:t<e}function MA(r,t,e,i,n,a){r=r||0;var o=e[1]-e[0];if(n!=null&&(n=Ii(n,[0,o])),a!=null&&(a=Math.max(a,n??0)),i==="all"){var s=Math.abs(t[1]-t[0]);s=Ii(s,[0,o]),n=a=Ii(s,[n,a]),i=0}t[0]=Ii(t[0],e),t[1]=Ii(t[1],e);var l=Yl(t,i);t[i]+=r;var u=n||0,f=e.slice();l.sign<0?f[0]+=u:f[1]-=u,t[i]=Ii(t[i],f);var h;return h=Yl(t,i),n!=null&&(h.sign!==l.sign||h.span<n)&&(t[1-i]=t[i]+l.sign*n),h=Yl(t,i),a!=null&&h.span>a&&(t[1-i]=t[i]+h.sign*a),t}function Yl(r,t){var e=r[t]-r[1-t];return{span:Math.abs(e),sign:e>0?-1:e<0?1:t?-1:1}}function Ii(r,t){return Math.min(t[1]!=null?t[1]:1/0,Math.max(t[0]!=null?t[0]:-1/0,r))}var Zr=_t(),Fd=U,Xl=K,CA=function(){function r(){this._dragging=!1,this.animationThreshold=15}return r.prototype.render=function(t,e,i,n){var a=e.get("value"),o=e.get("status");if(this._axisModel=t,this._axisPointerModel=e,this._api=i,!(!n&&this._lastValue===a&&this._lastStatus===o)){this._lastValue=a,this._lastStatus=o;var s=this._group,l=this._handle;if(!o||o==="hide"){s&&s.hide(),l&&l.hide();return}s&&s.show(),l&&l.show();var u={};this.makeElOption(u,a,t,e,i);var f=u.graphicKey;f!==this._lastGraphicKey&&this.clear(i),this._lastGraphicKey=f;var h=this._moveAnimation=this.determineAnimation(t,e);if(!s)s=this._group=new mt,this.createPointerEl(s,u,t,e),this.createLabelEl(s,u,t,e),i.getZr().add(s);else{var c=kt(zd,e,h);this.updatePointerEl(s,u,c),this.updateLabelEl(s,u,c,e)}Hd(s,e,!0),this._renderHandle(a)}},r.prototype.remove=function(t){this.clear(t)},r.prototype.dispose=function(t){this.clear(t)},r.prototype.determineAnimation=function(t,e){var i=e.get("animation"),n=t.axis,a=n.type==="category",o=e.get("snap");if(!o&&!a)return!1;if(i==="auto"||i==null){var s=this.animationThreshold;if(a&&n.getBandWidth()>s)return!0;if(o){var l=_h(t).seriesDataCount,u=n.getExtent();return Math.abs(u[0]-u[1])/l>s}return!1}return i===!0},r.prototype.makeElOption=function(t,e,i,n,a){},r.prototype.createPointerEl=function(t,e,i,n){var a=e.pointer;if(a){var o=Zr(t).pointerEl=new kw[a.type](Fd(e.pointer));t.add(o)}},r.prototype.createLabelEl=function(t,e,i,n){if(e.label){var a=Zr(t).labelEl=new yt(Fd(e.label));t.add(a),Vd(a,n)}},r.prototype.updatePointerEl=function(t,e,i){var n=Zr(t).pointerEl;n&&e.pointer&&(n.setStyle(e.pointer.style),i(n,{shape:e.pointer.shape}))},r.prototype.updateLabelEl=function(t,e,i,n){var a=Zr(t).labelEl;a&&(a.setStyle(e.label.style),i(a,{x:e.label.x,y:e.label.y}),Vd(a,n))},r.prototype._renderHandle=function(t){if(!(this._dragging||!this.updateHandleTransform)){var e=this._axisPointerModel,i=this._api.getZr(),n=this._handle,a=e.getModel("handle"),o=e.get("status");if(!a.get("show")||!o||o==="hide"){n&&i.remove(n),this._handle=null;return}var s;this._handle||(s=!0,n=this._handle=Nf(a.get("icon"),{cursor:"move",draggable:!0,onmousemove:function(u){ta(u.event)},onmousedown:Xl(this._onHandleDragMove,this,0,0),drift:Xl(this._onHandleDragMove,this),ondragend:Xl(this._onHandleDragEnd,this)}),i.add(n)),Hd(n,e,!1),n.setStyle(a.getItemStyle(null,["color","borderColor","borderWidth","opacity","shadowColor","shadowBlur","shadowOffsetX","shadowOffsetY"]));var l=a.get("size");N(l)||(l=[l,l]),n.scaleX=l[0]/2,n.scaleY=l[1]/2,om(this,"_doDispatchAxisPointer",a.get("throttle")||0,"fixRate"),this._moveHandleToValue(t,s)}},r.prototype._moveHandleToValue=function(t,e){zd(this._axisPointerModel,!e&&this._moveAnimation,this._handle,$l(this.getHandleTransform(t,this._axisModel,this._axisPointerModel)))},r.prototype._onHandleDragMove=function(t,e){var i=this._handle;if(i){this._dragging=!0;var n=this.updateHandleTransform($l(i),[t,e],this._axisModel,this._axisPointerModel);this._payloadInfo=n,i.stopAnimation(),i.attr($l(n)),Zr(i).lastProp=null,this._doDispatchAxisPointer()}},r.prototype._doDispatchAxisPointer=function(){var t=this._handle;if(t){var e=this._payloadInfo,i=this._axisModel;this._api.dispatchAction({type:"updateAxisPointer",x:e.cursorPoint[0],y:e.cursorPoint[1],tooltipOption:e.tooltipOption,axesInfo:[{axisDim:i.axis.dim,axisIndex:i.componentIndex}]})}},r.prototype._onHandleDragEnd=function(){this._dragging=!1;var t=this._handle;if(t){var e=this._axisPointerModel.get("value");this._moveHandleToValue(e),this._api.dispatchAction({type:"hideTip"})}},r.prototype.clear=function(t){this._lastValue=null,this._lastStatus=null;var e=t.getZr(),i=this._group,n=this._handle;e&&i&&(this._lastGraphicKey=null,i&&e.remove(i),n&&e.remove(n),this._group=null,this._handle=null,this._payloadInfo=null),Ou(this,"_doDispatchAxisPointer")},r.prototype.doClear=function(){},r.prototype.buildLabel=function(t,e,i){return i=i||0,{x:t[i],y:t[1-i],width:e[i],height:e[1-i]}},r}();function zd(r,t,e,i){w0(Zr(e).lastProp,i)||(Zr(e).lastProp=i,t?te(e,i,r):(e.stopAnimation(),e.attr(i)))}function w0(r,t){if(z(r)&&z(t)){var e=!0;return T(t,function(i,n){e=e&&w0(r[n],i)}),!!e}else return r===t}function Vd(r,t){r[t.get(["label","show"])?"show":"hide"]()}function $l(r){return{x:r.x||0,y:r.y||0,rotation:r.rotation||0}}function Hd(r,t,e){var i=t.get("z"),n=t.get("zlevel");r&&r.traverse(function(a){a.type!=="group"&&(i!=null&&(a.z=i),n!=null&&(a.zlevel=n),a.silent=e)})}function DA(r){var t=r.get("type"),e=r.getModel(t+"Style"),i;return t==="line"?(i=e.getLineStyle(),i.fill=null):t==="shadow"&&(i=e.getAreaStyle(),i.stroke=null),i}function AA(r,t,e,i,n){var a=e.get("value"),o=b0(a,t.axis,t.ecModel,e.get("seriesDataIndices"),{precision:e.get(["label","precision"]),formatter:e.get(["label","formatter"])}),s=e.getModel("label"),l=ga(s.get("padding")||0),u=s.getFont(),f=df(o,u),h=n.position,c=f.width+l[1]+l[3],v=f.height+l[0]+l[2],d=n.align;d==="right"&&(h[0]-=c),d==="center"&&(h[0]-=c/2);var y=n.verticalAlign;y==="bottom"&&(h[1]-=v),y==="middle"&&(h[1]-=v/2),IA(h,c,v,i);var p=s.get("backgroundColor");(!p||p==="auto")&&(p=t.get(["axisLine","lineStyle","color"])),r.label={x:h[0],y:h[1],style:he(s,{text:o,font:u,fill:s.getTextColor(),padding:l,backgroundColor:p}),z2:10}}function IA(r,t,e,i){var n=i.getWidth(),a=i.getHeight();r[0]=Math.min(r[0]+t,n)-t,r[1]=Math.min(r[1]+e,a)-e,r[0]=Math.max(r[0],0),r[1]=Math.max(r[1],0)}function b0(r,t,e,i,n){r=t.scale.parse(r);var a=t.scale.getLabel({value:r},{precision:n.precision}),o=n.formatter;if(o){var s={value:yh(t,{value:r}),axisDimension:t.dim,axisIndex:t.index,seriesData:[]};T(i,function(l){var u=e.getSeriesByIndex(l.seriesIndex),f=l.dataIndexInside,h=u&&u.getDataParams(f);h&&s.seriesData.push(h)}),V(o)?a=o.replace("{value}",a):Y(o)&&(a=o(s))}return a}function x0(r,t,e){var i=mr();return es(i,i,e.rotation),bo(i,i,e.position),Fi([r.dataToCoord(t),(e.labelOffset||0)+(e.labelDirection||1)*(e.labelMargin||0)],i)}function LA(r,t,e,i,n,a){var o=br.innerTextLayout(e.rotation,0,e.labelDirection);e.labelMargin=n.get(["label","margin"]),AA(t,i,n,a,{position:x0(i.axis,r,e),align:o.textAlign,verticalAlign:o.textVerticalAlign})}function PA(r,t,e){return e=e||0,{x1:r[e],y1:r[1-e],x2:t[e],y2:t[1-e]}}function RA(r,t,e){return e=e||0,{x:r[e],y:r[1-e],width:t[e],height:t[1-e]}}var EA=function(r){k(t,r);function t(){return r!==null&&r.apply(this,arguments)||this}return t.prototype.makeElOption=function(e,i,n,a,o){var s=n.axis,l=s.grid,u=a.get("type"),f=Gd(l,s).getOtherAxis(s).getGlobalExtent(),h=s.toGlobalCoord(s.dataToCoord(i,!0));if(u&&u!=="none"){var c=DA(a),v=OA[u](s,h,f);v.style=c,e.graphicKey=v.type,e.pointer=v}var d=Qu(l.model,n);LA(i,e,d,n,a,o)},t.prototype.getHandleTransform=function(e,i,n){var a=Qu(i.axis.grid.model,i,{labelInside:!1});a.labelMargin=n.get(["handle","margin"]);var o=x0(i.axis,e,a);return{x:o[0],y:o[1],rotation:a.rotation+(a.labelDirection<0?Math.PI:0)}},t.prototype.updateHandleTransform=function(e,i,n,a){var o=n.axis,s=o.grid,l=o.getGlobalExtent(!0),u=Gd(s,o).getOtherAxis(o).getGlobalExtent(),f=o.dim==="x"?0:1,h=[e.x,e.y];h[f]+=i[f],h[f]=Math.min(l[1],h[f]),h[f]=Math.max(l[0],h[f]);var c=(u[1]+u[0])/2,v=[c,c];v[f]=h[f];var d=[{verticalAlign:"middle"},{align:"center"}];return{x:h[0],y:h[1],rotation:e.rotation,cursorPoint:v,tooltipOption:d[f]}},t}(CA);function Gd(r,t){var e={};return e[t.dim+"AxisIndex"]=t.index,r.getCartesian(e)}var OA={line:function(r,t,e){var i=PA([t,e[0]],[t,e[1]],Wd(r));return{type:"Line",subPixelOptimize:!0,shape:i}},shadow:function(r,t,e){var i=Math.max(1,r.getBandWidth()),n=e[1]-e[0];return{type:"Rect",shape:RA([t-i/2,e[0]],[i,n],Wd(r))}}};function Wd(r){return r.dim==="x"?0:1}var kA=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.type="axisPointer",t.defaultOption={show:"auto",z:50,type:"line",snap:!1,triggerTooltip:!0,triggerEmphasis:!0,value:null,status:null,link:[],animation:null,animationDurationUpdate:200,lineStyle:{color:"#B9BEC9",width:1,type:"dashed"},shadowStyle:{color:"rgba(210,219,238,0.2)"},label:{show:!0,formatter:null,precision:"auto",margin:3,color:"#fff",padding:[5,7,5,7],backgroundColor:"auto",borderColor:null,borderWidth:0,borderRadius:3},handle:{show:!1,icon:"M10.7,11.9v-1.3H9.3v1.3c-4.9,0.3-8.8,4.4-8.8,9.4c0,5,3.9,9.1,8.8,9.4h1.3c4.9-0.3,8.8-4.4,8.8-9.4C19.5,16.3,15.6,12.2,10.7,11.9z M13.3,24.4H6.7v-1.2h6.6z M13.3,22H6.7v-1.2h6.6z M13.3,19.6H6.7v-1.2h6.6z",size:45,margin:50,color:"#333",shadowBlur:3,shadowColor:"#aaa",shadowOffsetX:0,shadowOffsetY:2,throttle:40}},t}(rt),Qe=_t(),BA=T;function T0(r,t,e){if(!Z.node){var i=t.getZr();Qe(i).records||(Qe(i).records={}),NA(i,t);var n=Qe(i).records[r]||(Qe(i).records[r]={});n.handler=e}}function NA(r,t){if(Qe(r).initialized)return;Qe(r).initialized=!0,e("click",kt(Ud,"click")),e("mousemove",kt(Ud,"mousemove")),e("globalout",zA);function e(i,n){r.on(i,function(a){var o=VA(t);BA(Qe(r).records,function(s){s&&n(s,a,o.dispatchAction)}),FA(o.pendings,t)})}}function FA(r,t){var e=r.showTip.length,i=r.hideTip.length,n;e?n=r.showTip[e-1]:i&&(n=r.hideTip[i-1]),n&&(n.dispatchAction=null,t.dispatchAction(n))}function zA(r,t,e){r.handler("leave",null,e)}function Ud(r,t,e,i){t.handler(r,e,i)}function VA(r){var t={showTip:[],hideTip:[]},e=function(i){var n=t[i.type];n?n.push(i):(i.dispatchAction=e,r.dispatchAction(i))};return{dispatchAction:e,pendings:t}}function rf(r,t){if(!Z.node){var e=t.getZr(),i=(Qe(e).records||{})[r];i&&(Qe(e).records[r]=null)}}var HA=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.render=function(e,i,n){var a=i.getComponent("tooltip"),o=e.get("triggerOn")||a&&a.get("triggerOn")||"mousemove|click";T0("axisPointer",n,function(s,l,u){o!=="none"&&(s==="leave"||o.indexOf(s)>=0)&&u({type:"updateAxisPointer",currTrigger:s,x:l&&l.offsetX,y:l&&l.offsetY})})},t.prototype.remove=function(e,i){rf("axisPointer",i)},t.prototype.dispose=function(e,i){rf("axisPointer",i)},t.type="axisPointer",t}(Xt);function M0(r,t){var e=[],i=r.seriesIndex,n;if(i==null||!(n=t.getSeriesByIndex(i)))return{point:[]};var a=n.getData(),o=ui(a,r);if(o==null||o<0||N(o))return{point:[]};var s=a.getItemGraphicEl(o),l=n.coordinateSystem;if(n.getTooltipPosition)e=n.getTooltipPosition(o)||[];else if(l&&l.dataToPoint)if(r.isStacked){var u=l.getBaseAxis(),f=l.getOtherAxis(u),h=f.dim,c=u.dim,v=h==="x"||h==="radius"?1:0,d=a.mapDimension(c),y=[];y[v]=a.get(d,o),y[1-v]=a.get(a.getCalculationInfo("stackResultDimension"),o),e=l.dataToPoint(y)||[]}else e=l.dataToPoint(a.getValues(H(l.dimensions,function(g){return a.mapDimension(g)}),o))||[];else if(s){var p=s.getBoundingRect().clone();p.applyTransform(s.transform),e=[p.x+p.width/2,p.y+p.height/2]}return{point:e,el:s}}var Yd=_t();function GA(r,t,e){var i=r.currTrigger,n=[r.x,r.y],a=r,o=r.dispatchAction||K(e.dispatchAction,e),s=t.getComponent("axisPointer").coordSysAxesInfo;if(s){_o(n)&&(n=M0({seriesIndex:a.seriesIndex,dataIndex:a.dataIndex},t).point);var l=_o(n),u=a.axesInfo,f=s.axesInfo,h=i==="leave"||_o(n),c={},v={},d={list:[],map:{}},y={showPointer:kt(UA,v),showTooltip:kt(YA,d)};T(s.coordSysMap,function(g,m){var _=l||g.containPoint(n);T(s.coordSysAxesInfo[m],function(S,b){var w=S.axis,x=qA(u,S);if(!h&&_&&(!u||x)){var C=x&&x.value;C==null&&!l&&(C=w.pointToData(n)),C!=null&&Xd(S,C,y,!1,c)}})});var p={};return T(f,function(g,m){var _=g.linkGroup;_&&!v[m]&&T(_.axesInfo,function(S,b){var w=v[b];if(S!==g&&w){var x=w.value;_.mapper&&(x=g.axis.scale.parse(_.mapper(x,$d(S),$d(g)))),p[g.key]=x}})}),T(p,function(g,m){Xd(f[m],g,y,!0,c)}),XA(v,f,c),$A(d,n,r,o),ZA(f,o,e),c}}function Xd(r,t,e,i,n){var a=r.axis;if(!(a.scale.isBlank()||!a.containData(t))){if(!r.involveSeries){e.showPointer(r,t);return}var o=WA(t,r),s=o.payloadBatch,l=o.snapToValue;s[0]&&n.seriesIndex==null&&O(n,s[0]),!i&&r.snap&&a.containData(l)&&l!=null&&(t=l),e.showPointer(r,t,s),e.showTooltip(r,o,l)}}function WA(r,t){var e=t.axis,i=e.dim,n=r,a=[],o=Number.MAX_VALUE,s=-1;return T(t.seriesModels,function(l,u){var f=l.getData().mapDimensionsAll(i),h,c;if(l.getAxisTooltipData){var v=l.getAxisTooltipData(f,r,e);c=v.dataIndices,h=v.nestestValue}else{if(c=l.getData().indicesOfNearest(f[0],r,e.type==="category"?.5:null),!c.length)return;h=l.getData().get(f[0],c[0])}if(!(h==null||!isFinite(h))){var d=r-h,y=Math.abs(d);y<=o&&((y<o||d>=0&&s<0)&&(o=y,s=d,n=h,a.length=0),T(c,function(p){a.push({seriesIndex:l.seriesIndex,dataIndexInside:p,dataIndex:l.getData().getRawIndex(p)})}))}}),{payloadBatch:a,snapToValue:n}}function UA(r,t,e,i){r[t.key]={value:e,payloadBatch:i}}function YA(r,t,e,i){var n=e.payloadBatch,a=t.axis,o=a.model,s=t.axisPointerModel;if(!(!t.triggerTooltip||!n.length)){var l=t.coordSys.model,u=aa(l),f=r.map[u];f||(f=r.map[u]={coordSysId:l.id,coordSysIndex:l.componentIndex,coordSysType:l.type,coordSysMainType:l.mainType,dataByAxis:[]},r.list.push(f)),f.dataByAxis.push({axisDim:a.dim,axisIndex:o.componentIndex,axisType:o.type,axisId:o.id,value:i,valueLabelOpt:{precision:s.get(["label","precision"]),formatter:s.get(["label","formatter"])},seriesDataIndices:n.slice()})}}function XA(r,t,e){var i=e.axesInfo=[];T(t,function(n,a){var o=n.axisPointerModel.option,s=r[a];s?(!n.useHandle&&(o.status="show"),o.value=s.value,o.seriesDataIndices=(s.payloadBatch||[]).slice()):!n.useHandle&&(o.status="hide"),o.status==="show"&&i.push({axisDim:n.axis.dim,axisIndex:n.axis.model.componentIndex,value:o.value})})}function $A(r,t,e,i){if(_o(t)||!r.list.length){i({type:"hideTip"});return}var n=((r.list[0].dataByAxis[0]||{}).seriesDataIndices||[])[0]||{};i({type:"showTip",escapeConnect:!0,x:t[0],y:t[1],tooltipOption:e.tooltipOption,position:e.position,dataIndexInside:n.dataIndexInside,dataIndex:n.dataIndex,seriesIndex:n.seriesIndex,dataByCoordSys:r.list})}function ZA(r,t,e){var i=e.getZr(),n="axisPointerLastHighlights",a=Yd(i)[n]||{},o=Yd(i)[n]={};T(r,function(u,f){var h=u.axisPointerModel.option;h.status==="show"&&u.triggerEmphasis&&T(h.seriesDataIndices,function(c){var v=c.seriesIndex+" | "+c.dataIndex;o[v]=c})});var s=[],l=[];T(a,function(u,f){!o[f]&&l.push(u)}),T(o,function(u,f){!a[f]&&s.push(u)}),l.length&&e.dispatchAction({type:"downplay",escapeConnect:!0,notBlur:!0,batch:l}),s.length&&e.dispatchAction({type:"highlight",escapeConnect:!0,notBlur:!0,batch:s})}function qA(r,t){for(var e=0;e<(r||[]).length;e++){var i=r[e];if(t.axis.dim===i.axisDim&&t.axis.model.componentIndex===i.axisIndex)return i}}function $d(r){var t=r.axis.model,e={},i=e.axisDim=r.axis.dim;return e.axisIndex=e[i+"AxisIndex"]=t.componentIndex,e.axisName=e[i+"AxisName"]=t.name,e.axisId=e[i+"AxisId"]=t.id,e}function _o(r){return!r||r[0]==null||isNaN(r[0])||r[1]==null||isNaN(r[1])}function C0(r){p0.registerAxisPointerClass("CartesianAxisPointer",EA),r.registerComponentModel(kA),r.registerComponentView(HA),r.registerPreprocessor(function(t){if(t){(!t.axisPointer||t.axisPointer.length===0)&&(t.axisPointer={});var e=t.axisPointer.link;e&&!N(e)&&(t.axisPointer.link=[e])}}),r.registerProcessor(r.PRIORITY.PROCESSOR.STATISTIC,function(t,e){t.getComponent("axisPointer").coordSysAxesInfo=eA(t,e)}),r.registerAction({type:"updateAxisPointer",event:"updateAxisPointer",update:":updateAxisPointer"},GA)}function gL(r){Ui(pA),Ui(C0)}var KA=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.type="tooltip",t.dependencies=["axisPointer"],t.defaultOption={z:60,show:!0,showContent:!0,trigger:"item",triggerOn:"mousemove|click",alwaysShowContent:!1,displayMode:"single",renderMode:"auto",confine:null,showDelay:0,hideDelay:100,transitionDuration:.4,enterable:!1,backgroundColor:"#fff",shadowBlur:10,shadowColor:"rgba(0, 0, 0, .2)",shadowOffsetX:1,shadowOffsetY:2,borderRadius:4,borderWidth:1,padding:null,extraCssText:"",axisPointer:{type:"line",axis:"auto",animation:"auto",animationDurationUpdate:200,animationEasingUpdate:"exponentialOut",crossStyle:{color:"#999",width:1,type:"dashed",textStyle:{}}},textStyle:{color:"#666",fontSize:14}},t}(rt);function D0(r){var t=r.get("confine");return t!=null?!!t:r.get("renderMode")==="richText"}function A0(r){if(Z.domSupported){for(var t=document.documentElement.style,e=0,i=r.length;e<i;e++)if(r[e]in t)return r[e]}}var I0=A0(["transform","webkitTransform","OTransform","MozTransform","msTransform"]),QA=A0(["webkitTransition","transition","OTransition","MozTransition","msTransition"]);function L0(r,t){if(!r)return t;t=Zf(t,!0);var e=r.indexOf(t);return r=e===-1?t:"-"+r.slice(0,e)+"-"+t,r.toLowerCase()}function jA(r,t){var e=r.currentStyle||document.defaultView&&document.defaultView.getComputedStyle(r);return e?e[t]:null}var JA=L0(QA,"transition"),Sh=L0(I0,"transform"),tI="position:absolute;display:block;border-style:solid;white-space:nowrap;z-index:9999999;"+(Z.transform3dSupported?"will-change:transform;":"");function eI(r){return r=r==="left"?"right":r==="right"?"left":r==="top"?"bottom":"top",r}function rI(r,t,e){if(!V(e)||e==="inside")return"";var i=r.get("backgroundColor"),n=r.get("borderWidth");t=hi(t);var a=eI(e),o=Math.max(Math.round(n)*1.5,6),s="",l=Sh+":",u;ot(["left","right"],a)>-1?(s+="top:50%",l+="translateY(-50%) rotate("+(u=a==="left"?-225:-45)+"deg)"):(s+="left:50%",l+="translateX(-50%) rotate("+(u=a==="top"?225:45)+"deg)");var f=u*Math.PI/180,h=o+n,c=h*Math.abs(Math.cos(f))+h*Math.abs(Math.sin(f)),v=Math.round(((c-Math.SQRT2*n)/2+Math.SQRT2*n-(c-h)/2)*100)/100;s+=";"+a+":-"+v+"px";var d=t+" solid "+n+"px;",y=["position:absolute;width:"+o+"px;height:"+o+"px;z-index:-1;",s+";"+l+";","border-bottom:"+d,"border-right:"+d,"background-color:"+i+";"];return'<div style="'+y.join("")+'"></div>'}function iI(r,t){var e="cubic-bezier(0.23,1,0.32,1)",i=" "+r/2+"s "+e,n="opacity"+i+",visibility"+i;return t||(i=" "+r+"s "+e,n+=Z.transformSupported?","+Sh+i:",left"+i+",top"+i),JA+":"+n}function Zd(r,t,e){var i=r.toFixed(0)+"px",n=t.toFixed(0)+"px";if(!Z.transformSupported)return e?"top:"+n+";left:"+i+";":[["top",n],["left",i]];var a=Z.transform3dSupported,o="translate"+(a?"3d":"")+"("+i+","+n+(a?",0":"")+")";return e?"top:0;left:0;"+Sh+":"+o+";":[["top",0],["left",0],[I0,o]]}function nI(r){var t=[],e=r.get("fontSize"),i=r.getTextColor();i&&t.push("color:"+i),t.push("font:"+r.getFont()),e&&t.push("line-height:"+Math.round(e*3/2)+"px");var n=r.get("textShadowColor"),a=r.get("textShadowBlur")||0,o=r.get("textShadowOffsetX")||0,s=r.get("textShadowOffsetY")||0;return n&&a&&t.push("text-shadow:"+o+"px "+s+"px "+a+"px "+n),T(["decoration","align"],function(l){var u=r.get(l);u&&t.push("text-"+l+":"+u)}),t.join(";")}function aI(r,t,e){var i=[],n=r.get("transitionDuration"),a=r.get("backgroundColor"),o=r.get("shadowBlur"),s=r.get("shadowColor"),l=r.get("shadowOffsetX"),u=r.get("shadowOffsetY"),f=r.getModel("textStyle"),h=Uy(r,"html"),c=l+"px "+u+"px "+o+"px "+s;return i.push("box-shadow:"+c),t&&n&&i.push(iI(n,e)),a&&i.push("background-color:"+a),T(["width","color","radius"],function(v){var d="border-"+v,y=Zf(d),p=r.get(y);p!=null&&i.push(d+":"+p+(v==="color"?"":"px"))}),i.push(nI(f)),h!=null&&i.push("padding:"+ga(h).join("px ")+"px"),i.join(";")+";"}function qd(r,t,e,i,n){var a=t&&t.painter;if(e){var o=a&&a.getViewportRoot();o&&Bb(r,o,e,i,n)}else{r[0]=i,r[1]=n;var s=a&&a.getViewportRootOffset();s&&(r[0]+=s.offsetLeft,r[1]+=s.offsetTop)}r[2]=r[0]/t.getWidth(),r[3]=r[1]/t.getHeight()}var oI=function(){function r(t,e){if(this._show=!1,this._styleCoord=[0,0,0,0],this._enterable=!0,this._alwaysShowContent=!1,this._firstShow=!0,this._longHide=!0,Z.wxa)return null;var i=document.createElement("div");i.domBelongToZr=!0,this.el=i;var n=this._zr=t.getZr(),a=e.appendTo,o=a&&(V(a)?document.querySelector(a):Gi(a)?a:Y(a)&&a(t.getDom()));qd(this._styleCoord,n,o,t.getWidth()/2,t.getHeight()/2),(o||t.getDom()).appendChild(i),this._api=t,this._container=o;var s=this;i.onmouseenter=function(){s._enterable&&(clearTimeout(s._hideTimeout),s._show=!0),s._inContent=!0},i.onmousemove=function(l){if(l=l||window.event,!s._enterable){var u=n.handler,f=n.painter.getViewportRoot();ne(f,l,!0),u.dispatch("mousemove",l)}},i.onmouseleave=function(){s._inContent=!1,s._enterable&&s._show&&s.hideLater(s._hideDelay)}}return r.prototype.update=function(t){if(!this._container){var e=this._api.getDom(),i=jA(e,"position"),n=e.style;n.position!=="absolute"&&i!=="absolute"&&(n.position="relative")}var a=t.get("alwaysShowContent");a&&this._moveIfResized(),this._alwaysShowContent=a,this.el.className=t.get("className")||""},r.prototype.show=function(t,e){clearTimeout(this._hideTimeout),clearTimeout(this._longHideTimeout);var i=this.el,n=i.style,a=this._styleCoord;i.innerHTML?n.cssText=tI+aI(t,!this._firstShow,this._longHide)+Zd(a[0],a[1],!0)+("border-color:"+hi(e)+";")+(t.get("extraCssText")||"")+(";pointer-events:"+(this._enterable?"auto":"none")):n.display="none",this._show=!0,this._firstShow=!1,this._longHide=!1},r.prototype.setContent=function(t,e,i,n,a){var o=this.el;if(t==null){o.innerHTML="";return}var s="";if(V(a)&&i.get("trigger")==="item"&&!D0(i)&&(s=rI(i,n,a)),V(t))o.innerHTML=t+s;else if(t){o.innerHTML="",N(t)||(t=[t]);for(var l=0;l<t.length;l++)Gi(t[l])&&t[l].parentNode!==o&&o.appendChild(t[l]);if(s&&o.childNodes.length){var u=document.createElement("div");u.innerHTML=s,o.appendChild(u)}}},r.prototype.setEnterable=function(t){this._enterable=t},r.prototype.getSize=function(){var t=this.el;return[t.offsetWidth,t.offsetHeight]},r.prototype.moveTo=function(t,e){var i=this._styleCoord;if(qd(i,this._zr,this._container,t,e),i[0]!=null&&i[1]!=null){var n=this.el.style,a=Zd(i[0],i[1]);T(a,function(o){n[o[0]]=o[1]})}},r.prototype._moveIfResized=function(){var t=this._styleCoord[2],e=this._styleCoord[3];this.moveTo(t*this._zr.getWidth(),e*this._zr.getHeight())},r.prototype.hide=function(){var t=this,e=this.el.style;e.visibility="hidden",e.opacity="0",Z.transform3dSupported&&(e.willChange=""),this._show=!1,this._longHideTimeout=setTimeout(function(){return t._longHide=!0},500)},r.prototype.hideLater=function(t){this._show&&!(this._inContent&&this._enterable)&&!this._alwaysShowContent&&(t?(this._hideDelay=t,this._show=!1,this._hideTimeout=setTimeout(K(this.hide,this),t)):this.hide())},r.prototype.isShow=function(){return this._show},r.prototype.dispose=function(){clearTimeout(this._hideTimeout),clearTimeout(this._longHideTimeout);var t=this.el.parentNode;t&&t.removeChild(this.el),this.el=this._container=null},r}(),sI=function(){function r(t){this._show=!1,this._styleCoord=[0,0,0,0],this._alwaysShowContent=!1,this._enterable=!0,this._zr=t.getZr(),Qd(this._styleCoord,this._zr,t.getWidth()/2,t.getHeight()/2)}return r.prototype.update=function(t){var e=t.get("alwaysShowContent");e&&this._moveIfResized(),this._alwaysShowContent=e},r.prototype.show=function(){this._hideTimeout&&clearTimeout(this._hideTimeout),this.el.show(),this._show=!0},r.prototype.setContent=function(t,e,i,n,a){var o=this;z(t)&&Gt(""),this.el&&this._zr.remove(this.el);var s=i.getModel("textStyle");this.el=new yt({style:{rich:e.richTextStyles,text:t,lineHeight:22,borderWidth:1,borderColor:n,textShadowColor:s.get("textShadowColor"),fill:i.get(["textStyle","color"]),padding:Uy(i,"richText"),verticalAlign:"top",align:"left"},z:i.get("z")}),T(["backgroundColor","borderRadius","shadowColor","shadowBlur","shadowOffsetX","shadowOffsetY"],function(u){o.el.style[u]=i.get(u)}),T(["textShadowBlur","textShadowOffsetX","textShadowOffsetY"],function(u){o.el.style[u]=s.get(u)||0}),this._zr.add(this.el);var l=this;this.el.on("mouseover",function(){l._enterable&&(clearTimeout(l._hideTimeout),l._show=!0),l._inContent=!0}),this.el.on("mouseout",function(){l._enterable&&l._show&&l.hideLater(l._hideDelay),l._inContent=!1})},r.prototype.setEnterable=function(t){this._enterable=t},r.prototype.getSize=function(){var t=this.el,e=this.el.getBoundingRect(),i=Kd(t.style);return[e.width+i.left+i.right,e.height+i.top+i.bottom]},r.prototype.moveTo=function(t,e){var i=this.el;if(i){var n=this._styleCoord;Qd(n,this._zr,t,e),t=n[0],e=n[1];var a=i.style,o=vr(a.borderWidth||0),s=Kd(a);i.x=t+o+s.left,i.y=e+o+s.top,i.markRedraw()}},r.prototype._moveIfResized=function(){var t=this._styleCoord[2],e=this._styleCoord[3];this.moveTo(t*this._zr.getWidth(),e*this._zr.getHeight())},r.prototype.hide=function(){this.el&&this.el.hide(),this._show=!1},r.prototype.hideLater=function(t){this._show&&!(this._inContent&&this._enterable)&&!this._alwaysShowContent&&(t?(this._hideDelay=t,this._show=!1,this._hideTimeout=setTimeout(K(this.hide,this),t)):this.hide())},r.prototype.isShow=function(){return this._show},r.prototype.dispose=function(){this._zr.remove(this.el)},r}();function vr(r){return Math.max(0,r)}function Kd(r){var t=vr(r.shadowBlur||0),e=vr(r.shadowOffsetX||0),i=vr(r.shadowOffsetY||0);return{left:vr(t-e),right:vr(t+e),top:vr(t-i),bottom:vr(t+i)}}function Qd(r,t,e,i){r[0]=e,r[1]=i,r[2]=r[0]/t.getWidth(),r[3]=r[1]/t.getHeight()}var lI=new gt({shape:{x:-1,y:-1,width:2,height:2}}),uI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.init=function(e,i){if(!(Z.node||!i.getDom())){var n=e.getComponent("tooltip"),a=this._renderMode=AS(n.get("renderMode"));this._tooltipContent=a==="richText"?new sI(i):new oI(i,{appendTo:n.get("appendToBody",!0)?"body":n.get("appendTo",!0)})}},t.prototype.render=function(e,i,n){if(!(Z.node||!n.getDom())){this.group.removeAll(),this._tooltipModel=e,this._ecModel=i,this._api=n;var a=this._tooltipContent;a.update(e),a.setEnterable(e.get("enterable")),this._initGlobalListener(),this._keepShow(),this._renderMode!=="richText"&&e.get("transitionDuration")?om(this,"_updatePosition",50,"fixRate"):Ou(this,"_updatePosition")}},t.prototype._initGlobalListener=function(){var e=this._tooltipModel,i=e.get("triggerOn");T0("itemTooltip",this._api,K(function(n,a,o){i!=="none"&&(i.indexOf(n)>=0?this._tryShow(a,o):n==="leave"&&this._hide(o))},this))},t.prototype._keepShow=function(){var e=this._tooltipModel,i=this._ecModel,n=this._api,a=e.get("triggerOn");if(this._lastX!=null&&this._lastY!=null&&a!=="none"&&a!=="click"){var o=this;clearTimeout(this._refreshUpdateTimeout),this._refreshUpdateTimeout=setTimeout(function(){!n.isDisposed()&&o.manuallyShowTip(e,i,n,{x:o._lastX,y:o._lastY,dataByCoordSys:o._lastDataByCoordSys})})}},t.prototype.manuallyShowTip=function(e,i,n,a){if(!(a.from===this.uid||Z.node||!n.getDom())){var o=jd(a,n);this._ticket="";var s=a.dataByCoordSys,l=cI(a,i,n);if(l){var u=l.el.getBoundingRect().clone();u.applyTransform(l.el.transform),this._tryShow({offsetX:u.x+u.width/2,offsetY:u.y+u.height/2,target:l.el,position:a.position,positionDefault:"bottom"},o)}else if(a.tooltip&&a.x!=null&&a.y!=null){var f=lI;f.x=a.x,f.y=a.y,f.update(),tt(f).tooltipConfig={name:null,option:a.tooltip},this._tryShow({offsetX:a.x,offsetY:a.y,target:f},o)}else if(s)this._tryShow({offsetX:a.x,offsetY:a.y,position:a.position,dataByCoordSys:s,tooltipOption:a.tooltipOption},o);else if(a.seriesIndex!=null){if(this._manuallyAxisShowTip(e,i,n,a))return;var h=M0(a,i),c=h.point[0],v=h.point[1];c!=null&&v!=null&&this._tryShow({offsetX:c,offsetY:v,target:h.el,position:a.position,positionDefault:"bottom"},o)}else a.x!=null&&a.y!=null&&(n.dispatchAction({type:"updateAxisPointer",x:a.x,y:a.y}),this._tryShow({offsetX:a.x,offsetY:a.y,position:a.position,target:n.getZr().findHover(a.x,a.y).target},o))}},t.prototype.manuallyHideTip=function(e,i,n,a){var o=this._tooltipContent;this._tooltipModel&&o.hideLater(this._tooltipModel.get("hideDelay")),this._lastX=this._lastY=this._lastDataByCoordSys=null,a.from!==this.uid&&this._hide(jd(a,n))},t.prototype._manuallyAxisShowTip=function(e,i,n,a){var o=a.seriesIndex,s=a.dataIndex,l=i.getComponent("axisPointer").coordSysAxesInfo;if(!(o==null||s==null||l==null)){var u=i.getSeriesByIndex(o);if(u){var f=u.getData(),h=mn([f.getItemModel(s),u,(u.coordinateSystem||{}).model],this._tooltipModel);if(h.get("trigger")==="axis")return n.dispatchAction({type:"updateAxisPointer",seriesIndex:o,dataIndex:s,position:a.position}),!0}}},t.prototype._tryShow=function(e,i){var n=e.target,a=this._tooltipModel;if(a){this._lastX=e.offsetX,this._lastY=e.offsetY;var o=e.dataByCoordSys;if(o&&o.length)this._showAxisTooltip(o,e);else if(n){var s=tt(n);if(s.ssrType==="legend")return;this._lastDataByCoordSys=null;var l,u;ki(n,function(f){if(tt(f).dataIndex!=null)return l=f,!0;if(tt(f).tooltipConfig!=null)return u=f,!0},!0),l?this._showSeriesItemTooltip(e,l,i):u?this._showComponentItemTooltip(e,u,i):this._hide(i)}else this._lastDataByCoordSys=null,this._hide(i)}},t.prototype._showOrMove=function(e,i){var n=e.get("showDelay");i=K(i,this),clearTimeout(this._showTimout),n>0?this._showTimout=setTimeout(i,n):i()},t.prototype._showAxisTooltip=function(e,i){var n=this._ecModel,a=this._tooltipModel,o=[i.offsetX,i.offsetY],s=mn([i.tooltipOption],a),l=this._renderMode,u=[],f=Jn("section",{blocks:[],noHeader:!0}),h=[],c=new cl;T(e,function(m){T(m.dataByAxis,function(_){var S=n.getComponent(_.axisDim+"Axis",_.axisIndex),b=_.value;if(!(!S||b==null)){var w=b0(b,S.axis,n,_.seriesDataIndices,_.valueLabelOpt),x=Jn("section",{header:w,noHeader:!be(w),sortBlocks:!0,blocks:[]});f.blocks.push(x),T(_.seriesDataIndices,function(C){var D=n.getSeriesByIndex(C.seriesIndex),A=C.dataIndexInside,M=D.getDataParams(A);if(!(M.dataIndex<0)){M.axisDim=_.axisDim,M.axisIndex=_.axisIndex,M.axisType=_.axisType,M.axisId=_.axisId,M.axisValue=yh(S.axis,{value:b}),M.axisValueLabel=w,M.marker=c.makeTooltipMarker("item",hi(M.color),l);var I=$v(D.formatTooltip(A,!0,null)),L=I.frag;if(L){var P=mn([D],a).get("valueFormatter");x.blocks.push(P?O({valueFormatter:P},L):L)}I.text&&h.push(I.text),u.push(M)}})}})}),f.blocks.reverse(),h.reverse();var v=i.position,d=s.get("order"),y=Qv(f,c,l,d,n.get("useUTC"),s.get("textStyle"));y&&h.unshift(y);var p=l==="richText"?`

`:"<br/>",g=h.join(p);this._showOrMove(s,function(){this._updateContentNotChangedOnAxis(e,u)?this._updatePosition(s,v,o[0],o[1],this._tooltipContent,u):this._showTooltipContent(s,g,u,Math.random()+"",o[0],o[1],v,null,c)})},t.prototype._showSeriesItemTooltip=function(e,i,n){var a=this._ecModel,o=tt(i),s=o.seriesIndex,l=a.getSeriesByIndex(s),u=o.dataModel||l,f=o.dataIndex,h=o.dataType,c=u.getData(h),v=this._renderMode,d=e.positionDefault,y=mn([c.getItemModel(f),u,l&&(l.coordinateSystem||{}).model],this._tooltipModel,d?{position:d}:null),p=y.get("trigger");if(!(p!=null&&p!=="item")){var g=u.getDataParams(f,h),m=new cl;g.marker=m.makeTooltipMarker("item",hi(g.color),v);var _=$v(u.formatTooltip(f,!1,h)),S=y.get("order"),b=y.get("valueFormatter"),w=_.frag,x=w?Qv(b?O({valueFormatter:b},w):w,m,v,S,a.get("useUTC"),y.get("textStyle")):_.text,C="item_"+u.name+"_"+f;this._showOrMove(y,function(){this._showTooltipContent(y,x,g,C,e.offsetX,e.offsetY,e.position,e.target,m)}),n({type:"showTip",dataIndexInside:f,dataIndex:c.getRawIndex(f),seriesIndex:s,from:this.uid})}},t.prototype._showComponentItemTooltip=function(e,i,n){var a=this._renderMode==="html",o=tt(i),s=o.tooltipConfig,l=s.option||{},u=l.encodeHTMLContent;if(V(l)){var f=l;l={content:f,formatter:f},u=!0}u&&a&&l.content&&(l=U(l),l.content=Wt(l.content));var h=[l],c=this._ecModel.getComponent(o.componentMainType,o.componentIndex);c&&h.push(c),h.push({formatter:l.content});var v=e.positionDefault,d=mn(h,this._tooltipModel,v?{position:v}:null),y=d.get("content"),p=Math.random()+"",g=new cl;this._showOrMove(d,function(){var m=U(d.get("formatterParams")||{});this._showTooltipContent(d,y,m,p,e.offsetX,e.offsetY,e.position,i,g)}),n({type:"showTip",from:this.uid})},t.prototype._showTooltipContent=function(e,i,n,a,o,s,l,u,f){if(this._ticket="",!(!e.get("showContent")||!e.get("show"))){var h=this._tooltipContent;h.setEnterable(e.get("enterable"));var c=e.get("formatter");l=l||e.get("position");var v=i,d=this._getNearestPoint([o,s],n,e.get("trigger"),e.get("borderColor")),y=d.color;if(c)if(V(c)){var p=e.ecModel.get("useUTC"),g=N(n)?n[0]:n,m=g&&g.axisType&&g.axisType.indexOf("time")>=0;v=c,m&&(v=pa(g.axisValue,v,p)),v=qf(v,n,!0)}else if(Y(c)){var _=K(function(S,b){S===this._ticket&&(h.setContent(b,f,e,y,l),this._updatePosition(e,l,o,s,h,n,u))},this);this._ticket=a,v=c(n,a,_)}else v=c;h.setContent(v,f,e,y,l),h.show(e,y),this._updatePosition(e,l,o,s,h,n,u)}},t.prototype._getNearestPoint=function(e,i,n,a){if(n==="axis"||N(i))return{color:a||(this._renderMode==="html"?"#fff":"none")};if(!N(i))return{color:a||i.color||i.borderColor}},t.prototype._updatePosition=function(e,i,n,a,o,s,l){var u=this._api.getWidth(),f=this._api.getHeight();i=i||e.get("position");var h=o.getSize(),c=e.get("align"),v=e.get("verticalAlign"),d=l&&l.getBoundingRect().clone();if(l&&d.applyTransform(l.transform),Y(i)&&(i=i([n,a],s,o.el,d,{viewSize:[u,f],contentSize:h.slice()})),N(i))n=Ot(i[0],u),a=Ot(i[1],f);else if(z(i)){var y=i;y.width=h[0],y.height=h[1];var p=Zi(y,{width:u,height:f});n=p.x,a=p.y,c=null,v=null}else if(V(i)&&l){var g=vI(i,d,h,e.get("borderWidth"));n=g[0],a=g[1]}else{var g=fI(n,a,o,u,f,c?null:20,v?null:20);n=g[0],a=g[1]}if(c&&(n-=Jd(c)?h[0]/2:c==="right"?h[0]:0),v&&(a-=Jd(v)?h[1]/2:v==="bottom"?h[1]:0),D0(e)){var g=hI(n,a,o,u,f);n=g[0],a=g[1]}o.moveTo(n,a)},t.prototype._updateContentNotChangedOnAxis=function(e,i){var n=this._lastDataByCoordSys,a=this._cbParamsList,o=!!n&&n.length===e.length;return o&&T(n,function(s,l){var u=s.dataByAxis||[],f=e[l]||{},h=f.dataByAxis||[];o=o&&u.length===h.length,o&&T(u,function(c,v){var d=h[v]||{},y=c.seriesDataIndices||[],p=d.seriesDataIndices||[];o=o&&c.value===d.value&&c.axisType===d.axisType&&c.axisId===d.axisId&&y.length===p.length,o&&T(y,function(g,m){var _=p[m];o=o&&g.seriesIndex===_.seriesIndex&&g.dataIndex===_.dataIndex}),a&&T(c.seriesDataIndices,function(g){var m=g.seriesIndex,_=i[m],S=a[m];_&&S&&S.data!==_.data&&(o=!1)})})}),this._lastDataByCoordSys=e,this._cbParamsList=i,!!o},t.prototype._hide=function(e){this._lastDataByCoordSys=null,e({type:"hideTip",from:this.uid})},t.prototype.dispose=function(e,i){Z.node||!i.getDom()||(Ou(this,"_updatePosition"),this._tooltipContent.dispose(),rf("itemTooltip",i))},t.type="tooltip",t}(Xt);function mn(r,t,e){var i=t.ecModel,n;e?(n=new xt(e,i,i),n=new xt(t.option,n,i)):n=t;for(var a=r.length-1;a>=0;a--){var o=r[a];o&&(o instanceof xt&&(o=o.get("tooltip",!0)),V(o)&&(o={formatter:o}),o&&(n=new xt(o,n,i)))}return n}function jd(r,t){return r.dispatchAction||K(t.dispatchAction,t)}function fI(r,t,e,i,n,a,o){var s=e.getSize(),l=s[0],u=s[1];return a!=null&&(r+l+a+2>i?r-=l+a:r+=a),o!=null&&(t+u+o>n?t-=u+o:t+=o),[r,t]}function hI(r,t,e,i,n){var a=e.getSize(),o=a[0],s=a[1];return r=Math.min(r+o,i)-o,t=Math.min(t+s,n)-s,r=Math.max(r,0),t=Math.max(t,0),[r,t]}function vI(r,t,e,i){var n=e[0],a=e[1],o=Math.ceil(Math.SQRT2*i)+8,s=0,l=0,u=t.width,f=t.height;switch(r){case"inside":s=t.x+u/2-n/2,l=t.y+f/2-a/2;break;case"top":s=t.x+u/2-n/2,l=t.y-a-o;break;case"bottom":s=t.x+u/2-n/2,l=t.y+f+o;break;case"left":s=t.x-n-o,l=t.y+f/2-a/2;break;case"right":s=t.x+u+o,l=t.y+f/2-a/2}return[s,l]}function Jd(r){return r==="center"||r==="middle"}function cI(r,t,e){var i=Tf(r).queryOptionMap,n=i.keys()[0];if(!(!n||n==="series")){var a=da(t,n,i.get(n),{useDefault:!1,enableAll:!1,enableNone:!1}),o=a.models[0];if(o){var s=e.getViewOfComponentModel(o),l;if(s.group.traverse(function(u){var f=tt(u).tooltipConfig;if(f&&f.name===r.name)return l=u,!0}),l)return{componentMainType:n,componentIndex:o.componentIndex,el:l}}}}function yL(r){Ui(C0),r.registerComponentModel(KA),r.registerComponentView(uI),r.registerAction({type:"showTip",event:"showTip",update:"tooltip:manuallyShowTip"},At),r.registerAction({type:"hideTip",event:"hideTip",update:"tooltip:manuallyHideTip"},At)}var tp=T;function ep(r){if(r){for(var t in r)if(r.hasOwnProperty(t))return!0}}function rp(r,t,e){var i={};return tp(t,function(a){var o=i[a]=n();tp(r[a],function(s,l){if(It.isValidType(l)){var u={type:l,visual:s};e&&e(u,a),o[l]=new It(u),l==="opacity"&&(u=U(u),u.type="colorAlpha",o.__hidden.__alphaForOpacity=new It(u))}})}),i;function n(){var a=function(){};a.prototype.__hidden=a.prototype;var o=new a;return o}}function dI(r,t,e){var i;T(e,function(n){t.hasOwnProperty(n)&&ep(t[n])&&(i=!0)}),i&&T(e,function(n){t.hasOwnProperty(n)&&ep(t[n])?r[n]=U(t[n]):delete r[n]})}function pI(r,t,e,i){var n={};return T(r,function(a){var o=It.prepareVisualTypes(t[a]);n[a]=o}),{progress:function(o,s){var l;i!=null&&(l=s.getDimensionIndex(i));function u(b){return Dm(s,h,b)}function f(b,w){hC(s,h,b,w)}for(var h,c=s.getStore();(h=o.next())!=null;){var v=s.getRawDataItem(h);if(!(v&&v.visualMap===!1))for(var d=i!=null?c.get(l,h):h,y=e(d),p=t[y],g=n[y],m=0,_=g.length;m<_;m++){var S=g[m];p[S]&&p[S].applyVisual(d,u,f)}}}}}var gI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e.layoutMode={type:"box",ignoreSize:!0},e}return t.type="title",t.defaultOption={z:6,show:!0,text:"",target:"blank",subtext:"",subtarget:"blank",left:0,top:0,backgroundColor:"rgba(0,0,0,0)",borderColor:"#ccc",borderWidth:0,padding:5,itemGap:10,textStyle:{fontSize:18,fontWeight:"bold",color:"#464646"},subtextStyle:{fontSize:12,color:"#6E7079"}},t}(rt),yI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.render=function(e,i,n){if(this.group.removeAll(),!!e.get("show")){var a=this.group,o=e.getModel("textStyle"),s=e.getModel("subtextStyle"),l=e.get("textAlign"),u=Q(e.get("textBaseline"),e.get("textVerticalAlign")),f=new yt({style:he(o,{text:e.get("text"),fill:o.getTextColor()},{disableBox:!0}),z2:10}),h=f.getBoundingRect(),c=e.get("subtext"),v=new yt({style:he(s,{text:c,fill:s.getTextColor(),y:h.height+e.get("itemGap"),verticalAlign:"top"},{disableBox:!0}),z2:10}),d=e.get("link"),y=e.get("sublink"),p=e.get("triggerEvent",!0);f.silent=!d&&!p,v.silent=!y&&!p,d&&f.on("click",function(){Yv(d,"_"+e.get("target"))}),y&&v.on("click",function(){Yv(y,"_"+e.get("subtarget"))}),tt(f).eventData=tt(v).eventData=p?{componentType:"title",componentIndex:e.componentIndex}:null,a.add(f),c&&a.add(v);var g=a.getBoundingRect(),m=e.getBoxLayoutParams();m.width=g.width,m.height=g.height;var _=Zi(m,{width:n.getWidth(),height:n.getHeight()},e.get("padding"));l||(l=e.get("left")||e.get("right"),l==="middle"&&(l="center"),l==="right"?_.x+=_.width:l==="center"&&(_.x+=_.width/2)),u||(u=e.get("top")||e.get("bottom"),u==="center"&&(u="middle"),u==="bottom"?_.y+=_.height:u==="middle"&&(_.y+=_.height/2),u=u||"top"),a.x=_.x,a.y=_.y,a.markRedraw();var S={align:l,verticalAlign:u};f.setStyle(S),v.setStyle(S),g=a.getBoundingRect();var b=_.margin,w=e.getItemStyle(["color","opacity"]);w.fill=e.get("backgroundColor");var x=new gt({shape:{x:g.x-b[3],y:g.y-b[0],width:g.width+b[1]+b[3],height:g.height+b[0]+b[2],r:e.get("borderRadius")},style:w,subPixelOptimize:!0,silent:!0});a.add(x)}},t.type="title",t}(Xt);function mL(r){r.registerComponentModel(gI),r.registerComponentView(yI)}var P0={get:function(r,t,e){var i=U((mI[r]||{})[t]);return e&&N(i)?i[i.length-1]:i}},mI={color:{active:["#006edd","#e0ffff"],inactive:["rgba(0,0,0,0)"]},colorHue:{active:[0,360],inactive:[0,0]},colorSaturation:{active:[.3,1],inactive:[0,0]},colorLightness:{active:[.9,.5],inactive:[0,0]},colorAlpha:{active:[.3,1],inactive:[0,0]},opacity:{active:[.3,1],inactive:[0,0]},symbol:{active:["circle","roundRect","diamond"],inactive:["none"]},symbolSize:{active:[10,50],inactive:[0,0]}},ip=It.mapVisual,_I=It.eachVisual,SI=N,np=T,wI=wf,bI=ze,qo=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e.stateList=["inRange","outOfRange"],e.replacableOptionKeys=["inRange","outOfRange","target","controller","color"],e.layoutMode={type:"box",ignoreSize:!0},e.dataBound=[-1/0,1/0],e.targetVisuals={},e.controllerVisuals={},e}return t.prototype.init=function(e,i,n){this.mergeDefaultAndTheme(e,n)},t.prototype.optionUpdated=function(e,i){var n=this.option;!i&&dI(n,e,this.replacableOptionKeys),this.textStyleModel=this.getModel("textStyle"),this.resetItemSize(),this.completeVisualOption()},t.prototype.resetVisual=function(e){var i=this.stateList;e=K(e,this),this.controllerVisuals=rp(this.option.controller,i,e),this.targetVisuals=rp(this.option.target,i,e)},t.prototype.getItemSymbol=function(){return null},t.prototype.getTargetSeriesIndices=function(){var e=this.option.seriesIndex,i=[];return e==null||e==="all"?this.ecModel.eachSeries(function(n,a){i.push(a)}):i=Tt(e),i},t.prototype.eachTargetSeries=function(e,i){T(this.getTargetSeriesIndices(),function(n){var a=this.ecModel.getSeriesByIndex(n);a&&e.call(i,a)},this)},t.prototype.isTargetSeries=function(e){var i=!1;return this.eachTargetSeries(function(n){n===e&&(i=!0)}),i},t.prototype.formatValueText=function(e,i,n){var a=this.option,o=a.precision,s=this.dataBound,l=a.formatter,u;n=n||["<",">"],N(e)&&(e=e.slice(),u=!0);var f=i?e:u?[h(e[0]),h(e[1])]:h(e);if(V(l))return l.replace("{value}",u?f[0]:f).replace("{value2}",u?f[1]:f);if(Y(l))return u?l(e[0],e[1]):l(e);if(u)return e[0]===s[0]?n[0]+" "+f[1]:e[1]===s[1]?n[1]+" "+f[0]:f[0]+" - "+f[1];return f;function h(c){return c===s[0]?"min":c===s[1]?"max":(+c).toFixed(Math.min(o,20))}},t.prototype.resetExtent=function(){var e=this.option,i=wI([e.min,e.max]);this._dataExtent=i},t.prototype.getDataDimensionIndex=function(e){var i=this.option.dimension;if(i!=null)return e.getDimensionIndex(i);for(var n=e.dimensions,a=n.length-1;a>=0;a--){var o=n[a],s=e.getDimensionInfo(o);if(!s.isCalculationCoord)return s.storeDimIndex}},t.prototype.getExtent=function(){return this._dataExtent.slice()},t.prototype.completeVisualOption=function(){var e=this.ecModel,i=this.option,n={inRange:i.inRange,outOfRange:i.outOfRange},a=i.target||(i.target={}),o=i.controller||(i.controller={});J(a,n),J(o,n);var s=this.isCategory();l.call(this,a),l.call(this,o),u.call(this,a,"inRange","outOfRange"),f.call(this,o);function l(h){SI(i.color)&&!h.inRange&&(h.inRange={color:i.color.slice().reverse()}),h.inRange=h.inRange||{color:e.get("gradientColor")}}function u(h,c,v){var d=h[c],y=h[v];d&&!y&&(y=h[v]={},np(d,function(p,g){if(It.isValidType(g)){var m=P0.get(g,"inactive",s);m!=null&&(y[g]=m,g==="color"&&!y.hasOwnProperty("opacity")&&!y.hasOwnProperty("colorAlpha")&&(y.opacity=[0,0]))}}))}function f(h){var c=(h.inRange||{}).symbol||(h.outOfRange||{}).symbol,v=(h.inRange||{}).symbolSize||(h.outOfRange||{}).symbolSize,d=this.get("inactiveColor"),y=this.getItemSymbol(),p=y||"roundRect";np(this.stateList,function(g){var m=this.itemSize,_=h[g];_||(_=h[g]={color:s?d:[d]}),_.symbol==null&&(_.symbol=c&&U(c)||(s?p:[p])),_.symbolSize==null&&(_.symbolSize=v&&U(v)||(s?m[0]:[m[0],m[0]])),_.symbol=ip(_.symbol,function(w){return w==="none"?p:w});var S=_.symbolSize;if(S!=null){var b=-1/0;_I(S,function(w){w>b&&(b=w)}),_.symbolSize=ip(S,function(w){return bI(w,[0,b],[0,m[0]],!0)})}},this)}},t.prototype.resetItemSize=function(){this.itemSize=[parseFloat(this.get("itemWidth")),parseFloat(this.get("itemHeight"))]},t.prototype.isCategory=function(){return!!this.option.categories},t.prototype.setSelected=function(e){},t.prototype.getSelected=function(){return null},t.prototype.getValueState=function(e){return null},t.prototype.getVisualMeta=function(e){return null},t.type="visualMap",t.dependencies=["series"],t.defaultOption={show:!0,z:4,seriesIndex:"all",min:0,max:200,left:0,right:null,top:null,bottom:0,itemWidth:null,itemHeight:null,inverse:!1,orient:"vertical",backgroundColor:"rgba(0,0,0,0)",borderColor:"#ccc",contentColor:"#5793f3",inactiveColor:"#aaa",borderWidth:0,padding:5,textGap:10,precision:0,textStyle:{color:"#333"}},t}(rt),ap=[20,140],xI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.optionUpdated=function(e,i){r.prototype.optionUpdated.apply(this,arguments),this.resetExtent(),this.resetVisual(function(n){n.mappingMethod="linear",n.dataExtent=this.getExtent()}),this._resetRange()},t.prototype.resetItemSize=function(){r.prototype.resetItemSize.apply(this,arguments);var e=this.itemSize;(e[0]==null||isNaN(e[0]))&&(e[0]=ap[0]),(e[1]==null||isNaN(e[1]))&&(e[1]=ap[1])},t.prototype._resetRange=function(){var e=this.getExtent(),i=this.option.range;!i||i.auto?(e.auto=1,this.option.range=e):N(i)&&(i[0]>i[1]&&i.reverse(),i[0]=Math.max(i[0],e[0]),i[1]=Math.min(i[1],e[1]))},t.prototype.completeVisualOption=function(){r.prototype.completeVisualOption.apply(this,arguments),T(this.stateList,function(e){var i=this.option.controller[e].symbolSize;i&&i[0]!==i[1]&&(i[0]=i[1]/3)},this)},t.prototype.setSelected=function(e){this.option.range=e.slice(),this._resetRange()},t.prototype.getSelected=function(){var e=this.getExtent(),i=wf((this.get("range")||[]).slice());return i[0]>e[1]&&(i[0]=e[1]),i[1]>e[1]&&(i[1]=e[1]),i[0]<e[0]&&(i[0]=e[0]),i[1]<e[0]&&(i[1]=e[0]),i},t.prototype.getValueState=function(e){var i=this.option.range,n=this.getExtent();return(i[0]<=n[0]||i[0]<=e)&&(i[1]>=n[1]||e<=i[1])?"inRange":"outOfRange"},t.prototype.findTargetDataIndices=function(e){var i=[];return this.eachTargetSeries(function(n){var a=[],o=n.getData();o.each(this.getDataDimensionIndex(o),function(s,l){e[0]<=s&&s<=e[1]&&a.push(l)},this),i.push({seriesId:n.id,dataIndex:a})},this),i},t.prototype.getVisualMeta=function(e){var i=op(this,"outOfRange",this.getExtent()),n=op(this,"inRange",this.option.range.slice()),a=[];function o(v,d){a.push({value:v,color:e(v,d)})}for(var s=0,l=0,u=n.length,f=i.length;l<f&&(!n.length||i[l]<=n[0]);l++)i[l]<n[s]&&o(i[l],"outOfRange");for(var h=1;s<u;s++,h=0)h&&a.length&&o(n[s],"outOfRange"),o(n[s],"inRange");for(var h=1;l<f;l++)(!n.length||n[n.length-1]<i[l])&&(h&&(a.length&&o(a[a.length-1].value,"outOfRange"),h=0),o(i[l],"outOfRange"));var c=a.length;return{stops:a,outerColors:[c?a[0].color:"transparent",c?a[c-1].color:"transparent"]}},t.type="visualMap.continuous",t.defaultOption=Hf(qo.defaultOption,{align:"auto",calculable:!1,hoverLink:!0,realtime:!0,handleIcon:"path://M-11.39,9.77h0a3.5,3.5,0,0,1-3.5,3.5h-22a3.5,3.5,0,0,1-3.5-3.5h0a3.5,3.5,0,0,1,3.5-3.5h22A3.5,3.5,0,0,1-11.39,9.77Z",handleSize:"120%",handleStyle:{borderColor:"#fff",borderWidth:1},indicatorIcon:"circle",indicatorSize:"50%",indicatorStyle:{borderColor:"#fff",borderWidth:2,shadowBlur:2,shadowOffsetX:1,shadowOffsetY:1,shadowColor:"rgba(0,0,0,0.2)"}}),t}(qo);function op(r,t,e){if(e[0]===e[1])return e.slice();for(var i=200,n=(e[1]-e[0])/i,a=e[0],o=[],s=0;s<=i&&a<e[1];s++)o.push(a),a+=n;return o.push(e[1]),o}var R0=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e.autoPositionValues={left:1,right:1,top:1,bottom:1},e}return t.prototype.init=function(e,i){this.ecModel=e,this.api=i},t.prototype.render=function(e,i,n,a){if(this.visualMapModel=e,e.get("show")===!1){this.group.removeAll();return}this.doRender(e,i,n,a)},t.prototype.renderBackground=function(e){var i=this.visualMapModel,n=ga(i.get("padding")||0),a=e.getBoundingRect();e.add(new gt({z2:-1,silent:!0,shape:{x:a.x-n[3],y:a.y-n[0],width:a.width+n[3]+n[1],height:a.height+n[0]+n[2]},style:{fill:i.get("backgroundColor"),stroke:i.get("borderColor"),lineWidth:i.get("borderWidth")}}))},t.prototype.getControllerVisual=function(e,i,n){n=n||{};var a=n.forceState,o=this.visualMapModel,s={};if(i==="color"){var l=o.get("contentColor");s.color=l}function u(v){return s[v]}function f(v,d){s[v]=d}var h=o.controllerVisuals[a||o.getValueState(e)],c=It.prepareVisualTypes(h);return T(c,function(v){var d=h[v];n.convertOpacityToAlpha&&v==="opacity"&&(v="colorAlpha",d=h.__alphaForOpacity),It.dependsOn(v,i)&&d&&d.applyVisual(e,u,f)}),s[i]},t.prototype.positionGroup=function(e){var i=this.visualMapModel,n=this.api;ex(e,i.getBoxLayoutParams(),{width:n.getWidth(),height:n.getHeight()})},t.prototype.doRender=function(e,i,n,a){},t.type="visualMap",t}(Xt),sp=[["left","right","width"],["top","bottom","height"]];function E0(r,t,e){var i=r.option,n=i.align;if(n!=null&&n!=="auto")return n;for(var a={width:t.getWidth(),height:t.getHeight()},o=i.orient==="horizontal"?1:0,s=sp[o],l=[0,null,10],u={},f=0;f<3;f++)u[sp[1-o][f]]=l[f],u[s[f]]=f===2?e[0]:i[s[f]];var h=[["x","width",3],["y","height",0]][o],c=Zi(u,a,i.padding);return s[(c.margin[h[2]]||0)+c[h[0]]+c[h[1]]*.5<a[h[1]]*.5?0:1]}function So(r,t){return T(r||[],function(e){e.dataIndex!=null&&(e.dataIndexInside=e.dataIndex,e.dataIndex=null),e.highlightKey="visualMap"+(t?t.componentIndex:"")}),r}var Pe=ze,TI=T,lp=Math.min,Zl=Math.max,MI=12,CI=6,DI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e._shapes={},e._dataInterval=[],e._handleEnds=[],e._hoverLinkDataIndices=[],e}return t.prototype.init=function(e,i){r.prototype.init.call(this,e,i),this._hoverLinkFromSeriesMouseOver=K(this._hoverLinkFromSeriesMouseOver,this),this._hideIndicator=K(this._hideIndicator,this)},t.prototype.doRender=function(e,i,n,a){(!a||a.type!=="selectDataRange"||a.from!==this.uid)&&this._buildView()},t.prototype._buildView=function(){this.group.removeAll();var e=this.visualMapModel,i=this.group;this._orient=e.get("orient"),this._useHandle=e.get("calculable"),this._resetInterval(),this._renderBar(i);var n=e.get("text");this._renderEndsText(i,n,0),this._renderEndsText(i,n,1),this._updateView(!0),this.renderBackground(i),this._updateView(),this._enableHoverLinkToSeries(),this._enableHoverLinkFromSeries(),this.positionGroup(i)},t.prototype._renderEndsText=function(e,i,n){if(i){var a=i[1-n];a=a!=null?a+"":"";var o=this.visualMapModel,s=o.get("textGap"),l=o.itemSize,u=this._shapes.mainGroup,f=this._applyTransform([l[0]/2,n===0?-s:l[1]+s],u),h=this._applyTransform(n===0?"bottom":"top",u),c=this._orient,v=this.visualMapModel.textStyleModel;this.group.add(new yt({style:he(v,{x:f[0],y:f[1],verticalAlign:c==="horizontal"?"middle":h,align:c==="horizontal"?h:"center",text:a})}))}},t.prototype._renderBar=function(e){var i=this.visualMapModel,n=this._shapes,a=i.itemSize,o=this._orient,s=this._useHandle,l=E0(i,this.api,a),u=n.mainGroup=this._createBarGroup(l),f=new mt;u.add(f),f.add(n.outOfRange=up()),f.add(n.inRange=up(null,s?hp(this._orient):null,K(this._dragHandle,this,"all",!1),K(this._dragHandle,this,"all",!0))),f.setClipPath(new gt({shape:{x:0,y:0,width:a[0],height:a[1],r:3}}));var h=i.textStyleModel.getTextRect("国"),c=Zl(h.width,h.height);s&&(n.handleThumbs=[],n.handleLabels=[],n.handleLabelPoints=[],this._createHandle(i,u,0,a,c,o),this._createHandle(i,u,1,a,c,o)),this._createIndicator(i,u,a,c,o),e.add(u)},t.prototype._createHandle=function(e,i,n,a,o,s){var l=K(this._dragHandle,this,n,!1),u=K(this._dragHandle,this,n,!0),f=Fe(e.get("handleSize"),a[0]),h=Je(e.get("handleIcon"),-f/2,-f/2,f,f,null,!0),c=hp(this._orient);h.attr({cursor:c,draggable:!0,drift:l,ondragend:u,onmousemove:function(g){ta(g.event)}}),h.x=a[0]/2,h.useStyle(e.getModel("handleStyle").getItemStyle()),h.setStyle({strokeNoScale:!0,strokeFirst:!0}),h.style.lineWidth*=2,h.ensureState("emphasis").style=e.getModel(["emphasis","handleStyle"]).getItemStyle(),If(h,!0),i.add(h);var v=this.visualMapModel.textStyleModel,d=new yt({cursor:c,draggable:!0,drift:l,onmousemove:function(g){ta(g.event)},ondragend:u,style:he(v,{x:0,y:0,text:""})});d.ensureState("blur").style={opacity:.1},d.stateTransition={duration:200},this.group.add(d);var y=[f,0],p=this._shapes;p.handleThumbs[n]=h,p.handleLabelPoints[n]=y,p.handleLabels[n]=d},t.prototype._createIndicator=function(e,i,n,a,o){var s=Fe(e.get("indicatorSize"),n[0]),l=Je(e.get("indicatorIcon"),-s/2,-s/2,s,s,null,!0);l.attr({cursor:"move",invisible:!0,silent:!0,x:n[0]/2});var u=e.getModel("indicatorStyle").getItemStyle();if(l instanceof tr){var f=l.style;l.useStyle(O({image:f.image,x:f.x,y:f.y,width:f.width,height:f.height},u))}else l.useStyle(u);i.add(l);var h=this.visualMapModel.textStyleModel,c=new yt({silent:!0,invisible:!0,style:he(h,{x:0,y:0,text:""})});this.group.add(c);var v=[(o==="horizontal"?a/2:CI)+n[0]/2,0],d=this._shapes;d.indicator=l,d.indicatorLabel=c,d.indicatorLabelPoint=v,this._firstShowIndicator=!0},t.prototype._dragHandle=function(e,i,n,a){if(this._useHandle){if(this._dragging=!i,!i){var o=this._applyTransform([n,a],this._shapes.mainGroup,!0);this._updateInterval(e,o[1]),this._hideIndicator(),this._updateView()}i===!this.visualMapModel.get("realtime")&&this.api.dispatchAction({type:"selectDataRange",from:this.uid,visualMapId:this.visualMapModel.id,selected:this._dataInterval.slice()}),i?!this._hovering&&this._clearHoverLinkToSeries():fp(this.visualMapModel)&&this._doHoverLinkToSeries(this._handleEnds[e],!1)}},t.prototype._resetInterval=function(){var e=this.visualMapModel,i=this._dataInterval=e.getSelected(),n=e.getExtent(),a=[0,e.itemSize[1]];this._handleEnds=[Pe(i[0],n,a,!0),Pe(i[1],n,a,!0)]},t.prototype._updateInterval=function(e,i){i=i||0;var n=this.visualMapModel,a=this._handleEnds,o=[0,n.itemSize[1]];MA(i,a,o,e,0);var s=n.getExtent();this._dataInterval=[Pe(a[0],o,s,!0),Pe(a[1],o,s,!0)]},t.prototype._updateView=function(e){var i=this.visualMapModel,n=i.getExtent(),a=this._shapes,o=[0,i.itemSize[1]],s=e?o:this._handleEnds,l=this._createBarVisual(this._dataInterval,n,s,"inRange"),u=this._createBarVisual(n,n,o,"outOfRange");a.inRange.setStyle({fill:l.barColor}).setShape("points",l.barPoints),a.outOfRange.setStyle({fill:u.barColor}).setShape("points",u.barPoints),this._updateHandle(s,l)},t.prototype._createBarVisual=function(e,i,n,a){var o={forceState:a,convertOpacityToAlpha:!0},s=this._makeColorGradient(e,o),l=[this.getControllerVisual(e[0],"symbolSize",o),this.getControllerVisual(e[1],"symbolSize",o)],u=this._createBarPoints(n,l);return{barColor:new hs(0,0,0,1,s),barPoints:u,handlesColor:[s[0].color,s[s.length-1].color]}},t.prototype._makeColorGradient=function(e,i){var n=100,a=[],o=(e[1]-e[0])/n;a.push({color:this.getControllerVisual(e[0],"color",i),offset:0});for(var s=1;s<n;s++){var l=e[0]+o*s;if(l>e[1])break;a.push({color:this.getControllerVisual(l,"color",i),offset:s/n})}return a.push({color:this.getControllerVisual(e[1],"color",i),offset:1}),a},t.prototype._createBarPoints=function(e,i){var n=this.visualMapModel.itemSize;return[[n[0]-i[0],e[0]],[n[0],e[0]],[n[0],e[1]],[n[0]-i[1],e[1]]]},t.prototype._createBarGroup=function(e){var i=this._orient,n=this.visualMapModel.get("inverse");return new mt(i==="horizontal"&&!n?{scaleX:e==="bottom"?1:-1,rotation:Math.PI/2}:i==="horizontal"&&n?{scaleX:e==="bottom"?-1:1,rotation:-Math.PI/2}:i==="vertical"&&!n?{scaleX:e==="left"?1:-1,scaleY:-1}:{scaleX:e==="left"?1:-1})},t.prototype._updateHandle=function(e,i){if(this._useHandle){var n=this._shapes,a=this.visualMapModel,o=n.handleThumbs,s=n.handleLabels,l=a.itemSize,u=a.getExtent();TI([0,1],function(f){var h=o[f];h.setStyle("fill",i.handlesColor[f]),h.y=e[f];var c=Pe(e[f],[0,l[1]],u,!0),v=this.getControllerVisual(c,"symbolSize");h.scaleX=h.scaleY=v/l[0],h.x=l[0]-v/2;var d=Fi(n.handleLabelPoints[f],On(h,this.group));s[f].setStyle({x:d[0],y:d[1],text:a.formatValueText(this._dataInterval[f]),verticalAlign:"middle",align:this._orient==="vertical"?this._applyTransform("left",n.mainGroup):"center"})},this)}},t.prototype._showIndicator=function(e,i,n,a){var o=this.visualMapModel,s=o.getExtent(),l=o.itemSize,u=[0,l[1]],f=this._shapes,h=f.indicator;if(h){h.attr("invisible",!1);var c={convertOpacityToAlpha:!0},v=this.getControllerVisual(e,"color",c),d=this.getControllerVisual(e,"symbolSize"),y=Pe(e,s,u,!0),p=l[0]-d/2,g={x:h.x,y:h.y};h.y=y,h.x=p;var m=Fi(f.indicatorLabelPoint,On(h,this.group)),_=f.indicatorLabel;_.attr("invisible",!1);var S=this._applyTransform("left",f.mainGroup),b=this._orient,w=b==="horizontal";_.setStyle({text:(n||"")+o.formatValueText(i),verticalAlign:w?S:"middle",align:w?"center":S});var x={x:p,y,style:{fill:v}},C={style:{x:m[0],y:m[1]}};if(o.ecModel.isAnimationEnabled()&&!this._firstShowIndicator){var D={duration:100,easing:"cubicInOut",additive:!0};h.x=g.x,h.y=g.y,h.animateTo(x,D),_.animateTo(C,D)}else h.attr(x),_.attr(C);this._firstShowIndicator=!1;var A=this._shapes.handleLabels;if(A)for(var M=0;M<A.length;M++)this.api.enterBlur(A[M])}},t.prototype._enableHoverLinkToSeries=function(){var e=this;this._shapes.mainGroup.on("mousemove",function(i){if(e._hovering=!0,!e._dragging){var n=e.visualMapModel.itemSize,a=e._applyTransform([i.offsetX,i.offsetY],e._shapes.mainGroup,!0,!0);a[1]=lp(Zl(0,a[1]),n[1]),e._doHoverLinkToSeries(a[1],0<=a[0]&&a[0]<=n[0])}}).on("mouseout",function(){e._hovering=!1,!e._dragging&&e._clearHoverLinkToSeries()})},t.prototype._enableHoverLinkFromSeries=function(){var e=this.api.getZr();this.visualMapModel.option.hoverLink?(e.on("mouseover",this._hoverLinkFromSeriesMouseOver,this),e.on("mouseout",this._hideIndicator,this)):this._clearHoverLinkFromSeries()},t.prototype._doHoverLinkToSeries=function(e,i){var n=this.visualMapModel,a=n.itemSize;if(n.option.hoverLink){var o=[0,a[1]],s=n.getExtent();e=lp(Zl(o[0],e),o[1]);var l=AI(n,s,o),u=[e-l,e+l],f=Pe(e,o,s,!0),h=[Pe(u[0],o,s,!0),Pe(u[1],o,s,!0)];u[0]<o[0]&&(h[0]=-1/0),u[1]>o[1]&&(h[1]=1/0),i&&(h[0]===-1/0?this._showIndicator(f,h[1],"< ",l):h[1]===1/0?this._showIndicator(f,h[0],"> ",l):this._showIndicator(f,f,"≈ ",l));var c=this._hoverLinkDataIndices,v=[];(i||fp(n))&&(v=this._hoverLinkDataIndices=n.findTargetDataIndices(h));var d=MS(c,v);this._dispatchHighDown("downplay",So(d[0],n)),this._dispatchHighDown("highlight",So(d[1],n))}},t.prototype._hoverLinkFromSeriesMouseOver=function(e){var i;if(ki(e.target,function(l){var u=tt(l);if(u.dataIndex!=null)return i=u,!0},!0),!!i){var n=this.ecModel.getSeriesByIndex(i.seriesIndex),a=this.visualMapModel;if(a.isTargetSeries(n)){var o=n.getData(i.dataType),s=o.getStore().get(a.getDataDimensionIndex(o),i.dataIndex);isNaN(s)||this._showIndicator(s,s)}}},t.prototype._hideIndicator=function(){var e=this._shapes;e.indicator&&e.indicator.attr("invisible",!0),e.indicatorLabel&&e.indicatorLabel.attr("invisible",!0);var i=this._shapes.handleLabels;if(i)for(var n=0;n<i.length;n++)this.api.leaveBlur(i[n])},t.prototype._clearHoverLinkToSeries=function(){this._hideIndicator();var e=this._hoverLinkDataIndices;this._dispatchHighDown("downplay",So(e,this.visualMapModel)),e.length=0},t.prototype._clearHoverLinkFromSeries=function(){this._hideIndicator();var e=this.api.getZr();e.off("mouseover",this._hoverLinkFromSeriesMouseOver),e.off("mouseout",this._hideIndicator)},t.prototype._applyTransform=function(e,i,n,a){var o=On(i,a?null:this.group);return N(e)?Fi(e,o,n):qg(e,o,n)},t.prototype._dispatchHighDown=function(e,i){i&&i.length&&this.api.dispatchAction({type:e,batch:i})},t.prototype.dispose=function(){this._clearHoverLinkFromSeries(),this._clearHoverLinkToSeries()},t.type="visualMap.continuous",t}(R0);function up(r,t,e,i){return new us({shape:{points:r},draggable:!!e,cursor:t,drift:e,onmousemove:function(n){ta(n.event)},ondragend:i})}function AI(r,t,e){var i=MI/2,n=r.get("hoverLinkDataSize");return n&&(i=Pe(n,t,e,!0)/2),i}function fp(r){var t=r.get("hoverLinkOnHandle");return!!(t??r.get("realtime"))}function hp(r){return r==="vertical"?"ns-resize":"ew-resize"}var II={type:"selectDataRange",event:"dataRangeSelected",update:"update"},LI=function(r,t){t.eachComponent({mainType:"visualMap",query:r},function(e){e.setSelected(r.selected)})},PI=[{createOnAllSeries:!0,reset:function(r,t){var e=[];return t.eachComponent("visualMap",function(i){var n=r.pipelineContext;!i.isTargetSeries(r)||n&&n.large||e.push(pI(i.stateList,i.targetVisuals,K(i.getValueState,i),i.getDataDimensionIndex(r.getData())))}),e}},{createOnAllSeries:!0,reset:function(r,t){var e=r.getData(),i=[];t.eachComponent("visualMap",function(n){if(n.isTargetSeries(r)){var a=n.getVisualMeta(K(RI,null,r,n))||{stops:[],outerColors:[]},o=n.getDataDimensionIndex(e);o>=0&&(a.dimension=o,i.push(a))}}),r.getData().setVisual("visualMeta",i)}}];function RI(r,t,e,i){for(var n=t.targetVisuals[i],a=It.prepareVisualTypes(n),o={color:Am(r.getData(),"color")},s=0,l=a.length;s<l;s++){var u=a[s],f=n[u==="opacity"?"__alphaForOpacity":u];f&&f.applyVisual(e,h,c)}return o.color;function h(v){return o[v]}function c(v,d){o[v]=d}}var vp=T;function EI(r){var t=r&&r.visualMap;N(t)||(t=t?[t]:[]),vp(t,function(e){if(e){Li(e,"splitList")&&!Li(e,"pieces")&&(e.pieces=e.splitList,delete e.splitList);var i=e.pieces;i&&N(i)&&vp(i,function(n){z(n)&&(Li(n,"start")&&!Li(n,"min")&&(n.min=n.start),Li(n,"end")&&!Li(n,"max")&&(n.max=n.end))})}})}function Li(r,t){return r&&r.hasOwnProperty&&r.hasOwnProperty(t)}var cp=!1;function O0(r){cp||(cp=!0,r.registerSubTypeDefaulter("visualMap",function(t){return!t.categories&&(!(t.pieces?t.pieces.length>0:t.splitNumber>0)||t.calculable)?"continuous":"piecewise"}),r.registerAction(II,LI),T(PI,function(t){r.registerVisual(r.PRIORITY.VISUAL.COMPONENT,t)}),r.registerPreprocessor(EI))}function OI(r){r.registerComponentModel(xI),r.registerComponentView(DI),O0(r)}var kI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e._pieceList=[],e}return t.prototype.optionUpdated=function(e,i){r.prototype.optionUpdated.apply(this,arguments),this.resetExtent();var n=this._mode=this._determineMode();this._pieceList=[],BI[this._mode].call(this,this._pieceList),this._resetSelected(e,i);var a=this.option.categories;this.resetVisual(function(o,s){n==="categories"?(o.mappingMethod="category",o.categories=U(a)):(o.dataExtent=this.getExtent(),o.mappingMethod="piecewise",o.pieceList=H(this._pieceList,function(l){return l=U(l),s!=="inRange"&&(l.visual=null),l}))})},t.prototype.completeVisualOption=function(){var e=this.option,i={},n=It.listVisualTypes(),a=this.isCategory();T(e.pieces,function(s){T(n,function(l){s.hasOwnProperty(l)&&(i[l]=1)})}),T(i,function(s,l){var u=!1;T(this.stateList,function(f){u=u||o(e,f,l)||o(e.target,f,l)},this),!u&&T(this.stateList,function(f){(e[f]||(e[f]={}))[l]=P0.get(l,f==="inRange"?"active":"inactive",a)})},this);function o(s,l,u){return s&&s[l]&&s[l].hasOwnProperty(u)}r.prototype.completeVisualOption.apply(this,arguments)},t.prototype._resetSelected=function(e,i){var n=this.option,a=this._pieceList,o=(i?n:e).selected||{};if(n.selected=o,T(a,function(l,u){var f=this.getSelectedMapKey(l);o.hasOwnProperty(f)||(o[f]=!0)},this),n.selectedMode==="single"){var s=!1;T(a,function(l,u){var f=this.getSelectedMapKey(l);o[f]&&(s?o[f]=!1:s=!0)},this)}},t.prototype.getItemSymbol=function(){return this.get("itemSymbol")},t.prototype.getSelectedMapKey=function(e){return this._mode==="categories"?e.value+"":e.index+""},t.prototype.getPieceList=function(){return this._pieceList},t.prototype._determineMode=function(){var e=this.option;return e.pieces&&e.pieces.length>0?"pieces":this.option.categories?"categories":"splitNumber"},t.prototype.setSelected=function(e){this.option.selected=U(e)},t.prototype.getValueState=function(e){var i=It.findPieceIndex(e,this._pieceList);return i!=null&&this.option.selected[this.getSelectedMapKey(this._pieceList[i])]?"inRange":"outOfRange"},t.prototype.findTargetDataIndices=function(e){var i=[],n=this._pieceList;return this.eachTargetSeries(function(a){var o=[],s=a.getData();s.each(this.getDataDimensionIndex(s),function(l,u){var f=It.findPieceIndex(l,n);f===e&&o.push(u)},this),i.push({seriesId:a.id,dataIndex:o})},this),i},t.prototype.getRepresentValue=function(e){var i;if(this.isCategory())i=e.value;else if(e.value!=null)i=e.value;else{var n=e.interval||[];i=n[0]===-1/0&&n[1]===1/0?0:(n[0]+n[1])/2}return i},t.prototype.getVisualMeta=function(e){if(this.isCategory())return;var i=[],n=["",""],a=this;function o(f,h){var c=a.getRepresentValue({interval:f});h||(h=a.getValueState(c));var v=e(c,h);f[0]===-1/0?n[0]=v:f[1]===1/0?n[1]=v:i.push({value:f[0],color:v},{value:f[1],color:v})}var s=this._pieceList.slice();if(!s.length)s.push({interval:[-1/0,1/0]});else{var l=s[0].interval[0];l!==-1/0&&s.unshift({interval:[-1/0,l]}),l=s[s.length-1].interval[1],l!==1/0&&s.push({interval:[l,1/0]})}var u=-1/0;return T(s,function(f){var h=f.interval;h&&(h[0]>u&&o([u,h[0]],"outOfRange"),o(h.slice()),u=h[1])},this),{stops:i,outerColors:n}},t.type="visualMap.piecewise",t.defaultOption=Hf(qo.defaultOption,{selected:null,minOpen:!1,maxOpen:!1,align:"auto",itemWidth:20,itemHeight:14,itemSymbol:"roundRect",pieces:null,categories:null,splitNumber:5,selectedMode:"multiple",itemGap:10,hoverLink:!0}),t}(qo),BI={splitNumber:function(r){var t=this.option,e=Math.min(t.precision,20),i=this.getExtent(),n=t.splitNumber;n=Math.max(parseInt(n,10),1),t.splitNumber=n;for(var a=(i[1]-i[0])/n;+a.toFixed(e)!==a&&e<5;)e++;t.precision=e,a=+a.toFixed(e),t.minOpen&&r.push({interval:[-1/0,i[0]],close:[0,0]});for(var o=0,s=i[0];o<n;s+=a,o++){var l=o===n-1?i[1]:s+a;r.push({interval:[s,l],close:[1,1]})}t.maxOpen&&r.push({interval:[i[1],1/0],close:[0,0]}),du(r),T(r,function(u,f){u.index=f,u.text=this.formatValueText(u.interval)},this)},categories:function(r){var t=this.option;T(t.categories,function(e){r.push({text:this.formatValueText(e,!0),value:e})},this),dp(t,r)},pieces:function(r){var t=this.option;T(t.pieces,function(e,i){z(e)||(e={value:e});var n={text:"",index:i};if(e.label!=null&&(n.text=e.label),e.hasOwnProperty("value")){var a=n.value=e.value;n.interval=[a,a],n.close=[1,1]}else{for(var o=n.interval=[],s=n.close=[0,0],l=[1,0,1],u=[-1/0,1/0],f=[],h=0;h<2;h++){for(var c=[["gte","gt","min"],["lte","lt","max"]][h],v=0;v<3&&o[h]==null;v++)o[h]=e[c[v]],s[h]=l[v],f[h]=v===2;o[h]==null&&(o[h]=u[h])}f[0]&&o[1]===1/0&&(s[0]=0),f[1]&&o[0]===-1/0&&(s[1]=0),o[0]===o[1]&&s[0]&&s[1]&&(n.value=o[0])}n.visual=It.retrieveVisuals(e),r.push(n)},this),dp(t,r),du(r),T(r,function(e){var i=e.close,n=[["<","≤"][i[1]],[">","≥"][i[0]]];e.text=e.text||this.formatValueText(e.value!=null?e.value:e.interval,!1,n)},this)}};function dp(r,t){var e=r.inverse;(r.orient==="vertical"?!e:e)&&t.reverse()}var NI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type=t.type,e}return t.prototype.doRender=function(){var e=this.group;e.removeAll();var i=this.visualMapModel,n=i.get("textGap"),a=i.textStyleModel,o=a.getFont(),s=a.getTextColor(),l=this._getItemAlign(),u=i.itemSize,f=this._getViewData(),h=f.endsText,c=oi(i.get("showLabel",!0),!h);h&&this._renderEndsText(e,h[0],u,c,l),T(f.viewPieceList,function(v){var d=v.piece,y=new mt;y.onclick=K(this._onItemClick,this,d),this._enableHoverLink(y,v.indexInModelPieceList);var p=i.getRepresentValue(d);if(this._createItemSymbol(y,p,[0,0,u[0],u[1]]),c){var g=this.visualMapModel.getValueState(p);y.add(new yt({style:{x:l==="right"?-n:u[0]+n,y:u[1]/2,text:d.text,verticalAlign:"middle",align:l,font:o,fill:s,opacity:g==="outOfRange"?.5:1}}))}e.add(y)},this),h&&this._renderEndsText(e,h[1],u,c,l),tx(i.get("orient"),e,i.get("itemGap")),this.renderBackground(e),this.positionGroup(e)},t.prototype._enableHoverLink=function(e,i){var n=this;e.on("mouseover",function(){return a("highlight")}).on("mouseout",function(){return a("downplay")});var a=function(o){var s=n.visualMapModel;s.option.hoverLink&&n.api.dispatchAction({type:o,batch:So(s.findTargetDataIndices(i),s)})}},t.prototype._getItemAlign=function(){var e=this.visualMapModel,i=e.option;if(i.orient==="vertical")return E0(e,this.api,e.itemSize);var n=i.align;return(!n||n==="auto")&&(n="left"),n},t.prototype._renderEndsText=function(e,i,n,a,o){if(i){var s=new mt,l=this.visualMapModel.textStyleModel;s.add(new yt({style:he(l,{x:a?o==="right"?n[0]:0:n[0]/2,y:n[1]/2,verticalAlign:"middle",align:a?o:"center",text:i})})),e.add(s)}},t.prototype._getViewData=function(){var e=this.visualMapModel,i=H(e.getPieceList(),function(s,l){return{piece:s,indexInModelPieceList:l}}),n=e.get("text"),a=e.get("orient"),o=e.get("inverse");return(a==="horizontal"?o:!o)?i.reverse():n&&(n=n.slice().reverse()),{viewPieceList:i,endsText:n}},t.prototype._createItemSymbol=function(e,i,n){e.add(Je(this.getControllerVisual(i,"symbol"),n[0],n[1],n[2],n[3],this.getControllerVisual(i,"color")))},t.prototype._onItemClick=function(e){var i=this.visualMapModel,n=i.option,a=n.selectedMode;if(a){var o=U(n.selected),s=i.getSelectedMapKey(e);a==="single"||a===!0?(o[s]=!0,T(o,function(l,u){o[u]=u===s})):o[s]=!o[s],this.api.dispatchAction({type:"selectDataRange",from:this.uid,visualMapId:this.visualMapModel.id,selected:o})}},t.type="visualMap.piecewise",t}(R0);function FI(r){r.registerComponentModel(kI),r.registerComponentView(NI),O0(r)}function _L(r){Ui(OI),Ui(FI)}var zI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="dataset",e}return t.prototype.init=function(e,i,n){r.prototype.init.call(this,e,i,n),this._sourceManager=new Fy(this),qv(this)},t.prototype.mergeOption=function(e,i){r.prototype.mergeOption.call(this,e,i),qv(this)},t.prototype.optionUpdated=function(){this._sourceManager.dirty()},t.prototype.getSourceManager=function(){return this._sourceManager},t.type="dataset",t.defaultOption={seriesLayoutBy:Be},t}(rt),VI=function(r){k(t,r);function t(){var e=r!==null&&r.apply(this,arguments)||this;return e.type="dataset",e}return t.type="dataset",t}(Xt);function SL(r){r.registerComponentModel(zI),r.registerComponentView(VI)}function HI(r){return Ss(null,r)}var GI={isDimensionStacked:fi,enableDataStack:xy,getStackedDimension:Ty};function WI(r,t){var e=t;t instanceof xt||(e=new xt(t));var i=a0(e);return i.setExtent(r[0],r[1]),qu(i,e),i}function UI(r){de(r,Jm)}function YI(r,t){return t=t||{},he(r,null,null,t.state!=="normal")}const wL=Object.freeze(Object.defineProperty({__proto__:null,createDimensions:Sb,createList:HI,createScale:WI,createSymbol:Je,createTextStyle:YI,dataStack:GI,enableHoverEmphasis:xg,getECData:tt,getLayoutRect:Zi,mixinAxisModelCommonMethods:UI},Symbol.toStringTag,{value:"Module"})),bL=Object.freeze(Object.defineProperty({__proto__:null,MAX_SAFE_INTEGER:hS,asc:wf,getPercentWithPrecision:lS,getPixelPrecision:ag,getPrecision:Oe,getPrecisionSafe:ng,isNumeric:sg,isRadianAroundZero:Xn,linearMap:ze,nice:xf,numericToNumber:$n,parseDate:ve,quantile:cS,quantity:og,quantityExponent:ns,reformIntervals:du,remRadian:bf,round:pt},Symbol.toStringTag,{value:"Module"})),xL=Object.freeze(Object.defineProperty({__proto__:null,format:pa,parse:ve},Symbol.toStringTag,{value:"Module"})),TL=Object.freeze(Object.defineProperty({__proto__:null,Arc:fs,BezierCurve:Ef,BoundingRect:et,Circle:ls,CompoundPath:zg,Ellipse:Lf,Group:mt,Image:tr,IncrementalDisplayable:Gg,Line:xr,LinearGradient:hs,Polygon:us,Polyline:Rf,RadialGradient:Hg,Rect:gt,Ring:Pf,Sector:Xi,Text:yt,clipPointsByRect:Qg,clipRectByRect:jg,createIcon:Nf,extendPath:Yg,extendShape:Ug,getShapeClass:Xg,getTransform:On,initProps:He,makeImage:kf,makePath:vs,mergePath:Zg,registerShape:pe,resizePath:Bf,updateProps:te},Symbol.toStringTag,{value:"Module"})),ML=Object.freeze(Object.defineProperty({__proto__:null,addCommas:$f,capitalFirst:jb,encodeHTML:Wt,formatTime:Qb,formatTpl:qf,getTextRect:Kb,getTooltipMarker:By,normalizeCssArray:ga,toCamelCase:Zf,truncateText:Pp},Symbol.toStringTag,{value:"Module"})),CL=Object.freeze(Object.defineProperty({__proto__:null,bind:K,clone:U,curry:kt,defaults:nt,each:T,extend:O,filter:wt,indexOf:ot,inherits:uf,isArray:N,isFunction:Y,isObject:z,isString:V,map:H,merge:J,reduce:Ne},Symbol.toStringTag,{value:"Module"}));function DL(r){var t=rt.extend(r);return rt.registerClass(t),t}function AL(r){var t=Xt.extend(r);return Xt.registerClass(t),t}function IL(r){var t=ce.extend(r);return ce.registerClass(t),t}function LL(r){var t=Jt.extend(r);return Jt.registerClass(t),t}function pp(r,t,e){var i=je.createCanvas(),n=t.getWidth(),a=t.getHeight(),o=i.style;return o&&(o.position="absolute",o.left="0",o.top="0",o.width=n+"px",o.height=a+"px",i.setAttribute("data-zr-dom-id",r)),i.width=n*e,i.height=a*e,i}var ql=function(r){k(t,r);function t(e,i,n){var a=r.call(this)||this;a.motionBlur=!1,a.lastFrameAlpha=.7,a.dpr=1,a.virtual=!1,a.config={},a.incremental=!1,a.zlevel=0,a.maxRepaintRectCount=5,a.__dirty=!0,a.__firstTimePaint=!0,a.__used=!1,a.__drawIndex=0,a.__startIndex=0,a.__endIndex=0,a.__prevStartIndex=null,a.__prevEndIndex=null;var o;n=n||Io,typeof e=="string"?o=pp(e,i,n):z(e)&&(o=e,e=o.id),a.id=e,a.dom=o;var s=o.style;return s&&(hf(o),o.onselectstart=function(){return!1},s.padding="0",s.margin="0",s.borderWidth="0"),a.painter=i,a.dpr=n,a}return t.prototype.getElementCount=function(){return this.__endIndex-this.__startIndex},t.prototype.afterBrush=function(){this.__prevStartIndex=this.__startIndex,this.__prevEndIndex=this.__endIndex},t.prototype.initContext=function(){this.ctx=this.dom.getContext("2d"),this.ctx.dpr=this.dpr},t.prototype.setUnpainted=function(){this.__firstTimePaint=!0},t.prototype.createBackBuffer=function(){var e=this.dpr;this.domBack=pp("back-"+this.id,this.painter,e),this.ctxBack=this.domBack.getContext("2d"),e!==1&&this.ctxBack.scale(e,e)},t.prototype.createRepaintRects=function(e,i,n,a){if(this.__firstTimePaint)return this.__firstTimePaint=!1,null;var o=[],s=this.maxRepaintRectCount,l=!1,u=new et(0,0,0,0);function f(m){if(!(!m.isFinite()||m.isZero()))if(o.length===0){var _=new et(0,0,0,0);_.copy(m),o.push(_)}else{for(var S=!1,b=1/0,w=0,x=0;x<o.length;++x){var C=o[x];if(C.intersect(m)){var D=new et(0,0,0,0);D.copy(C),D.union(m),o[x]=D,S=!0;break}else if(l){u.copy(m),u.union(C);var A=m.width*m.height,M=C.width*C.height,I=u.width*u.height,L=I-A-M;L<b&&(b=L,w=x)}}if(l&&(o[w].union(m),S=!0),!S){var _=new et(0,0,0,0);_.copy(m),o.push(_)}l||(l=o.length>=s)}}for(var h=this.__startIndex;h<this.__endIndex;++h){var c=e[h];if(c){var v=c.shouldBePainted(n,a,!0,!0),d=c.__isRendered&&(c.__dirty&Kt||!v)?c.getPrevPaintRect():null;d&&f(d);var y=v&&(c.__dirty&Kt||!c.__isRendered)?c.getPaintRect():null;y&&f(y)}}for(var h=this.__prevStartIndex;h<this.__prevEndIndex;++h){var c=i[h],v=c&&c.shouldBePainted(n,a,!0,!0);if(c&&(!v||!c.__zr)&&c.__isRendered){var d=c.getPrevPaintRect();d&&f(d)}}var p;do{p=!1;for(var h=0;h<o.length;){if(o[h].isZero()){o.splice(h,1);continue}for(var g=h+1;g<o.length;)o[h].intersect(o[g])?(p=!0,o[h].union(o[g]),o.splice(g,1)):g++;h++}}while(p);return this._paintRects=o,o},t.prototype.debugGetPaintRects=function(){return(this._paintRects||[]).slice()},t.prototype.resize=function(e,i){var n=this.dpr,a=this.dom,o=a.style,s=this.domBack;o&&(o.width=e+"px",o.height=i+"px"),a.width=e*n,a.height=i*n,s&&(s.width=e*n,s.height=i*n,n!==1&&this.ctxBack.scale(n,n))},t.prototype.clear=function(e,i,n){var a=this.dom,o=this.ctx,s=a.width,l=a.height;i=i||this.clearColor;var u=this.motionBlur&&!e,f=this.lastFrameAlpha,h=this.dpr,c=this;u&&(this.domBack||this.createBackBuffer(),this.ctxBack.globalCompositeOperation="copy",this.ctxBack.drawImage(a,0,0,s/h,l/h));var v=this.domBack;function d(y,p,g,m){if(o.clearRect(y,p,g,m),i&&i!=="transparent"){var _=void 0;if(sa(i)){var S=i.global||i.__width===g&&i.__height===m;_=S&&i.__canvasGradient||Hu(o,i,{x:0,y:0,width:g,height:m}),i.__canvasGradient=_,i.__width=g,i.__height=m}else Sp(i)&&(i.scaleX=i.scaleX||h,i.scaleY=i.scaleY||h,_=Gu(o,i,{dirty:function(){c.setUnpainted(),c.painter.refresh()}}));o.save(),o.fillStyle=_||i,o.fillRect(y,p,g,m),o.restore()}u&&(o.save(),o.globalAlpha=f,o.drawImage(v,y,p,g,m),o.restore())}!n||u?d(0,0,s,l):n.length&&T(n,function(y){d(y.x*h,y.y*h,y.width*h,y.height*h)})},t}(Ge),gp=1e5,Yr=314159,io=.01,XI=.001;function $I(r){return r?r.__builtin__?!0:!(typeof r.resize!="function"||typeof r.refresh!="function"):!1}function ZI(r,t){var e=document.createElement("div");return e.style.cssText=["position:relative","width:"+r+"px","height:"+t+"px","padding:0","margin:0","border-width:0"].join(";")+";",e}var qI=function(){function r(t,e,i,n){this.type="canvas",this._zlevelList=[],this._prevDisplayList=[],this._layers={},this._layerConfig={},this._needsManuallyCompositing=!1,this.type="canvas";var a=!t.nodeName||t.nodeName.toUpperCase()==="CANVAS";this._opts=i=O({},i||{}),this.dpr=i.devicePixelRatio||Io,this._singleCanvas=a,this.root=t;var o=t.style;o&&(hf(t),t.innerHTML=""),this.storage=e;var s=this._zlevelList;this._prevDisplayList=[];var l=this._layers;if(a){var f=t,h=f.width,c=f.height;i.width!=null&&(h=i.width),i.height!=null&&(c=i.height),this.dpr=i.devicePixelRatio||1,f.width=h*this.dpr,f.height=c*this.dpr,this._width=h,this._height=c;var v=new ql(f,this,this.dpr);v.__builtin__=!0,v.initContext(),l[Yr]=v,v.zlevel=Yr,s.push(Yr),this._domRoot=t}else{this._width=Za(t,0,i),this._height=Za(t,1,i);var u=this._domRoot=ZI(this._width,this._height);t.appendChild(u)}}return r.prototype.getType=function(){return"canvas"},r.prototype.isSingleCanvas=function(){return this._singleCanvas},r.prototype.getViewportRoot=function(){return this._domRoot},r.prototype.getViewportRootOffset=function(){var t=this.getViewportRoot();if(t)return{offsetLeft:t.offsetLeft||0,offsetTop:t.offsetTop||0}},r.prototype.refresh=function(t){var e=this.storage.getDisplayList(!0),i=this._prevDisplayList,n=this._zlevelList;this._redrawId=Math.random(),this._paintList(e,i,t,this._redrawId);for(var a=0;a<n.length;a++){var o=n[a],s=this._layers[o];if(!s.__builtin__&&s.refresh){var l=a===0?this._backgroundColor:null;s.refresh(l)}}return this._opts.useDirtyRect&&(this._prevDisplayList=e.slice()),this},r.prototype.refreshHover=function(){this._paintHoverList(this.storage.getDisplayList(!1))},r.prototype._paintHoverList=function(t){var e=t.length,i=this._hoverlayer;if(i&&i.clear(),!!e){for(var n={inHover:!0,viewWidth:this._width,viewHeight:this._height},a,o=0;o<e;o++){var s=t[o];s.__inHover&&(i||(i=this._hoverlayer=this.getLayer(gp)),a||(a=i.ctx,a.save()),jr(a,s,n,o===e-1))}a&&a.restore()}},r.prototype.getHoverLayer=function(){return this.getLayer(gp)},r.prototype.paintOne=function(t,e){Rm(t,e)},r.prototype._paintList=function(t,e,i,n){if(this._redrawId===n){i=i||!1,this._updateLayerStatus(t);var a=this._doPaintList(t,e,i),o=a.finished,s=a.needsRefreshHover;if(this._needsManuallyCompositing&&this._compositeManually(),s&&this._paintHoverList(t),o)this.eachLayer(function(u){u.afterBrush&&u.afterBrush()});else{var l=this;Vo(function(){l._paintList(t,e,i,n)})}}},r.prototype._compositeManually=function(){var t=this.getLayer(Yr).ctx,e=this._domRoot.width,i=this._domRoot.height;t.clearRect(0,0,e,i),this.eachBuiltinLayer(function(n){n.virtual&&t.drawImage(n.dom,0,0,e,i)})},r.prototype._doPaintList=function(t,e,i){for(var n=this,a=[],o=this._opts.useDirtyRect,s=0;s<this._zlevelList.length;s++){var l=this._zlevelList[s],u=this._layers[l];u.__builtin__&&u!==this._hoverlayer&&(u.__dirty||i)&&a.push(u)}for(var f=!0,h=!1,c=function(y){var p=a[y],g=p.ctx,m=o&&p.createRepaintRects(t,e,v._width,v._height),_=i?p.__startIndex:p.__drawIndex,S=!i&&p.incremental&&Date.now,b=S&&Date.now(),w=p.zlevel===v._zlevelList[0]?v._backgroundColor:null;if(p.__startIndex===p.__endIndex)p.clear(!1,w,m);else if(_===p.__startIndex){var x=t[_];(!x.incremental||!x.notClear||i)&&p.clear(!1,w,m)}_===-1&&(console.error("For some unknown reason. drawIndex is -1"),_=p.__startIndex);var C,D=function(L){var P={inHover:!1,allClipped:!1,prevEl:null,viewWidth:n._width,viewHeight:n._height};for(C=_;C<p.__endIndex;C++){var R=t[C];if(R.__inHover&&(h=!0),n._doPaintEl(R,p,o,L,P,C===p.__endIndex-1),S){var E=Date.now()-b;if(E>15)break}}P.prevElClipPaths&&g.restore()};if(m)if(m.length===0)C=p.__endIndex;else for(var A=v.dpr,M=0;M<m.length;++M){var I=m[M];g.save(),g.beginPath(),g.rect(I.x*A,I.y*A,I.width*A,I.height*A),g.clip(),D(I),g.restore()}else g.save(),D(),g.restore();p.__drawIndex=C,p.__drawIndex<p.__endIndex&&(f=!1)},v=this,d=0;d<a.length;d++)c(d);return Z.wxa&&T(this._layers,function(y){y&&y.ctx&&y.ctx.draw&&y.ctx.draw()}),{finished:f,needsRefreshHover:h}},r.prototype._doPaintEl=function(t,e,i,n,a,o){var s=e.ctx;if(i){var l=t.getPaintRect();(!n||l&&l.intersect(n))&&(jr(s,t,a,o),t.setPrevPaintRect(l))}else jr(s,t,a,o)},r.prototype.getLayer=function(t,e){this._singleCanvas&&!this._needsManuallyCompositing&&(t=Yr);var i=this._layers[t];return i||(i=new ql("zr_"+t,this,this.dpr),i.zlevel=t,i.__builtin__=!0,this._layerConfig[t]?J(i,this._layerConfig[t],!0):this._layerConfig[t-io]&&J(i,this._layerConfig[t-io],!0),e&&(i.virtual=e),this.insertLayer(t,i),i.initContext()),i},r.prototype.insertLayer=function(t,e){var i=this._layers,n=this._zlevelList,a=n.length,o=this._domRoot,s=null,l=-1;if(!i[t]&&$I(e)){if(a>0&&t>n[0]){for(l=0;l<a-1&&!(n[l]<t&&n[l+1]>t);l++);s=i[n[l]]}if(n.splice(l+1,0,t),i[t]=e,!e.virtual)if(s){var u=s.dom;u.nextSibling?o.insertBefore(e.dom,u.nextSibling):o.appendChild(e.dom)}else o.firstChild?o.insertBefore(e.dom,o.firstChild):o.appendChild(e.dom);e.painter||(e.painter=this)}},r.prototype.eachLayer=function(t,e){for(var i=this._zlevelList,n=0;n<i.length;n++){var a=i[n];t.call(e,this._layers[a],a)}},r.prototype.eachBuiltinLayer=function(t,e){for(var i=this._zlevelList,n=0;n<i.length;n++){var a=i[n],o=this._layers[a];o.__builtin__&&t.call(e,o,a)}},r.prototype.eachOtherLayer=function(t,e){for(var i=this._zlevelList,n=0;n<i.length;n++){var a=i[n],o=this._layers[a];o.__builtin__||t.call(e,o,a)}},r.prototype.getLayers=function(){return this._layers},r.prototype._updateLayerStatus=function(t){this.eachBuiltinLayer(function(h,c){h.__dirty=h.__used=!1});function e(h){a&&(a.__endIndex!==h&&(a.__dirty=!0),a.__endIndex=h)}if(this._singleCanvas)for(var i=1;i<t.length;i++){var n=t[i];if(n.zlevel!==t[i-1].zlevel||n.incremental){this._needsManuallyCompositing=!0;break}}var a=null,o=0,s,l;for(l=0;l<t.length;l++){var n=t[l],u=n.zlevel,f=void 0;s!==u&&(s=u,o=0),n.incremental?(f=this.getLayer(u+XI,this._needsManuallyCompositing),f.incremental=!0,o=1):f=this.getLayer(u+(o>0?io:0),this._needsManuallyCompositing),f.__builtin__||Qo("ZLevel "+u+" has been used by unkown layer "+f.id),f!==a&&(f.__used=!0,f.__startIndex!==l&&(f.__dirty=!0),f.__startIndex=l,f.incremental?f.__drawIndex=-1:f.__drawIndex=l,e(l),a=f),n.__dirty&Kt&&!n.__inHover&&(f.__dirty=!0,f.incremental&&f.__drawIndex<0&&(f.__drawIndex=l))}e(l),this.eachBuiltinLayer(function(h,c){!h.__used&&h.getElementCount()>0&&(h.__dirty=!0,h.__startIndex=h.__endIndex=h.__drawIndex=0),h.__dirty&&h.__drawIndex<0&&(h.__drawIndex=h.__startIndex)})},r.prototype.clear=function(){return this.eachBuiltinLayer(this._clearLayer),this},r.prototype._clearLayer=function(t){t.clear()},r.prototype.setBackgroundColor=function(t){this._backgroundColor=t,T(this._layers,function(e){e.setUnpainted()})},r.prototype.configLayer=function(t,e){if(e){var i=this._layerConfig;i[t]?J(i[t],e,!0):i[t]=e;for(var n=0;n<this._zlevelList.length;n++){var a=this._zlevelList[n];if(a===t||a===t+io){var o=this._layers[a];J(o,i[t],!0)}}}},r.prototype.delLayer=function(t){var e=this._layers,i=this._zlevelList,n=e[t];n&&(n.dom.parentNode.removeChild(n.dom),delete e[t],i.splice(ot(i,t),1))},r.prototype.resize=function(t,e){if(this._domRoot.style){var i=this._domRoot;i.style.display="none";var n=this._opts,a=this.root;if(t!=null&&(n.width=t),e!=null&&(n.height=e),t=Za(a,0,n),e=Za(a,1,n),i.style.display="",this._width!==t||e!==this._height){i.style.width=t+"px",i.style.height=e+"px";for(var o in this._layers)this._layers.hasOwnProperty(o)&&this._layers[o].resize(t,e);this.refresh(!0)}this._width=t,this._height=e}else{if(t==null||e==null)return;this._width=t,this._height=e,this.getLayer(Yr).resize(t,e)}return this},r.prototype.clearLayer=function(t){var e=this._layers[t];e&&e.clear()},r.prototype.dispose=function(){this.root.innerHTML="",this.root=this.storage=this._domRoot=this._layers=null},r.prototype.getRenderedCanvas=function(t){if(t=t||{},this._singleCanvas&&!this._compositeManually)return this._layers[Yr].dom;var e=new ql("image",this,t.pixelRatio||this.dpr);e.initContext(),e.clear(!1,t.backgroundColor||this._backgroundColor);var i=e.ctx;if(t.pixelRatio<=this.dpr){this.refresh();var n=e.dom.width,a=e.dom.height;this.eachLayer(function(h){h.__builtin__?i.drawImage(h.dom,0,0,n,a):h.renderToCanvas&&(i.save(),h.renderToCanvas(i),i.restore())})}else for(var o={inHover:!1,viewWidth:this._width,viewHeight:this._height},s=this.storage.getDisplayList(!0),l=0,u=s.length;l<u;l++){var f=s[l];jr(i,f,o,l===u-1)}return e.dom},r.prototype.getWidth=function(){return this._width},r.prototype.getHeight=function(){return this._height},r}();function PL(r){r.registerPainter("canvas",qI)}export{vh as $,YD as A,Zm as B,Jt as C,fL as D,cL as E,TL as F,wL as G,oL as H,Rm as I,jI as J,bL as K,pL as L,xt as M,qi as N,jC as O,WC as P,JC as Q,jm as R,_b as S,Dy as T,tD as U,KC as V,QC as W,Km as X,Qm as Y,qm as Z,eD as _,rL as a,di as a0,vL as a1,o_ as a2,oh as a3,xL as a4,CL as a5,JI as a6,nL as a7,QI as a8,iL as a9,KI as aa,PL as b,gL as c,yL as d,mL as e,SL as f,_L as g,rt as h,eL as i,Xt as j,ce as k,tL as l,sL as m,dL as n,aL as o,lL as p,qC as q,uL as r,Z as s,LL as t,Ui as u,DL as v,AL as w,IL as x,ML as y,hL as z};
